Package: emburden
Title: Energy Burden Analysis Using Net Energy Return Methodology
Version: 0.6.2
Authors@R: 
    person("Eric", "Scheier", , "eric@scheier.org", role = c("aut", "cre", "cph"))
Description: Calculate and analyze household energy burden using the Net Energy
    Return aggregation methodology. Functions support weighted statistical
    calculations across geographic and demographic cohorts, with utilities for
    formatting results into publication-ready tables. Methods are based on
    Scheier & Kittner (2022) <doi:10.1038/s41467-021-27673-y>.
License: AGPL (>= 3)
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: dplyr, httr, rappdirs, readr, rlang, scales, spatstat.univar,
        stats, stringr, tibble, tidyr
Suggests: covr, DBI, httptest2, jsonlite, kableExtra, knitr, mockery,
        rmarkdown, RSQLite, rticles, testthat (>= 3.0.0), tinytex,
        withr
VignetteBuilder: knitr
LazyData: true
URL: https://github.com/ericscheier/emburden,
        https://ericscheier.info/emburden/
BugReports: https://github.com/ericscheier/emburden/issues
NeedsCompilation: no
Packaged: 2026-05-18 19:59:08 UTC; runner
Author: Eric Scheier [aut, cre, cph]
Maintainer: Eric Scheier <eric@scheier.org>
Repository: CRAN
Date/Publication: 2026-05-18 20:40:02 UTC
Built: R 4.6.0; ; 2026-05-18 22:54:00 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,energy_burden_comparison)
export("%>%")
export(calculate_weighted_metrics)
export(check_data_sources)
export(clear_all_cache)
export(clear_dataset_cache)
export(colorize)
export(compare_energy_burden)
export(dear_func)
export(energy_burden_func)
export(eroi_func)
export(get_dataset_info)
export(get_income_brackets)
export(lead_to_poverty)
export(list_cohort_columns)
export(list_income_brackets)
export(list_states)
export(load_census_tract_data)
export(load_cohort_data)
export(neb_func)
export(ner_func)
export(process_lead_cohort_data)
export(raw_to_lead)
export(to_big)
export(to_billion_dollar)
export(to_dollar)
export(to_million)
export(to_percent)
importFrom(dplyr,"%>%")
importFrom(stats,weighted.mean)

# Testing Framework for emburden Package

This directory contains comprehensive tests for the emburden package, including unit tests, integration tests, and a local test runner.

## Running Tests

### Quick Local Test Run

From the package root directory:

```bash
Rscript tests/run-tests-locally.R
```

Or from R console:

```r
source("tests/run-tests-locally.R")
```

### Full R CMD Check

To run complete package checks (like GitHub Actions):

```bash
RUN_CMD_CHECK=true Rscript tests/run-tests-locally.R
```

### Run Specific Test Files

From R console:

```r
devtools::load_all()
testthat::test_file("tests/testthat/test-energy-metrics.R")
```

### Coverage Report

```r
covr::package_coverage()
covr::report()  # Opens HTML report in browser
```

## Test Organization

### Phase 1: Critical Tests (Implemented)

- **test-file-validation.R**: Data quality checks, edge cases, fixture generation
- **test-energy-metrics.R**: Core NER, energy burden, and related calculations

### Phase 2: High Priority (Planned)

- **test-data-loaders.R**: HTTP mocking, database tests, file loading
- **test-schema-normalization.R**: Vintage compatibility, bracket aggregation

### Phase 3: Medium Priority (Planned)

- **test-integration.R**: End-to-end workflows
- **test-formatting.R**: Output formatting functions

## Helper Files

- **helper-fixtures.R**: Test data generation and cleanup utilities
  - `create_sample_lead_data()`: Generate realistic test data
  - `create_corrupted_fpl_data()`: Test all-NA income_bracket bug
  - `create_test_database()`: SQLite database for testing
  - `cleanup_test_files()`: Remove temporary files

## Writing New Tests

### Template for New Test File

```r
# Phase X: Description
# Tests for specific functionality

test_that("descriptive test name", {
  # Arrange: Set up test data
  data <- create_sample_lead_data(n = 50)

  # Act: Perform the operation
  result <- some_function(data)

  # Assert: Check expectations
  expect_equal(result$value, expected_value)
  expect_true(some_condition)
})
```

### Best Practices

1. **Use fixtures**: Don't rely on external data - use `create_sample_lead_data()`
2. **Test edge cases**: Zero, negative, NA, Inf values
3. **Clean up**: Use `cleanup_test_files()` to remove temporary files
4. **Skip when appropriate**: Use `skip_if_offline()` for network tests
5. **Descriptive names**: Test names should describe what's being tested

## Coverage Goals

| Component | Target | Current |
|-----------|--------|---------|
| Data loaders | 95% | TBD |
| Energy metrics | 95% | TBD |
| Data processing | 90% | TBD |
| Validation functions | 100% | TBD |
| Overall package | 85% | TBD |

## Continuous Integration

Tests run automatically on:
- Pull requests targeting `main` branch
- Pushes to `main` branch
- Multiple OS/R version combinations (macOS, Windows, Ubuntu)

See `.github/workflows/R-CMD-check.yml` and `.github/workflows/test-coverage.yml` for details.

## Debugging Failed Tests

### View Detailed Output

```r
devtools::test(reporter = "check")  # More verbose than default
```

### Interactive Debugging

```r
devtools::load_all()
library(testthat)

# Set breakpoint in your test
test_that("my failing test", {
  browser()  # Execution will pause here
  # ... test code ...
})
```

### Check Test Data

```r
# Generate and inspect test data
data <- create_sample_lead_data(n = 10)
str(data)
summary(data)
```

## Test Data Fixtures

Test fixtures are generated on-the-fly using `helper-fixtures.R`. This ensures:

1. **No large data files in git**: Fixtures are created during test runs
2. **Reproducibility**: Seed values ensure consistent test data
3. **Flexibility**: Easy to modify fixture characteristics
4. **Speed**: Minimal data creation for fast tests

### Example Fixture Usage

```r
# Create standard test data
data <- create_sample_lead_data(n = 100, dataset = "ami", vintage = "2022")

# Create edge case data
edge_data <- create_edge_case_data()

# Create corrupted data (for testing validation)
corrupted <- create_corrupted_fpl_data(n = 50)
```

## Performance Benchmarks

Target test suite execution time:
- Unit tests: < 30 seconds
- Integration tests: < 60 seconds
- Full suite with coverage: < 2 minutes

## Adding Tests for New Features

When adding new features:

1. Write tests first (TDD approach preferred)
2. Ensure new code has >85% coverage
3. Add integration tests for major features
4. Update this README with new test files

## Troubleshooting

### "Package not installed" Error

```bash
# Install in development mode
Rscript -e "devtools::install()"
```

### Missing Test Dependencies

```r
# Install testing packages
install.packages(c("testthat", "covr", "withr", "mockery"))
```

### Tests Pass Locally but Fail in CI

- Check R version differences
- Verify all dependencies in DESCRIPTION
- Check for hardcoded paths
- Ensure fixtures are platform-independent

## Additional Resources

- [testthat documentation](https://testthat.r-lib.org/)
- [covr package](https://covr.r-lib.org/)
- [R Packages book - Testing chapter](https://r-pkgs.org/testing-basics.html)
- [Testing spec document](.dev/testing-spec.md)

## ----eval=FALSE---------------------------------------------------------------
# calculate_weighted_metrics(
#   data,
#   group_columns = c("state", "income_bracket"),
#   metric_name = "ner"
# )

## ----eval=FALSE---------------------------------------------------------------
# library(emburden)
# 
# # Energy metric calculations
# energy_burden_func(gross_income, energy_spending)
# ner_func(gross_income, energy_spending)  # Net Energy Return
# eroi_func(gross_income, energy_spending)  # EROI
# dear_func(gross_income, energy_spending)  # DEAR
# 
# # Statistical aggregation
# calculate_weighted_metrics(
#   graph_data,
#   group_columns = "state",
#   metric_name = "ner"
# )

## ----eval=FALSE---------------------------------------------------------------
# # Load census tract data (auto-downloads if not available)
# nc_tracts <- load_census_tract_data(states = "NC")
# 
# # Load cohort data by income bracket
# nc_ami <- load_cohort_data(
#   dataset = "ami",
#   states = "NC",
#   vintage = "2022"
# )
# 
# # Compare vintages
# comparison <- compare_energy_burden(
#   dataset = "ami",
#   states = "NC",
#   group_by = "state"
# )

## ----basic-comparison, eval=FALSE---------------------------------------------
# library(emburden)
# 
# # Compare North Carolina energy burden: 2018 vs 2022
# nc_comparison <- compare_energy_burden(
#   dataset = "ami",
#   states = "NC",
#   group_by = "income_bracket"
# )
# 
# # View formatted comparison table
# print(nc_comparison)

## ----comparison-metrics, eval=FALSE-------------------------------------------
# # Energy burden in 2018 and 2022
# nc_comparison$neb_2018
# nc_comparison$neb_2022
# 
# # Change in energy burden (percentage points)
# nc_comparison$neb_change_pp
# 
# # Net Energy Return values
# nc_comparison$ner_2018
# nc_comparison$ner_2022
# 
# # Household counts
# nc_comparison$households_2018
# nc_comparison$households_2022

## ----state-level, eval=FALSE--------------------------------------------------
# # Overall state comparison
# nc_state <- compare_energy_burden(
#   dataset = "ami",
#   states = "NC",
#   group_by = "none"
# )
# 
# # Extract key findings
# cat(sprintf(
#   "North Carolina energy burden changed from %.1f%% (2018) to %.1f%% (2022)\n",
#   nc_state$neb_2018 * 100,
#   nc_state$neb_2022 * 100
# ))
# 
# cat(sprintf(
#   "Change: %+.2f percentage points\n",
#   nc_state$neb_change_pp * 100
# ))

## ----income-bracket, eval=FALSE-----------------------------------------------
# # Compare by income bracket
# nc_income <- compare_energy_burden(
#   dataset = "ami",
#   states = "NC",
#   group_by = "income_bracket"
# )
# 
# # Visualize changes
# library(ggplot2)
# 
# ggplot(nc_income, aes(x = income_bracket, y = neb_change_pp * 100)) +
#   geom_col(fill = "steelblue") +
#   geom_hline(yintercept = 0, linetype = "dashed") +
#   labs(
#     title = "Change in Energy Burden by Income Bracket",
#     subtitle = "North Carolina, 2018 to 2022",
#     x = "Income Bracket (% of Area Median Income)",
#     y = "Change in Energy Burden (percentage points)"
#   ) +
#   theme_minimal()

## ----multi-state, eval=FALSE--------------------------------------------------
# # Compare Southern states
# southern_states <- compare_energy_burden(
#   dataset = "ami",
#   states = c("NC", "SC", "GA", "FL"),
#   group_by = "state"
# )
# 
# # Which states improved most?
# southern_states %>%
#   arrange(neb_change_pp) %>%
#   select(state_abbr, neb_2018, neb_2022, neb_change_pp)
# 
# # Visualize state comparison
# ggplot(southern_states, aes(x = reorder(state_abbr, neb_2022),
#                              y = neb_2022 * 100)) +
#   geom_col(fill = "darkgreen") +
#   geom_point(aes(y = neb_2018 * 100), color = "red", size = 3) +
#   labs(
#     title = "Energy Burden by State: 2022 (bars) vs 2018 (points)",
#     x = "State",
#     y = "Energy Burden (%)"
#   ) +
#   theme_minimal()

## ----housing-tenure, eval=FALSE-----------------------------------------------
# # Compare by housing tenure
# nc_tenure <- compare_energy_burden(
#   dataset = "ami",
#   states = "NC",
#   group_by = "housing_tenure"
# )
# 
# # Calculate the renter-owner gap
# gap_2018 <- nc_tenure$neb_2018[nc_tenure$housing_tenure == "RENTER"] -
#             nc_tenure$neb_2018[nc_tenure$housing_tenure == "OWNER"]
# 
# gap_2022 <- nc_tenure$neb_2022[nc_tenure$housing_tenure == "RENTER"] -
#             nc_tenure$neb_2022[nc_tenure$housing_tenure == "OWNER"]
# 
# cat(sprintf(
#   "Renter-Owner energy burden gap: %.2f pp (2018) → %.2f pp (2022)\n",
#   gap_2018 * 100,
#   gap_2022 * 100
# ))

## ----fpl-analysis, eval=FALSE-------------------------------------------------
# # Use FPL dataset instead of AMI
# nc_fpl <- compare_energy_burden(
#   dataset = "fpl",
#   states = "NC",
#   group_by = "income_bracket"
# )
# 
# # Compare poverty vs non-poverty households
# nc_fpl %>%
#   filter(income_bracket %in% c("Below Federal Poverty Line",
#                                 "Above Federal Poverty Line")) %>%
#   select(income_bracket, neb_2018, neb_2022, neb_change_pp)

## ----tract-level, eval=FALSE--------------------------------------------------
# # Load 2022 census tract data
# nc_tracts_2022 <- load_census_tract_data(
#   states = "NC",
#   vintage = "2022"
# )
# 
# # Calculate county-level statistics
# nc_counties <- calculate_weighted_metrics(
#   nc_tracts_2022,
#   group_columns = "county_name",
#   metric_name = "ner"
# )
# 
# # Identify counties with highest energy burden
# nc_counties %>%
#   mutate(energy_burden = 1 / (ner + 1)) %>%
#   arrange(desc(energy_burden)) %>%
#   head(10) %>%
#   select(county_name, energy_burden, household_count)

## ----split-incentive-example, eval=FALSE--------------------------------------
# # Quantify the renter-owner gap
# tenure_comparison <- compare_energy_burden(
#   dataset = "ami",
#   states = "all",  # National analysis
#   group_by = "housing_tenure"
# )
# 
# # Calculate disparity
# renter_burden <- tenure_comparison$neb_2022[
#   tenure_comparison$housing_tenure == "RENTER"
# ]
# owner_burden <- tenure_comparison$neb_2022[
#   tenure_comparison$housing_tenure == "OWNER"
# ]
# 
# disparity_ratio <- renter_burden / owner_burden

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----installation, eval=FALSE-------------------------------------------------
# # install.packages("devtools")
# devtools::install_github("ericscheier/emburden")

## ----setup--------------------------------------------------------------------
library(emburden)
library(dplyr)

## ----single-household---------------------------------------------------------
# Calculate energy burden for a single household
gross_income <- 50000
energy_spending <- 3000

# Method 1: Direct energy burden
eb <- energy_burden_func(gross_income, energy_spending)
print(paste("Energy Burden:", scales::percent(eb)))

# Method 2: Via Net Energy Return (mathematically identical)
nh <- ner_func(gross_income, energy_spending)
neb <- 1 / (nh + 1)
print(paste("Net Energy Burden:", scales::percent(neb)))
print(paste("Net Energy Return:", round(nh, 2)))

## ----load-data, eval=FALSE----------------------------------------------------
# # Load census tract data for North Carolina
# nc_tracts <- load_census_tract_data(states = "NC")
# 
# # Load household cohort data by Area Median Income
# nc_ami <- load_cohort_data(dataset = "ami", states = "NC")
# 
# # View structure
# head(nc_ami)

## ----cohort-example, eval=FALSE-----------------------------------------------
# # Calculate mean income and spending from totals
# nc_data <- nc_ami %>%
#   mutate(
#     mean_income = total_income / households,
#     mean_energy_spending = (total_electricity_spend +
#                            coalesce(total_gas_spend, 0) +
#                            coalesce(total_other_spend, 0)) / households
#   ) %>%
#   filter(!is.na(mean_income), !is.na(mean_energy_spending), households > 0) %>%
#   mutate(
#     eb = energy_burden_func(mean_income, mean_energy_spending),
#     nh = ner_func(mean_income, mean_energy_spending),
#     neb = neb_func(mean_income, mean_energy_spending)
#   )

## ----wrong-aggregation, eval=FALSE--------------------------------------------
# # ❌ WRONG: Direct averaging of energy burden introduces ~1-5% error
# eb_wrong <- weighted.mean(nc_data$eb, nc_data$households)

## ----correct-aggregation, eval=FALSE------------------------------------------
# # ✅ CORRECT Method 1: Aggregate using Nh, then convert to NEB
# nh_mean <- weighted.mean(nc_data$nh, nc_data$households)
# neb_correct <- 1 / (1 + nh_mean)
# 
# # ✅ CORRECT Method 2: Use neb_func() with weights (simpler!)
# # neb_func() automatically uses the Nh method internally
# neb_correct2 <- neb_func(nc_data$mean_income,
#                          nc_data$mean_energy_spending,
#                          weights = nc_data$households)
# 
# print(paste("Correct NEB (manual Nh):", scales::percent(neb_correct)))
# print(paste("Correct NEB (neb_func): ", scales::percent(neb_correct2)))
# # Both give identical results!

## ----by-income, eval=FALSE----------------------------------------------------
# # Method 1: Manual Nh aggregation
# nc_by_income <- nc_data %>%
#   group_by(income_bracket) %>%
#   summarise(
#     households = sum(households),
#     nh_mean = weighted.mean(nh, households),
#     neb = 1 / (1 + nh_mean),  # Correct aggregation
#     .groups = "drop"
#   )
# 
# # Method 2: Using neb_func() with weights (simpler!)
# nc_by_income2 <- nc_data %>%
#   group_by(income_bracket) %>%
#   summarise(
#     neb = neb_func(mean_income, mean_energy_spending, weights = households),
#     households = sum(households),
#     .groups = "drop"
#   )
# 
# print(nc_by_income)

## ----high-burden, eval=FALSE--------------------------------------------------
# # 6% energy burden corresponds to Nh = 15.67
# high_burden_threshold <- 15.67
# 
# high_burden_households <- sum(nc_data$households[nc_data$nh < high_burden_threshold])
# total_households <- sum(nc_data$households)
# high_burden_pct <- (high_burden_households / total_households) * 100
# 
# print(paste("Households with >6% energy burden:",
#             scales::percent(high_burden_pct/100)))

## ----weighted-metrics, eval=FALSE---------------------------------------------
# results <- calculate_weighted_metrics(
#   graph_data = nc_ami,
#   group_columns = "income_bracket",
#   metric_name = "ner",
#   metric_cutoff_level = 15.67,  # 6% burden threshold
#   upper_quantile_view = 0.95,
#   lower_quantile_view = 0.05
# )
# 
# # Format for publication
# results$formatted_median <- to_percent(results$metric_median)
# print(results)

## ----temporal-comparison, eval=FALSE------------------------------------------
# # Compare by income bracket
# comparison <- compare_energy_burden(
#   dataset = "ami",
#   states = "NC",
#   group_by = "income_bracket"
# )
# 
# # View results
# print(comparison)
# 
# # The function automatically:
# # - Loads both 2018 and 2022 data
# # - Normalizes schema differences (4 vs 6 AMI brackets)
# # - Performs proper Nh-based aggregation
# # - Calculates changes in energy burden
# 
# # Grouping options:
# # - "income_bracket": Compare by AMI/FPL brackets (default)
# # - "state": Compare multiple states
# # - "none": Overall state-level comparison
# 
# # Example: State-level comparison
# state_comparison <- compare_energy_burden(
#   dataset = "ami",
#   states = "NC",
#   group_by = "none"
# )
# 
# # Access specific metrics
# state_comparison$neb_2018         # 2018 energy burden
# state_comparison$neb_2022         # 2022 energy burden
# state_comparison$neb_change_pp    # Change in percentage points
# state_comparison$neb_change_pct   # Relative change percentage

## ----housing-analysis, eval=FALSE---------------------------------------------
# # Load data with housing characteristics
# nc_housing <- load_cohort_data(dataset = "ami", states = "NC")
# 
# # Analyze energy burden by tenure and heating fuel
# housing_analysis <- nc_housing %>%
#   filter(!is.na(TEN), !is.na(`TEN-HFL`)) %>%
#   mutate(
#     mean_income = total_income / households,
#     mean_energy_spending = (total_electricity_spend +
#                            coalesce(total_gas_spend, 0) +
#                            coalesce(total_other_spend, 0)) / households,
#     nh = ner_func(mean_income, mean_energy_spending)
#   ) %>%
#   group_by(TEN, `TEN-HFL`) %>%
#   summarise(
#     total_households = sum(households),
#     nh_mean = weighted.mean(nh, households),
#     neb = 1 / (1 + nh_mean),
#     .groups = "drop"
#   ) %>%
#   arrange(desc(neb))
# 
# # View the top 10 tenure-heating fuel combinations with highest burden
# head(housing_analysis, 10)

## ----building-analysis, eval=FALSE--------------------------------------------
# # Analyze by building characteristics
# building_analysis <- nc_housing %>%
#   filter(!is.na(`TEN-YBL6`), !is.na(`TEN-BLD`)) %>%
#   mutate(
#     mean_income = total_income / households,
#     mean_energy_spending = (total_electricity_spend +
#                            coalesce(total_gas_spend, 0) +
#                            coalesce(total_other_spend, 0)) / households,
#     nh = ner_func(mean_income, mean_energy_spending)
#   ) %>%
#   group_by(`TEN-YBL6`, `TEN-BLD`) %>%
#   summarise(
#     total_households = sum(households),
#     nh_mean = weighted.mean(nh, households),
#     neb = 1 / (1 + nh_mean),
#     .groups = "drop"
#   )
# 
# # Identify building age/type combinations with highest burden
# high_burden_buildings <- building_analysis %>%
#   filter(neb > 0.06) %>%  # Above 6% burden threshold
#   arrange(desc(neb))
# 
# print(high_burden_buildings)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(emburden)
library(dplyr)

## ----simple-example-----------------------------------------------------------
# Three households with different income/spending patterns
households <- data.frame(
  id = 1:3,
  income = c(30000, 50000, 100000),
  spending = c(3000, 3500, 4000)
)

households$eb <- energy_burden_func(households$income, households$spending)
print(households)

## ----wrong-aggregation--------------------------------------------------------
# Attempt 1: Simple arithmetic mean (WRONG!)
mean_eb_wrong <- mean(households$eb)
print(paste("Simple mean EB:", scales::percent(mean_eb_wrong)))

## ----weighted-wrong-----------------------------------------------------------
# Attempt 2: Weighted arithmetic mean (STILL WRONG!)
# Let's weight by number of households (all equal here, but principle matters)
weights <- c(100, 150, 200)  # Different household counts

eb_arithmetic_mean <- weighted.mean(households$eb, weights)
print(paste("Weighted arithmetic mean EB:", scales::percent(eb_arithmetic_mean)))

## ----harmonic-mean------------------------------------------------------------
# Correct aggregation: Weighted harmonic mean
eb_harmonic <- 1 / weighted.mean(1 / households$eb, weights)
print(paste("Weighted harmonic mean EB:", scales::percent(eb_harmonic)))

# Verify by calculating from totals
total_spending <- sum(households$spending * weights)
total_income <- sum(households$income * weights)
eb_from_totals <- total_spending / total_income
print(paste("EB from totals:", scales::percent(eb_from_totals)))

# These should be identical
print(paste("Difference:", abs(eb_harmonic - eb_from_totals)))

## ----nh-identity--------------------------------------------------------------
# Starting from Nh = (G - S) / S
# Let's solve for EB = S / G

# Nh = (G - S) / S
# Nh = G/S - S/S
# Nh = G/S - 1
# Nh + 1 = G/S
# 1 / (Nh + 1) = S/G = EB

# Therefore: EB = 1 / (Nh + 1)

# Verify with our example
households$nh <- ner_func(households$income, households$spending)
households$eb_from_nh <- 1 / (households$nh + 1)

# Compare to original EB
households$identical <- all.equal(households$eb, households$eb_from_nh)
print(households[, c("id", "eb", "eb_from_nh", "nh")])

## ----nh-aggregation-----------------------------------------------------------
# Step 1: Calculate Nh for each household
nh_values <- ner_func(households$income, households$spending)

# Step 2: Arithmetic weighted mean (simple!)
nh_mean <- weighted.mean(nh_values, weights)
print(paste("Weighted mean Nh:", round(nh_mean, 2)))

# Step 3: Convert back to EB
eb_from_nh <- 1 / (nh_mean + 1)
print(paste("EB from Nh method:", scales::percent(eb_from_nh)))

# Compare to harmonic mean result
print(paste("EB from harmonic mean:", scales::percent(eb_harmonic)))
print(paste("Difference:", abs(eb_from_nh - eb_harmonic)))

## ----computational-comparison-------------------------------------------------
# Simulate larger dataset
set.seed(42)
n <- 10000
large_data <- data.frame(
  income = rlnorm(n, meanlog = 10.8, sdlog = 0.8),  # Log-normal income distribution
  spending = NA
)
large_data$spending <- pmin(
  rlnorm(n, meanlog = 8.2, sdlog = 0.5),  # Log-normal spending
  large_data$income * 0.5  # Cap at 50% of income
)
weights <- sample(50:500, n, replace = TRUE)

# Method 1: Nh with arithmetic mean
system.time({
  nh <- ner_func(large_data$income, large_data$spending)
  nh_mean <- weighted.mean(nh, weights)
  eb_nh <- 1 / (1 + nh_mean)
})

# Method 2: Harmonic mean
system.time({
  eb_direct <- energy_burden_func(large_data$income, large_data$spending)
  eb_harmonic <- 1 / weighted.mean(1 / eb_direct, weights)
})

# Verify results are identical
print(paste("EB via Nh:", scales::percent(eb_nh)))
print(paste("EB via harmonic mean:", scales::percent(eb_harmonic)))
print(paste("Difference:", abs(eb_nh - eb_harmonic)))

## ----numerical-stability------------------------------------------------------
# Households with very low energy burden
low_burden <- data.frame(
  income = c(200000, 500000, 1000000),
  spending = c(2000, 3000, 5000)  # Very low spending relative to income
)

low_burden$eb <- energy_burden_func(low_burden$income, low_burden$spending)
low_burden$nh <- ner_func(low_burden$income, low_burden$spending)

print("Energy Burden (direct):")
print(low_burden$eb)

print("\nReciprocal of EB (used in harmonic mean):")
print(1 / low_burden$eb)  # Very large numbers!

print("\nNet Energy Return (Nh):")
print(low_burden$nh)  # More reasonable range

## ----error-demonstration------------------------------------------------------
# Use realistic North Carolina-like data
set.seed(123)
n_households <- 5000

realistic_data <- data.frame(
  income_bracket = sample(
    c("0-30% AMI", "30-50% AMI", "50-80% AMI", "80-100% AMI", "100%+ AMI"),
    n_households,
    replace = TRUE,
    prob = c(0.15, 0.12, 0.20, 0.10, 0.43)
  ),
  income = rlnorm(n_households, meanlog = 10.8, sdlog = 0.8),
  households = sample(10:100, n_households, replace = TRUE)
)

realistic_data$spending <- realistic_data$income * rlnorm(
  n_households,
  meanlog = log(0.05),
  sdlog = 0.6
)

# Calculate metrics
realistic_data <- realistic_data %>%
  mutate(
    eb = energy_burden_func(income, spending),
    nh = ner_func(income, spending),
    neb = neb_func(income, spending)
  )

# WRONG: Arithmetic mean of EB
eb_wrong <- weighted.mean(realistic_data$eb, realistic_data$households)

# CORRECT: Via Nh
nh_mean <- weighted.mean(realistic_data$nh, realistic_data$households)
eb_correct <- 1 / (1 + nh_mean)

# Calculate error
absolute_error <- eb_wrong - eb_correct
relative_error_pct <- (absolute_error / eb_correct) * 100

cat(sprintf("WRONG (arithmetic mean EB): %.2f%%\n", eb_wrong * 100))
cat(sprintf("CORRECT (via Nh method):   %.2f%%\n", eb_correct * 100))
cat(sprintf("Absolute error:             %.4f\n", absolute_error))
cat(sprintf("Relative error:             %.2f%%\n", relative_error_pct))

## ----practical-workflow-------------------------------------------------------
# Step 1: Calculate Nh for all observations
data_with_metrics <- realistic_data %>%
  mutate(
    nh = ner_func(income, spending),
    neb = neb_func(income, spending)  # Same as eb, but emphasizes proper aggregation
  )

# Step 2: Aggregate by groups using arithmetic weighted mean
by_bracket <- data_with_metrics %>%
  group_by(income_bracket) %>%
  summarise(
    total_households = sum(households),
    nh_mean = weighted.mean(nh, households),
    neb = 1 / (1 + nh_mean),  # Correct aggregation
    .groups = "drop"
  ) %>%
  arrange(desc(neb))

print(by_bracket)

# Step 3: Identify high-burden households
high_burden_threshold <- 0.06  # 6% energy burden threshold
nh_threshold <- (1 / high_burden_threshold) - 1  # = 15.67

high_burden_count <- sum(
  data_with_metrics$households[data_with_metrics$nh < nh_threshold]
)
total_households <- sum(data_with_metrics$households)

cat(sprintf("\nHouseholds with >6%% energy burden: %.1f%%\n",
            (high_burden_count / total_households) * 100))

# Global variable bindings to satisfy R CMD check
utils::globalVariables(c(
  "geoid", "geo_id", "income_bracket", "AMI150", "AMI68",
  "TEN", "TEN-YBL6", "TEN-BLD", "TEN-HFL"
))

#' Load DOE LEAD Tool Cohort Data
#'
#' Load household energy burden cohort data with automatic fallback:
#' 1. Try local database
#' 2. Fall back to local CSV files
#' 3. Auto-download from OpenEI if neither exists
#' 4. Auto-import downloaded data to database for future use
#'
#' @param dataset Character, either "ami" (Area Median Income) or "fpl"
#'   (Federal Poverty Line)
#' @param states Character vector of state abbreviations to filter by (optional)
#' @param counties Character vector of county names or FIPS codes to filter by (optional).
#'   County names are matched case-insensitively. Requires `states` to be specified.
#' @param vintage Character, data vintage: "2018" or "2022" (default "2022")
#' @param income_brackets Character vector of income brackets to filter by (optional)
#' @param verbose Logical, print status messages (default TRUE)
#' @param ... Additional filter expressions passed to dplyr::filter() for dynamic filtering.
#'   Allows filtering by any column in the dataset using tidyverse syntax.
#'   Example: `households > 100, total_income > 50000`
#'
#' @return A tibble with columns:
#'   - geoid: Census tract identifier
#'   - income_bracket: Income bracket label
#'   - households: Number of households
#'   - total_income: Total household income ($)
#'   - total_electricity_spend: Total electricity spending ($)
#'   - total_gas_spend: Total gas spending ($)
#'   - total_other_spend: Total other fuel spending ($)
#'   - TEN: Housing tenure category (1=Owned free/clear, 2=Owned with mortgage,
#'     3=Rented, 4=Occupied without rent). Enables analysis of energy burden
#'     differences between renters and owners.
#'   - TEN-YBL6: Housing tenure crossed with year structure built (6 categories).
#'     Allows analysis of how building age and ownership status interact to affect
#'     energy burden (e.g., older rental units vs newer owner-occupied homes).
#'   - TEN-BLD: Housing tenure crossed with building type (e.g., single-family,
#'     multi-unit). Enables analysis of energy burden across different housing
#'     structures and ownership patterns.
#'   - TEN-HFL: Housing tenure crossed with primary heating fuel type (e.g., gas,
#'     electric, oil). Critical for analyzing how heating fuel choice and tenure
#'     status jointly influence energy costs and burden.
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Single state (fast, good for learning)
#' nc_ami <- load_cohort_data(dataset = "ami", states = "NC")
#'
#' # Load specific vintage
#' nc_2018 <- load_cohort_data(dataset = "ami", states = "NC", vintage = "2018")
#' }
#'
#' \donttest{
#' if (interactive()) {
#'   # Multiple states (regional analysis - requires data download)
#'   southeast <- load_cohort_data(dataset = "fpl", states = c("NC", "SC", "GA", "FL"))
#'
#'   # Nationwide (all 51 states - no filter)
#'   us_data <- load_cohort_data(dataset = "ami", vintage = "2022")
#'
#' # Filter to specific income brackets
#' low_income <- load_cohort_data(
#'   dataset = "ami",
#'   states = "NC",
#'   income_brackets = c("0-30% AMI", "30-50% AMI")
#' )
#'
#' # Filter to specific counties within a state
#' triangle <- load_cohort_data(
#'   dataset = "fpl",
#'   states = "NC",
#'   counties = c("Orange", "Durham", "Wake")
#' )
#'
#' # Or use county FIPS codes
#' orange <- load_cohort_data(
#'   dataset = "fpl",
#'   states = "NC",
#'   counties = "37135"
#' )
#'
#' # Use dynamic filtering for custom criteria
#' high_burden <- load_cohort_data(
#'   dataset = "ami",
#'   states = "NC",
#'   households > 100,
#'   total_electricity_spend / total_income > 0.06
#' )
#'
#' # Analyze energy burden by housing characteristics
#' # Compare renters vs owners by heating fuel type
#' nc_housing <- load_cohort_data(dataset = "ami", states = "NC")
#' library(dplyr)
#'
#' # Group by tenure and heating fuel to analyze energy burden patterns
#' housing_analysis <- nc_housing %>%
#'   filter(!is.na(TEN), !is.na(`TEN-HFL`)) %>%
#'   group_by(TEN, `TEN-HFL`) %>%
#'   summarise(
#'     total_households = sum(households),
#'     avg_energy_burden = weighted.mean(
#'       (total_electricity_spend + total_gas_spend + total_other_spend) / total_income,
#'       w = households,
#'       na.rm = TRUE
#'     ),
#'     .groups = "drop"
#'   )
#' }
#' }
load_cohort_data <- function(dataset = c("ami", "fpl"),
                              states = NULL,
                              counties = NULL,
                              vintage = "2022",
                              income_brackets = NULL,
                              verbose = TRUE,
                              ...) {

  # Validate inputs
  dataset <- match.arg(dataset)
  if (!vintage %in% c("2018", "2022")) {
    stop("vintage must be '2018' or '2022'")
  }

  if (verbose) {
    message("Loading ", vintage, " ", toupper(dataset), " cohort data...")
  }

  # Try database first (unless disabled via environment variable)
  data <- if (Sys.getenv("EMBURDEN_NO_DATABASE") == "1") {
    if (verbose) {
      message("  \u26A0  Database caching disabled (EMBURDEN_NO_DATABASE=1)")
    }
    NULL  # Skip database, go directly to CSV/OpenEI
  } else {
    try_load_from_database(
      dataset = dataset,
      vintage = vintage,
      verbose = verbose
    )
  }

  # Check database data for corruption (warn but don't auto-delete)
  if (!is.null(data)) {
    corruption_check <- detect_database_corruption(
      data = data,
      dataset = dataset,
      vintage = vintage,
      states = states,
      verbose = verbose
    )

    # If corrupted, discard and try other sources
    if (corruption_check$is_corrupted) {
      if (verbose) {
        message("  \u26A0  Database data appears corrupted, will try other sources...")
      }
      data <- NULL  # Discard corrupted data, try CSV/OpenEI
    }
  }

  # If database fails or corrupted, try CSV
  if (is.null(data)) {
    data <- try_load_from_csv(
      dataset = dataset,
      vintage = vintage,
      verbose = verbose
    )
  }

  # If CSV fails, try Zenodo first (faster, more reliable), then OpenEI
  if (is.null(data)) {
    if (verbose) {
      message("Data not found locally.")
    }

    # Try Zenodo first (pre-processed, compressed, faster)
    data <- download_from_zenodo(
      dataset = dataset,
      vintage = vintage,
      verbose = verbose
    )

    # If Zenodo fails, fall back to OpenEI (original source)
    if (is.null(data)) {
      if (verbose) {
        message("Downloading from OpenEI (original source)...")
      }
      data <- download_lead_data(
        dataset = dataset,
        vintage = vintage,
        states = states,
        verbose = verbose
      )
    }

    # Try to import to database for future use
    if (!is.null(data)) {
      try_import_to_database(
        data = data,
        dataset = dataset,
        vintage = vintage,
        verbose = verbose
      )
    }
  }

  if (is.null(data)) {
    stop("Failed to load data from any source (database, CSV, or OpenEI)")
  }

  # Filter by states if requested
  if (!is.null(states)) {
    # Extract state FIPS from geoid (first 2 digits)
    state_fips <- get_state_fips(states)
    data <- data |>
      dplyr::filter(substr(as.character(geoid), 1, 2) %in% state_fips)

    if (verbose) {
      message("Filtered to state(s): ", paste(states, collapse = ", "))
    }
  }

  # Filter by counties if requested
  if (!is.null(counties)) {
    if (is.null(states)) {
      warning("County filtering requires 'states' parameter. Ignoring 'counties' parameter.")
    } else {
      # Extract county FIPS from geoid (characters 3-5)
      # Support both county names and FIPS codes
      county_fips <- get_county_fips(counties, states)

      if (length(county_fips) > 0) {
        data <- data |>
          dplyr::filter(substr(as.character(geoid), 3, 5) %in% county_fips)

        if (verbose) {
          message("Filtered to ", length(county_fips), " county/counties")
        }
      } else {
        warning("No matching counties found for the specified names/FIPS codes")
      }
    }
  }

  # Filter by income brackets if requested
  if (!is.null(income_brackets)) {
    data <- data |>
      dplyr::filter(income_bracket %in% income_brackets)

    if (verbose) {
      message("Filtered to ", length(income_brackets), " income bracket(s)")
    }
  }

  # Apply dynamic filters if provided
  filter_exprs <- rlang::enquos(...)
  if (length(filter_exprs) > 0) {
    for (filter_expr in filter_exprs) {
      data <- data |>
        dplyr::filter(!!filter_expr)
    }

    if (verbose) {
      message("Applied ", length(filter_exprs), " custom filter(s)")
    }
  }

  if (verbose) {
    message("Loaded ", nrow(data), " cohort records")
  }

  return(data)
}


#' Load Census Tract Data
#'
#' Load census tract demographics and utility service territory information
#' with automatic fallback to CSV or OpenEI download.
#'
#' @param states Character vector of state abbreviations to filter by (optional)
#' @param verbose Logical, print status messages (default TRUE)
#'
#' @return A tibble with columns:
#'   - geoid: Census tract identifier
#'   - state_abbr: State abbreviation
#'   - county_name: County name
#'   - tract_name: Tract name
#'   - utility_name: Electric utility serving this tract
#'   - Additional demographic columns
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (interactive()) {
#'   # Single state (requires census data download)
#'   nc_tracts <- load_census_tract_data(states = "NC")
#'
#'   # Multiple states (regional)
#'   southeast <- load_census_tract_data(states = c("NC", "SC", "GA", "FL"))
#'
#'   # Nationwide (all ~73,000 census tracts)
#'   us_tracts <- load_census_tract_data()  # No filter = all states
#' }
#' }
load_census_tract_data <- function(states = NULL, verbose = TRUE) {

  if (verbose) {
    message("Loading census tract data...")
  }

  # Try database first
  data <- try_load_tracts_from_database(verbose = verbose)

  # If database fails, try CSV
  if (is.null(data)) {
    data <- try_load_tracts_from_csv(verbose = verbose)
  }

  # If CSV fails, try Zenodo first, then OpenEI
  if (is.null(data)) {
    if (verbose) {
      message("Data not found locally.")
    }

    # Try Zenodo first
    data <- download_tracts_from_zenodo(verbose = verbose)

    # If Zenodo fails, fall back to OpenEI
    if (is.null(data)) {
      if (verbose) {
        message("Downloading from OpenEI (original source)...")
      }
      data <- download_census_tract_data(verbose = verbose)
    }

    # Try to import to database for future use
    if (!is.null(data)) {
      try_import_tracts_to_database(data = data, verbose = verbose)
    }
  }

  if (is.null(data)) {
    stop("Failed to load census tract data from any source")
  }

  # Filter by states if requested
  if (!is.null(states)) {
    data <- data |>
      dplyr::filter(state_abbr %in% states)

    if (verbose) {
      message("Filtered to state(s): ", paste(states, collapse = ", "))
    }
  }

  if (verbose) {
    message("Loaded ", nrow(data), " census tracts")
  }

  return(data)
}


#' Check Available Data Sources
#'
#' Check which data sources are available locally (database, CSV files, or
#' will require download from OpenEI).
#'
#' @param verbose Logical, print detailed status (default TRUE)
#'
#' @return A list with status of each data source
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Check what data is available
#' check_data_sources()
#' }
check_data_sources <- function(verbose = TRUE) {

  # Check database
  db_path <- find_emburden_db()
  db_available <- !is.null(db_path) && file.exists(db_path)

  if (db_available && requireNamespace("DBI", quietly = TRUE) &&
      requireNamespace("RSQLite", quietly = TRUE)) {
    tryCatch({
      conn <- DBI::dbConnect(RSQLite::SQLite(), db_path)
      tables <- DBI::dbListTables(conn)
      DBI::dbDisconnect(conn)
      db_tables <- tables
    }, error = function(e) {
      db_available <- FALSE
      db_tables <- character(0)
    })
  } else {
    db_tables <- character(0)
  }

  # Check CSV files
  csv_files <- list.files(
    path = "data",
    pattern = "^(CohortData|CensusTractData|very_clean_data).*\\.csv$",
    full.names = TRUE
  )

  result <- list(
    database = list(
      available = db_available,
      path = if (db_available) db_path else NULL,
      tables = db_tables
    ),
    csv_files = list(
      available = length(csv_files) > 0,
      files = basename(csv_files)
    ),
    download_required = !db_available && length(csv_files) == 0
  )

  if (verbose) {
    cat("\n")
    cat("Data Source Status\n")
    cat(strrep("=", 60), "\n")

    cat("\nLocal database:\n")
    if (result$database$available) {
      cat("  \u2713 Available at:", result$database$path, "\n")
      if (length(result$database$tables) > 0) {
        cat("  Tables:", paste(result$database$tables, collapse = ", "), "\n")
      }
    } else {
      cat("  \u2717 Not found\n")
    }

    cat("\nCSV Files (data/):\n")
    if (result$csv_files$available) {
      cat("  \u2713 Found", length(csv_files), "CSV file(s):\n")
      for (f in result$csv_files$files) {
        cat("    -", f, "\n")
      }
    } else {
      cat("  \u2717 No CSV files found\n")
    }

    cat("\n")
    if (result$download_required) {
      cat("\u26A0 No local data found. Data will be downloaded from OpenEI on first use.\n")
    } else {
      cat("\u2713 Local data available! No download required.\n")
    }
    cat("\n")
  }

  invisible(result)
}


# Internal helper functions ------------------------------------------------

#' Try to load cohort data from database
#' @keywords internal
try_load_from_database <- function(dataset, vintage, verbose = FALSE) {

  # Check if database packages are available
  if (!requireNamespace("DBI", quietly = TRUE) ||
      !requireNamespace("RSQLite", quietly = TRUE)) {
    if (verbose) {
      message("  DBI/RSQLite not available, skipping database")
    }
    return(NULL)
  }

  # Find database
  db_path <- find_emburden_db()
  if (is.null(db_path) || !file.exists(db_path)) {
    if (verbose) {
      message("  Database not found, trying CSV...")
    }
    return(NULL)
  }

  # Determine table name
  table_name <- paste0("lead_", vintage, "_", dataset, "_cohorts")

  tryCatch({
    conn <- DBI::dbConnect(RSQLite::SQLite(), db_path)
    on.exit(DBI::dbDisconnect(conn))

    # Check if table exists
    if (!DBI::dbExistsTable(conn, table_name)) {
      if (verbose) {
        message("  Table '", table_name, "' not found in database")
      }
      return(NULL)
    }

    # Load data
    data <- DBI::dbReadTable(conn, table_name) |>
      tibble::as_tibble()

    # Standardize column names (create total_* columns if needed)
    data <- standardize_cohort_columns(data, dataset, vintage)

    if (verbose) {
      message("  \u2713 Loaded from database")
    }

    return(data)

  }, error = function(e) {
    if (verbose) {
      message("  Database error: ", e$message)
    }
    return(NULL)
  })
}


#' Try to load cohort data from CSV
#' @keywords internal
try_load_from_csv <- function(dataset, vintage, verbose = FALSE) {

  # Construct possible CSV filenames
  dataset_upper <- toupper(dataset)

  # Get cache directory for downloaded files
  cache_dir <- get_cache_dir()

  # Try multiple naming conventions used in different data sources
  # ORDER MATTERS: Try most specific/processed formats first
  possible_files <- c(
    # very_clean_data format (with vintage) - THIS IS THE CORRECT FORMAT, TRY FIRST!
    # Matches: "very_clean_data_ami_census tracts_2022.csv", "very_clean_data_ami_census tracts_2022_nc.csv", etc.
    list.files("data", pattern = paste0("^very_clean_data_", dataset, "_census tracts_", vintage, ".*\\.csv$"),
               full.names = TRUE, ignore.case = TRUE),
    # Legacy CohortData format (no vintage)
    file.path("data", paste0("CohortData_",
                             ifelse(dataset == "ami", "AreaMedianIncome", "FederalPovertyLine"),
                             ".csv")),
    # Downloaded files in cache directory (from download_lead_data function)
    # Format: "lead_2022_ami.csv", "lead_2018_fpl.csv"
    file.path(cache_dir, paste0("lead_", vintage, "_", dataset, ".csv")),
    # replica_lead format: "replica_lead_AMI_CENSUS TRACTS_2022_NC.csv"
    list.files("data", pattern = paste0("^replica_lead_", dataset_upper, "_CENSUS TRACTS_", vintage, ".*\\.csv$"),
               full.names = TRUE, ignore.case = TRUE),
    # in_poverty format: "in_poverty_data_FPL_CENSUS TRACTS_2022_NC.csv"
    list.files("data", pattern = paste0("^in_poverty_data_", dataset_upper, "_CENSUS TRACTS_", vintage, ".*\\.csv$"),
               full.names = TRUE, ignore.case = TRUE),
    # State-prefixed format: "NC AMI Census Tracts 2022.csv" - TRY LAST (raw data format)
    list.files("data", pattern = paste0("^[A-Z]{2} ", dataset_upper, " Census Tracts ", vintage, "\\.csv$"),
               full.names = TRUE, ignore.case = TRUE)
  )

  # Flatten list (list.files returns vectors, c() can nest them)
  possible_files <- unlist(possible_files)

  for (csv_file in possible_files) {
    if (file.exists(csv_file)) {
      tryCatch({
        if (verbose) {
          message("  Reading CSV: ", basename(csv_file))
        }

        data <- readr::read_csv(
          csv_file,
          show_col_types = FALSE,
          col_types = readr::cols(
            .default = readr::col_guess()
          )
        )

        # Standardize column names
        data <- standardize_cohort_columns(data, dataset, vintage)

        # Validate that income_bracket exists and has valid data
        # Skip files where income_bracket is missing or all NA (incomplete processed files)
        if (!"income_bracket" %in% names(data)) {
          if (verbose) {
            message("  \u2717 Skipping file (missing income_bracket column): ", basename(csv_file))
          }
          next
        }

        if (all(is.na(data$income_bracket))) {
          if (verbose) {
            message("  \u2717 Skipping file (income_bracket all NA): ", basename(csv_file))
          }
          next
        }

        if (verbose) {
          message("  \u2713 Loaded from CSV")
        }

        return(data)

      }, error = function(e) {
        if (verbose) {
          message("  CSV read error: ", e$message)
        }
      })
    }
  }

  if (verbose) {
    message("  No CSV files found, will download...")
  }

  return(NULL)
}


#' Download LEAD data from OpenEI
#' @keywords internal
download_lead_data <- function(dataset, vintage, states = NULL, verbose = FALSE) {

  if (!requireNamespace("httr", quietly = TRUE)) {
    stop("Package 'httr' required for downloading from OpenEI. Install with: install.packages('httr')")
  }

  # For 2018, data is distributed as state-specific ZIP files
  # For 2022, data is available as direct CSV downloads
  if (vintage == "2018") {
    # If no states specified OR multiple states requested, download all/merge
    # This provides uniform API - nationwide data works same way for both vintages
    if (is.null(states) || length(states) == 0 || length(states) > 1) {
      # Use provided states or get all states if none specified
      states_to_download <- if (is.null(states) || length(states) == 0) {
        get_all_states()
      } else {
        states
      }

      if (verbose) {
        message("Downloading 2018 data for ", length(states_to_download), " states...")
        message("Note: This downloads and merges ", length(states_to_download), " separate ZIP files")
        message("This is a one-time download. Subsequent uses load from cache.")
        if (is.null(states) || length(states) == 0) {
          message("TIP: For faster downloads, use Zenodo (automatic via load_cohort_data)")
        }
      }

      return(download_and_merge_states(dataset, vintage, states_to_download, verbose))
    }

    # Single state requested - use first state
    state <- toupper(states[1])

    # ZIP file URL pattern
    # Note: Arizona has a non-standard filename with " (1)" suffix
    if (state == "AZ") {
      zip_url <- "https://data.openei.org/files/573/AZ-2018-LEAD-data%20(1).zip"
    } else {
      zip_url <- paste0("https://data.openei.org/files/573/", state, "-2018-LEAD-data.zip")
    }

    #CSV file name inside ZIP
    dataset_upper <- toupper(dataset)
    csv_filename <- paste0(state, " ", dataset_upper, " Census Tracts 2018.csv")

    if (verbose) {
      message("  Downloading 2018 ZIP from: ", zip_url)
      message("  Will extract: ", csv_filename)
    }

    url <- zip_url
    is_zip <- TRUE

  } else if (vintage == "2022") {
    # 2022: AMI uses direct CSV, FPL uses state ZIP files
    if (dataset == "fpl") {
      # If no states specified OR multiple states requested, download all/merge
      # This provides uniform API - nationwide data works same way for both datasets
      if (is.null(states) || length(states) == 0 || length(states) > 1) {
        # Use provided states or get all states if none specified
        states_to_download <- if (is.null(states) || length(states) == 0) {
          get_all_states()
        } else {
          states
        }

        if (verbose) {
          message("Downloading FPL data for ", length(states_to_download), " states...")
          message("Note: This downloads and merges ", length(states_to_download), " separate ZIP files")
          message("This is a one-time download. Subsequent uses load from cache.")
          if (is.null(states) || length(states) == 0) {
            message("TIP: For faster downloads, use Zenodo (automatic via load_cohort_data)")
          }
        }

        return(download_and_merge_states(dataset, vintage, states_to_download, verbose))
      }

      # Single state requested - use first state
      state <- toupper(states[1])

      # ZIP file URL pattern for 2022
      zip_url <- paste0("https://data.openei.org/files/6219/", state, "-2022-LEAD-data.zip")

      # CSV file name inside ZIP (note the space in filename)
      dataset_upper <- toupper(dataset)
      csv_filename <- paste0(state, " ", dataset_upper, " Census Tracts 2022.csv")

      if (verbose) {
        message("  Downloading 2022 FPL ZIP from: ", zip_url)
        message("  Will extract: ", csv_filename)
      }

      url <- zip_url
      is_zip <- TRUE

    } else {
      # AMI: Also uses state ZIP files (same as FPL)
      # If no states specified OR multiple states requested, download all/merge
      if (is.null(states) || length(states) == 0 || length(states) > 1) {
        # Use provided states or get all states if none specified
        states_to_download <- if (is.null(states) || length(states) == 0) {
          get_all_states()
        } else {
          states
        }

        if (verbose) {
          message("Downloading AMI data for ", length(states_to_download), " states...")
          message("Note: This downloads and merges ", length(states_to_download), " separate ZIP files")
          message("This is a one-time download. Subsequent uses load from cache.")
          if (is.null(states) || length(states) == 0) {
            message("TIP: For faster downloads, use Zenodo (automatic via load_cohort_data)")
          }
        }

        return(download_and_merge_states(dataset, vintage, states_to_download, verbose))
      }

      # Single state requested - use first state
      state <- toupper(states[1])

      # ZIP file URL pattern for 2022
      zip_url <- paste0("https://data.openei.org/files/6219/", state, "-2022-LEAD-data.zip")

      # CSV file name inside ZIP (note the space in filename)
      dataset_upper <- toupper(dataset)
      csv_filename <- paste0(state, " ", dataset_upper, " Census Tracts 2022.csv")

      if (verbose) {
        message("  Downloading 2022 AMI ZIP from: ", zip_url)
        message("  Will extract: ", csv_filename)
      }

      url <- zip_url
      is_zip <- TRUE
    }

  } else {
    stop("Unsupported vintage: ", vintage, ". Supported: 2018, 2022")
  }

  # Get cache directory
  cache_dir <- get_cache_dir()
  temp_file <- file.path(cache_dir, paste0("lead_", vintage, "_", dataset, "_raw.csv"))
  cache_file <- file.path(cache_dir, paste0("lead_", vintage, "_", dataset, ".csv"))

  # Warn user about download size (first-time only)
  if (is_zip) {
    message("\nDownloading LEAD data from OpenEI...")
    message("Note: ZIP files are typically 150-250 MB. This is a one-time download.")
    message("Data will be cached at: ", cache_dir)
    message("Subsequent uses will load from cache (much faster).\n")
  } else {
    message("\nDownloading LEAD data from OpenEI...")
    message("Note: CSV files are typically 50-150 MB. This is a one-time download.")
    message("Data will be cached at: ", cache_dir)
    message("Subsequent uses will load from cache (much faster).\n")
  }

  # Download with progress
  tryCatch({
    if (is_zip) {
      # Download ZIP file first
      zip_file <- file.path(cache_dir, paste0("lead_", vintage, "_", dataset, "_temp.zip"))

      if (verbose) {
        message("  Downloading ZIP file...")
      }

      response <- httr::GET(
        url,
        httr::progress(),
        httr::write_disk(zip_file, overwrite = TRUE)
      )

      if (httr::http_error(response)) {
        status_code <- httr::status_code(response)
        stop(
          "Download failed with HTTP status ", status_code, "\n",
          if (status_code == 404) {
            "  File not found at OpenEI. The data may have been moved or is unavailable.\n"
          } else if (status_code >= 500) {
            "  OpenEI server error. Try again later.\n"
          } else {
            "  Check your internet connection and try again.\n"
          }
        )
      }

      # Extract specific CSV from ZIP
      if (verbose) {
        message("  Extracting: ", csv_filename)
      }

      # List files in ZIP to verify
      zip_contents <- utils::unzip(zip_file, list = TRUE)

      if (!csv_filename %in% zip_contents$Name) {
        # Try to find a matching file (case-insensitive)
        matching_files <- grep(csv_filename, zip_contents$Name, ignore.case = TRUE, value = TRUE)
        if (length(matching_files) > 0) {
          csv_filename <- matching_files[1]
          if (verbose) {
            message("  Using matched file: ", csv_filename)
          }
        } else {
          stop("CSV file '", csv_filename, "' not found in ZIP. Available files: ",
               paste(zip_contents$Name, collapse = ", "))
        }
      }

      # Extract to temp_file location
      utils::unzip(zip_file, files = csv_filename, exdir = cache_dir, overwrite = TRUE)

      # Move extracted file to expected location
      extracted_path <- file.path(cache_dir, csv_filename)
      if (file.exists(extracted_path)) {
        file.rename(extracted_path, temp_file)
      } else {
        stop("Failed to extract ", csv_filename, " from ZIP")
      }

      # Clean up ZIP file
      unlink(zip_file)

    } else {
      # Direct CSV download (2022 behavior)
      response <- httr::GET(
        url,
        httr::progress(),
        httr::write_disk(temp_file, overwrite = TRUE)
      )

      if (httr::http_error(response)) {
        status_code <- httr::status_code(response)
        stop(
          "Download failed with HTTP status ", status_code, "\n",
          if (status_code == 404) {
            "  File not found at OpenEI. The data may have been moved or is unavailable.\n"
          } else if (status_code >= 500) {
            "  OpenEI server error. Try again later.\n"
          } else {
            "  Check your internet connection and try again.\n"
          }
        )
      }
    }

    # Read the downloaded file
    raw_data <- readr::read_csv(
      temp_file,
      show_col_types = FALSE,
      col_types = readr::cols(
        .default = readr::col_guess()
      )
    )

    # Check if data needs processing (has raw microdata format)
    # Raw microdata has: FIP, HINCP, ELEP, GASP (individual records)
    # Aggregated cohort has: FIP, HINCP*UNITS or HINCP.UNITS (pre-aggregated)
    # Note: 2022 data uses period (.) while some older formats use asterisk (*)
    is_raw_microdata <- "HINCP" %in% names(raw_data) &&
      !"HINCP*UNITS" %in% names(raw_data) &&
      !"HINCP.UNITS" %in% names(raw_data)
    is_aggregated_cohort <- "FIP" %in% names(raw_data) &&
      (any(grepl("\\*UNITS$", names(raw_data))) || any(grepl("\\.UNITS$", names(raw_data))))

    if (is_raw_microdata) {
      if (verbose) {
        message("  Processing raw microdata into cohort format...")
      }

      # Process raw → clean format using the pipeline
      data <- process_lead_cohort_data(
        data = raw_data,
        dataset = dataset,
        vintage = vintage,
        aggregate_poverty = FALSE  # Keep cohort-level detail
      )

    } else if (is_aggregated_cohort) {
      if (verbose) {
        message("  Data is aggregated cohort format, aggregating and standardizing...")
      }

      # First, aggregate data by census tract and income bracket
      # (2022 data has multiple rows per tract/bracket for different housing characteristics)
      data <- aggregate_cohort_data(raw_data, dataset, vintage, verbose = verbose)

      # Then standardize column names
      data <- standardize_cohort_columns(data, dataset, vintage)

      # Ensure geoid is character and properly padded
      if ("geoid" %in% names(data)) {
        data$geoid <- stringr::str_pad(as.character(data$geoid), width = 11, side = "left", pad = "0")
      }

    } else {
      if (verbose) {
        message("  Data appears pre-processed, using as-is...")
      }

      # Data is already processed, use as-is
      data <- raw_data

      # Ensure geoid is character and properly padded
      if ("geoid" %in% names(data)) {
        data$geoid <- stringr::str_pad(as.character(data$geoid), width = 11, side = "left", pad = "0")
      }
    }

    # Save processed data to cache
    readr::write_csv(data, cache_file)

    # Clean up temporary raw file
    if (file.exists(temp_file)) {
      unlink(temp_file)
    }

    # Import to database for faster subsequent loads
    if (verbose) {
      message("  Importing to database...")
    }
    try_import_to_database(data, dataset, vintage, verbose = verbose)

    if (verbose) {
      message("  \u2713 Downloaded, processed, and cached successfully")
    }

    return(data)

  }, error = function(e) {
    error_msg <- paste0(
      "\n", strrep("=", 60), "\n",
      "ERROR: Failed to download LEAD data\n",
      strrep("=", 60), "\n\n",
      "Details: ", e$message, "\n\n",
      "Possible solutions:\n",
      "  1. Check your internet connection\n",
      "  2. Verify OpenEI data availability at https://data.openei.org/\n",
      "  3. Try again later (OpenEI servers may be temporarily unavailable)\n",
      "  4. Check if you need to install 'httr' package: install.packages('httr')\n\n",
      "If the problem persists, please file an issue at:\n",
      "  https://github.com/ericscheier/emburden/issues\n",
      strrep("=", 60), "\n"
    )

    message(error_msg)
    return(NULL)
  })
}


#' Try to load census tract data from database
#' @keywords internal
try_load_tracts_from_database <- function(verbose = FALSE) {

  # Check if database packages are available
  if (!requireNamespace("DBI", quietly = TRUE) ||
      !requireNamespace("RSQLite", quietly = TRUE)) {
    if (verbose) {
      message("  DBI/RSQLite not available, skipping database")
    }
    return(NULL)
  }

  # Find database
  db_path <- find_emburden_db()
  if (is.null(db_path) || !file.exists(db_path)) {
    if (verbose) {
      message("  Database not found, trying CSV...")
    }
    return(NULL)
  }

  tryCatch({
    conn <- DBI::dbConnect(RSQLite::SQLite(), db_path)
    on.exit(DBI::dbDisconnect(conn))

    # Check for census tract table
    possible_tables <- c("census_tracts", "lead_census_tracts", "CensusTractData")

    for (table_name in possible_tables) {
      if (DBI::dbExistsTable(conn, table_name)) {
        data <- DBI::dbReadTable(conn, table_name) |>
          tibble::as_tibble()

        if (verbose) {
          message("  \u2713 Loaded from database table '", table_name, "'")
        }

        return(data)
      }
    }

    if (verbose) {
      message("  No census tract table found in database")
    }
    return(NULL)

  }, error = function(e) {
    if (verbose) {
      message("  Database error: ", e$message)
    }
    return(NULL)
  })
}


#' Try to load census tract data from CSV
#' @keywords internal
try_load_tracts_from_csv <- function(verbose = FALSE) {

  csv_file <- file.path("data", "CensusTractData.csv")

  if (!file.exists(csv_file)) {
    if (verbose) {
      message("  CSV file not found: ", csv_file)
    }
    return(NULL)
  }

  tryCatch({
    if (verbose) {
      message("  Reading CSV: ", basename(csv_file))
    }

    data <- readr::read_csv(
      csv_file,
      show_col_types = FALSE,
      col_types = readr::cols(
        geoid = readr::col_character(),
        .default = readr::col_guess()
      )
    )

    if (verbose) {
      message("  \u2713 Loaded from CSV")
    }

    return(data)

  }, error = function(e) {
    if (verbose) {
      message("  CSV read error: ", e$message)
    }
    return(NULL)
  })
}


#' Download census tract data from OpenEI
#' @keywords internal
download_census_tract_data <- function(verbose = FALSE) {

  if (!requireNamespace("httr", quietly = TRUE)) {
    stop("Package 'httr' required for downloading. Install with: install.packages('httr')")
  }

  # OpenEI URL for census tract data (using 2022 as latest)
  url <- "https://data.openei.org/files/6219/lead_census_tracts_2022.csv"

  if (verbose) {
    message("  Downloading from: ", url)
  }

  # Get cache directory
  cache_dir <- get_cache_dir()
  cache_file <- file.path(cache_dir, "lead_census_tracts.csv")

  tryCatch({
    response <- httr::GET(
      url,
      httr::progress(),
      httr::write_disk(cache_file, overwrite = TRUE)
    )

    if (httr::http_error(response)) {
      stop("Download failed with status ", httr::status_code(response))
    }

    # Read the downloaded file
    data <- readr::read_csv(
      cache_file,
      show_col_types = FALSE,
      col_types = readr::cols(
        geoid = readr::col_character(),
        .default = readr::col_guess()
      )
    )

    if (verbose) {
      message("  \u2713 Downloaded and cached successfully")
    }

    return(data)

  }, error = function(e) {
    if (verbose) {
      message("  Download error: ", e$message)
    }
    return(NULL)
  })
}


#' Try to import cohort data to database
#' @keywords internal
try_import_to_database <- function(data, dataset, vintage, verbose = FALSE) {

  # Check if database packages are available
  if (!requireNamespace("DBI", quietly = TRUE) ||
      !requireNamespace("RSQLite", quietly = TRUE)) {
    if (verbose) {
      message("  DBI/RSQLite not available, skipping database import")
    }
    return(FALSE)
  }

  # Find or create database
  db_path <- find_emburden_db()
  if (is.null(db_path)) {
    # Create in default location
    db_path <- file.path("data", "emburden_db.sqlite")
    dir.create("data", showWarnings = FALSE, recursive = TRUE)
  }

  # Validate data BEFORE caching to prevent corruption
  validation <- validate_before_caching(
    data = data,
    dataset = dataset,
    vintage = vintage,
    expected_states = 51,  # Assume nationwide; will be relaxed for filtered data
    strict = FALSE  # Don't throw errors, just check
  )

  if (!validation$valid) {
    if (verbose) {
      message("  \u26A0  Data validation failed, will NOT cache to database:")
      for (issue in validation$issues) {
        message("     - ", issue)
      }
      message("  This prevents corrupted data from being cached.")
    }
    return(FALSE)
  }

  table_name <- paste0("lead_", vintage, "_", dataset, "_cohorts")

  tryCatch({
    conn <- DBI::dbConnect(RSQLite::SQLite(), db_path)
    on.exit(DBI::dbDisconnect(conn))

    DBI::dbWriteTable(conn, table_name, data, overwrite = TRUE)

    if (verbose) {
      message("  \u2713 Imported to database table '", table_name, "'")
    }

    return(TRUE)

  }, error = function(e) {
    if (verbose) {
      message("  Database import error: ", e$message)
    }
    return(FALSE)
  })
}


#' Try to import census tract data to database
#' @keywords internal
try_import_tracts_to_database <- function(data, verbose = FALSE) {

  if (!requireNamespace("DBI", quietly = TRUE) ||
      !requireNamespace("RSQLite", quietly = TRUE)) {
    return(FALSE)
  }

  db_path <- find_emburden_db()
  if (is.null(db_path)) {
    db_path <- file.path("data", "emburden_db.sqlite")
    dir.create("data", showWarnings = FALSE, recursive = TRUE)
  }

  tryCatch({
    conn <- DBI::dbConnect(RSQLite::SQLite(), db_path)
    on.exit(DBI::dbDisconnect(conn))

    DBI::dbWriteTable(conn, "census_tracts", data, overwrite = TRUE)

    if (verbose) {
      message("  \u2713 Imported to database table 'census_tracts'")
    }

    return(TRUE)

  }, error = function(e) {
    if (verbose) {
      message("  Database import error: ", e$message)
    }
    return(FALSE)
  })
}


#' Find emburden_db.sqlite database
#' @keywords internal
find_emburden_db <- function() {

  # Check environment variable first
  env_path <- Sys.getenv("EMBURDEN_DB_PATH")
  if (nzchar(env_path) && file.exists(env_path)) {
    return(env_path)
  }

  # Check local data directory
  local_path <- file.path("data", "emburden_db.sqlite")
  if (file.exists(local_path)) {
    return(local_path)
  }

  return(NULL)
}


#' Get cache directory for downloaded files
#' @keywords internal
get_cache_dir <- function() {

  # During R CMD check, tests, or CRAN checks, use tempdir()
  # This satisfies CRAN policy: no writes to user's home directory during check
  is_checking <- !identical(Sys.getenv("NOT_CRAN"), "true") ||
                 nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_"))

  if (is_checking) {
    # Use tempdir() during R CMD check, CRAN checks, and automated testing
    cache_dir <- file.path(tempdir(), "emburden_cache")
  } else if (requireNamespace("rappdirs", quietly = TRUE)) {
    # Use persistent cache directory during interactive use
    cache_dir <- rappdirs::user_cache_dir("emburden")
  } else {
    # Fallback if rappdirs not available
    cache_dir <- file.path(tempdir(), "emburden_cache")
  }

  dir.create(cache_dir, showWarnings = FALSE, recursive = TRUE)
  return(cache_dir)
}


#' Aggregate cohort data by census tract and income bracket
#' @keywords internal
aggregate_cohort_data <- function(data, dataset, vintage, verbose = FALSE) {

  # Determine income bracket column name
  # FPL data uses FPL150, AMI data may use different column
  income_col <- if ("FPL150" %in% names(data)) {
    "FPL150"
  } else if ("AMI" %in% names(data)) {
    "AMI"
  } else {
    # Try to find any column that looks like an income bracket
    grep("fpl|ami|income.*bracket", names(data), ignore.case = TRUE, value = TRUE)[1]
  }

  if (is.null(income_col) || !income_col %in% names(data)) {
    if (verbose) {
      message("  Warning: Could not identify income bracket column, skipping aggregation")
    }
    return(data)
  }

  # Identify the aggregation columns (columns ending with .UNITS or *UNITS)
  units_cols <- grep("\\.(UNITS|HINCP|ELEP|GASP|FULP)$|\\.UNITS$|\\*UNITS$",
                     names(data), value = TRUE)

  if (length(units_cols) == 0) {
    if (verbose) {
      message("  Warning: No aggregation columns found, skipping aggregation")
    }
    return(data)
  }

  # Core aggregation columns
  agg_cols <- c("UNITS", "HINCP.UNITS", "ELEP.UNITS", "GASP.UNITS", "FULP.UNITS",
                "HINCP*UNITS", "ELEP*UNITS", "GASP*UNITS", "FULP*UNITS")
  agg_cols <- intersect(agg_cols, names(data))

  if (length(agg_cols) == 0) {
    if (verbose) {
      message("  Warning: No standard aggregation columns found, skipping aggregation")
    }
    return(data)
  }

  if (verbose) {
    message("  Aggregating ", nrow(data), " rows by FIP and ", income_col, "...")
  }

  # Identify housing characteristic columns to preserve
  housing_cols <- c("TEN", "TEN-YBL6", "TEN-BLD", "TEN-HFL")
  housing_cols <- intersect(housing_cols, names(data))

  # Group by FIP, income_bracket, AND housing characteristics to preserve dimensions
  group_cols <- c("FIP", income_col, housing_cols)

  if (verbose) {
    if (length(housing_cols) > 0) {
      message("  Preserving housing dimensions: ", paste(housing_cols, collapse = ", "))
    } else {
      message("  No housing dimension columns found, aggregating without them")
    }
  }

  # Aggregate by summing across the specified grouping
  aggregated <- data |>
    dplyr::group_by(dplyr::across(dplyr::all_of(group_cols))) |>
    dplyr::summarise(
      dplyr::across(dplyr::all_of(agg_cols), ~sum(.x, na.rm = TRUE)),
      .groups = "drop"
    )

  if (verbose) {
    message("  Aggregated to ", nrow(aggregated), " rows")
  }

  return(aggregated)
}


#' Standardize cohort column names across vintages
#' @keywords internal
standardize_cohort_columns <- function(data, dataset, vintage) {

  # Handle raw data that uses FIP instead of geoid
  if ("FIP" %in% names(data) && !"geoid" %in% names(data)) {
    data <- data |>
      dplyr::rename(geoid = FIP)
  }

  # Handle legacy data that uses geo_id instead of geoid
  if ("geo_id" %in% names(data) && !"geoid" %in% names(data)) {
    data <- data |>
      dplyr::rename(geoid = geo_id)
  }

  # Ensure geoid is character
  if ("geoid" %in% names(data)) {
    data$geoid <- as.character(data$geoid)
  }

  # Handle aggregated cohort format column names
  # These columns come from ZIP files (aggregated format)
  # Note: 2022 uses period (HINCP.UNITS), older formats use asterisk (HINCP*UNITS)

  # Income bracket column (check multiple formats)
  # FPL datasets: 2022 uses FPL150, 2018 uses FPL15
  # AMI datasets: 2022 uses AMI150, 2018 uses AMI68
  if ("FPL150" %in% names(data) && !"income_bracket" %in% names(data)) {
    data <- data |>
      dplyr::rename(income_bracket = FPL150)
  } else if ("FPL15" %in% names(data) && !"income_bracket" %in% names(data)) {
    data <- data |>
      dplyr::rename(income_bracket = FPL15)
  } else if ("AMI150" %in% names(data) && !"income_bracket" %in% names(data)) {
    data <- data |>
      dplyr::rename(income_bracket = AMI150)
  } else if ("AMI68" %in% names(data) && !"income_bracket" %in% names(data)) {
    data <- data |>
      dplyr::rename(income_bracket = AMI68)
  }

  # Households column
  if ("UNITS" %in% names(data) && !"households" %in% names(data)) {
    data <- data |>
      dplyr::rename(households = UNITS)
  }

  # Total income column (check both period and asterisk formats)
  if ("HINCP.UNITS" %in% names(data) && !"total_income" %in% names(data)) {
    data <- data |>
      dplyr::rename(total_income = HINCP.UNITS)
  } else if ("HINCP*UNITS" %in% names(data) && !"total_income" %in% names(data)) {
    data <- data |>
      dplyr::rename(total_income = `HINCP*UNITS`)
  }

  # Total electricity spend column (check both formats)
  if ("ELEP.UNITS" %in% names(data) && !"total_electricity_spend" %in% names(data)) {
    data <- data |>
      dplyr::rename(total_electricity_spend = ELEP.UNITS)
  } else if ("ELEP*UNITS" %in% names(data) && !"total_electricity_spend" %in% names(data)) {
    data <- data |>
      dplyr::rename(total_electricity_spend = `ELEP*UNITS`)
  }

  # Total gas spend column (check both formats)
  if ("GASP.UNITS" %in% names(data) && !"total_gas_spend" %in% names(data)) {
    data <- data |>
      dplyr::rename(total_gas_spend = GASP.UNITS)
  } else if ("GASP*UNITS" %in% names(data) && !"total_gas_spend" %in% names(data)) {
    data <- data |>
      dplyr::rename(total_gas_spend = `GASP*UNITS`)
  }

  # Total other fuel spend column (check both formats)
  if ("FULP.UNITS" %in% names(data) && !"total_other_spend" %in% names(data)) {
    data <- data |>
      dplyr::rename(total_other_spend = FULP.UNITS)
  } else if ("FULP*UNITS" %in% names(data) && !"total_other_spend" %in% names(data)) {
    data <- data |>
      dplyr::rename(total_other_spend = `FULP*UNITS`)
  }

  # Standardize income bracket column name (for processed data)
  income_col <- if (dataset == "ami") "ami_bracket" else "fpl_bracket"
  if (income_col %in% names(data) && !"income_bracket" %in% names(data)) {
    data <- data |>
      dplyr::rename(income_bracket = !!income_col)
  }

  # Standardize income bracket values across vintages
  # Map 2018 percentage-based brackets to 2022 categorical brackets
  if ("income_bracket" %in% names(data)) {
    data <- data |>
      dplyr::mutate(
        income_bracket = dplyr::case_when(
          # Map 2018 AMI percentage brackets to standard categories
          income_bracket == "0-30%" ~ "very_low",
          income_bracket == "30-60%" ~ "low_mod",
          income_bracket == "60-80%" ~ "low_mod",
          income_bracket == "80-100%" ~ "mid_high",
          income_bracket == "100%+" ~ "mid_high",
          # Keep 2022 brackets as-is
          income_bracket %in% c("very_low", "low_mod", "mid_high") ~ income_bracket,
          # For FPL brackets, keep as-is (not standardizing FPL yet)
          TRUE ~ income_bracket
        )
      )
  }

  # Create total_* columns from per-household columns if needed
  # The "total" columns represent household-weighted sums for proper aggregation
  if ("income" %in% names(data) && !"total_income" %in% names(data)) {
    data$total_income <- data$income * data$households
  }

  if ("electricity_spend" %in% names(data) && !"total_electricity_spend" %in% names(data)) {
    data$total_electricity_spend <- data$electricity_spend * data$households
  }

  if ("gas_spend" %in% names(data) && !"total_gas_spend" %in% names(data)) {
    data$total_gas_spend <- data$gas_spend * data$households
  }

  if ("other_spend" %in% names(data) && !"total_other_spend" %in% names(data)) {
    data$total_other_spend <- data$other_spend * data$households
  }

  # Ensure required columns exist
  required_cols <- c("geoid", "income_bracket", "households",
                     "total_income", "total_electricity_spend")

  missing_cols <- setdiff(required_cols, names(data))
  if (length(missing_cols) > 0) {
    warning("Missing expected columns: ", paste(missing_cols, collapse = ", "))
  }

  return(data)
}


#' Get state FIPS codes from abbreviations
#' @keywords internal
get_state_fips <- function(state_abbrs) {

  # State FIPS lookup table
  state_fips_table <- c(
    AL = "01", AK = "02", AZ = "04", AR = "05", CA = "06",
    CO = "08", CT = "09", DE = "10", FL = "12", GA = "13",
    HI = "15", ID = "16", IL = "17", IN = "18", IA = "19",
    KS = "20", KY = "21", LA = "22", ME = "23", MD = "24",
    MA = "25", MI = "26", MN = "27", MS = "28", MO = "29",
    MT = "30", NE = "31", NV = "32", NH = "33", NJ = "34",
    NM = "35", NY = "36", NC = "37", ND = "38", OH = "39",
    OK = "40", OR = "41", PA = "42", RI = "44", SC = "45",
    SD = "46", TN = "47", TX = "48", UT = "49", VT = "50",
    VA = "51", WA = "53", WV = "54", WI = "55", WY = "56",
    DC = "11", PR = "72"
  )

  fips <- state_fips_table[toupper(state_abbrs)]

  if (any(is.na(fips))) {
    missing <- state_abbrs[is.na(fips)]
    stop("Invalid state abbreviation(s): ", paste(missing, collapse = ", "))
  }

  return(unname(fips))
}

#' Get all state abbreviations
#' @return Character vector of all 51 state abbreviations (50 states + DC)
#' @keywords internal
get_all_states <- function() {
  c(
    "AL", "AK", "AZ", "AR", "CA", "CO", "CT", "DE", "FL", "GA",
    "HI", "ID", "IL", "IN", "IA", "KS", "KY", "LA", "ME", "MD",
    "MA", "MI", "MN", "MS", "MO", "MT", "NE", "NV", "NH", "NJ",
    "NM", "NY", "NC", "ND", "OH", "OK", "OR", "PA", "RI", "SC",
    "SD", "TN", "TX", "UT", "VT", "VA", "WA", "WV", "WI", "WY",
    "DC"
  )
}

#' Download and merge data from multiple states
#' @param dataset Character, "ami" or "fpl"
#' @param vintage Character, "2018" or "2022"
#' @param states Character vector of state abbreviations
#' @param verbose Logical, print progress messages
#' @return Combined tibble with data from all states
#' @keywords internal
download_and_merge_states <- function(dataset, vintage, states, verbose = TRUE) {

  if (verbose) {
    message(sprintf("Downloading %s %s data for %d states...", vintage, dataset, length(states)))
  }

  # Download each state's data
  all_data <- list()
  failed_states <- character()

  for (i in seq_along(states)) {
    state <- states[i]

    if (verbose) {
      message(sprintf("[%d/%d] Downloading %s...", i, length(states), state))
    }

    # Download single state
    tryCatch({
      state_data <- download_lead_data(
        dataset = dataset,
        vintage = vintage,
        states = state,
        verbose = FALSE  # Suppress individual state messages
      )

      if (!is.null(state_data) && nrow(state_data) > 0) {
        all_data[[state]] <- state_data
      } else {
        failed_states <- c(failed_states, state)
      }

    }, error = function(e) {
      warning(sprintf("Failed to download %s: %s", state, e$message))
      failed_states <- c(failed_states, state)
    })
  }

  if (length(all_data) == 0) {
    stop("Failed to download data from any state")
  }

  if (length(failed_states) > 0 && verbose) {
    message(sprintf("Warning: Failed to download %d state(s): %s",
                    length(failed_states), paste(failed_states, collapse = ", ")))
  }

  # Merge all state data
  if (verbose) {
    message(sprintf("Merging data from %d states...", length(all_data)))
  }

  combined_data <- dplyr::bind_rows(all_data)

  if (verbose) {
    message(sprintf("Successfully merged %s rows from %d states",
                    format(nrow(combined_data), big.mark = ","),
                    length(all_data)))
  }

  # Save merged data to cache
  cache_dir <- get_cache_dir()
  cache_file <- file.path(cache_dir, paste0("lead_", vintage, "_", dataset, ".csv"))

  if (verbose) {
    message("  Caching merged nationwide data...")
  }

  readr::write_csv(combined_data, cache_file)

  # Import to database for faster subsequent loads
  if (verbose) {
    message("  Importing to database...")
  }
  try_import_to_database(combined_data, dataset, vintage, verbose = verbose)

  if (verbose) {
    message("  \u2713 Downloaded, merged, and cached successfully")
  }

  return(combined_data)
}

#' Convert county identifiers to FIPS codes
#'
#' Supports both 3-digit county FIPS codes and 5-digit state+county FIPS codes.
#' County names can be matched from the orange_county_sample or nc_sample datasets.
#'
#' @param counties Character vector of county identifiers (FIPS codes or names)
#' @param states Character vector of state abbreviations for context
#'
#' @return Character vector of 3-digit county FIPS codes
#' @keywords internal
get_county_fips <- function(counties, states) {

  # NC county lookup table (for common counties)
  nc_county_table <- c(
    Orange = "135", Durham = "063", Wake = "183",
    Mecklenburg = "119", Guilford = "081", Forsyth = "067",
    Cumberland = "051", Buncombe = "021", Gaston = "071",
    Union = "179", Iredell = "097", Cabarrus = "025",
    Rowan = "159", Catawba = "035", Alamance = "001",
    Randolph = "151", Johnston = "101", Davidson = "057",
    Onslow = "133"
  )

  # Process each county identifier
  fips_codes <- character(length(counties))

  for (i in seq_along(counties)) {
    county <- counties[i]

    # Check if already a 3-digit FIPS code
    if (grepl("^\\d{3}$", county)) {
      fips_codes[i] <- county
    }
    # Check if 5-digit state+county FIPS (extract county part)
    else if (grepl("^\\d{5}$", county)) {
      fips_codes[i] <- substr(county, 3, 5)
    }
    # Try county name lookup (NC only for now)
    else if ("NC" %in% toupper(states)) {
      # Case-insensitive lookup
      county_title <- tools::toTitleCase(tolower(county))
      if (county_title %in% names(nc_county_table)) {
        fips_codes[i] <- nc_county_table[county_title]
      } else {
        warning("County name '", county, "' not found in lookup table. Please use 3-digit FIPS code.")
        fips_codes[i] <- NA_character_
      }
    }
    else {
      warning("County name lookups currently only supported for NC. Please use 3-digit FIPS code for '", county, "'.")
      fips_codes[i] <- NA_character_
    }
  }

  # Remove NAs
  fips_codes <- fips_codes[!is.na(fips_codes)]

  return(fips_codes)
}

test_that("get_state_fips converts valid state abbreviations", {
  # Single state
  expect_equal(get_state_fips("NC"), "37")
  expect_equal(get_state_fips("CA"), "06")
  expect_equal(get_state_fips("NY"), "36")

  # Multiple states
  result <- get_state_fips(c("NC", "SC", "VA"))
  expect_equal(result, c("37", "45", "51"))

  # Case insensitive
  expect_equal(get_state_fips("nc"), "37")
  expect_equal(get_state_fips("Nc"), "37")

  # DC and territories
  expect_equal(get_state_fips("DC"), "11")
  expect_equal(get_state_fips("PR"), "72")
})

test_that("get_state_fips returns unnamed vector", {
  result <- get_state_fips(c("NC", "SC"))

  # Should be unnamed
  expect_null(names(result))
  expect_equal(length(result), 2)
})

test_that("get_state_fips handles all 50 states plus DC and PR", {
  # Test a sample of states across different FIPS codes
  states <- c("AL", "AK", "AZ", "AR", "CA", "FL", "GA", "HI", "IL", "TX")
  fips <- c("01", "02", "04", "05", "06", "12", "13", "15", "17", "48")

  result <- get_state_fips(states)
  expect_equal(result, fips)
})

test_that("get_state_fips errors on invalid state abbreviation", {
  expect_error(
    get_state_fips("XX"),
    "Invalid state abbreviation.*XX"
  )

  expect_error(
    get_state_fips(c("NC", "INVALID", "SC")),
    "Invalid state abbreviation.*INVALID"
  )

  # Multiple invalid
  expect_error(
    get_state_fips(c("XX", "YY")),
    "Invalid state abbreviation"
  )
})

test_that("standardize_cohort_columns renames FIP to geoid", {
  data <- data.frame(
    FIP = c("37183020100", "37051003400"),
    income_bracket = c("very_low", "low_mod"),
    households = c(100, 150),
    total_income = c(2500000, 6000000),
    total_electricity_spend = c(120000, 180000)
  )

  result <- standardize_cohort_columns(data, "ami", "2022")

  expect_true("geoid" %in% names(result))
  expect_false("FIP" %in% names(result))
  expect_equal(result$geoid, c("37183020100", "37051003400"))
})

test_that("standardize_cohort_columns ensures geoid is character", {
  data <- data.frame(
    geoid = c(37183020100, 37051003400),  # Numeric
    income_bracket = c("very_low", "low_mod"),
    households = c(100, 150),
    total_income = c(2500000, 6000000),
    total_electricity_spend = c(120000, 180000)
  )

  result <- standardize_cohort_columns(data, "ami", "2022")

  expect_type(result$geoid, "character")
  expect_equal(result$geoid, c("37183020100", "37051003400"))
})

test_that("standardize_cohort_columns renames aggregated cohort columns", {
  data <- data.frame(
    geoid = c("37183020100", "37051003400"),
    FPL150 = c("Below", "Above"),
    UNITS = c(100, 150),
    `HINCP*UNITS` = c(5000000, 7500000),
    `ELEP*UNITS` = c(120000, 180000),
    `GASP*UNITS` = c(80000, 90000),
    `FULP*UNITS` = c(10000, 15000),
    check.names = FALSE
  )

  result <- standardize_cohort_columns(data, "fpl", "2022")

  # Check renamed columns
  expect_true("income_bracket" %in% names(result))
  expect_true("households" %in% names(result))
  expect_true("total_income" %in% names(result))
  expect_true("total_electricity_spend" %in% names(result))
  expect_true("total_gas_spend" %in% names(result))
  expect_true("total_other_spend" %in% names(result))

  # Check old names removed
  expect_false("FPL150" %in% names(result))
  expect_false("UNITS" %in% names(result))
})

test_that("standardize_cohort_columns renames dataset-specific income columns", {
  # AMI dataset
  ami_data <- data.frame(
    geoid = "37183020100",
    ami_bracket = "very_low",
    households = 100,
    total_income = 2500000,
    total_electricity_spend = 120000
  )

  ami_result <- standardize_cohort_columns(ami_data, "ami", "2022")
  expect_true("income_bracket" %in% names(ami_result))
  expect_false("ami_bracket" %in% names(ami_result))
  expect_equal(ami_result$income_bracket, "very_low")

  # FPL dataset
  fpl_data <- data.frame(
    geoid = "37183020100",
    fpl_bracket = "0-100%",
    households = 100,
    total_income = 2500000,
    total_electricity_spend = 120000
  )

  fpl_result <- standardize_cohort_columns(fpl_data, "fpl", "2022")
  expect_true("income_bracket" %in% names(fpl_result))
  expect_false("fpl_bracket" %in% names(fpl_result))
  expect_equal(fpl_result$income_bracket, "0-100%")
})

test_that("standardize_cohort_columns maps 2018 AMI brackets to standard categories", {
  data <- data.frame(
    geoid = rep("37183020100", 5),
    income_bracket = c("0-30%", "30-60%", "60-80%", "80-100%", "100%+"),
    households = c(50, 60, 70, 80, 90),
    total_income = c(750000, 2400000, 4200000, 6400000, 9000000),
    total_electricity_spend = c(60000, 72000, 84000, 96000, 108000)
  )

  result <- standardize_cohort_columns(data, "ami", "2018")

  # Check mappings
  expect_equal(result$income_bracket[1], "very_low")     # 0-30%
  expect_equal(result$income_bracket[2], "low_mod")      # 30-60%
  expect_equal(result$income_bracket[3], "low_mod")      # 60-80%
  expect_equal(result$income_bracket[4], "mid_high")     # 80-100%
  expect_equal(result$income_bracket[5], "mid_high")     # 100%+
})

test_that("standardize_cohort_columns preserves 2022 AMI brackets", {
  data <- data.frame(
    geoid = rep("37183020100", 3),
    income_bracket = c("very_low", "low_mod", "mid_high"),
    households = c(50, 60, 70),
    total_income = c(750000, 2400000, 4200000),
    total_electricity_spend = c(60000, 72000, 84000)
  )

  result <- standardize_cohort_columns(data, "ami", "2022")

  # Should remain unchanged
  expect_equal(result$income_bracket, c("very_low", "low_mod", "mid_high"))
})

test_that("standardize_cohort_columns creates total_* columns from per-household columns", {
  data <- data.frame(
    geoid = rep("37183020100", 2),
    income_bracket = c("very_low", "low_mod"),
    households = c(100, 150),
    income = c(25000, 40000),
    electricity_spend = c(1200, 1500),
    gas_spend = c(800, 900),
    other_spend = c(100, 150)
  )

  result <- standardize_cohort_columns(data, "ami", "2022")

  # Check total_* columns were created
  expect_true("total_income" %in% names(result))
  expect_true("total_electricity_spend" %in% names(result))
  expect_true("total_gas_spend" %in% names(result))
  expect_true("total_other_spend" %in% names(result))

  # Check calculations
  expect_equal(result$total_income[1], 25000 * 100)
  expect_equal(result$total_income[2], 40000 * 150)
  expect_equal(result$total_electricity_spend[1], 1200 * 100)
  expect_equal(result$total_electricity_spend[2], 1500 * 150)
})

test_that("standardize_cohort_columns does not overwrite existing total_* columns", {
  data <- data.frame(
    geoid = "37183020100",
    income_bracket = "very_low",
    households = 100,
    income = 25000,
    total_income = 3000000,  # Pre-existing, different value
    total_electricity_spend = 120000
  )

  result <- standardize_cohort_columns(data, "ami", "2022")

  # Should keep the existing value
  expect_equal(result$total_income, 3000000)
})

test_that("standardize_cohort_columns warns about missing required columns", {
  # Data missing several required columns
  data <- data.frame(
    geoid = "37183020100",
    households = 100
    # Missing: income_bracket, total_income, total_electricity_spend
  )

  expect_warning(
    standardize_cohort_columns(data, "ami", "2022"),
    "Missing expected columns"
  )
})

test_that("standardize_cohort_columns handles complete valid input silently", {
  data <- data.frame(
    geoid = "37183020100",
    income_bracket = "very_low",
    households = 100,
    total_income = 2500000,
    total_electricity_spend = 120000,
    total_gas_spend = 80000,
    total_other_spend = 10000
  )

  # Should not warn or error
  expect_silent(standardize_cohort_columns(data, "ami", "2022"))
})

test_that("standardize_cohort_columns handles empty dataframe", {
  data <- data.frame(
    geoid = character(),
    income_bracket = character(),
    households = numeric(),
    total_income = numeric(),
    total_electricity_spend = numeric()
  )

  result <- standardize_cohort_columns(data, "ami", "2022")

  expect_equal(nrow(result), 0)
  expect_true("geoid" %in% names(result))
})

test_that("standardize_cohort_columns preserves non-target columns", {
  data <- data.frame(
    geoid = "37183020100",
    income_bracket = "very_low",
    households = 100,
    total_income = 2500000,
    total_electricity_spend = 120000,
    custom_column = "test_value",
    another_col = 42
  )

  result <- standardize_cohort_columns(data, "ami", "2022")

  # Custom columns should be preserved
  expect_true("custom_column" %in% names(result))
  expect_true("another_col" %in% names(result))
  expect_equal(result$custom_column, "test_value")
  expect_equal(result$another_col, 42)
})

# Phase 2: Data Loader Tests
# Tests for load_cohort_data, load_census_tract_data, and related functions
#
# Data Coverage: Package supports all 51 US states (50 states + DC) with ~73,000 census tracts
# and 2.3+ million cohort records. Tests use mocked data for speed but validate
# nationwide functionality.

test_that("load_cohort_data validates dataset parameter", {
  expect_error(
    load_cohort_data(dataset = "invalid", verbose = FALSE),
    "should be one of"
  )
})

test_that("load_cohort_data validates vintage parameter", {
  expect_error(
    load_cohort_data(dataset = "ami", vintage = "2020", verbose = FALSE),
    "vintage must be '2018' or '2022'"
  )
})

test_that("load_cohort_data handles missing data with download fallback", {
  # Mock all local sources to fail (simulating no local data)
  mockery::stub(load_cohort_data, "try_load_from_database", NULL)
  mockery::stub(load_cohort_data, "try_load_from_csv", NULL)

  # Mock Zenodo to fail (so we test OpenEI fallback)
  mockery::stub(load_cohort_data, "download_from_zenodo", NULL)

  # Mock successful download as fallback
  fallback_data <- data.frame(
    geoid = c("37051003400"),
    income_bracket = c("very_low"),
    households = c(100),
    total_income = c(2500000),
    total_electricity_spend = c(120000)
  )
  mockery::stub(load_cohort_data, "download_lead_data", fallback_data)
  mockery::stub(load_cohort_data, "try_import_to_database", TRUE)

  # Should successfully load via download fallback
  result <- load_cohort_data(
    dataset = "ami",
    vintage = "2022",
    verbose = FALSE
  )

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 1)
  expect_true("geoid" %in% names(result))
})

test_that("load_cohort_data fails gracefully when all sources unavailable", {
  # Mock all sources to fail
  mockery::stub(load_cohort_data, "try_load_from_database", NULL)
  mockery::stub(load_cohort_data, "try_load_from_csv", NULL)
  mockery::stub(load_cohort_data, "download_from_zenodo", NULL)
  mockery::stub(load_cohort_data, "download_lead_data", NULL)

  # Should error with informative message
  expect_error(
    load_cohort_data(dataset = "ami", vintage = "2022", verbose = FALSE),
    "Failed to load data from any source"
  )
})

test_that("load_census_tract_data accepts valid parameters", {
  # load_census_tract_data signature: (states = NULL, verbose = TRUE)
  # It doesn't validate dataset or vintage, just states
  # Just verify the function is callable
  expect_type(load_census_tract_data, "closure")
})

test_that("data loader helper functions exist and are callable", {
  # Verify main exported functions are defined

  # Check that the main exported functions exist
  expect_true(exists("load_cohort_data"))
  expect_true(exists("load_census_tract_data"))

  # Verify they are functions
  expect_type(load_cohort_data, "closure")
  expect_type(load_census_tract_data, "closure")
})

test_that("load_cohort_data returns expected structure with mocked data", {
  # Unit test version - always runs with fixtures
  fixture_data <- data.frame(
    geoid = c("37051003400", "37183020100", "45001020100"),
    income_bracket = c("very_low", "low_mod", "very_low"),
    households = c(100, 150, 120),
    total_income = c(2500000, 6000000, 3000000),
    total_electricity_spend = c(120000, 180000, 144000),
    total_gas_spend = c(40000, 60000, 48000),
    total_other_spend = c(10000, 15000, 12000)
  )

  # Mock data sources
  mockery::stub(load_cohort_data, "try_load_from_database", NULL)
  mockery::stub(load_cohort_data, "try_load_from_csv", fixture_data)

  # Load with mocked data
  result <- load_cohort_data(
    dataset = "ami",
    states = NULL,
    vintage = "2022",
    verbose = FALSE
  )

  # Verify structure
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 3)

  # Check for expected columns
  expected_cols <- c("geoid", "income_bracket", "households",
                     "total_income", "total_electricity_spend")
  expect_true(all(expected_cols %in% names(result)))

  # Verify data integrity
  expect_type(result$geoid, "character")
  expect_type(result$income_bracket, "character")
  expect_type(result$households, "double")
})

test_that("load_cohort_data returns expected structure when data exists", {
  skip_if_not(file.exists("data"), "No data directory found")

  # Check if any ami data files exist
  ami_files <- list.files("data", pattern = "ami.*\\.csv", full.names = TRUE)
  skip_if(length(ami_files) == 0, "No AMI data files found for testing")

  # Try to load data
  result <- try(load_cohort_data(
    dataset = "ami",
    states = NULL,
    vintage = "2022",
    verbose = FALSE
  ), silent = TRUE)

  if (!inherits(result, "try-error") && !is.null(result)) {
    expect_s3_class(result, "data.frame")

    # Check for expected columns (at minimum)
    expected_cols <- c("geoid", "income_bracket")
    expect_true(all(expected_cols %in% names(result)))
  }
})

test_that("state filtering works correctly", {
  # Create test fixture data with multiple states
  fixture_data <- data.frame(
    geoid = c("37051003400", "37183020100", "45001020100", "13001020100"),
    income_bracket = c("very_low", "low_mod", "very_low", "low_mod"),
    households = c(100, 150, 120, 130),
    total_income = c(2500000, 6000000, 3000000, 5200000),
    total_electricity_spend = c(120000, 180000, 144000, 156000)
  )

  # Mock try_load_from_database to return NULL (skip database)
  mockery::stub(load_cohort_data, "try_load_from_database", NULL)

  # Mock try_load_from_csv to return our fixture data
  mockery::stub(load_cohort_data, "try_load_from_csv", fixture_data)

  # Test NC filter (geoid starts with "37")
  result_nc <- load_cohort_data(dataset = "ami", states = "NC", vintage = "2022", verbose = FALSE)

  # Should only have NC records (geoid starting with "37")
  expect_equal(nrow(result_nc), 2)
  expect_true(all(substr(result_nc$geoid, 1, 2) == "37"))

  # Test SC filter (geoid starts with "45")
  result_sc <- load_cohort_data(dataset = "ami", states = "SC", vintage = "2022", verbose = FALSE)

  expect_equal(nrow(result_sc), 1)
  expect_true(all(substr(result_sc$geoid, 1, 2) == "45"))

  # Test multiple states
  result_multi <- load_cohort_data(dataset = "ami", states = c("NC", "GA"), vintage = "2022", verbose = FALSE)

  expect_equal(nrow(result_multi), 3)  # NC (2) + GA (1)
  expect_true(all(substr(result_multi$geoid, 1, 2) %in% c("37", "13")))
})

test_that("income bracket filtering works correctly", {
  # Create test fixture data with multiple income brackets
  fixture_data <- data.frame(
    geoid = rep("37051003400", 4),
    income_bracket = c("very_low", "low_mod", "mid_high", "very_low"),
    households = c(100, 150, 200, 110),
    total_income = c(1500000, 4500000, 8000000, 1650000),
    total_electricity_spend = c(90000, 135000, 160000, 99000)
  )

  # Mock the data loader functions
  mockery::stub(load_cohort_data, "try_load_from_database", NULL)
  mockery::stub(load_cohort_data, "try_load_from_csv", fixture_data)

  # Test filtering to single bracket
  result_single <- load_cohort_data(
    dataset = "ami",
    income_brackets = "very_low",
    vintage = "2022",
    verbose = FALSE
  )

  expect_equal(nrow(result_single), 2)
  expect_true(all(result_single$income_bracket == "very_low"))

  # Test filtering to multiple brackets
  result_multi <- load_cohort_data(
    dataset = "ami",
    income_brackets = c("very_low", "low_mod"),
    vintage = "2022",
    verbose = FALSE
  )

  expect_equal(nrow(result_multi), 3)
  expect_true(all(result_multi$income_bracket %in% c("very_low", "low_mod")))
  expect_false(any(result_multi$income_bracket == "mid_high"))
})

test_that("load_cohort_data handles corrupt data files", {
  # Mock try_load_from_database to return NULL
  mockery::stub(load_cohort_data, "try_load_from_database", NULL)

  # Mock try_load_from_csv to simulate corrupt file (return NULL)
  mockery::stub(load_cohort_data, "try_load_from_csv", NULL)

  # Mock Zenodo to fail (so we test OpenEI fallback)
  mockery::stub(load_cohort_data, "download_from_zenodo", NULL)

  # Mock download_lead_data to return valid data (fallback)
  valid_data <- data.frame(
    geoid = c("37051003400"),
    income_bracket = c("very_low"),
    households = c(100),
    total_income = c(2500000),
    total_electricity_spend = c(120000)
  )
  mockery::stub(load_cohort_data, "download_lead_data", valid_data)

  # Mock database import to succeed silently
  mockery::stub(load_cohort_data, "try_import_to_database", TRUE)

  # Should fall back to download when CSV is corrupt/unavailable
  result <- load_cohort_data(dataset = "ami", vintage = "2022", verbose = FALSE)

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 1)
  expect_true("geoid" %in% names(result))
})

test_that("database fallback works when CSV unavailable", {
  # Create valid database response
  db_data <- data.frame(
    geoid = c("37051003400", "37183020100"),
    income_bracket = c("very_low", "low_mod"),
    households = c(100, 150),
    total_income = c(2500000, 6000000),
    total_electricity_spend = c(120000, 180000)
  )

  # Mock all download sources to fail, only database succeeds
  mockery::stub(load_cohort_data, "try_load_from_csv", NULL)
  mockery::stub(load_cohort_data, "download_from_zenodo", NULL)
  mockery::stub(load_cohort_data, "download_lead_data", NULL)  # Also mock OpenEI fallback
  mockery::stub(load_cohort_data, "try_load_from_database", db_data)

  # Mock corruption detection to pass (2-row data is valid for testing)
  mockery::stub(load_cohort_data, "detect_database_corruption", list(is_corrupted = FALSE))

  # Should use database data when all download sources are unavailable

  result <- load_cohort_data(dataset = "ami", vintage = "2022", verbose = FALSE)

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 2)
  expect_true(all(c("geoid", "income_bracket") %in% names(result)))
})

test_that("download fallback works when local data unavailable", {
  # Mock all local sources to fail
  mockery::stub(load_cohort_data, "try_load_from_database", NULL)
  mockery::stub(load_cohort_data, "try_load_from_csv", NULL)

  # Mock Zenodo to fail (so we test OpenEI fallback)
  mockery::stub(load_cohort_data, "download_from_zenodo", NULL)

  # Mock successful download
  download_data <- data.frame(
    geoid = c("37051003400"),
    income_bracket = c("very_low"),
    households = c(100),
    total_income = c(2500000),
    total_electricity_spend = c(120000)
  )
  mockery::stub(load_cohort_data, "download_lead_data", download_data)

  # Mock database import
  mockery::stub(load_cohort_data, "try_import_to_database", TRUE)

  result <- load_cohort_data(dataset = "ami", vintage = "2022", verbose = FALSE)

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 1)
  expect_true("geoid" %in% names(result))
})

test_that("load_cohort_data handles FPL data correctly", {
  # Note: aggregate_poverty is a parameter of process_lead_cohort_data, not load_cohort_data
  # This test verifies FPL data loading works correctly

  fpl_data <- data.frame(
    geoid = c("37051003400", "37183020100"),
    income_bracket = c("0-100% FPL", "100-150% FPL"),
    households = c(100, 150),
    total_income = c(1000000, 2250000),
    total_electricity_spend = c(80000, 112500)
  )

  mockery::stub(load_cohort_data, "try_load_from_database", NULL)
  mockery::stub(load_cohort_data, "try_load_from_csv", fpl_data)

  result <- load_cohort_data(dataset = "fpl", vintage = "2022", verbose = FALSE)

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 2)
  expect_true(all(grepl("FPL", result$income_bracket)))
})

test_that("vintage-specific schema handling works", {
  # Test that both 2018 and 2022 data produce consistent schema

  # 2018 data with old percentage-based brackets
  data_2018 <- data.frame(
    geoid = c("37051003400", "37183020100"),
    income_bracket = c("0-30%", "80-100%"),
    households = c(100, 150),
    total_income = c(1500000, 7200000),
    total_electricity_spend = c(90000, 180000)
  )

  # 2022 data with new categorical brackets
  data_2022 <- data.frame(
    geoid = c("37051003400", "37183020100"),
    income_bracket = c("very_low", "mid_high"),
    households = c(100, 150),
    total_income = c(1500000, 7200000),
    total_electricity_spend = c(90000, 180000)
  )

  # Test 2018 loading
  mockery::stub(load_cohort_data, "try_load_from_database", NULL)
  mockery::stub(load_cohort_data, "try_load_from_csv", data_2018)

  result_2018 <- load_cohort_data(dataset = "ami", vintage = "2018", verbose = FALSE)

  expect_s3_class(result_2018, "data.frame")
  expect_true(all(c("geoid", "income_bracket", "households") %in% names(result_2018)))

  # Test 2022 loading
  mockery::stub(load_cohort_data, "try_load_from_csv", data_2022)

  result_2022 <- load_cohort_data(dataset = "ami", vintage = "2022", verbose = FALSE)

  expect_s3_class(result_2022, "data.frame")
  expect_true(all(c("geoid", "income_bracket", "households") %in% names(result_2022)))

  # Both should have same schema
  expect_equal(sort(names(result_2018)), sort(names(result_2022)))
})

# Census Tract Data Tests -----------------------------------------------

test_that("load_census_tract_data handles missing data gracefully", {
  # Mock all sources to fail
  mockery::stub(load_census_tract_data, "try_load_tracts_from_database", NULL)
  mockery::stub(load_census_tract_data, "try_load_tracts_from_csv", NULL)

  # Mock successful download as fallback
  tract_data <- data.frame(
    geoid = c("37051003400", "37183020100"),
    state_abbr = c("NC", "NC"),
    county_name = c("Wake", "Durham"),
    tract_name = c("Tract 34", "Tract 201"),
    utility_name = c("Duke Energy", "Duke Energy"),
    stringsAsFactors = FALSE
  )
  mockery::stub(load_census_tract_data, "download_census_tract_data", tract_data)
  mockery::stub(load_census_tract_data, "try_import_tracts_to_database", TRUE)

  result <- load_census_tract_data(verbose = FALSE)

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 2)
  expect_true("geoid" %in% names(result))
  expect_true("state_abbr" %in% names(result))
})

test_that("load_census_tract_data filters by state", {
  # Create tract data for multiple states
  tract_data <- data.frame(
    geoid = c("37051003400", "45001020100", "13001020100"),
    state_abbr = c("NC", "SC", "GA"),
    county_name = c("Wake", "Berkeley", "Fulton"),
    utility_name = c("Duke Energy", "SCE&G", "Georgia Power"),
    stringsAsFactors = FALSE
  )

  mockery::stub(load_census_tract_data, "try_load_tracts_from_database", NULL)
  mockery::stub(load_census_tract_data, "try_load_tracts_from_csv", tract_data)

  # Test single state filter
  result <- load_census_tract_data(states = "NC", verbose = FALSE)

  expect_equal(nrow(result), 1)
  expect_equal(result$state_abbr[1], "NC")

  # Test multiple state filter
  mockery::stub(load_census_tract_data, "try_load_tracts_from_csv", tract_data)
  result_multi <- load_census_tract_data(states = c("NC", "SC"), verbose = FALSE)

  expect_equal(nrow(result_multi), 2)
  expect_true(all(result_multi$state_abbr %in% c("NC", "SC")))
})

# Edge Cases & Additional Coverage --------------------------------------

test_that("load_cohort_data handles state filters that match no data", {
  # All data is NC, but we request GA
  fixture_data <- data.frame(
    geoid = c("37051003400", "37183020100"),  # All NC (starts with 37)
    income_bracket = c("very_low", "low_mod"),
    households = c(100, 150),
    total_income = c(2500000, 6000000),
    total_electricity_spend = c(120000, 180000)
  )

  mockery::stub(load_cohort_data, "try_load_from_database", NULL)
  mockery::stub(load_cohort_data, "try_load_from_csv", fixture_data)

  # Request GA data when only NC exists
  result <- load_cohort_data(
    dataset = "ami",
    states = "GA",  # Georgia FIPS = 13, won't match any NC data
    vintage = "2022",
    verbose = FALSE
  )

  # Should return empty data.frame, not error
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 0)
})

test_that("load_cohort_data handles income bracket filters with no matches", {
  # Data has only very_low and low_mod
  fixture_data <- data.frame(
    geoid = c("37051003400", "37183020100"),
    income_bracket = c("very_low", "low_mod"),
    households = c(100, 150),
    total_income = c(2500000, 6000000),
    total_electricity_spend = c(120000, 180000)
  )

  mockery::stub(load_cohort_data, "try_load_from_database", NULL)
  mockery::stub(load_cohort_data, "try_load_from_csv", fixture_data)

  # Request mid_high when only very_low and low_mod exist
  result <- load_cohort_data(
    dataset = "ami",
    income_brackets = "mid_high",
    vintage = "2022",
    verbose = FALSE
  )

  # Should return empty data.frame
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 0)
})

test_that("check_data_sources returns proper structure", {
  result <- check_data_sources(verbose = FALSE)

  # Verify it returns a list
  expect_type(result, "list")

  # Check for expected top-level components
  expect_true("database" %in% names(result))
  expect_true("csv_files" %in% names(result))
  expect_true("download_required" %in% names(result))

  # Check database component structure
  expect_true("available" %in% names(result$database))
  expect_type(result$database$available, "logical")

  # Check CSV files component structure
  expect_true("available" %in% names(result$csv_files))
  expect_type(result$csv_files$available, "logical")

  # Check download_required is logical
  expect_type(result$download_required, "logical")
})

test_that("load_cohort_data produces expected verbose messages", {
  fixture_data <- data.frame(
    geoid = "37051003400",
    income_bracket = "very_low",
    households = 100,
    total_income = 2500000,
    total_electricity_spend = 120000
  )

  mockery::stub(load_cohort_data, "try_load_from_database", NULL)
  mockery::stub(load_cohort_data, "try_load_from_csv", fixture_data)

  # Test verbose output for basic loading
  expect_message(
    load_cohort_data(dataset = "ami", vintage = "2022", verbose = TRUE),
    "Loading 2022 AMI cohort data"
  )

  # Test verbose output for state filtering
  mockery::stub(load_cohort_data, "try_load_from_csv", fixture_data)
  expect_message(
    load_cohort_data(dataset = "ami", states = "NC", vintage = "2022", verbose = TRUE),
    "Filtered to state.*NC"
  )
})

test_that("load_cohort_data returns columns with correct types", {
  fixture_data <- data.frame(
    geoid = "37051003400",
    income_bracket = "very_low",
    households = 100L,
    total_income = 2500000,
    total_electricity_spend = 120000,
    stringsAsFactors = FALSE
  )

  mockery::stub(load_cohort_data, "try_load_from_database", NULL)
  mockery::stub(load_cohort_data, "try_load_from_csv", fixture_data)

  result <- load_cohort_data(dataset = "ami", vintage = "2022", verbose = FALSE)

  # Verify column types
  expect_type(result$geoid, "character")
  expect_type(result$income_bracket, "character")
  # households can be integer or double depending on R's coercion
  expect_true(is.numeric(result$households))
  expect_type(result$total_income, "double")
  expect_type(result$total_electricity_spend, "double")
})

test_that("Package supports all 51 US states (nationwide coverage)", {
  # Verify that all 51 states (50 states + DC) are supported
  all_states <- list_states()

  expect_length(all_states, 51)
  expect_type(all_states, "character")

  # Check key states are included
  expect_true("NC" %in% all_states)
  expect_true("CA" %in% all_states)
  expect_true("TX" %in% all_states)
  expect_true("NY" %in% all_states)
  expect_true("DC" %in% all_states)

  # Verify PR is NOT included (not in LEAD data)
  expect_false("PR" %in% all_states)

  # All should be 2-character uppercase codes
  expect_true(all(nchar(all_states) == 2))
  expect_true(all(all_states == toupper(all_states)))
})

# Tests for Housing Dimension Preservation Feature
# Tests that TEN, TEN-YBL6, TEN-BLD, and TEN-HFL columns are preserved
# during cohort data aggregation

library(testthat)

# Global variables to avoid R CMD check notes
geoid <- income_bracket <- TEN <- `TEN-YBL6` <- `TEN-BLD` <- `TEN-HFL` <- NULL
UNITS <- `HINCP.UNITS` <- `ELEP.UNITS` <- `GASP.UNITS` <- `FULP.UNITS` <- NULL


# Helper: Create test data with housing dimensions
create_housing_dimension_data <- function(n = 100, include_housing_cols = TRUE) {
  set.seed(42)

  data <- data.frame(
    FIP = rep(c("37001", "37003", "37005"), length.out = n),
    AMI = sample(c("very_low", "low_mod", "mid_high"), n, replace = TRUE),
    UNITS = rpois(n, 10),
    `HINCP.UNITS` = rpois(n, 500000),
    `ELEP.UNITS` = rpois(n, 1200),
    `GASP.UNITS` = rpois(n, 800),
    `FULP.UNITS` = rpois(n, 400),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  if (include_housing_cols) {
    data$TEN <- sample(c("1", "2", "3", "4"), n, replace = TRUE)
    data$`TEN-YBL6` <- sample(c("1-1", "1-2", "2-1", "2-2"), n, replace = TRUE)
    data$`TEN-BLD` <- sample(c("1-A", "1-B", "2-A", "2-B"), n, replace = TRUE)
    data$`TEN-HFL` <- sample(c("1-H1", "1-H2", "2-H1", "2-H2"), n, replace = TRUE)
  }

  return(data)
}


# ============================================================================
# TEST SUITE: Housing Dimension Column Preservation
# ============================================================================

test_that("aggregate_cohort_data preserves all housing dimension columns", {
  # Setup
  data <- create_housing_dimension_data(n = 200)

  # Execute - call internal function
  result <- emburden:::aggregate_cohort_data(data, "ami", "2022", verbose = FALSE)

  # Verify all housing columns are present
  expect_true("TEN" %in% names(result))
  expect_true("TEN-YBL6" %in% names(result))
  expect_true("TEN-BLD" %in% names(result))
  expect_true("TEN-HFL" %in% names(result))
})


test_that("aggregate_cohort_data groups by housing dimensions", {
  # Setup - create data with known pattern
  data <- data.frame(
    FIP = rep("37001", 4),
    AMI = rep("low_mod", 4),
    TEN = c("1", "1", "2", "2"),
    `TEN-YBL6` = c("1-1", "1-1", "2-1", "2-1"),
    `TEN-BLD` = c("1-A", "1-A", "2-A", "2-A"),
    `TEN-HFL` = c("1-H1", "1-H1", "2-H1", "2-H1"),
    UNITS = c(10, 15, 20, 25),
    `HINCP.UNITS` = c(100, 150, 200, 250),
    `ELEP.UNITS` = c(50, 75, 100, 125),
    `GASP.UNITS` = c(30, 45, 60, 75),
    `FULP.UNITS` = c(20, 30, 40, 50),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  # Execute
  result <- emburden:::aggregate_cohort_data(data, "ami", "2022", verbose = FALSE)

  # Verify - should result in 2 rows (one for each TEN value)
  # because all other housing dimensions align with TEN
  expect_equal(nrow(result), 2)
  expect_equal(sum(result$UNITS), 70)  # 10+15+20+25

  # Check that distinct housing dimension combinations are preserved
  expect_setequal(result$TEN, c("1", "2"))
})


test_that("housing dimensions create separate aggregation groups", {
  # Setup - same FIP and income, but different housing dimensions
  data <- data.frame(
    FIP = rep("37001", 3),
    AMI = rep("low_mod", 3),
    TEN = c("1", "1", "1"),  # Same tenure
    `TEN-HFL` = c("1-H1", "1-H2", "1-H3"),  # Different heating fuel
    UNITS = c(100, 200, 300),
    `HINCP.UNITS` = c(1000, 2000, 3000),
    `ELEP.UNITS` = c(500, 1000, 1500),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  # Execute
  result <- emburden:::aggregate_cohort_data(data, "ami", "2022", verbose = FALSE)

  # Verify - should result in 3 rows (one per heating fuel)
  expect_equal(nrow(result), 3)
  expect_setequal(result$`TEN-HFL`, c("1-H1", "1-H2", "1-H3"))
})


test_that("aggregation sums correctly when grouped by housing dimensions", {
  # Setup
  data <- data.frame(
    FIP = rep("37001", 4),
    AMI = rep("very_low", 4),
    TEN = c("1", "1", "2", "2"),
    `TEN-HFL` = c("1-H1", "1-H1", "2-H1", "2-H1"),
    UNITS = c(10, 20, 30, 40),
    `HINCP.UNITS` = c(100, 200, 300, 400),
    `ELEP.UNITS` = c(50, 100, 150, 200),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  # Execute
  result <- emburden:::aggregate_cohort_data(data, "ami", "2022", verbose = FALSE)

  # Verify - two groups: TEN=1 and TEN=2
  expect_equal(nrow(result), 2)

  ten1_row <- result[result$TEN == "1", ]
  expect_equal(ten1_row$UNITS, 30)  # 10 + 20
  expect_equal(ten1_row$`HINCP.UNITS`, 300)  # 100 + 200
  expect_equal(ten1_row$`ELEP.UNITS`, 150)  # 50 + 100

  ten2_row <- result[result$TEN == "2", ]
  expect_equal(ten2_row$UNITS, 70)  # 30 + 40
  expect_equal(ten2_row$`HINCP.UNITS`, 700)  # 300 + 400
  expect_equal(ten2_row$`ELEP.UNITS`, 350)  # 150 + 200
})


test_that("backward compatibility: works without housing dimension columns", {
  # Setup - data without housing dimensions
  data <- create_housing_dimension_data(n = 100, include_housing_cols = FALSE)

  # Execute
  result <- emburden:::aggregate_cohort_data(data, "ami", "2022", verbose = FALSE)

  # Verify - should still work, just without housing columns
  expect_false("TEN" %in% names(result))
  expect_false("TEN-YBL6" %in% names(result))
  expect_false("TEN-BLD" %in% names(result))
  expect_false("TEN-HFL" %in% names(result))

  # Should still have aggregation columns
  expect_true("UNITS" %in% names(result))
  expect_true("FIP" %in% names(result))
  expect_true("AMI" %in% names(result))
})


test_that("partial housing columns: some present, some missing", {
  # Setup - only TEN and TEN-HFL present
  data <- data.frame(
    FIP = rep("37001", 4),
    AMI = rep("low_mod", 4),
    TEN = c("1", "1", "2", "2"),
    `TEN-HFL` = c("1-H1", "1-H1", "2-H1", "2-H1"),
    # TEN-YBL6 and TEN-BLD intentionally missing
    UNITS = c(10, 20, 30, 40),
    `HINCP.UNITS` = c(100, 200, 300, 400),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  # Execute
  result <- emburden:::aggregate_cohort_data(data, "ami", "2022", verbose = FALSE)

  # Verify - only present housing columns are preserved
  expect_true("TEN" %in% names(result))
  expect_true("TEN-HFL" %in% names(result))
  expect_false("TEN-YBL6" %in% names(result))
  expect_false("TEN-BLD" %in% names(result))
})


# ============================================================================
# TEST SUITE: Integration with load_cohort_data
# ============================================================================

test_that("housing columns survive full data loading pipeline (mock test)", {
  skip("Requires real data or comprehensive mocking")

  # This test would verify that housing columns are preserved through
  # the entire load_cohort_data -> aggregate_cohort_data pipeline.
  # Skipped because it requires either:
  # 1. Real LEAD data files
  # 2. Complex mocking of the entire loading pipeline

  # Future enhancement: Mock the database/CSV loading to return data
  # with housing dimensions, then verify they survive to the final output
})


# ============================================================================
# TEST SUITE: Data Types and Edge Cases
# ============================================================================

test_that("housing dimension columns maintain character type", {
  # Setup
  data <- create_housing_dimension_data(n = 50)

  # Execute
  result <- emburden:::aggregate_cohort_data(data, "ami", "2022", verbose = FALSE)

  # Verify column types
  expect_type(result$TEN, "character")
  expect_type(result$`TEN-YBL6`, "character")
  expect_type(result$`TEN-BLD`, "character")
  expect_type(result$`TEN-HFL`, "character")
})


test_that("NA values in housing dimensions are handled correctly", {
  # Setup - include some NA values
  data <- create_housing_dimension_data(n = 50)
  data$TEN[1:5] <- NA
  data$`TEN-HFL`[6:10] <- NA

  # Execute
  result <- emburden:::aggregate_cohort_data(data, "ami", "2022", verbose = FALSE)

  # Verify - NA groups should be created (dplyr groups NAs together)
  expect_true(any(is.na(result$TEN)))
  expect_true(any(is.na(result$`TEN-HFL`)))
})


test_that("works with FPL dataset (different income bracket column)", {
  # Setup - FPL data uses FPL150 instead of AMI
  data <- data.frame(
    FIP = rep(c("37001", "37003"), each = 4),
    FPL150 = rep(c("0-100%", "100-150%"), 4),
    TEN = sample(c("1", "2"), 8, replace = TRUE),
    `TEN-HFL` = sample(c("1-H1", "2-H1"), 8, replace = TRUE),
    UNITS = rpois(8, 10),
    `HINCP.UNITS` = rpois(8, 50000),
    `ELEP.UNITS` = rpois(8, 1000),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  # Execute
  result <- emburden:::aggregate_cohort_data(data, "fpl", "2022", verbose = FALSE)

  # Verify housing columns are preserved with FPL data too
  expect_true("TEN" %in% names(result))
  expect_true("TEN-HFL" %in% names(result))
  expect_true("FPL150" %in% names(result))
})


test_that("large number of housing dimension combinations", {
  # Setup - create data with many unique housing dimension combinations
  n <- 500
  set.seed(123)
  data <- data.frame(
    FIP = sample(sprintf("37%03d", 1:20), n, replace = TRUE),
    AMI = sample(c("very_low", "low_mod", "mid_high"), n, replace = TRUE),
    TEN = sample(c("1", "2", "3", "4"), n, replace = TRUE),
    `TEN-YBL6` = sample(paste0(1:4, "-", 1:6), n, replace = TRUE),
    `TEN-BLD` = sample(paste0(1:4, "-", LETTERS[1:10]), n, replace = TRUE),
    `TEN-HFL` = sample(paste0(1:4, "-H", 1:8), n, replace = TRUE),
    UNITS = rpois(n, 5),
    `HINCP.UNITS` = rpois(n, 50000),
    `ELEP.UNITS` = rpois(n, 1000),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  # Execute
  result <- emburden:::aggregate_cohort_data(data, "ami", "2022", verbose = FALSE)

  # Verify
  # Result should have fewer rows than input (aggregation happened)
  expect_lt(nrow(result), nrow(data))

  # But should preserve the variety of housing dimensions
  expect_gt(length(unique(result$TEN)), 1)
  expect_gt(length(unique(result$`TEN-HFL`)), 1)

  # All housing columns present
  expect_true(all(c("TEN", "TEN-YBL6", "TEN-BLD", "TEN-HFL") %in% names(result)))
})


# ============================================================================
# TEST SUITE: Verbose Output
# ============================================================================

test_that("verbose mode reports housing dimension preservation", {
  # Setup
  data <- create_housing_dimension_data(n = 50)

  # Execute with verbose = TRUE and capture messages
  expect_message(
    emburden:::aggregate_cohort_data(data, "ami", "2022", verbose = TRUE),
    "Preserving housing dimensions"
  )

  # Verify the message contains column names
  expect_message(
    emburden:::aggregate_cohort_data(data, "ami", "2022", verbose = TRUE),
    "TEN"
  )
})


test_that("verbose mode reports when housing dimensions are absent", {
  # Setup - no housing columns
  data <- create_housing_dimension_data(n = 50, include_housing_cols = FALSE)

  # Execute with verbose = TRUE
  expect_message(
    emburden:::aggregate_cohort_data(data, "ami", "2022", verbose = TRUE),
    "No housing dimension columns found"
  )
})

#' Cache and Database Management Utilities
#'
#' @name cache_utils
#' @keywords internal
NULL

#' Get the emburden cache directory
#' @keywords internal
get_cache_dir <- function() {
  # During R CMD check, tests, or CRAN checks, use tempdir()
  # This satisfies CRAN policy: no writes to user's home directory during check
  is_checking <- !identical(Sys.getenv("NOT_CRAN"), "true") ||
                 nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_"))

  if (is_checking) {
    cache_dir <- file.path(tempdir(), "emburden_cache")
  } else {
    cache_dir <- rappdirs::user_cache_dir("emburden", "emburden")
  }

  dir.create(cache_dir, showWarnings = FALSE, recursive = TRUE)
  return(cache_dir)
}

#' Get the emburden database directory
#' @keywords internal
get_database_dir <- function() {
  # During R CMD check, tests, or CRAN checks, use tempdir()
  # This satisfies CRAN policy: no writes to user's home directory during check
  is_checking <- !identical(Sys.getenv("NOT_CRAN"), "true") ||
                 nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_"))

  if (is_checking) {
    db_dir <- file.path(tempdir(), "emburden_db")
  } else {
    db_dir <- rappdirs::user_data_dir("emburden", "emburden")
  }

  dir.create(db_dir, showWarnings = FALSE, recursive = TRUE)
  return(db_dir)
}

#' Get the full path to the emburden database file
#' @keywords internal
get_database_path <- function() {
  file.path(get_database_dir(), "emburden_db.sqlite")
}

#' Detect potentially corrupted database data
#'
#' Checks if loaded data appears corrupted (too small, missing states, missing columns).
#' **Does NOT automatically delete** - only warns and provides recommendations.
#'
#' @param data Data frame to check
#' @param dataset Character, "ami" or "fpl"
#' @param vintage Character, "2018" or "2022"
#' @param states Character vector of expected states (NULL = all US states)
#' @param verbose Logical, print warnings
#'
#' @return List with: is_corrupted (logical), issues (character vector), recommendation (character)
#' @keywords internal
detect_database_corruption <- function(data, dataset, vintage, states = NULL, verbose = TRUE) {

  if (is.null(data) || nrow(data) == 0) {
    return(list(
      is_corrupted = TRUE,
      issues = "Data is NULL or empty",
      recommendation = "Skip database and load from CSV/OpenEI"
    ))
  }

  issues <- character()

  # Expected states (all US if not specified)
  expected_states <- if (is.null(states)) 51 else length(unique(states))

  # Check 1: Suspiciously small dataset
  # Nationwide datasets should have >100k rows, single state >500 rows
  min_expected_rows <- if (expected_states == 1) 500 else 100000

  if (nrow(data) < min_expected_rows) {
    issues <- c(issues, sprintf(
      "Dataset too small: %s rows (expected >%s)",
      format(nrow(data), big.mark = ","),
      format(min_expected_rows, big.mark = ",")
    ))
  }

  # Check 2: Missing required columns
  required_cols <- c("geoid", "income_bracket", "households")
  missing_cols <- setdiff(required_cols, names(data))

  if (length(missing_cols) > 0) {
    issues <- c(issues, sprintf(
      "Missing required columns: %s",
      paste(missing_cols, collapse = ", ")
    ))
  }

  # Check 3: State coverage (if geoid available)
  if ("geoid" %in% names(data)) {
    state_fips <- unique(substr(as.character(data$geoid), 1, 2))
    actual_states <- length(state_fips)

    # For nationwide, expect at least 80% of states (40+ out of 51)
    if (expected_states > 10 && actual_states < expected_states * 0.8) {
      issues <- c(issues, sprintf(
        "Incomplete state coverage: %d states found (expected ~%d)",
        actual_states, expected_states
      ))
    }
  }

  # Check 4: state_abbr column exists and has data
  if ("state_abbr" %in% names(data)) {
    unique_states <- length(unique(data$state_abbr))
    if (expected_states > 10 && unique_states < expected_states * 0.8) {
      issues <- c(issues, sprintf(
        "state_abbr column shows only %d states (expected ~%d)",
        unique_states, expected_states
      ))
    }
  }

  is_corrupted <- length(issues) > 0

  # Generate recommendation
  recommendation <- if (is_corrupted) {
    paste(
      "Database data appears corrupted.",
      "Recommendation:",
      "  1. Delete database table for this dataset, OR",
      "  2. Delete entire database file if multiple datasets affected, OR",
      sprintf("  3. Run: clear_dataset_cache('%s', '%s')", dataset, vintage),
      sep = "\n"
    )
  } else {
    "Data appears valid"
  }

  # Print warning if corrupted
  if (is_corrupted && verbose) {
    message("\n\u26A0  WARNING: Potential database corruption detected")
    message("   Dataset: ", toupper(dataset), " ", vintage)
    message("   Issues:")
    for (issue in issues) {
      message("     - ", issue)
    }
    message("\n", recommendation, "\n")
  }

  list(
    is_corrupted = is_corrupted,
    issues = issues,
    recommendation = recommendation
  )
}

#' Validate data before caching to database
#'
#' Performs comprehensive validation BEFORE data is saved to database or cache.
#' Prevents corrupted data from being cached in the first place.
#'
#' @param data Data frame to validate
#' @param dataset Character, "ami" or "fpl"
#' @param vintage Character, "2018" or "2022"
#' @param expected_states Integer, expected number of states (51 for nationwide)
#' @param strict Logical, if TRUE throws errors; if FALSE returns list with validation results
#'
#' @return If strict=FALSE, returns list with: valid (logical), issues (character vector)
#'         If strict=TRUE, throws error on validation failure
#' @keywords internal
validate_before_caching <- function(data, dataset, vintage, expected_states = 51, strict = TRUE) {

  issues <- character()

  # Check 1: Data exists
  if (is.null(data) || nrow(data) == 0) {
    issues <- c(issues, "Data is NULL or empty")
  } else {

    # Check 2: Required columns present
    required_cols <- c("geoid", "income_bracket", "households",
                       "total_income", "total_electricity_spend")
    missing_cols <- setdiff(required_cols, names(data))

    if (length(missing_cols) > 0) {
      issues <- c(issues, sprintf(
        "Missing required columns: %s",
        paste(missing_cols, collapse = ", ")
      ))
    }

    # Check 3: Minimum row count (varies by scope)
    min_rows <- if (expected_states == 1) 500 else 100000
    if (nrow(data) < min_rows) {
      issues <- c(issues, sprintf(
        "Dataset too small: %s rows (expected >%s)",
        format(nrow(data), big.mark = ","),
        format(min_rows, big.mark = ",")
      ))
    }

    # Check 4: State coverage (for nationwide datasets)
    if (expected_states > 10 && "geoid" %in% names(data)) {
      state_fips <- unique(substr(as.character(data$geoid), 1, 2))
      actual_states <- length(state_fips)

      if (actual_states < expected_states * 0.9) {  # Require 90%+ coverage
        issues <- c(issues, sprintf(
          "Incomplete state coverage: %d states (expected %d)",
          actual_states, expected_states
        ))
      }
    }

    # Check 5: Income bracket has detailed values (not binary)
    if ("income_bracket" %in% names(data)) {
      unique_brackets <- length(unique(data$income_bracket))
      if (unique_brackets < 3) {
        issues <- c(issues, sprintf(
          "Income brackets appear binary (%d unique values, expected 5+)",
          unique_brackets
        ))
      }
    }

    # Check 6: No all-NA columns
    na_cols <- names(data)[sapply(data, function(x) all(is.na(x)))]
    if (length(na_cols) > 0) {
      issues <- c(issues, sprintf(
        "Columns with all NA values: %s",
        paste(na_cols, collapse = ", ")
      ))
    }
  }

  valid <- length(issues) == 0

  # Handle strict mode
  if (strict && !valid) {
    stop(
      "Data validation failed before caching:\n",
      paste("  -", issues, collapse = "\n"),
      "\n\nData will NOT be cached to prevent corruption."
    )
  }

  list(
    valid = valid,
    issues = issues
  )
}

#' Clear cache for a specific dataset
#'
#' Removes cached CSV files and database entries for a specific dataset/vintage.
#' Useful when you know a specific dataset is corrupted.
#'
#' @param dataset Character, "ami" or "fpl"
#' @param vintage Character, "2018" or "2022"
#' @param verbose Logical, print progress messages
#'
#' @return Invisibly returns number of items cleared
#' @export
#'
#' @examples
#' \donttest{
#' # Clear corrupted AMI 2018 cache
#' clear_dataset_cache("ami", "2018")
#'
#' # Clear FPL 2022 cache
#' clear_dataset_cache("fpl", "2022", verbose = TRUE)
#' }
clear_dataset_cache <- function(dataset = c("ami", "fpl"), vintage = c("2018", "2022"), verbose = TRUE) {

  dataset <- match.arg(dataset)
  vintage <- match.arg(vintage)

  if (verbose) {
    message("Clearing cache for ", toupper(dataset), " ", vintage, "...")
  }

  cleared <- 0

  # 1. Clear CSV cache files
  cache_dir <- get_cache_dir()
  cache_files <- c(
    file.path(cache_dir, sprintf("lead_%s_%s.csv", vintage, dataset)),
    file.path(cache_dir, sprintf("lead_%s_%s_temp.zip", vintage, dataset))
  )

  for (f in cache_files) {
    if (file.exists(f)) {
      unlink(f)
      cleared <- cleared + 1
      if (verbose) message("  \u2713 Deleted: ", basename(f))
    }
  }

  # 2. Clear database table
  db_path <- get_database_path()

  if (file.exists(db_path)) {
    # Try multiple table name formats
    table_names <- c(
      sprintf("%s_cohorts_%s", dataset, vintage),
      sprintf("lead_%s_%s_cohorts", vintage, dataset),
      sprintf("lead_%s_cohorts_%s", dataset, vintage)
    )

    tryCatch({
      conn <- DBI::dbConnect(RSQLite::SQLite(), db_path)

      for (table_name in table_names) {
        if (DBI::dbExistsTable(conn, table_name)) {
          DBI::dbExecute(conn, sprintf("DROP TABLE IF EXISTS %s", table_name))
          cleared <- cleared + 1
          if (verbose) message("  \u2713 Deleted database table: ", table_name)
        }
      }

      DBI::dbDisconnect(conn)
    }, error = function(e) {
      if (verbose) message("  \u26A0  Could not access database: ", e$message)
    })
  }

  if (verbose) {
    message("\u2713 Cleared ", cleared, " cache item(s) for ", toupper(dataset), " ", vintage)
  }

  invisible(cleared)
}

#' Clear all emburden cache and database
#'
#' Nuclear option: clears ALL cached data and database.
#' Use with caution - will require re-downloading all data.
#'
#' @param confirm Logical, must be TRUE to proceed (safety check)
#' @param verbose Logical, print progress messages
#'
#' @return Invisibly returns list with: cache_cleared (logical), db_cleared (logical)
#' @export
#'
#' @examples
#' \donttest{
#' # Clear everything (requires confirm = TRUE)
#' clear_all_cache(confirm = TRUE)
#' }
clear_all_cache <- function(confirm = FALSE, verbose = TRUE) {

  if (!confirm) {
    stop(
      "This will delete ALL cached data and the database.\n",
      "All data will need to be re-downloaded from OpenEI.\n",
      "To proceed, call: clear_all_cache(confirm = TRUE)"
    )
  }

  if (verbose) {
    message("Clearing ALL emburden cache and database...")
  }

  results <- list(cache_cleared = FALSE, db_cleared = FALSE)

  # 1. Clear cache directory
  cache_dir <- get_cache_dir()
  if (dir.exists(cache_dir)) {
    unlink(cache_dir, recursive = TRUE)
    results$cache_cleared <- TRUE
    if (verbose) message("  \u2713 Deleted cache directory: ", cache_dir)
  }

  # 2. Clear database file
  db_path <- get_database_path()
  if (file.exists(db_path)) {
    unlink(db_path)
    results$db_cleared <- TRUE
    if (verbose) message("  \u2713 Deleted database: ", db_path)
  }

  if (verbose) {
    message("\u2713 All cache and database cleared")
    message("  Note: Data will be re-downloaded from OpenEI on next use")
  }

  invisible(results)
}

# Integration Tests for Zenodo Downloads
#
# These tests actually download data from Zenodo to verify:
# 1. URLs are accessible
# 2. MD5 checksums match
# 3. Data loads correctly
#
# IMPORTANT: These tests are SKIPPED by default because they:
# - Require network access
# - Download large files (>100 MB total)
# - Take several minutes to complete
#
# To run these tests manually before a release:
#   testthat::test_file("tests/testthat/test-zenodo-integration.R")
# Or with an environment variable:
#   EMBURDEN_RUN_INTEGRATION_TESTS=1 R CMD check

test_that("Zenodo integration tests are skipped unless explicitly enabled", {
  skip_on_cran()
  skip_on_ci()

  # Skip unless explicitly requested
  run_integration <- Sys.getenv("EMBURDEN_RUN_INTEGRATION_TESTS", "0")
  skip_if(run_integration != "1", "Integration tests disabled. Set EMBURDEN_RUN_INTEGRATION_TESTS=1 to enable.")

  # If we got here, integration tests are enabled
  cat("\n\n")
  cat("==========================================\n")
  cat("  ZENODO INTEGRATION TESTS\n")
  cat("  (Full downloads + validation)\n")
  cat("==========================================\n\n")
})


test_that("Zenodo AMI 2022 download works with correct checksum", {
  skip_on_cran()
  skip_on_ci()
  run_integration <- Sys.getenv("EMBURDEN_RUN_INTEGRATION_TESTS", "0")
  skip_if(run_integration != "1", "Integration tests disabled")

  cat("Downloading AMI 2022 from Zenodo...\n")

  # Clear cache to force fresh download
  clear_dataset_cache("ami", "2022", verbose = FALSE)

  # Download from Zenodo (verbose=TRUE to show progress)
  data <- download_from_zenodo("ami", "2022", verbose = TRUE)

  # Should have successfully downloaded
  expect_false(is.null(data))
  expect_s3_class(data, "data.frame")

  # Check data structure
  expect_true("geoid" %in% names(data))
  expect_true("income_bracket" %in% names(data))
  expect_true("households" %in% names(data))

  # Should have substantial data
  expect_gt(nrow(data), 100000)

  cat("  SUCCESS: AMI 2022 downloaded and validated\n\n")
})


test_that("Zenodo FPL 2022 download works with correct checksum", {
  skip_on_cran()
  skip_on_ci()
  run_integration <- Sys.getenv("EMBURDEN_RUN_INTEGRATION_TESTS", "0")
  skip_if(run_integration != "1", "Integration tests disabled")

  cat("Downloading FPL 2022 from Zenodo...\n")

  clear_dataset_cache("fpl", "2022", verbose = FALSE)
  data <- download_from_zenodo("fpl", "2022", verbose = TRUE)

  expect_false(is.null(data))
  expect_s3_class(data, "data.frame")
  expect_gt(nrow(data), 100000)

  cat("  SUCCESS: FPL 2022 downloaded and validated\n\n")
})


test_that("Zenodo AMI 2018 download works with correct checksum", {
  skip_on_cran()
  skip_on_ci()
  run_integration <- Sys.getenv("EMBURDEN_RUN_INTEGRATION_TESTS", "0")
  skip_if(run_integration != "1", "Integration tests disabled")

  cat("Downloading AMI 2018 from Zenodo...\n")

  clear_dataset_cache("ami", "2018", verbose = FALSE)
  data <- download_from_zenodo("ami", "2018", verbose = TRUE)

  expect_false(is.null(data))
  expect_s3_class(data, "data.frame")
  expect_gt(nrow(data), 100000)

  cat("  SUCCESS: AMI 2018 downloaded and validated\n\n")
})


test_that("Zenodo FPL 2018 download works with correct checksum", {
  skip_on_cran()
  skip_on_ci()
  run_integration <- Sys.getenv("EMBURDEN_RUN_INTEGRATION_TESTS", "0")
  skip_if(run_integration != "1", "Integration tests disabled")

  cat("Downloading FPL 2018 from Zenodo...\n")

  clear_dataset_cache("fpl", "2018", verbose = FALSE)
  data <- download_from_zenodo("fpl", "2018", verbose = TRUE)

  expect_false(is.null(data))
  expect_s3_class(data, "data.frame")
  expect_gt(nrow(data), 100000)

  cat("  SUCCESS: FPL 2018 downloaded and validated\n\n")
})


test_that("All Zenodo datasets have different data (no duplicates)", {
  skip_on_cran()
  skip_on_ci()
  run_integration <- Sys.getenv("EMBURDEN_RUN_INTEGRATION_TESTS", "0")
  skip_if(run_integration != "1", "Integration tests disabled")

  cat("Verifying all datasets are distinct...\n")

  # Load all 4 datasets
  ami_2022 <- download_from_zenodo("ami", "2022", verbose = FALSE)
  fpl_2022 <- download_from_zenodo("fpl", "2022", verbose = FALSE)
  ami_2018 <- download_from_zenodo("ami", "2018", verbose = FALSE)
  fpl_2018 <- download_from_zenodo("fpl", "2018", verbose = FALSE)

  # Check row counts are different (would be identical if cached same data)
  expect_false(nrow(ami_2022) == nrow(ami_2018))
  expect_false(nrow(fpl_2022) == nrow(fpl_2018))

  # Check income bracket distributions differ between AMI and FPL
  ami_2022_brackets <- unique(ami_2022$income_bracket)
  fpl_2022_brackets <- unique(fpl_2022$income_bracket)
  expect_false(identical(sort(ami_2022_brackets), sort(fpl_2022_brackets)))

  cat("  SUCCESS: All datasets are distinct\n\n")
})


test_that("Downloaded data matches state-manifest.json metadata", {
  skip_on_cran()
  skip_on_ci()
  run_integration <- Sys.getenv("EMBURDEN_RUN_INTEGRATION_TESTS", "0")
  skip_if(run_integration != "1", "Integration tests disabled")

  cat("Cross-checking with state-manifest.json...\n")

  # Read manifest
  manifest_path <- system.file("../../zenodo-upload-nationwide/state-manifest.json", package = "emburden")
  if (!file.exists(manifest_path)) {
    skip("state-manifest.json not available in package")
  }

  manifest <- jsonlite::read_json(manifest_path)

  # Check AMI 2022 row count matches
  ami_2022 <- download_from_zenodo("ami", "2022", verbose = FALSE)
  expect_equal(nrow(ami_2022), manifest$nationwide$ami_2022$rows,
               tolerance = 100)  # Allow small variance

  cat("  SUCCESS: Data matches manifest metadata\n\n")
})


# Cleanup after integration tests
test_that("Cleanup after integration tests", {
  skip_on_cran()
  skip_on_ci()
  run_integration <- Sys.getenv("EMBURDEN_RUN_INTEGRATION_TESTS", "0")
  skip_if(run_integration != "1", "Integration tests disabled")

  cat("\n")
  cat("==========================================\n")
  cat("  INTEGRATION TESTS COMPLETE\n")
  cat("==========================================\n\n")
  cat("All Zenodo downloads validated successfully!\n")
  cat("Safe to proceed with release.\n\n")
})

#' Process raw LEAD data into clean format
#'
#' Converts raw LEAD data downloaded from OpenEI into a standardized clean format
#' suitable for analysis. Handles both 2016 (SH) and 2018+ ACS vintages.
#'
#' @param data A data frame of raw LEAD data from OpenEI
#' @param vintage Character string indicating the ACS vintage year ("2016", "2018", "2022", etc.)
#'
#' @return A data frame with standardized column names:
#'   \item{geoid}{11-digit census tract GEOID as character}
#'   \item{state_abbr}{2-letter state abbreviation (2018+ only)}
#'   \item{housing_tenure}{Housing tenure category}
#'   \item{year_constructed}{Year building was constructed category}
#'   \item{building_type}{Building type category}
#'   \item{min_units}{Minimum number of units in building}
#'   \item{detached}{Whether building is detached (1/0)}
#'   \item{primary_heating_fuel}{Primary heating fuel type}
#'   \item{income_bracket}{Income bracket category (depends on dataset: AMI, FPL, etc.)}
#'   \item{households}{Number of households}
#'   \item{income}{Annual income}
#'   \item{electricity_spend}{Annual electricity spending}
#'   \item{gas_spend}{Annual gas spending}
#'   \item{other_spend}{Annual other fuel spending}
#'
#' @keywords internal
#' @export
raw_to_lead <- function(data, vintage) {

  if (is.character(vintage)) {
    vintage <- as.integer(vintage)
  }

  # Handle 2018+ format (most common)
  if (vintage >= 2018) {

    # Extract building type ranges for min_units and detached
    bld_types <- unique(data$BLD)
    bld_ranges <- stringr::str_extract_all(bld_types, "[0-9]+", simplify = TRUE)
    bld_ranges <- apply(bld_ranges, c(1, 2), as.numeric)
    bld_ranges <- data.frame(bld_ranges)
    names(bld_ranges) <- c("min_units", "max_units")
    bld_ranges$BLD <- bld_types
    bld_ranges$detached <- as.numeric(stringr::str_detect(bld_ranges$BLD, "DETACHED"))

    # Merge building attributes
    data <- merge(data, bld_ranges[c("BLD", "min_units", "detached")],
                  by = "BLD",
                  all.x = TRUE)

    # Identify income bracket column (AMI68, FPL15, FPL150, SMI, etc.)
    possible_colnames <- c("AMI68", "FPL15", "FPL150", "SMI")
    income_bracket_colname <- names(data)[stringr::str_detect(names(data), paste(possible_colnames, collapse = "|"))]

    # Ensure geoid (FIP) is properly formatted as 11-digit character
    data$FIP <- stringr::str_pad(as.character(data$FIP), width = 11, side = "left", pad = "0")

    # Select and rename columns to standard names
    data <- dplyr::select(data,
                          geoid = `FIP`,
                          state_abbr = `ABV`,
                          housing_tenure = `TEN`,
                          year_constructed = `YBL6`,
                          building_type = `BLD`,
                          min_units,
                          detached,
                          primary_heating_fuel = `HFL`,
                          income_bracket = !!rlang::sym(income_bracket_colname),
                          households = `UNITS`,
                          income = `HINCP`,
                          electricity_spend = `ELEP`,
                          gas_spend = `GASP`,
                          other_spend = `FULP`)

  } else {
    # Handle 2016 (SH) format
    # Note: This is a simplified version - full implementation would include
    # data dictionary parsing and more complex transformations
    stop("2016 vintage processing not fully implemented yet. Please use 2018+ data.")
  }

  return(data)
}


#' Aggregate LEAD data by poverty status
#'
#' Consolidates LEAD cohort data by poverty status, aggregating households
#' and computing weighted averages for income and spending.
#'
#' @param data A data frame of processed LEAD data (output from raw_to_lead)
#' @param dataset Character string indicating income metric: "ami", "fpl", or "smi"
#'
#' @return A data frame aggregated by geoid, poverty status, housing attributes
#'
#' @keywords internal
#' @export
lead_to_poverty <- function(data, dataset) {

  # Determine poverty cutoff based on income metric
  if (tolower(dataset) %in% c("fpl", "fpl15")) {
    poverty_cutoff <- "0-100%"
    poverty_label_below <- "Below Federal Poverty Line"
    poverty_label_above <- "Above Federal Poverty Line"
  } else if (tolower(dataset) %in% c("ami", "ami68")) {
    poverty_cutoff <- "very_low"  # Below 80% of AMI
    poverty_label_below <- "Below AMI Poverty Line"
    poverty_label_above <- "Above AMI Poverty Line"
  } else {
    stop("Unknown dataset type: ", dataset, ". Must be 'ami' or 'fpl'.")
  }

  # Create binary poverty indicator
  data$income_bracket <- as.factor(ifelse(
    data$income_bracket == poverty_cutoff,
    poverty_label_below,
    poverty_label_above
  ))

  # Consolidate housing tenure
  data$housing_tenure <- dplyr::recode_factor(data$housing_tenure,
                                              `OWNER` = "owned",
                                              `RENTER` = "rented")

  # Create number of units category
  data$number_of_units <- as.factor(ifelse(
    data$min_units > 1,
    "multi-family",
    "single-family"
  ))

  # Group columns for aggregation
  group_columns <- c("geoid",
                     "primary_heating_fuel",
                     "income_bracket",
                     "number_of_units",
                     "housing_tenure")

  # Aggregate by group, computing weighted averages
  data <- data |>
    dplyr::group_by(dplyr::across(dplyr::all_of(group_columns))) |>
    dplyr::mutate(group_households = sum(as.numeric(households), na.rm = TRUE)) |>
    dplyr::summarise(
      households = sum(as.numeric(households), na.rm = TRUE),
      income = sum(as.numeric(income) * group_households, na.rm = TRUE) /
        sum(group_households, na.rm = TRUE),
      electricity_spend = sum(as.numeric(electricity_spend) * group_households, na.rm = TRUE) /
        sum(group_households, na.rm = TRUE),
      gas_spend = sum(as.numeric(gas_spend) * group_households, na.rm = TRUE) /
        sum(group_households, na.rm = TRUE),
      other_spend = sum(as.numeric(other_spend) * group_households, na.rm = TRUE) /
        sum(group_households, na.rm = TRUE),
      pct_detached = sum(as.numeric(detached) * group_households, na.rm = TRUE) /
        sum(group_households, na.rm = TRUE),
      .groups = "drop"
    ) |>
    dplyr::ungroup()

  return(data)
}


#' Process raw LEAD data into analysis-ready format with energy metrics
#'
#' This is the main processing workflow that:
#' 1. Converts raw OpenEI data to clean format
#' 2. Optionally aggregates by poverty status
#' 3. Adds energy burden and related metrics
#' 4. Filters out zero-energy records
#'
#' @param data A data frame of raw LEAD data from OpenEI
#' @param dataset Character string indicating dataset type ("ami" or "fpl")
#' @param vintage Character string indicating ACS vintage year
#' @param aggregate_poverty Logical; if TRUE, aggregate to poverty status level
#'
#' @return A data frame ready for analysis with all energy metrics
#'
#' @keywords internal
#' @export
process_lead_cohort_data <- function(data, dataset, vintage, aggregate_poverty = FALSE) {

  # Step 1: Convert raw → clean format
  data <- raw_to_lead(data, vintage)

  # Step 2: Optionally aggregate by poverty status
  if (aggregate_poverty) {
    data <- lead_to_poverty(data, dataset)
  }

  # Step 3: Calculate derived metrics
  # Ensure numeric columns
  data <- data |>
    dplyr::mutate(
      income = as.numeric(income),
      electricity_spend = as.numeric(electricity_spend),
      gas_spend = as.numeric(gas_spend),
      other_spend = as.numeric(other_spend),
      households = as.numeric(households)
    )

  # Calculate total energy cost
  data$energy_cost <- rowSums(dplyr::select(data, electricity_spend, gas_spend, other_spend), na.rm = TRUE)
  data$energy_cost <- ifelse(abs(data$energy_cost) < 1, 0, data$energy_cost)

  # Calculate energy burden (as fraction, not percentage)
  data$energy_burden <- ifelse(data$income > 0, data$energy_cost / data$income, NA_real_)

  # Filter out zero-energy records (required for analysis)
  data <- data[data$energy_cost != 0, ]

  return(data)
}

test_that("raw_to_lead processes 2018 data correctly", {
  # Create mock 2018+ format raw data
  # Note: BLD uses numeric range format that the function expects
  raw_data <- data.frame(
    FIP = c("37183020100", "37051003400"),
    ABV = c("NC", "NC"),
    TEN = c("OWNER", "RENTER"),
    YBL6 = c("2000-2009", "1990-1999"),
    BLD = c("1 1 DETACHED", "2 4"),  # Numeric ranges: "1-1" and "2-4"
    HFL = c("Natural gas", "Electricity"),
    AMI68 = c("0-30% AMI", "30-50% AMI"),
    UNITS = c(100, 150),
    HINCP = c(25000, 35000),
    ELEP = c(1200, 1500),
    GASP = c(800, 0),
    FULP = c(200, 100)
  )

  result <- raw_to_lead(raw_data, "2018")

  # Check column names are standardized
  expect_true("geoid" %in% names(result))
  expect_true("state_abbr" %in% names(result))
  expect_true("income_bracket" %in% names(result))
  expect_true("households" %in% names(result))

  # Check geoid is properly formatted (11 digits)
  expect_equal(nchar(result$geoid[1]), 11)
  expect_equal(result$geoid[1], "37183020100")

  # Check min_units and detached are extracted
  expect_true("min_units" %in% names(result))
  expect_true("detached" %in% names(result))
  expect_equal(result$detached[1], 1)  # Has DETACHED keyword
  expect_equal(result$detached[2], 0)  # No DETACHED keyword

  # Check data values
  expect_equal(result$households, c(100, 150))
  expect_equal(result$income, c(25000, 35000))
})

test_that("raw_to_lead handles short geoids with padding", {
  raw_data <- data.frame(
    FIP = c("1234567890", "123456789"),  # 10 and 9 digits
    ABV = c("NC", "NC"),
    TEN = c("OWNER", "OWNER"),
    YBL6 = c("2000-2009", "2000-2009"),
    BLD = c("1 1 DETACHED", "1 1 DETACHED"),
    HFL = c("Natural gas", "Electricity"),
    AMI68 = c("0-30% AMI", "30-50% AMI"),
    UNITS = c(100, 150),
    HINCP = c(25000, 35000),
    ELEP = c(1200, 1500),
    GASP = c(800, 0),
    FULP = c(200, 100)
  )

  result <- raw_to_lead(raw_data, "2018")

  # All geoids should be 11 digits with zero padding
  expect_equal(result$geoid[1], "01234567890")
  expect_equal(result$geoid[2], "00123456789")
})

test_that("raw_to_lead rejects 2016 vintage", {
  raw_data <- data.frame(FIP = "37183020100")

  expect_error(
    raw_to_lead(raw_data, "2016"),
    "2016 vintage processing not fully implemented"
  )
})

test_that("lead_to_poverty creates binary poverty indicator for FPL", {
  # Mock processed LEAD data
  data <- data.frame(
    geoid = rep("37183020100", 4),
    primary_heating_fuel = rep("Natural gas", 4),
    income_bracket = c("0-100%", "100-150%", "150-200%", "200%+"),
    households = c(50, 75, 100, 125),
    income = c(15000, 30000, 45000, 75000),
    electricity_spend = c(1200, 1400, 1600, 1800),
    gas_spend = c(800, 900, 1000, 1100),
    other_spend = c(100, 150, 200, 250),
    min_units = c(1, 1, 1, 1),
    detached = c(1, 1, 1, 1),
    housing_tenure = rep("OWNER", 4),
    year_constructed = rep("2000-2009", 4),
    building_type = rep("1 1 DETACHED", 4)
  )

  result <- lead_to_poverty(data, "fpl")

  # Check poverty indicator was created
  expect_true("income_bracket" %in% names(result))
  expect_true(is.factor(result$income_bracket))

  # Check levels
  poverty_levels <- levels(result$income_bracket)
  expect_true("Below Federal Poverty Line" %in% poverty_levels)
  expect_true("Above Federal Poverty Line" %in% poverty_levels)

  # Check aggregation occurred
  expect_true(nrow(result) <= nrow(data))
})

test_that("lead_to_poverty creates binary poverty indicator for AMI", {
  data <- data.frame(
    geoid = rep("37183020100", 3),
    primary_heating_fuel = rep("Natural gas", 3),
    income_bracket = c("very_low", "low_mod", "mid_high"),
    households = c(50, 75, 100),
    income = c(15000, 35000, 60000),
    electricity_spend = c(1200, 1400, 1600),
    gas_spend = c(800, 900, 1000),
    other_spend = c(100, 150, 200),
    min_units = c(1, 1, 1),
    detached = c(1, 1, 1),
    housing_tenure = rep("OWNER", 3),
    year_constructed = rep("2000-2009", 3),
    building_type = rep("1 1 DETACHED", 3)
  )

  result <- lead_to_poverty(data, "ami")

  # Check levels
  poverty_levels <- levels(result$income_bracket)
  expect_true("Below AMI Poverty Line" %in% poverty_levels)
  expect_true("Above AMI Poverty Line" %in% poverty_levels)
})

test_that("lead_to_poverty consolidates housing tenure", {
  data <- data.frame(
    geoid = rep("37183020100", 2),
    primary_heating_fuel = rep("Natural gas", 2),
    income_bracket = c("0-100%", "100-150%"),
    housing_tenure = c("OWNER", "RENTER"),
    households = c(50, 75),
    income = c(15000, 30000),
    electricity_spend = c(1200, 1400),
    gas_spend = c(800, 900),
    other_spend = c(100, 150),
    min_units = c(1, 1),
    detached = c(1, 1),
    year_constructed = rep("2000-2009", 2),
    building_type = rep("1 1 DETACHED", 2)
  )

  result <- lead_to_poverty(data, "fpl")

  # Check housing tenure was recoded
  expect_true(is.factor(result$housing_tenure))
  expect_true(all(result$housing_tenure %in% c("owned", "rented")))
})

test_that("lead_to_poverty creates number_of_units category", {
  data <- data.frame(
    geoid = rep("37183020100", 2),
    primary_heating_fuel = rep("Natural gas", 2),
    income_bracket = c("0-100%", "100-150%"),
    housing_tenure = rep("OWNER", 2),
    min_units = c(1, 5),  # Single vs multi-family
    detached = c(1, 0),
    households = c(50, 75),
    income = c(15000, 30000),
    electricity_spend = c(1200, 1400),
    gas_spend = c(800, 900),
    other_spend = c(100, 150),
    year_constructed = rep("2000-2009", 2),
    building_type = rep("1 1 DETACHED", 2)
  )

  result <- lead_to_poverty(data, "fpl")

  # Check number_of_units was created
  expect_true("number_of_units" %in% names(result))
  expect_true(is.factor(result$number_of_units))
  expect_true(all(result$number_of_units %in% c("single-family", "multi-family")))
})

test_that("process_lead_cohort_data calculates energy_burden correctly", {
  raw_data <- data.frame(
    FIP = c("37183020100", "37051003400"),
    ABV = c("NC", "NC"),
    TEN = c("OWNER", "RENTER"),
    YBL6 = c("2000-2009", "1990-1999"),
    BLD = c("1 1 DETACHED", "2 4"),
    HFL = c("Natural gas", "Electricity"),
    AMI68 = c("0-30% AMI", "30-50% AMI"),
    UNITS = c(100, 150),
    HINCP = c(50000, 40000),
    ELEP = c(1200, 1500),
    GASP = c(800, 0),
    FULP = c(200, 100)
  )

  result <- process_lead_cohort_data(raw_data, "ami", "2018", aggregate_poverty = FALSE)

  # Check energy_cost was calculated
  expect_true("energy_cost" %in% names(result))
  expect_equal(result$energy_cost[1], 1200 + 800 + 200)  # 2200
  expect_equal(result$energy_cost[2], 1500 + 0 + 100)    # 1600

  # Check energy_burden was calculated
  expect_true("energy_burden" %in% names(result))
  expect_equal(result$energy_burden[1], 2200 / 50000)
  expect_equal(result$energy_burden[2], 1600 / 40000)
})

test_that("process_lead_cohort_data filters zero-energy records", {
  raw_data <- data.frame(
    FIP = c("37183020100", "37051003400", "37119000100"),
    ABV = c("NC", "NC", "NC"),
    TEN = c("OWNER", "RENTER", "OWNER"),
    YBL6 = c("2000-2009", "1990-1999", "2010-2019"),
    BLD = rep("1 1 DETACHED", 3),
    HFL = rep("Natural gas", 3),
    AMI68 = rep("0-30% AMI", 3),
    UNITS = c(100, 150, 200),
    HINCP = c(50000, 40000, 60000),
    ELEP = c(1200, 0, 1800),
    GASP = c(800, 0, 900),
    FULP = c(200, 0, 0)  # Second record has zero energy
  )

  result <- process_lead_cohort_data(raw_data, "ami", "2018", aggregate_poverty = FALSE)

  # Should have filtered out the zero-energy record
  expect_equal(nrow(result), 2)
  expect_false("37051003400" %in% result$geoid)
})

test_that("process_lead_cohort_data handles aggregate_poverty = TRUE", {
  raw_data <- data.frame(
    FIP = rep("37183020100", 4),
    ABV = rep("NC", 4),
    TEN = rep("OWNER", 4),
    YBL6 = rep("2000-2009", 4),
    BLD = rep("1 1 DETACHED", 4),
    HFL = rep("Natural gas", 4),
    FPL15 = c("0-100%", "100-150%", "150-200%", "200%+"),
    UNITS = c(50, 75, 100, 125),
    HINCP = c(15000, 30000, 45000, 75000),
    ELEP = c(1200, 1400, 1600, 1800),
    GASP = c(800, 900, 1000, 1100),
    FULP = c(100, 150, 200, 250)
  )

  result <- process_lead_cohort_data(raw_data, "fpl", "2018", aggregate_poverty = TRUE)

  # Should have aggregated to poverty status level
  expect_true(nrow(result) < 4)
  expect_true("income_bracket" %in% names(result))

  # Check poverty labels were applied
  poverty_levels <- unique(as.character(result$income_bracket))
  expect_true(any(grepl("Poverty Line", poverty_levels)))
})

test_that("process_lead_cohort_data handles NA income gracefully", {
  raw_data <- data.frame(
    FIP = c("37183020100", "37051003400"),
    ABV = c("NC", "NC"),
    TEN = c("OWNER", "RENTER"),
    YBL6 = c("2000-2009", "1990-1999"),
    BLD = rep("1 1 DETACHED", 2),
    HFL = rep("Natural gas", 2),
    AMI68 = rep("0-30% AMI", 2),
    UNITS = c(100, 150),
    HINCP = c(NA, 40000),  # NA income
    ELEP = c(1200, 1500),
    GASP = c(800, 0),
    FULP = c(200, 100)
  )

  result <- process_lead_cohort_data(raw_data, "ami", "2018", aggregate_poverty = FALSE)

  # Check NA handling
  expect_true(is.na(result$energy_burden[1]))
  expect_false(is.na(result$energy_burden[2]))
})


# ============================================================================
# Housing Dimension Column Tests
# ============================================================================

test_that("raw_to_lead preserves housing dimension columns", {
  raw_data <- data.frame(
    FIP = c("37183020100", "37051003400"),
    ABV = c("NC", "NC"),
    TEN = c("OWNER", "RENTER"),
    YBL6 = c("2000-2009", "1990-1999"),
    BLD = c("1 1 DETACHED", "2 4"),
    HFL = c("Natural gas", "Electricity"),
    AMI68 = c("0-30% AMI", "30-50% AMI"),
    UNITS = c(100, 150),
    HINCP = c(25000, 35000),
    ELEP = c(1200, 1500),
    GASP = c(800, 0),
    FULP = c(200, 100)
  )

  result <- raw_to_lead(raw_data, "2018")

  # Verify individual housing dimension columns are preserved
  expect_true("housing_tenure" %in% names(result))
  expect_true("year_constructed" %in% names(result))
  expect_true("building_type" %in% names(result))
  expect_true("primary_heating_fuel" %in% names(result))

  # Verify values are preserved
  expect_equal(result$housing_tenure, c("OWNER", "RENTER"))
  expect_equal(result$year_constructed, c("2000-2009", "1990-1999"))
  expect_equal(result$primary_heating_fuel, c("Natural gas", "Electricity"))
})


test_that("process_lead_cohort_data preserves housing dimension columns", {
  raw_data <- data.frame(
    FIP = rep("37183020100", 4),
    ABV = rep("NC", 4),
    TEN = c("OWNER", "OWNER", "RENTER", "RENTER"),
    YBL6 = c("2000-2009", "1990-1999", "2000-2009", "1990-1999"),
    BLD = c("1 1 DETACHED", "2 4", "1 1 DETACHED", "2 4"),
    HFL = c("Natural gas", "Electricity", "Natural gas", "Electricity"),
    AMI68 = rep("0-30% AMI", 4),
    UNITS = c(100, 150, 120, 180),
    HINCP = c(25000, 35000, 28000, 32000),
    ELEP = c(1200, 1500, 1300, 1400),
    GASP = c(800, 0, 700, 0),
    FULP = c(200, 100, 150, 120)
  )

  result <- process_lead_cohort_data(raw_data, "ami", "2018", aggregate_poverty = FALSE)

  # Verify housing dimension columns are present in output
  expect_true("housing_tenure" %in% names(result))
  expect_true("year_constructed" %in% names(result))
  expect_true("building_type" %in% names(result))
  expect_true("primary_heating_fuel" %in% names(result))

  # Verify distinct combinations are preserved
  expect_equal(nrow(result), 4)  # All 4 combinations should be distinct
  expect_setequal(unique(result$housing_tenure), c("OWNER", "RENTER"))
  expect_setequal(unique(result$year_constructed), c("2000-2009", "1990-1999"))
})


test_that("process_lead_cohort_data with aggregate_poverty preserves housing dims", {
  raw_data <- data.frame(
    FIP = rep("37183020100", 4),
    ABV = rep("NC", 4),
    TEN = c("OWNER", "OWNER", "RENTER", "RENTER"),
    YBL6 = rep("2000-2009", 4),
    BLD = rep("1 1 DETACHED", 4),
    HFL = c("Natural gas", "Natural gas", "Electricity", "Electricity"),
    FPL15 = c("0-100%", "100-150%", "0-100%", "100-150%"),
    UNITS = c(50, 75, 60, 80),
    HINCP = c(15000, 30000, 18000, 32000),
    ELEP = c(1200, 1400, 1300, 1500),
    GASP = c(800, 900, 0, 0),
    FULP = c(100, 150, 120, 160)
  )

  result <- process_lead_cohort_data(raw_data, "fpl", "2018", aggregate_poverty = TRUE)

  # Even with aggregation, housing characteristics should be preserved
  expect_true("housing_tenure" %in% names(result))
  expect_true("primary_heating_fuel" %in% names(result))

  # Check that different housing characteristics create separate groups
  # With 2 tenure types, 2 fuel types, and 2 poverty categories = up to 8 groups
  # But we only have 4 input rows, so max 4 groups
  expect_true(nrow(result) <= 4)
  expect_gt(nrow(result), 1)  # Should have multiple groups
})

#' Format Number as Dollar Amount
#'
#' Converts numeric values to formatted dollar strings with appropriate
#' decimal places and thousand separators.
#'
#' @param x Numeric vector to format
#' @param latex Logical indicating whether to escape dollar sign for LaTeX
#'   (default: FALSE)
#'
#' @returns Character vector of formatted dollar amounts
#'
#' @export
#'
#' @examples
#' # Format dollar amounts
#' to_dollar(c(1000, 2500.50, 10000))
#'
#' # LaTeX-escaped format
#' to_dollar(c(1000, 2500.50), latex = TRUE)
to_dollar <- function(x, latex = FALSE) {
  # Handle NA before formatting
  if (all(is.na(x))) {
    return(rep("", length(x)))
  }

  if (latex) {
    y <- scales::label_dollar(largest_with_cents = 10, prefix = "\\$")(x)
  } else {
    y <- scales::label_dollar(largest_with_cents = 10)(x)
  }
  y[is.na(x)] <- ""
  return(y)
}

#' Format Number as Percentage
#'
#' Converts numeric values to formatted percentage strings with no decimal
#' places by default.
#'
#' @param x Numeric vector to format (as proportions, not percentages)
#' @param latex Logical indicating whether to escape percent sign for LaTeX
#'   (default: FALSE)
#'
#' @returns Character vector of formatted percentages
#'
#' @export
#'
#' @examples
#' # Format percentages
#' to_percent(c(0.25, 0.50, 0.123))
#'
#' # LaTeX-escaped format
#' to_percent(c(0.25, 0.50), latex = TRUE)
to_percent <- function(x, latex = FALSE) {
  # Handle NA before formatting
  if (all(is.na(x))) {
    return(rep("", length(x)))
  }

  if (latex) {
    y <- scales::label_percent(accuracy = 1, big.mark = ",", suffix = "\\%")(x)
  } else {
    y <- scales::label_percent(accuracy = 1, big.mark = ",")(x)
  }
  y[is.na(x)] <- ""
  return(y)
}

#' Format Large Numbers with Thousand Separators
#'
#' Converts numeric values to formatted strings with thousand separators (commas).
#'
#' @param x Numeric vector to format
#'
#' @returns Character vector of formatted numbers
#'
#' @export
#'
#' @examples
#' # Format large numbers
#' to_big(c(1000, 25000, 1000000))
to_big <- function(x) {
  # Handle NA before formatting
  if (all(is.na(x))) {
    return(rep("", length(x)))
  }

  y <- scales::label_comma(accuracy = 1, big.mark = ",")(x)
  y[is.na(x)] <- ""
  return(y)
}

#' Format Numbers in Millions
#'
#' Converts large numeric values to millions format with appropriate suffix.
#' Values less than 1 million are shown in thousands.
#'
#' @param x Numeric vector to format
#' @param suffix Character string to append after "million" (default: " million")
#' @param override_to_k Logical indicating whether to show values < 1M as
#'   thousands (default: TRUE)
#'
#' @returns Character vector of formatted numbers with "million" or "k" suffix
#'
#' @export
#'
#' @examples
#' # Format in millions
#' to_million(c(5000, 1000000, 2500000))
to_million <- function(x, suffix = " million", override_to_k = TRUE) {
  # Handle NA before formatting
  if (all(is.na(x))) {
    return(rep("", length(x)))
  }

  y <- ifelse(abs(x) < 10^6,
    scales::label_number(accuracy = 1, suffix = "k")(x * 10^-3),
    scales::label_number(accuracy = 0.1, suffix = suffix)(x * 10^-6)
  )
  y[is.na(x)] <- ""
  return(y)
}

#' Format Dollar Amounts in Billions
#'
#' Converts large dollar values to billions format with dollar sign prefix.
#' Values less than 1 billion are shown in millions.
#'
#' @param x Numeric vector to format
#' @param suffix Character string to append after "billion" (default: " billion")
#' @param override_to_k Logical (currently unused, kept for compatibility)
#'
#' @returns Character vector of formatted dollar amounts with "billion" or "m" suffix
#'
#' @export
#'
#' @examples
#' # Format in billions
#' to_billion_dollar(c(5000000, 1000000000, 2500000000))
to_billion_dollar <- function(x, suffix = " billion", override_to_k = TRUE) {
  # Handle NA before formatting
  if (all(is.na(x))) {
    return(rep("", length(x)))
  }

  y <- ifelse(abs(x) < 10^9,
    scales::label_number(accuracy = 1, suffix = "m", prefix = "$")(x * 10^-6),
    scales::label_number(accuracy = 0.1, suffix = suffix, prefix = "$")(x * 10^-9)
  )
  y[is.na(x)] <- ""
  return(y)
}

#' Colorize Text for Knitted Documents
#'
#' Wraps text in color formatting appropriate for the output format (LaTeX or HTML).
#' This function is intended for use within R Markdown/knitr documents.
#'
#' @param x Character string to colorize
#' @param color Character string specifying the color name (e.g., "red", "blue")
#'
#' @returns Character string wrapped in LaTeX or HTML color commands, or
#'   unchanged if output format is neither
#'
#' @details
#' This function detects the knitr output format and applies appropriate color
#' formatting. For LaTeX output, it uses `\\textcolor{}`. For HTML output, it
#' uses `<span style='color: ...'>`.
#'
#' @export
#'
#' @examples
#' \donttest{
#' # In an R Markdown document:
#' colorize("Important text", "red")
#' }
colorize <- function(x, color) {
  if (requireNamespace("knitr", quietly = TRUE)) {
    if (knitr::is_latex_output()) {
      sprintf("\\textcolor{%s}{%s}", color, x)
    } else if (knitr::is_html_output()) {
      sprintf("<span style='color: %s;'>%s</span>", color, x)
    } else {
      x
    }
  } else {
    x
  }
}

test_that("to_dollar formats correctly", {
  # Basic formatting
  expect_equal(to_dollar(1000), "$1,000")
  # Note: largest_with_cents=10 means only values < 10 show cents
  expect_equal(to_dollar(2500.50), "$2,500")  # Rounds to nearest dollar
  expect_equal(to_dollar(5.50), "$5.50")  # Small values show cents
  expect_equal(to_dollar(10000), "$10,000")

  # LaTeX escaped
  latex_result <- to_dollar(1000, latex = TRUE)
  expect_true(grepl("\\\\\\$", latex_result))

  # Vector inputs
  result <- to_dollar(c(1000, 2500, 10000))
  expect_length(result, 3)

  # NA handling
  expect_equal(to_dollar(NA), "")

  # Zero
  expect_equal(to_dollar(0), "$0")
})

test_that("to_percent formats correctly", {
  # Basic formatting (proportions to percentages)
  expect_equal(to_percent(0.25), "25%")
  expect_equal(to_percent(0.50), "50%")
  expect_equal(to_percent(0.123), "12%")

  # LaTeX escaped
  latex_result <- to_percent(0.25, latex = TRUE)
  expect_true(grepl("\\\\%", latex_result))

  # Vector inputs
  result <- to_percent(c(0.25, 0.50, 0.75))
  expect_length(result, 3)

  # NA handling
  expect_equal(to_percent(NA), "")

  # Edge cases
  expect_equal(to_percent(0), "0%")
  expect_equal(to_percent(1), "100%")
})

test_that("to_big formats correctly", {
  # Thousand separators
  expect_equal(to_big(1000), "1,000")
  expect_equal(to_big(25000), "25,000")
  expect_equal(to_big(1000000), "1,000,000")

  # No decimals
  result <- to_big(1234.56)
  expect_false(grepl("\\.", result))

  # NA handling
  expect_equal(to_big(NA), "")

  # Negative numbers
  expect_true(grepl("-", to_big(-1000)))
})

test_that("to_million formats correctly", {
  # Values less than 1 million
  expect_true(grepl("k$", to_million(5000)))

  # Values in millions
  result <- to_million(2500000)
  expect_true(grepl("million", result))

  # NA handling
  expect_equal(to_million(NA), "")
})

test_that("to_billion_dollar formats correctly", {
  # Values less than 1 billion
  result <- to_billion_dollar(5000000)
  expect_true(grepl("\\$", result))
  expect_true(grepl("m$", result))

  # Values in billions
  result <- to_billion_dollar(2500000000)
  expect_true(grepl("\\$", result))
  expect_true(grepl("billion", result))

  # NA handling
  expect_equal(to_billion_dollar(NA), "")
})

test_that("colorize works without knitr", {
  # Without knitr loaded, should return unchanged text
  result <- colorize("test text", "red")
  expect_type(result, "character")
})

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

# Declare global variables used in NSE (tidyverse/data.table) operations
# to avoid R CMD check NOTEs
utils::globalVariables(c(
  # PUMS/Census variables (raw_to_lead, standardize_cohort_columns)
  "ABV", "BLD", "ELEP", "ELEP*UNITS", "FIP", "FPL150", "FULP", "FULP*UNITS",
  "GASP", "GASP*UNITS", "HFL", "HINCP", "HINCP*UNITS", "TEN", "UNITS", "YBL6",

  # Computed variables (process_lead_cohort_data, lead_to_poverty)
  "detached", "electricity_spend", "gas_spend", "income", "min_units", "other_spend",

  # Aggregation variables (compare_energy_burden)
  "total_income_sum", "total_spend_sum"
))

#' Harmonize Income Brackets Across Vintages
#'
#' @description
#' Harmonizes income bracket categories when comparing data across different
#' vintage years. This is necessary because some datasets (particularly AMI)
#' have different bracket definitions across years.
#'
#' @details
#' ## Dataset-Specific Bracket Definitions
#'
#' ### AMI (Area Median Income)
#' - **2018**: 3 brackets
#'   - `very_low`: Very low income (typically <50% AMI)
#'   - `low_mod`: Low to moderate income (typically 50-80% AMI)
#'   - `mid_high`: Middle to high income (typically >80% AMI)
#' - **2022**: 5 brackets
#'   - `very_low`: Very low income (same as 2018)
#'   - `low_mod`: Low to moderate income (same as 2018)
#'   - `mid_high`: Middle to high income (narrower than 2018)
#'   - `100-150%`: 100-150% of AMI (new in 2022)
#'   - `150%+`: Above 150% of AMI (new in 2022)
#'
#' ### FPL (Federal Poverty Level)
#' - **Both 2018 and 2022**: 5 brackets
#'   - `0-100%`: Below poverty line
#'   - `100-150%`: 100-150% of FPL
#'   - `150-200%`: 150-200% of FPL
#'   - `200-400%`: 200-400% of FPL
#'   - `400%+`: Above 400% of FPL
#'
#' @param data A data frame containing income bracket data
#' @param dataset Character, either "ami" or "fpl"
#' @param vintage Integer, the year of the data vintage
#' @param strict_matching Logical, if TRUE (default) only keeps brackets that
#'   exist in both vintages being compared. If FALSE, keeps all brackets.
#' @param comparison_vintages Integer vector of length 2, the vintages being
#'   compared (e.g., c(2018, 2022)). Required when strict_matching = TRUE.
#'
#' @return A list with components:
#'   - `data`: The harmonized data frame
#'   - `warnings`: Character vector of any warnings about bracket mismatches
#'   - `dropped_brackets`: Character vector of brackets that were dropped
#'
#' @keywords internal
harmonize_income_brackets <- function(data,
                                       dataset,
                                       vintage,
                                       strict_matching = TRUE,
                                       comparison_vintages = NULL) {

  # Validate inputs
  if (!dataset %in% c("ami", "fpl")) {
    stop("dataset must be either 'ami' or 'fpl'")
  }

  if (!"income_bracket" %in% names(data)) {
    stop("data must contain an 'income_bracket' column")
  }

  # Define known bracket sets for each dataset and vintage
  known_brackets <- list(
    ami = list(
      `2018` = c("very_low", "low_mod", "mid_high"),
      `2022` = c("very_low", "low_mod", "mid_high", "100-150%", "150%+")
    ),
    fpl = list(
      `2018` = c("0-100%", "100-150%", "150-200%", "200-400%", "400%+"),
      `2022` = c("0-100%", "100-150%", "150-200%", "200-400%", "400%+")
    )
  )

  # Get expected brackets for this dataset/vintage
  expected_brackets <- known_brackets[[dataset]][[as.character(vintage)]]

  # Initialize results
  warnings_out <- character(0)
  dropped_brackets <- character(0)
  harmonized_data <- data

  # Check for unexpected brackets
  actual_brackets <- unique(data$income_bracket)
  unexpected <- setdiff(actual_brackets, expected_brackets)
  if (length(unexpected) > 0) {
    warnings_out <- c(
      warnings_out,
      sprintf(
        "Unexpected income brackets found in %s %d data: %s",
        toupper(dataset),
        vintage,
        paste(unexpected, collapse = ", ")
      )
    )
  }

  # Apply strict matching if requested
  if (strict_matching) {
    if (is.null(comparison_vintages) || length(comparison_vintages) != 2) {
      stop("comparison_vintages must be provided when strict_matching = TRUE")
    }

    # Get brackets for both vintages
    vintage1_brackets <- known_brackets[[dataset]][[as.character(comparison_vintages[1])]]
    vintage2_brackets <- known_brackets[[dataset]][[as.character(comparison_vintages[2])]]

    # Find common brackets
    common_brackets <- intersect(vintage1_brackets, vintage2_brackets)

    # Find brackets unique to each vintage
    only_vintage1 <- setdiff(vintage1_brackets, vintage2_brackets)
    only_vintage2 <- setdiff(vintage2_brackets, vintage1_brackets)

    # Create warning if there are mismatched brackets
    if (length(only_vintage1) > 0 || length(only_vintage2) > 0) {
      mismatch_msg <- sprintf(
        "Income bracket mismatch in %s dataset:",
        toupper(dataset)
      )

      if (length(only_vintage1) > 0) {
        mismatch_msg <- paste0(
          mismatch_msg,
          sprintf(
            "\n  - Only in %d: %s",
            comparison_vintages[1],
            paste(only_vintage1, collapse = ", ")
          )
        )
      }

      if (length(only_vintage2) > 0) {
        mismatch_msg <- paste0(
          mismatch_msg,
          sprintf(
            "\n  - Only in %d: %s",
            comparison_vintages[2],
            paste(only_vintage2, collapse = ", ")
          )
        )
      }

      mismatch_msg <- paste0(
        mismatch_msg,
        sprintf(
          "\n  - Keeping only common brackets: %s",
          paste(common_brackets, collapse = ", ")
        )
      )

      warnings_out <- c(warnings_out, mismatch_msg)
    }

    # Filter data to only common brackets
    brackets_to_drop <- setdiff(actual_brackets, common_brackets)
    if (length(brackets_to_drop) > 0) {
      dropped_brackets <- brackets_to_drop
      harmonized_data <- harmonized_data |>
        dplyr::filter(income_bracket %in% common_brackets)
    }
  }

  return(list(
    data = harmonized_data,
    warnings = warnings_out,
    dropped_brackets = dropped_brackets
  ))
}


#' Get Available Income Brackets for a Dataset and Vintage
#'
#' @description
#' Returns the expected income brackets for a given dataset and vintage year.
#' Useful for understanding what brackets are available before running analyses.
#'
#' @param dataset Character, either "ami" or "fpl"
#' @param vintage Integer, the year of the data vintage (e.g., 2018, 2022)
#'
#' @return Character vector of income bracket names
#'
#' @examples
#' # Get AMI brackets for 2022
#' get_income_brackets("ami", 2022)
#'
#' # Get FPL brackets for 2018
#' get_income_brackets("fpl", 2018)
#'
#' @export
get_income_brackets <- function(dataset, vintage) {
  # Validate inputs
  if (!dataset %in% c("ami", "fpl")) {
    stop("dataset must be either 'ami' or 'fpl'")
  }

  # Define known bracket sets
  known_brackets <- list(
    ami = list(
      `2018` = c("very_low", "low_mod", "mid_high"),
      `2022` = c("very_low", "low_mod", "mid_high", "100-150%", "150%+")
    ),
    fpl = list(
      `2018` = c("0-100%", "100-150%", "150-200%", "200-400%", "400%+"),
      `2022` = c("0-100%", "100-150%", "150-200%", "200-400%", "400%+")
    )
  )

  # Get brackets for this dataset/vintage
  brackets <- known_brackets[[dataset]][[as.character(vintage)]]

  if (is.null(brackets)) {
    stop(sprintf("No bracket definitions found for %s vintage %d", dataset, vintage))
  }

  return(brackets)
}

# Phase 1: Compare Energy Burden Tests
# Comprehensive tests for temporal comparison functionality

test_that("compare_energy_burden returns proper structure", {
  # Use existing test data if available, otherwise create sample data
  skip_if_not_installed("dplyr")

  # Create mock data for 2018 and 2022
  data_2018 <- create_sample_lead_data(n = 100, seed = 1, dataset = "ami", vintage = "2018")
  data_2022 <- create_sample_lead_data(n = 100, seed = 2, dataset = "ami", vintage = "2022")

  # Mock the load_cohort_data function to return our test data
  # In real use, this would download from OpenEI or load from cache

  # For now, test the underlying calculation logic
  # Calculate weighted metrics for each vintage
  metrics_2018 <- data_2018 %>%
    dplyr::group_by(income_bracket) %>%
    dplyr::summarise(
      ner_2018 = weighted.mean(ner, households, na.rm = TRUE),
      households_2018 = sum(households, na.rm = TRUE),
      .groups = "drop"
    )

  metrics_2022 <- data_2022 %>%
    dplyr::group_by(income_bracket) %>%
    dplyr::summarise(
      ner_2022 = weighted.mean(ner, households, na.rm = TRUE),
      households_2022 = sum(households, na.rm = TRUE),
      .groups = "drop"
    )

  # Merge to create comparison
  comparison <- dplyr::full_join(metrics_2018, metrics_2022, by = "income_bracket")

  # Should have required columns
  expect_true("income_bracket" %in% names(comparison))
  expect_true("ner_2018" %in% names(comparison))
  expect_true("ner_2022" %in% names(comparison))
  expect_true("households_2018" %in% names(comparison))
  expect_true("households_2022" %in% names(comparison))
})

test_that("compare_energy_burden calculates energy burden from NER", {
  skip_if_not_installed("dplyr")

  data <- data.frame(
    income_bracket = c("0-30%", "30-60%", "60-80%"),
    ner_2018 = c(5, 10, 15),
    ner_2022 = c(4, 9, 14),
    households_2018 = c(1000, 2000, 3000),
    households_2022 = c(1100, 2100, 3100)
  )

  # Calculate energy burden from NER: EB = 1 / (NER + 1)
  data$neb_2018 <- 1 / (data$ner_2018 + 1)
  data$neb_2022 <- 1 / (data$ner_2022 + 1)

  # Check calculations
  expect_equal(data$neb_2018[1], 1/6)  # NER=5 -> EB=1/6
  expect_equal(data$neb_2018[2], 1/11) # NER=10 -> EB=1/11
  expect_equal(data$neb_2018[3], 1/16) # NER=15 -> EB=1/16

  expect_equal(data$neb_2022[1], 1/5)  # NER=4 -> EB=1/5
  expect_equal(data$neb_2022[2], 1/10) # NER=9 -> EB=1/10
  expect_equal(data$neb_2022[3], 1/15) # NER=14 -> EB=1/15
})

test_that("compare_energy_burden calculates change correctly", {
  skip_if_not_installed("dplyr")

  data <- data.frame(
    income_bracket = c("0-30%", "30-60%"),
    neb_2018 = c(0.10, 0.05),  # 10%, 5%
    neb_2022 = c(0.12, 0.04)   # 12%, 4%
  )

  # Calculate change in percentage points
  data$neb_change_pp <- data$neb_2022 - data$neb_2018

  expect_equal(data$neb_change_pp[1], 0.02)   # +2 percentage points
  expect_equal(data$neb_change_pp[2], -0.01)  # -1 percentage point

  # Calculate percentage change
  data$neb_change_pct <- ((data$neb_2022 - data$neb_2018) / data$neb_2018) * 100

  expect_equal(data$neb_change_pct[1], 20)   # 20% increase
  expect_equal(data$neb_change_pct[2], -20)  # 20% decrease
})

test_that("weighted mean calculations are correct", {
  # Test that weighted mean is calculated properly
  values <- c(5, 10, 15)
  weights <- c(100, 200, 300)

  wm <- weighted.mean(values, weights)

  # Manual calculation: (5*100 + 10*200 + 15*300) / (100+200+300)
  #                   = (500 + 2000 + 4500) / 600
  #                   = 7000 / 600 = 11.67
  expect_equal(wm, 7000 / 600)
})

test_that("comparison handles all income brackets correctly", {
  skip_if_not_installed("dplyr")

  # Create data with all AMI brackets
  data_2018 <- create_sample_lead_data(n = 500, seed = 100, dataset = "ami", vintage = "2018")
  data_2022 <- create_sample_lead_data(n = 500, seed = 101, dataset = "ami", vintage = "2022")

  ami_brackets <- c("0-30%", "30-60%", "60-80%", "80-100%", "100%+")

  # Verify both datasets have all brackets
  expect_true(all(ami_brackets %in% data_2018$income_bracket))
  expect_true(all(ami_brackets %in% data_2022$income_bracket))

  # Calculate metrics by bracket for both vintages
  metrics_2018 <- data_2018 %>%
    dplyr::group_by(income_bracket) %>%
    dplyr::summarise(
      ner_2018 = weighted.mean(ner, households, na.rm = TRUE),
      .groups = "drop"
    )

  metrics_2022 <- data_2022 %>%
    dplyr::group_by(income_bracket) %>%
    dplyr::summarise(
      ner_2022 = weighted.mean(ner, households, na.rm = TRUE),
      .groups = "drop"
    )

  # Both should have all 5 brackets
  expect_equal(nrow(metrics_2018), 5)
  expect_equal(nrow(metrics_2022), 5)
})

test_that("comparison handles FPL data correctly", {
  skip_if_not_installed("dplyr")

  # Create FPL data
  data_2018 <- create_sample_lead_data(n = 200, seed = 200, dataset = "fpl", vintage = "2018")
  data_2022 <- create_sample_lead_data(n = 200, seed = 201, dataset = "fpl", vintage = "2022")

  fpl_brackets <- c("0-100%", "100-150%", "150-200%", "200%+")

  # Verify both datasets have FPL brackets
  expect_true(all(data_2018$income_bracket %in% fpl_brackets))
  expect_true(all(data_2022$income_bracket %in% fpl_brackets))
})

test_that("comparison with aggregation to poverty status works", {
  skip_if_not_installed("dplyr")

  # Create FPL data
  data_fpl <- create_sample_lead_data(n = 200, seed = 300, dataset = "fpl", vintage = "2022")

  # Aggregate to binary poverty status
  data_fpl$poverty_status <- ifelse(
    data_fpl$income_bracket == "0-100%",
    "Below Federal Poverty Line",
    "Above Federal Poverty Line"
  )

  # Calculate weighted metrics by poverty status
  poverty_metrics <- data_fpl %>%
    dplyr::group_by(poverty_status) %>%
    dplyr::summarise(
      ner = weighted.mean(ner, households, na.rm = TRUE),
      households = sum(households, na.rm = TRUE),
      .groups = "drop"
    )

  # Should have exactly 2 groups
  expect_equal(nrow(poverty_metrics), 2)
  expect_true("Below Federal Poverty Line" %in% poverty_metrics$poverty_status)
  expect_true("Above Federal Poverty Line" %in% poverty_metrics$poverty_status)
})

test_that("comparison handles missing data gracefully", {
  skip_if_not_installed("dplyr")

  # Create scenario where 2018 has more brackets than 2022
  data_2018 <- data.frame(
    income_bracket = c("0-30%", "30-60%", "60-80%"),
    ner = c(5, 10, 15),
    households = c(100, 200, 300)
  )

  data_2022 <- data.frame(
    income_bracket = c("0-30%", "30-60%"),  # Missing 60-80%
    ner = c(4, 9),
    households = c(110, 210)
  )

  # Full join should handle this
  comparison <- dplyr::full_join(
    data_2018,
    data_2022,
    by = "income_bracket",
    suffix = c("_2018", "_2022")
  )

  # Should have 3 rows (all brackets from both years)
  expect_equal(nrow(comparison), 3)

  # The 60-80% bracket should have NA for 2022 values
  row_60_80 <- comparison[comparison$income_bracket == "60-80%", ]
  expect_true(is.na(row_60_80$ner_2022))
})

test_that("state-level aggregation works correctly", {
  skip_if_not_installed("dplyr")

  # Create multi-state data
  data <- create_sample_lead_data(n = 300, seed = 400, vintage = "2022")
  data$state_abbr <- sample(c("NC", "SC", "GA"), nrow(data), replace = TRUE)

  # Aggregate by state
  state_metrics <- data %>%
    dplyr::group_by(state_abbr) %>%
    dplyr::summarise(
      ner = weighted.mean(ner, households, na.rm = TRUE),
      mean_income = weighted.mean(income, households, na.rm = TRUE),
      mean_energy_cost = weighted.mean(energy_cost, households, na.rm = TRUE),
      total_households = sum(households, na.rm = TRUE),
      .groups = "drop"
    )

  # Should have 3 states
  expect_equal(nrow(state_metrics), 3)
  expect_true(all(c("NC", "SC", "GA") %in% state_metrics$state_abbr))

  # All metrics should be finite
  expect_true(all(is.finite(state_metrics$ner)))
  expect_true(all(is.finite(state_metrics$mean_income)))
  expect_true(all(is.finite(state_metrics$mean_energy_cost)))
})

test_that("housing tenure comparison works", {
  skip_if_not_installed("dplyr")

  data_2018 <- create_sample_lead_data(n = 200, seed = 500, vintage = "2018")
  data_2022 <- create_sample_lead_data(n = 200, seed = 501, vintage = "2022")

  # Group by housing tenure
  tenure_2018 <- data_2018 %>%
    dplyr::group_by(housing_tenure) %>%
    dplyr::summarise(
      ner_2018 = weighted.mean(ner, households, na.rm = TRUE),
      .groups = "drop"
    )

  tenure_2022 <- data_2022 %>%
    dplyr::group_by(housing_tenure) %>%
    dplyr::summarise(
      ner_2022 = weighted.mean(ner, households, na.rm = TRUE),
      .groups = "drop"
    )

  # Both should have OWNER and RENTER
  expect_true("OWNER" %in% tenure_2018$housing_tenure)
  expect_true("RENTER" %in% tenure_2018$housing_tenure)
  expect_true("OWNER" %in% tenure_2022$housing_tenure)
  expect_true("RENTER" %in% tenure_2022$housing_tenure)
})

test_that("overall state comparison (no grouping) works", {
  skip_if_not_installed("dplyr")

  data_2018 <- create_sample_lead_data(n = 500, seed = 600, vintage = "2018")
  data_2022 <- create_sample_lead_data(n = 500, seed = 601, vintage = "2022")

  # Overall weighted mean (no grouping)
  overall_2018 <- weighted.mean(data_2018$ner, data_2018$households, na.rm = TRUE)
  overall_2022 <- weighted.mean(data_2022$ner, data_2022$households, na.rm = TRUE)

  # Should get single values
  expect_length(overall_2018, 1)
  expect_length(overall_2022, 1)
  expect_true(is.finite(overall_2018))
  expect_true(is.finite(overall_2022))
})

test_that("energy burden at poverty threshold is consistent", {
  # NER of 9 should equal 10% energy burden
  ner_threshold <- 9
  eb_threshold <- 1 / (ner_threshold + 1)

  expect_equal(eb_threshold, 0.1)

  # Verify reverse calculation
  ner_from_eb <- (1 / 0.1) - 1
  expect_equal(ner_from_eb, 9)
})

test_that("poverty rate calculation from NER", {
  skip_if_not_installed("dplyr")

  data <- create_sample_lead_data(n = 1000, seed = 700, vintage = "2022")

  # Calculate poverty rate (NER < 9 = energy burden > 10%)
  data$in_energy_poverty <- data$ner < 9

  # Weighted poverty rate
  poverty_rate <- weighted.mean(
    as.numeric(data$in_energy_poverty),
    data$households,
    na.rm = TRUE
  )

  # Should be between 0 and 1
  expect_gte(poverty_rate, 0)
  expect_lte(poverty_rate, 1)

  # Count by poverty status
  poverty_summary <- data %>%
    dplyr::group_by(in_energy_poverty) %>%
    dplyr::summarise(
      count = dplyr::n(),
      total_households = sum(households, na.rm = TRUE),
      .groups = "drop"
    )

  # Should have both TRUE and FALSE (some in poverty, some not)
  expect_equal(nrow(poverty_summary), 2)
})

test_that("temporal comparison shows expected trends", {
  skip_if_not_installed("dplyr")

  # Create data where 2022 has systematically lower energy burden than 2018
  data_2018 <- create_sample_lead_data(n = 200, seed = 800, vintage = "2018")
  data_2022 <- create_sample_lead_data(n = 200, seed = 800, vintage = "2022")  # Same seed

  # Artificially make 2022 better (higher NER = lower burden)
  data_2022$energy_cost <- data_2022$energy_cost * 0.9  # 10% reduction in costs
  data_2022$ner <- (data_2022$income - data_2022$energy_cost) / data_2022$energy_cost
  data_2022$energy_burden <- data_2022$energy_cost / data_2022$income

  # Calculate overall metrics
  ner_2018 <- weighted.mean(data_2018$ner, data_2018$households, na.rm = TRUE)
  ner_2022 <- weighted.mean(data_2022$ner, data_2022$households, na.rm = TRUE)

  # 2022 should have higher NER (better conditions)
  expect_gt(ner_2022, ner_2018)

  # Convert to energy burden
  eb_2018 <- 1 / (ner_2018 + 1)
  eb_2022 <- 1 / (ner_2022 + 1)

  # 2022 should have lower energy burden
  expect_lt(eb_2022, eb_2018)
})

test_that("comparison handles edge case: identical data", {
  skip_if_not_installed("dplyr")

  # Same data for both vintages
  data_2018 <- create_sample_lead_data(n = 100, seed = 900, vintage = "2018")
  data_2022 <- create_sample_lead_data(n = 100, seed = 900, vintage = "2022")  # Same seed

  # Calculate metrics
  ner_2018 <- weighted.mean(data_2018$ner, data_2018$households, na.rm = TRUE)
  ner_2022 <- weighted.mean(data_2022$ner, data_2022$households, na.rm = TRUE)

  # Should be identical (or very close due to floating point)
  expect_equal(ner_2018, ner_2022, tolerance = 1e-10)

  # Change should be zero
  change <- ner_2022 - ner_2018
  expect_equal(change, 0, tolerance = 1e-10)
})

test_that("comparison correctly uses household counts as weights", {
  # Small dataset to verify weighting manually
  data <- data.frame(
    income = c(10000, 50000, 100000),
    energy_cost = c(2000, 5000, 10000),
    households = c(100, 1, 1)  # First group has 100x weight
  )

  data$ner <- (data$income - data$energy_cost) / data$energy_cost

  # Weighted mean should be dominated by first group
  wm <- weighted.mean(data$ner, data$households)

  # Manual calculation
  ner_values <- c(4, 9, 9)  # (10000-2000)/2000=4, (50000-5000)/5000=9, etc.
  manual_wm <- (4*100 + 9*1 + 9*1) / (100 + 1 + 1)
  # = (400 + 9 + 9) / 102 = 418/102 ≈ 4.098

  expect_equal(wm, manual_wm, tolerance = 0.01)

  # Should be much closer to 4 than to 9 due to weighting
  expect_lt(abs(wm - 4), abs(wm - 9))
})

# ==============================================================================
# MVP DEMO INTEGRATION TEST
# ==============================================================================
# This test validates the MVP demo command: compare_energy_burden('fpl', 'NC', 'income_bracket')
#It mocks data loading to test the full end-to-end flow without requiring downloads.

test_that("MVP demo: compare_energy_burden('fpl', 'NC', 'income_bracket') works end-to-end", {
  skip_if_not_installed("mockery")
  skip_if_not_installed("dplyr")

  # Create realistic FPL test data for NC
  fpl_2018 <- create_sample_lead_data(n = 500, seed = 2018, dataset = "fpl", vintage = "2018")
  fpl_2022 <- create_sample_lead_data(n = 500, seed = 2022, dataset = "fpl", vintage = "2022")

  # Add required columns for compare_energy_burden
  fpl_2018$total_income <- fpl_2018$income * fpl_2018$households
  fpl_2018$total_electricity_spend <- fpl_2018$electricity_spend * fpl_2018$households
  fpl_2018$total_gas_spend <- fpl_2018$gas_spend * fpl_2018$households
  fpl_2018$total_other_spend <- fpl_2018$other_spend * fpl_2018$households

  fpl_2022$total_income <- fpl_2022$income * fpl_2022$households
  fpl_2022$total_electricity_spend <- fpl_2022$electricity_spend * fpl_2022$households
  fpl_2022$total_gas_spend <- fpl_2022$gas_spend * fpl_2022$households
  fpl_2022$total_other_spend <- fpl_2022$other_spend * fpl_2022$households

  # Mock load_cohort_data to return our test data
  mock_load <- mockery::mock(fpl_2018, fpl_2022, cycle = TRUE)

  mockery::stub(compare_energy_burden, 'load_cohort_data', mock_load)

  # Execute MVP demo command (format=FALSE to get numeric values for testing)
  result <- compare_energy_burden('fpl', 'NC', 'income_bracket', format = FALSE)

  # Verify structure
  expect_s3_class(result, "data.frame")
  expect_true(nrow(result) > 0)

  # Should have all required columns
  expected_cols <- c("income_bracket", "neb_2018", "neb_2022",
                     "change_pp", "change_pct", "households_2018", "households_2022")
  expect_true(all(expected_cols %in% names(result)))

  # Should have FPL income brackets
  fpl_brackets <- c("0-100%", "100-150%", "150-200%", "200%+")
  expect_true(all(result$income_bracket %in% fpl_brackets))

  # NEB values should be numeric
  expect_type(result$neb_2018, "double")
  expect_type(result$neb_2022, "double")

  # Change columns should exist and be numeric
  expect_true("change_pp" %in% names(result))
  expect_true("change_pct" %in% names(result))
  expect_type(result$change_pp, "double")

  # Verify mocking was called correctly
  mockery::expect_called(mock_load, 2)  # Once for 2018, once for 2022
})

# ==============================================================================
# BRACKET HARMONIZATION TESTS
# ==============================================================================

test_that("get_income_brackets returns correct brackets for each dataset/vintage", {
  # AMI 2018
  ami_2018_brackets <- get_income_brackets("ami", 2018)
  expect_equal(length(ami_2018_brackets), 3)
  expect_true(all(c("very_low", "low_mod", "mid_high") %in% ami_2018_brackets))

  # AMI 2022
  ami_2022_brackets <- get_income_brackets("ami", 2022)
  expect_equal(length(ami_2022_brackets), 5)
  expect_true(all(c("very_low", "low_mod", "mid_high", "100-150%", "150%+") %in% ami_2022_brackets))

  # FPL 2018
  fpl_2018_brackets <- get_income_brackets("fpl", 2018)
  expect_equal(length(fpl_2018_brackets), 5)
  expect_true(all(c("0-100%", "100-150%", "150-200%", "200-400%", "400%+") %in% fpl_2018_brackets))

  # FPL 2022
  fpl_2022_brackets <- get_income_brackets("fpl", 2022)
  expect_equal(length(fpl_2022_brackets), 5)
  expect_true(all(c("0-100%", "100-150%", "150-200%", "200-400%", "400%+") %in% fpl_2022_brackets))
})

test_that("get_income_brackets errors on invalid inputs", {
  expect_error(get_income_brackets("invalid", 2018))
  expect_error(get_income_brackets("ami", 2030))  # Unknown vintage
})

test_that("harmonize_income_brackets identifies bracket mismatches", {
  # Create mock AMI 2022 data
  mock_data <- data.frame(
    geoid = rep("01001020100", 5),
    income_bracket = c("very_low", "low_mod", "mid_high", "100-150%", "150%+"),
    households = c(100, 200, 300, 400, 500)
  )

  # Harmonize with strict matching (using ::: to access internal function)
  result <- emburden:::harmonize_income_brackets(
    data = mock_data,
    dataset = "ami",
    vintage = 2022,
    strict_matching = TRUE,
    comparison_vintages = c(2018, 2022)
  )

  # Should drop the 2022-only brackets
  expect_equal(length(result$warnings), 1)
  expect_true(grepl("mismatch", result$warnings[1], ignore.case = TRUE))
  expect_equal(length(result$dropped_brackets), 2)
  expect_true(all(c("100-150%", "150%+") %in% result$dropped_brackets))

  # Harmonized data should only have 3 brackets
  expect_equal(nrow(result$data), 3)
  expect_true(all(result$data$income_bracket %in% c("very_low", "low_mod", "mid_high")))
})

test_that("harmonize_income_brackets handles FPL data without warnings", {
  # Create mock FPL data (consistent across vintages)
  mock_data <- data.frame(
    geoid = rep("01001020100", 5),
    income_bracket = c("0-100%", "100-150%", "150-200%", "200-400%", "400%+"),
    households = c(100, 200, 300, 400, 500)
  )

  # Harmonize FPL data (using ::: to access internal function)
  result <- emburden:::harmonize_income_brackets(
    data = mock_data,
    dataset = "fpl",
    vintage = 2022,
    strict_matching = TRUE,
    comparison_vintages = c(2018, 2022)
  )

  # Should have no warnings or dropped brackets (FPL is consistent)
  expect_equal(length(result$warnings), 0)
  expect_equal(length(result$dropped_brackets), 0)
  expect_equal(nrow(result$data), 5)
})

# Helper functions for creating test fixtures
# These functions generate sample data for testing without requiring
# external data sources or API calls

#' Create sample LEAD data for testing
#'
#' @param n Number of rows to generate
#' @param seed Random seed for reproducibility
#' @param dataset Either "ami" or "fpl"
#' @param vintage Either "2018" or "2022"
#' @return A data.frame with sample LEAD data
create_sample_lead_data <- function(n = 100, seed = 123, dataset = "ami", vintage = "2022") {
  set.seed(seed)

  # Generate realistic income brackets based on dataset
  if (dataset == "ami") {
    income_brackets <- c("0-30%", "30-60%", "60-80%", "80-100%", "100%+")
    # Income ranges by bracket (rough estimates)
    income_ranges <- list(
      "0-30%" = c(0, 25000),
      "30-60%" = c(25000, 50000),
      "60-80%" = c(50000, 65000),
      "80-100%" = c(65000, 85000),
      "100%+" = c(85000, 200000)
    )
  } else {  # fpl
    income_brackets <- c("0-100%", "100-150%", "150-200%", "200%+")
    income_ranges <- list(
      "0-100%" = c(0, 27000),
      "100-150%" = c(27000, 40000),
      "150-200%" = c(40000, 54000),
      "200%+" = c(54000, 150000)
    )
  }

  # Generate data
  data <- data.frame(
    geoid = sprintf("37%03d%06d",
                    sample(1:100, n, replace = TRUE),
                    sample(1:999999, n, replace = TRUE)),
    state_abbr = "NC",
    county_name = sample(c("Wake", "Mecklenburg", "Durham", "Guilford", "Forsyth"),
                        n, replace = TRUE),
    income_bracket = sample(income_brackets, n, replace = TRUE),
    stringsAsFactors = FALSE
  )

  # Generate income based on bracket
  data$income <- mapply(function(bracket) {
    range <- income_ranges[[bracket]]
    runif(1, range[1], range[2])
  }, data$income_bracket)

  # Energy costs correlate weakly with income (but not perfectly)
  data$energy_cost <- pmax(800, data$income * runif(n, 0.02, 0.08) + rnorm(n, 0, 300))

  # Other fields
  data$electricity_spend <- data$energy_cost * runif(n, 0.5, 0.7)
  data$gas_spend <- data$energy_cost * runif(n, 0.2, 0.4)
  data$other_spend <- data$energy_cost - data$electricity_spend - data$gas_spend

  data$households <- pmax(1, round(rnorm(n, 100, 50)))
  data$housing_tenure <- sample(c("OWNER", "RENTER"), n, replace = TRUE, prob = c(0.65, 0.35))
  data$primary_heating_fuel <- sample(
    c("Electricity", "Natural gas", "Fuel oil", "Propane"),
    n, replace = TRUE, prob = c(0.5, 0.3, 0.1, 0.1)
  )
  data$building_type <- ifelse(
    sample(c(TRUE, FALSE), n, replace = TRUE, prob = c(0.7, 0.3)),
    "Single-Family", "Multi-Family"
  )

  # Calculate derived metrics
  data$net_income <- data$income - data$energy_cost
  data$ner <- data$net_income / data$energy_cost
  data$energy_burden <- data$energy_cost / data$income

  # Add vintage identifier
  data$vintage <- vintage

  return(data)
}

#' Create corrupted test data with all-NA income_bracket
#'
#' This replicates the bug reported in issue #15
create_corrupted_fpl_data <- function(n = 100, seed = 456) {
  data <- create_sample_lead_data(n, seed, dataset = "fpl")
  data$income_bracket <- NA_character_
  return(data)
}

#' Create incomplete schema data (missing required columns)
create_incomplete_schema_data <- function(n = 100, seed = 789) {
  data <- create_sample_lead_data(n, seed)
  # Remove critical column
  data$income <- NULL
  return(data)
}

#' Create test data with edge cases
create_edge_case_data <- function() {
  data.frame(
    geoid = c("37001000001", "37001000002", "37001000003", "37001000004"),
    income_bracket = c("0-30%", "0-30%", "100%+", "0-30%"),
    income = c(0, 5000, 150000, -1000),  # Zero, low, high, negative
    energy_cost = c(1000, 0, 3000, 2000),  # Normal, zero, high, normal
    households = c(100, 200, 50, 0),  # Normal, normal, low, zero (invalid)
    housing_tenure = c("OWNER", "RENTER", "OWNER", NA),
    stringsAsFactors = FALSE
  )
}

#' Create a temporary test cache directory
#'
#' @return Path to temporary directory
create_test_cache <- function() {
  cache_dir <- file.path(tempdir(), "emburden_test_cache", paste0("test_", Sys.getpid()))
  dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
  return(cache_dir)
}

#' Write sample data to CSV file
#'
#' @param data Data frame to write
#' @param filename Filename (will be placed in temp directory)
#' @return Path to created file
write_test_csv <- function(data, filename) {
  filepath <- file.path(tempdir(), filename)
  write.csv(data, filepath, row.names = FALSE)
  return(filepath)
}

#' Create sample database for testing
#'
#' @return Path to created SQLite database
create_test_database <- function() {
  if (!requireNamespace("DBI", quietly = TRUE) ||
      !requireNamespace("RSQLite", quietly = TRUE)) {
    skip("DBI and RSQLite required for database tests")
  }

  db_path <- file.path(tempdir(), paste0("test_emburden_", Sys.getpid(), ".db"))

  con <- DBI::dbConnect(RSQLite::SQLite(), db_path)

  # Create tables for different datasets/vintages
  DBI::dbWriteTable(con, "ami_2022", create_sample_lead_data(500, seed = 1, dataset = "ami", vintage = "2022"))
  DBI::dbWriteTable(con, "ami_2018", create_sample_lead_data(500, seed = 2, dataset = "ami", vintage = "2018"))
  DBI::dbWriteTable(con, "fpl_2022", create_sample_lead_data(500, seed = 3, dataset = "fpl", vintage = "2022"))
  DBI::dbWriteTable(con, "fpl_2018", create_sample_lead_data(500, seed = 4, dataset = "fpl", vintage = "2018"))

  DBI::dbDisconnect(con)

  return(db_path)
}

#' Clean up test files and directories
#'
#' @param paths Character vector of paths to remove
cleanup_test_files <- function(paths) {
  for (path in paths) {
    if (file.exists(path)) {
      if (dir.exists(path)) {
        unlink(path, recursive = TRUE)
      } else {
        file.remove(path)
      }
    }
  }
}

#' Skip test if offline (no internet connection)
skip_if_offline <- function() {
  # Simple internet connectivity check without external dependencies
  has_internet <- tryCatch({
    con <- url("http://www.google.com", open = "rb", timeout = 2)
    close(con)
    TRUE
  }, error = function(e) {
    FALSE
  })

  if (!has_internet) {
    skip("No internet connection available")
  }
}

#' Skip test if specific package not available
skip_if_not_installed <- function(pkg) {
  if (!requireNamespace(pkg, quietly = TRUE)) {
    skip(paste0("Package '", pkg, "' not available"))
  }
}

test_that("JSS vignette can be built", {
  skip_on_cran()
  skip_if_not_installed("rticles")
  skip_if_not_installed("rmarkdown")

  # Determine if we're running from package source or installed package
  # During development: vignettes are in vignettes/ directory
  # During R CMD check: package is installed but we're in tests/testthat/

  # Try to find vignette source file
  # First, try the development path (when running tests from package root)
  vignette_path <- "../../vignettes/jss-emburden.Rmd"

  if (!file.exists(vignette_path)) {
    # If not in development mode, try finding it in the installed package location
    # During R CMD check, the source files are available in the check directory
    pkg_path <- find.package("emburden", quiet = TRUE)
    if (length(pkg_path) > 0) {
      vignette_path <- file.path(pkg_path, "vignettes", "jss-emburden.Rmd")
    }
  }

  # Skip test if we can't find the vignette source
  # This can happen in certain installation scenarios
  skip_if_not(
    file.exists(vignette_path),
    message = "JSS vignette source file not found - skipping test"
  )

  # Test that vignette can be rendered without errors
  # We don't actually build the PDF in tests (too slow), just verify no parsing errors
  expect_silent({
    # Parse the Rmd to check for syntax errors
    rmarkdown::yaml_front_matter(vignette_path)
  })

  # Verify references.bib exists
  bib_path <- "../../vignettes/references.bib"
  if (!file.exists(bib_path)) {
    pkg_path <- find.package("emburden", quiet = TRUE)
    if (length(pkg_path) > 0) {
      bib_path <- file.path(pkg_path, "vignettes", "references.bib")
    }
  }

  expect_true(
    file.exists(bib_path),
    info = "Bibliography file should exist"
  )
})

# Phase 1: File Validation Tests
# Tests for data quality checks and validation functions

test_that("sample data generator creates valid structure", {
  data <- create_sample_lead_data(n = 50)

  expect_s3_class(data, "data.frame")
  expect_equal(nrow(data), 50)
  expect_true("geoid" %in% names(data))
  expect_true("income_bracket" %in% names(data))
  expect_true("income" %in% names(data))
  expect_true("energy_cost" %in% names(data))
  expect_true("households" %in% names(data))
})

test_that("sample AMI data has correct income brackets", {
  data <- create_sample_lead_data(n = 100, dataset = "ami")

  expected_brackets <- c("0-30%", "30-60%", "60-80%", "80-100%", "100%+")
  expect_true(all(data$income_bracket %in% expected_brackets))
})

test_that("sample FPL data has correct income brackets", {
  data <- create_sample_lead_data(n = 100, dataset = "fpl")

  expected_brackets <- c("0-100%", "100-150%", "150-200%", "200%+")
  expect_true(all(data$income_bracket %in% expected_brackets))
})

test_that("energy cost correlates positively with income", {
  data <- create_sample_lead_data(n = 200)

  # Should have positive correlation (though not perfect)
  cor_value <- cor(data$income, data$energy_cost, use = "complete.obs")
  expect_gt(cor_value, 0)
  expect_lt(cor_value, 1)  # Not perfect correlation
})

test_that("corrupted data has all-NA income_bracket", {
  data <- create_corrupted_fpl_data(n = 100)

  expect_true(all(is.na(data$income_bracket)))
  expect_equal(nrow(data), 100)
  expect_true("income" %in% names(data))
})

test_that("incomplete schema data is missing required column", {
  data <- create_incomplete_schema_data(n = 50)

  expect_false("income" %in% names(data))
  expect_true("income_bracket" %in% names(data))
})

test_that("edge case data includes problematic values", {
  data <- create_edge_case_data()

  # Check for zero income
  expect_true(any(data$income == 0))

  # Check for zero energy cost
  expect_true(any(data$energy_cost == 0))

  # Check for zero/invalid households
  expect_true(any(data$households == 0))

  # Check for NA housing tenure
  expect_true(any(is.na(data$housing_tenure)))
})

test_that("NER calculation handles zero income correctly", {
  # When income is zero, NER should be -1
  result <- ner_func(0, 1000)
  expect_equal(result, -1)
})

test_that("NER calculation handles zero energy cost correctly", {
  # When energy cost is zero, NER should be Inf
  result <- ner_func(50000, 0)
  expect_equal(result, Inf)
})

test_that("NER calculation handles negative income", {
  # Negative income should still produce finite result
  result <- ner_func(-1000, 2000)
  expect_true(is.finite(result))
  expect_equal(result, -1.5)  # (-1000 - 2000) / 2000 = -1.5
})

test_that("energy burden calculation handles edge cases", {
  # Zero income -> Inf burden
  expect_equal(energy_burden_func(0, 1000), Inf)

  # Zero cost -> 0 burden
  expect_equal(energy_burden_func(50000, 0), 0)

  # Normal case
  expect_equal(energy_burden_func(50000, 5000), 0.1)
})

test_that("energy burden and NER are mathematically consistent", {
  income <- 50000
  cost <- 5000

  eb <- energy_burden_func(income, cost)
  ner <- ner_func(income, cost)

  # eb = 1 / (ner + 1)
  expect_equal(eb, 1 / (ner + 1), tolerance = 1e-10)

  # ner = (1 / eb) - 1
  expect_equal(ner, (1 / eb) - 1, tolerance = 1e-10)
})

test_that("household counts validation catches negative values", {
  data <- data.frame(
    households = c(100, 200, -50, 300)
  )

  # Should have negative household count
  expect_true(any(data$households < 0))
})

test_that("household counts validation catches zero values", {
  data <- data.frame(
    households = c(100, 0, 200, 300)
  )

  # Should have zero household count
  expect_true(any(data$households == 0))
})

test_that("income validation catches negative values", {
  data <- create_edge_case_data()

  # Should have negative income
  expect_true(any(data$income < 0))
})

test_that("test CSV writing works", {
  data <- create_sample_lead_data(n = 10)
  filepath <- write_test_csv(data, "test_sample.csv")

  expect_true(file.exists(filepath))

  # Read it back
  read_data <- read.csv(filepath, stringsAsFactors = FALSE)
  expect_equal(nrow(read_data), 10)

  # Cleanup
  file.remove(filepath)
})

test_that("test cache directory creation works", {
  cache_dir <- create_test_cache()

  expect_true(dir.exists(cache_dir))

  # Cleanup
  unlink(cache_dir, recursive = TRUE)
})

test_that("cleanup function removes test files", {
  # Create test files
  test_file1 <- write_test_csv(create_sample_lead_data(10), "cleanup_test1.csv")
  test_file2 <- write_test_csv(create_sample_lead_data(10), "cleanup_test2.csv")
  test_dir <- create_test_cache()

  # Verify they exist
  expect_true(file.exists(test_file1))
  expect_true(file.exists(test_file2))
  expect_true(dir.exists(test_dir))

  # Cleanup
  cleanup_test_files(c(test_file1, test_file2, test_dir))

  # Verify removed
  expect_false(file.exists(test_file1))
  expect_false(file.exists(test_file2))
  expect_false(dir.exists(test_dir))
})

test_that("required column check works", {
  data <- create_sample_lead_data(n = 20)

  # Should have all required columns
  required_cols <- c("geoid", "income", "energy_cost", "income_bracket", "households")
  expect_true(all(required_cols %in% names(data)))

  # Missing column test
  data_incomplete <- data
  data_incomplete$income <- NULL
  expect_false("income" %in% names(data_incomplete))
})

test_that("income bracket validation for AMI data", {
  data <- create_sample_lead_data(n = 100, dataset = "ami")

  ami_brackets <- c("0-30%", "30-60%", "60-80%", "80-100%", "100%+")

  # All values should be valid AMI brackets
  expect_true(all(data$income_bracket %in% ami_brackets))

  # Should have variety of brackets (not all the same)
  expect_gt(length(unique(data$income_bracket)), 1)
})

test_that("income bracket validation for FPL data", {
  data <- create_sample_lead_data(n = 100, dataset = "fpl")

  fpl_brackets <- c("0-100%", "100-150%", "150-200%", "200%+")

  # All values should be valid FPL brackets
  expect_true(all(data$income_bracket %in% fpl_brackets))

  # Should have variety of brackets
  expect_gt(length(unique(data$income_bracket)), 1)
})

test_that("vintage field is set correctly", {
  data_2018 <- create_sample_lead_data(n = 50, vintage = "2018")
  data_2022 <- create_sample_lead_data(n = 50, vintage = "2022")

  expect_equal(unique(data_2018$vintage), "2018")
  expect_equal(unique(data_2022$vintage), "2022")
})

test_that("geoid format is valid", {
  data <- create_sample_lead_data(n = 100)

  # All geoids should start with 37 (NC FIPS code)
  expect_true(all(startsWith(data$geoid, "37")))

  # All geoids should be 11 characters (FIPS code format)
  expect_true(all(nchar(data$geoid) == 11))
})

test_that("housing tenure values are valid", {
  data <- create_sample_lead_data(n = 100)

  valid_tenure <- c("OWNER", "RENTER")
  expect_true(all(data$housing_tenure %in% valid_tenure | is.na(data$housing_tenure)))
})

test_that("primary heating fuel values are realistic", {
  data <- create_sample_lead_data(n = 100)

  valid_fuels <- c("Electricity", "Natural gas", "Fuel oil", "Propane")
  expect_true(all(data$primary_heating_fuel %in% valid_fuels | is.na(data$primary_heating_fuel)))
})

test_that("building type values are valid", {
  data <- create_sample_lead_data(n = 100)

  valid_types <- c("Single-Family", "Multi-Family")
  expect_true(all(data$building_type %in% valid_types | is.na(data$building_type)))
})

test_that("derived metrics are calculated correctly", {
  data <- create_sample_lead_data(n = 50)

  # Check net_income calculation
  expect_equal(data$net_income, data$income - data$energy_cost)

  # Check NER calculation
  expected_ner <- (data$income - data$energy_cost) / data$energy_cost
  expect_equal(data$ner, expected_ner)

  # Check energy burden calculation
  expected_eb <- data$energy_cost / data$income
  expect_equal(data$energy_burden, expected_eb)
})

#' Calculate Weighted Metrics for Energy Burden Analysis
#'
#' Calculates weighted statistical metrics (mean, median, quantiles) for a
#' specified energy metric, with optional grouping by geographic or demographic
#' categories. This is the primary function for aggregating household-level
#' energy burden data using proper weighting by household counts.
#'
#' @param graph_data A data frame containing household energy burden data with
#'   columns for the metric of interest, household counts, and optional grouping
#'   variables
#' @param group_columns Character vector of column names to group by, or NULL
#'   for no grouping (calculates overall statistics)
#' @param metric_name Character string specifying the column name of the metric
#'   to analyze (e.g., "ner" for Net Energy Return)
#' @param metric_cutoff_level Numeric value defining the poverty threshold for
#'   the metric (e.g., 15.67 for Nh corresponding to 6% energy burden)
#' @param upper_quantile_view Numeric between 0 and 1 specifying the upper
#'   quantile to calculate (default: 1.0 for maximum)
#' @param lower_quantile_view Numeric between 0 and 1 specifying the lower
#'   quantile to calculate (default: 0.0 for minimum)
#'
#' @returns A data frame with one row per group (or one row if ungrouped)
#'   containing:
#'   \item{household_count}{Total number of households in the group}
#'   \item{households_below_cutoff}{Number of households below poverty threshold}
#'   \item{pct_in_group_below_cutoff}{Proportion of group below threshold}
#'   \item{metric_mean}{Weighted mean of the metric}
#'   \item{metric_median}{Weighted median of the metric}
#'   \item{metric_upper}{Upper quantile value}
#'   \item{metric_lower}{Lower quantile value}
#'   \item{metric_max}{Maximum value in group}
#'   \item{metric_min}{Minimum value in group}
#'
#' @details
#' This function requires the `spatstat` package for weighted quantile
#' calculations. It automatically handles missing values and ensures that
#' statistics are only calculated when sufficient data points exist (n >= 3).
#'
#' The function adds an "All" category row that aggregates across all groups,
#' in addition to the individual group statistics.
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Calculate metrics for NC cooperatives using Nh
#' library(dplyr)
#'
#' # Sample data
#' data <- data.frame(
#'   cooperative = rep(c("Coop A", "Coop B"), each = 3),
#'   ner = c(20, 15, 25, 18, 22, 12),
#'   households = c(1000, 500, 750, 900, 600, 400)
#' )
#'
#' # Calculate weighted metrics by cooperative
#' results <- calculate_weighted_metrics(
#'   graph_data = data,
#'   group_columns = "cooperative",
#'   metric_name = "ner",
#'   metric_cutoff_level = 15.67,
#'   upper_quantile_view = 0.95,
#'   lower_quantile_view = 0.05
#' )
#' }
calculate_weighted_metrics <- function(graph_data,
                                       group_columns,
                                       metric_name,
                                       metric_cutoff_level,
                                       upper_quantile_view = 1.0,
                                       lower_quantile_view = 0.0) {
  weighted_metrics <- grouped_weighted_metrics(
    graph_data,
    group_columns = NULL,
    metric_name,
    metric_cutoff_level,
    upper_quantile_view,
    lower_quantile_view
  )

  if (!is.null(group_columns)) {
    all_groups <- as.data.frame(matrix(rep("All", length(group_columns)), nrow = 1))
    names(all_groups) <- group_columns
    weighted_metrics <- tibble::as_tibble(cbind(all_groups, weighted_metrics))

    grouped_weighted_metrics <- grouped_weighted_metrics(
      graph_data,
      group_columns = group_columns,
      metric_name,
      metric_cutoff_level,
      upper_quantile_view,
      lower_quantile_view
    ) %>%
      dplyr::ungroup() %>%
      dplyr::mutate(dplyr::across(dplyr::all_of(group_columns), as.factor))

    weighted_metrics <- dplyr::bind_rows(grouped_weighted_metrics, weighted_metrics) %>%
      dplyr::ungroup() %>%
      dplyr::mutate(dplyr::across(dplyr::all_of(group_columns), as.factor))
  } else {
    weighted_metrics <- data.frame(
      group = as.factor(rep("All", nrow(weighted_metrics))),
      weighted_metrics
    )
  }

  return(weighted_metrics)
}

#' Calculate Grouped Weighted Metrics (Internal)
#'
#' Internal function to calculate weighted statistics for grouped or ungrouped
#' data. Used by [calculate_weighted_metrics()].
#'
#' @inheritParams calculate_weighted_metrics
#'
#' @returns A data frame with weighted statistics for each group
#'
#' @keywords internal
#' @noRd
grouped_weighted_metrics <- function(graph_data,
                                     group_columns,
                                     metric_name,
                                     metric_cutoff_level,
                                     upper_quantile_view = 1.0,
                                     lower_quantile_view = 0.0) {
  weighted_metrics <- graph_data %>%
    dplyr::filter(is.finite(!!rlang::sym(metric_name)), .preserve = TRUE) %>%
    {
      if (!is.null(group_columns)) {
        dplyr::group_by(., dplyr::across(dplyr::all_of(group_columns)))
      } else {
        .
      }
    } %>%
    dplyr::summarise(
      household_count = sum(households),
      total_na = sum(is.na(!!rlang::sym(metric_name)) * households, na.rm = TRUE),
      households_below_cutoff =
        sum((!!rlang::sym(metric_name) < metric_cutoff_level) * households, na.rm = TRUE),
      metric_max = max(!!rlang::sym(metric_name), na.rm = TRUE),
      metric_min = min(!!rlang::sym(metric_name), na.rm = TRUE),
      metric_mean = if (sum(!is.na(households * !!rlang::sym(metric_name))) < 3 ||
        all(households == 0)) {
        NA
      } else {
        stats::weighted.mean(x = !!rlang::sym(metric_name), w = households, na.rm = TRUE)
      },
      metric_median = if (sum(!is.na(households * !!rlang::sym(metric_name))) < 3 ||
        all(households == 0)) {
        NA
      } else {
        spatstat.univar::weighted.quantile(
          x = !!rlang::sym(metric_name), w = households,
          probs = c(.5), na.rm = TRUE
        )
      },
      metric_upper = if (sum(!is.na(households * !!rlang::sym(metric_name))) < 3 ||
        all(households == 0)) {
        NA
      } else {
        spatstat.univar::weighted.quantile(
          x = !!rlang::sym(metric_name), w = households,
          probs = c(upper_quantile_view), na.rm = TRUE
        )
      },
      metric_lower = if (sum(!is.na(households * !!rlang::sym(metric_name))) < 3 ||
        all(households == 0)) {
        NA
      } else {
        spatstat.univar::weighted.quantile(
          x = !!rlang::sym(metric_name), w = households,
          probs = c(lower_quantile_view), na.rm = TRUE
        )
      },
      .groups = "drop"
    ) %>%
    dplyr::mutate(
      households_pct = household_count / sum(household_count),
      pct_in_group_below_cutoff = households_below_cutoff / household_count,
      pct_total_below_cutoff = households_below_cutoff / sum(households_below_cutoff)
    )

  return(weighted_metrics)
}

#' Filter and Prepare Graph Data (Internal)
#'
#' Internal function to filter and add grouping metadata to energy burden data.
#' Used for preparing data before visualization or further analysis.
#'
#' @inheritParams calculate_weighted_metrics
#'
#' @returns A data frame with additional columns for group percentiles and weights
#'
#' @keywords internal
#' @noRd
filter_graph_data <- function(clean_data, group_columns, metric_name) {
  graph_data <- clean_data %>%
    {
      if (!is.null(group_columns)) {
        dplyr::group_by(., dplyr::across(dplyr::all_of(group_columns)))
      } else {
        .
      }
    } %>%
    dplyr::mutate(group_households = sum(households, na.rm = TRUE)) %>%
    dplyr::mutate(group_household_weights = ifelse(group_households == 0, 0, households / group_households)) %>%
    dplyr::arrange(!!rlang::sym(metric_name)) %>%
    dplyr::mutate(
      group_percentile = cumsum(households * group_household_weights),
      overall_percentile = cumsum(households) / sum(households)
    ) %>%
    {
      if (!is.null(group_columns)) {
        tidyr::unite(., "group_name", dplyr::all_of(group_columns), remove = FALSE, sep = "+", na.rm = FALSE)
      } else {
        dplyr::mutate(., group_name = "all")
      }
    } %>%
    dplyr::ungroup()

  return(graph_data)
}

test_that("calculate_weighted_metrics calculates basic statistics", {
  # Mock data
  data <- data.frame(
    ner = c(10, 15, 20, 25, 30),
    households = c(100, 150, 200, 250, 300)
  )

  result <- calculate_weighted_metrics(
    graph_data = data,
    group_columns = NULL,
    metric_name = "ner",
    metric_cutoff_level = 15,
    upper_quantile_view = 0.95,
    lower_quantile_view = 0.05
  )

  # Check required columns exist
  expect_true("household_count" %in% names(result))
  expect_true("households_below_cutoff" %in% names(result))
  expect_true("metric_mean" %in% names(result))
  expect_true("metric_median" %in% names(result))
  expect_true("metric_max" %in% names(result))
  expect_true("metric_min" %in% names(result))

  # Check values
  expect_equal(result$household_count, 1000)
  expect_equal(result$households_below_cutoff, 100)  # Only ner=10 (cutoff is <, not <=)
  expect_equal(result$metric_max, 30)
  expect_equal(result$metric_min, 10)
})

test_that("calculate_weighted_metrics groups by specified columns", {
  # Mock data with groups
  data <- data.frame(
    cooperative = rep(c("Coop A", "Coop B"), each = 3),
    ner = c(10, 15, 20, 12, 18, 25),
    households = c(100, 150, 200, 120, 180, 240)
  )

  result <- calculate_weighted_metrics(
    graph_data = data,
    group_columns = "cooperative",
    metric_name = "ner",
    metric_cutoff_level = 15,
    upper_quantile_view = 1.0,
    lower_quantile_view = 0.0
  )

  # Should have 3 rows: "All" plus 2 cooperatives
  expect_equal(nrow(result), 3)

  # Check grouping column exists
  expect_true("cooperative" %in% names(result))

  # Check "All" row exists
  expect_true("All" %in% result$cooperative)

  # Check grouped rows exist
  expect_true("Coop A" %in% result$cooperative)
  expect_true("Coop B" %in% result$cooperative)

  # Verify household counts
  coop_a_row <- result[result$cooperative == "Coop A", ]
  coop_b_row <- result[result$cooperative == "Coop B", ]

  expect_equal(coop_a_row$household_count, 450)  # 100 + 150 + 200
  expect_equal(coop_b_row$household_count, 540)  # 120 + 180 + 240
})

test_that("calculate_weighted_metrics handles cutoff threshold correctly", {
  data <- data.frame(
    ner = c(5, 10, 15, 20, 25),  # 2 below 15, 1 at 15, 2 above
    households = c(100, 100, 100, 100, 100)
  )

  result <- calculate_weighted_metrics(
    graph_data = data,
    group_columns = NULL,
    metric_name = "ner",
    metric_cutoff_level = 15,
    upper_quantile_view = 1.0,
    lower_quantile_view = 0.0
  )

  # Cutoff is < not <=, so only values < 15 count
  expect_equal(result$households_below_cutoff, 200)  # 5 and 10 only

  # Check percentage
  expect_equal(result$pct_in_group_below_cutoff, 0.4)  # 200/500
})

test_that("calculate_weighted_metrics calculates weighted mean correctly", {
  # Data where weighted mean differs from unweighted
  # Need at least 3 data points for function to calculate mean
  data <- data.frame(
    ner = c(10, 20, 30),
    households = c(900, 50, 50)  # Heavily weighted toward 10
  )

  result <- calculate_weighted_metrics(
    graph_data = data,
    group_columns = NULL,
    metric_name = "ner",
    metric_cutoff_level = 0,
    upper_quantile_view = 1.0,
    lower_quantile_view = 0.0
  )

  # Weighted mean should be close to 10
  expected_mean <- (10 * 900 + 20 * 50 + 30 * 50) / 1000  # = 11.5
  expect_equal(result$metric_mean, expected_mean)

  # Unweighted mean would be 20, so this confirms weighting works
  expect_true(result$metric_mean < 15)
})

test_that("calculate_weighted_metrics handles NA values", {
  data <- data.frame(
    ner = c(10, NA, 20, 30),
    households = c(100, 150, 200, 250)
  )

  result <- calculate_weighted_metrics(
    graph_data = data,
    group_columns = NULL,
    metric_name = "ner",
    metric_cutoff_level = 15,
    upper_quantile_view = 1.0,
    lower_quantile_view = 0.0
  )

  # Should calculate statistics excluding NA
  expect_false(is.na(result$metric_mean))
  expect_false(is.na(result$metric_median))

  # Function filters to finite values, so household count excludes NA rows
  expect_equal(result$household_count, 550)  # 100 + 200 + 250

  # total_na should be 0 since NA rows are filtered before summarise
  expect_equal(result$total_na, 0)
})

test_that("calculate_weighted_metrics requires minimum data points", {
  # Less than 3 non-NA data points
  data <- data.frame(
    ner = c(10, NA),
    households = c(100, 150)
  )

  result <- calculate_weighted_metrics(
    graph_data = data,
    group_columns = NULL,
    metric_name = "ner",
    metric_cutoff_level = 15,
    upper_quantile_view = 1.0,
    lower_quantile_view = 0.0
  )

  # With < 3 data points, mean/median should be NA
  expect_true(is.na(result$metric_mean))
  expect_true(is.na(result$metric_median))

  # Household count only includes finite values (NA filtered out)
  expect_equal(result$household_count, 100)
})

test_that("calculate_weighted_metrics with multiple grouping columns", {
  data <- data.frame(
    state = rep(c("NC", "SC"), each = 4),
    fuel_type = rep(c("Gas", "Electric"), 4),
    ner = c(10, 15, 12, 18, 20, 25, 22, 28),
    households = rep(100, 8)
  )

  result <- calculate_weighted_metrics(
    graph_data = data,
    group_columns = c("state", "fuel_type"),
    metric_name = "ner",
    metric_cutoff_level = 15,
    upper_quantile_view = 1.0,
    lower_quantile_view = 0.0
  )

  # Should have: 1 "All" + 4 combinations (NC/Gas, NC/Electric, SC/Gas, SC/Electric)
  expect_equal(nrow(result), 5)

  # Check all expected combinations exist
  expect_true(all(c("state", "fuel_type") %in% names(result)))

  # Check "All" row
  all_row <- result[result$state == "All" & result$fuel_type == "All", ]
  expect_equal(nrow(all_row), 1)
  expect_equal(all_row$household_count, 800)
})

test_that("calculate_weighted_metrics calculates quantiles correctly", {
  # Data with known quantiles
  data <- data.frame(
    ner = 1:100,
    households = rep(10, 100)  # Equal weights
  )

  result <- calculate_weighted_metrics(
    graph_data = data,
    group_columns = NULL,
    metric_name = "ner",
    metric_cutoff_level = 0,
    upper_quantile_view = 0.95,
    lower_quantile_view = 0.05
  )

  # Check quantiles are reasonable
  expect_true(result$metric_upper > result$metric_median)
  expect_true(result$metric_lower < result$metric_median)
  expect_true(result$metric_upper <= 100)
  expect_true(result$metric_lower >= 1)

  # 95th percentile should be around 95
  expect_true(result$metric_upper > 90)

  # 5th percentile should be around 5
  expect_true(result$metric_lower < 10)
})

test_that("calculate_weighted_metrics handles infinite values", {
  data <- data.frame(
    ner = c(10, 15, Inf, -Inf, 20),
    households = c(100, 150, 200, 250, 300)
  )

  # Should filter out infinite values
  result <- calculate_weighted_metrics(
    graph_data = data,
    group_columns = NULL,
    metric_name = "ner",
    metric_cutoff_level = 15,
    upper_quantile_view = 1.0,
    lower_quantile_view = 0.0
  )

  # Statistics should be calculated only on finite values
  expect_false(is.infinite(result$metric_mean))
  expect_false(is.infinite(result$metric_median))
  expect_false(is.infinite(result$metric_max))
  expect_false(is.infinite(result$metric_min))
})

test_that("calculate_weighted_metrics percentage calculations are correct", {
  data <- data.frame(
    group = c("A", "A", "B", "B"),
    ner = c(10, 20, 5, 15),
    households = c(100, 100, 100, 100)
  )

  result <- calculate_weighted_metrics(
    graph_data = data,
    group_columns = "group",
    metric_name = "ner",
    metric_cutoff_level = 15,
    upper_quantile_view = 1.0,
    lower_quantile_view = 0.0
  )

  # Find group A row
  group_a <- result[result$group == "A", ]

  # Group A: 1 out of 2 records below cutoff (ner=10)
  expect_equal(group_a$households_below_cutoff, 100)
  expect_equal(group_a$pct_in_group_below_cutoff, 0.5)

  # Check households_pct (proportion of total)
  expect_equal(group_a$households_pct, 0.5)  # 200/400
})

test_that("calculate_weighted_metrics with zero households handled", {
  data <- data.frame(
    ner = c(10, 15, 20),
    households = c(0, 0, 0)  # All zero
  )

  result <- calculate_weighted_metrics(
    graph_data = data,
    group_columns = NULL,
    metric_name = "ner",
    metric_cutoff_level = 15,
    upper_quantile_view = 1.0,
    lower_quantile_view = 0.0
  )

  # With all zero households, statistics should be NA
  expect_true(is.na(result$metric_mean))
  expect_true(is.na(result$metric_median))

  # But household count can still be 0
  expect_equal(result$household_count, 0)
})

#' North Carolina Complete Energy Burden Sample Data
#'
#' A comprehensive dataset containing energy burden data for all counties in North Carolina.
#' This dataset includes both Federal Poverty Line (FPL) and Area Median Income (AMI) cohort
#' data for 2018 and 2022 vintages, aggregated to the census tract × income bracket level.
#'
#' This sample data provides full state coverage for more comprehensive analysis, testing,
#' and demonstrations. For lightweight quick demos, see \code{\link{orange_county_sample}}.
#'
#' @format A named list with 4 data frames:
#' \describe{
#'   \item{fpl_2018}{Federal Poverty Line cohort data for 2018 (~10,805 rows)}
#'   \item{fpl_2022}{Federal Poverty Line cohort data for 2022 (~13,185 rows)}
#'   \item{ami_2018}{Area Median Income cohort data for 2018 (~6,484 rows)}
#'   \item{ami_2022}{Area Median Income cohort data for 2022 (~5,091 rows)}
#' }
#'
#' Each data frame contains:
#' \describe{
#'   \item{geoid}{11-digit census tract identifier (character)}
#'   \item{income_bracket}{Income bracket category (character)}
#'   \item{households}{Number of households in this cohort (numeric)}
#'   \item{total_income}{Total household income in dollars (numeric)}
#'   \item{total_electricity_spend}{Total electricity spending in dollars (numeric)}
#'   \item{total_gas_spend}{Total gas spending in dollars (numeric)}
#'   \item{total_other_spend}{Total other fuel spending in dollars (numeric)}
#' }
#'
#' @details
#' **North Carolina** (all 100 counties):
#' \itemize{
#'   \item 2018: 2,163 census tracts
#'   \item 2022: 2,642 census tracts (tract boundaries changed)
#' }
#'
#' **Income Brackets**:
#' \itemize{
#'   \item FPL: 0-100%, 100-150%, 150-200%, 200-400%, 400%+
#'   \item AMI: Varies by vintage (4-6 categories)
#' }
#'
#' **Size**: 1.3 MB compressed (.rda)
#'
#' @source
#' U.S. Department of Energy Low-Income Energy Affordability Data (LEAD) Tool
#' \itemize{
#'   \item 2018 vintage: \url{https://data.openei.org/submissions/573}
#'   \item 2022 vintage: \url{https://data.openei.org/submissions/6219}
#' }
#'
#' @examples
#' # Load sample data
#' data(nc_sample)
#'
#' # View structure
#' names(nc_sample)
#'
#' # Analyze energy burden by county
#' library(dplyr)
#'
#' # Extract county FIPS (first 5 digits of geoid)
#' nc_sample$fpl_2022 %>%
#'   mutate(county_fips = substr(geoid, 1, 5)) %>%
#'   group_by(county_fips, income_bracket) %>%
#'   summarise(
#'     households = sum(households),
#'     avg_energy_burden = sum(total_electricity_spend + total_gas_spend + total_other_spend) /
#'                         sum(total_income),
#'     .groups = "drop"
#'   ) %>%
#'   filter(county_fips == "37183")  # Wake County
#'
#' # Compare urban vs rural counties
#' urban_counties <- c("37119", "37063", "37183")  # Mecklenburg, Durham, Wake
#' rural_counties <- c("37069", "37095", "37131")  # Franklin, Hyde, Northampton
#'
#' nc_sample$fpl_2022 %>%
#'   mutate(
#'     county_fips = substr(geoid, 1, 5),
#'     region = case_when(
#'       county_fips %in% urban_counties ~ "Urban",
#'       county_fips %in% rural_counties ~ "Rural",
#'       TRUE ~ "Other"
#'     )
#'   ) %>%
#'   filter(region != "Other") %>%
#'   group_by(region, income_bracket) %>%
#'   summarise(
#'     households = sum(households),
#'     energy_burden = sum(total_electricity_spend + total_gas_spend + total_other_spend) /
#'                     sum(total_income),
#'     .groups = "drop"
#'   )
#'
#' @seealso
#' \itemize{
#'   \item \code{\link{orange_county_sample}} - Lightweight sample (94 KB) for quick demos
#'   \item \code{\link{load_cohort_data}} - Load data for any state with county filtering
#'   \item \code{\link{compare_energy_burden}} - Compare energy burden across vintages
#'   \item \code{\link{calculate_weighted_metrics}} - Calculate weighted metrics with grouping
#' }
"nc_sample"

# Database Helper Functions
# Functions for managing production and test databases safely

#' Get Database Path
#'
#' Returns the path to the database, with protection against accidental deletion.
#' For tests, use a separate test database.
#'
#' @param test Logical, whether to use test database (default FALSE)
#' @return Path to database file
#' @keywords internal
get_db_path <- function(test = FALSE) {
  # During R CMD check, tests, or CRAN checks, use tempdir()
  is_checking <- !identical(Sys.getenv("NOT_CRAN"), "true") ||
                 nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_"))

  if (is_checking) {
    cache_dir <- file.path(tempdir(), "emburden_cache")
  } else {
    cache_dir <- rappdirs::user_cache_dir("emburden")
  }

  if (test) {
    # Test database - safe to delete
    db_name <- "emburden_test_db.sqlite"
  } else {
    # Production database - PROTECTED
    db_name <- "emburden_db.sqlite"
  }

  file.path(cache_dir, db_name)
}

#' Check if Database Exists
#'
#' @param test Logical, check test database instead of production
#' @return Logical, TRUE if database exists
#' @keywords internal
db_exists <- function(test = FALSE) {
  db_path <- get_db_path(test = test)
  file.exists(db_path)
}

#' Delete Database (PROTECTED)
#'
#' Deletes a database with safety checks. Production database requires
#' explicit confirmation.
#'
#' @param test Logical, delete test database (default TRUE)
#' @param confirm Logical, must be TRUE to delete production database
#' @return Logical, TRUE if deleted successfully
#' @keywords internal
delete_db <- function(test = TRUE, confirm = FALSE) {

  if (!test && !confirm) {
    stop(
      "Cannot delete production database without confirmation!\n",
      "To delete production database, use: delete_db(test = FALSE, confirm = TRUE)\n",
      "This should ONLY be done if you know what you're doing."
    )
  }

  db_path <- get_db_path(test = test)

  if (!file.exists(db_path)) {
    message("Database does not exist: ", db_path)
    return(FALSE)
  }

  db_type <- if (test) "TEST" else "PRODUCTION"
  message("Deleting ", db_type, " database: ", db_path)

  unlink(db_path)

  if (file.exists(db_path)) {
    warning("Failed to delete database: ", db_path)
    return(FALSE)
  }

  message("Successfully deleted ", db_type, " database")
  return(TRUE)
}

#' Get Database Connection
#'
#' @param test Logical, connect to test database instead of production
#' @return DBI connection object
#' @keywords internal
get_db_connection <- function(test = FALSE) {
  if (!requireNamespace("DBI", quietly = TRUE)) {
    stop("Package 'DBI' required for database operations")
  }
  if (!requireNamespace("RSQLite", quietly = TRUE)) {
    stop("Package 'RSQLite' required for database operations")
  }

  db_path <- get_db_path(test = test)
  cache_dir <- dirname(db_path)

  if (!dir.exists(cache_dir)) {
    dir.create(cache_dir, recursive = TRUE)
  }

  DBI::dbConnect(RSQLite::SQLite(), db_path)
}

#' Backup Production Database
#'
#' Creates a timestamped backup of the production database
#'
#' @return Path to backup file, or NULL if no database exists
#' @keywords internal
backup_db <- function() {
  prod_db <- get_db_path(test = FALSE)

  if (!file.exists(prod_db)) {
    message("No production database to backup")
    return(NULL)
  }

  # Create backup filename with timestamp
  timestamp <- format(Sys.time(), "%Y%m%d_%H%M%S")
  backup_dir <- file.path(dirname(prod_db), "backups")
  dir.create(backup_dir, showWarnings = FALSE, recursive = TRUE)

  backup_file <- file.path(
    backup_dir,
    paste0("emburden_db_backup_", timestamp, ".sqlite")
  )

  # Copy database
  file.copy(prod_db, backup_file, overwrite = FALSE)

  if (file.exists(backup_file)) {
    size_mb <- round(file.size(backup_file) / 1024^2, 2)
    message("Database backed up successfully!")
    message("  Location: ", backup_file)
    message("  Size: ", size_mb, " MB")
    return(backup_file)
  } else {
    warning("Backup failed!")
    return(NULL)
  }
}

#' Clear Test Database and Cache
#'
#' Safe function to clear test database and cache for testing.
#' NEVER touches production database.
#'
#' @keywords internal
clear_test_environment <- function() {
  message("Clearing test environment...")

  # Delete test database
  test_db <- get_db_path(test = TRUE)
  if (file.exists(test_db)) {
    unlink(test_db)
    message("  - Deleted test database")
  }

  # Clear test cache (but not production!)
  # During R CMD check, tests, or CRAN checks, use tempdir()
  is_checking <- !identical(Sys.getenv("NOT_CRAN"), "true") ||
                 nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_"))

  if (is_checking) {
    cache_dir <- file.path(tempdir(), "emburden_cache")
  } else {
    cache_dir <- rappdirs::user_cache_dir("emburden")
  }

  test_csv_pattern <- "test_.*\\.csv$"

  if (dir.exists(cache_dir)) {
    test_files <- list.files(
      cache_dir,
      pattern = test_csv_pattern,
      full.names = TRUE
    )

    if (length(test_files) > 0) {
      unlink(test_files)
      message("  - Deleted ", length(test_files), " test cache files")
    }
  }

  message("Test environment cleared (production data untouched)")
  invisible(TRUE)
}

# Test Zenodo Download Functionality
#
# These tests verify that Zenodo downloads work correctly WITHOUT
# touching the production database.

test_that("Zenodo configuration is valid", {
  config <- get_zenodo_config()

  # Check DOIs are configured
  expect_type(config$concept_doi, "character")
  expect_type(config$version_doi, "character")
  expect_true(grepl("^10\\.5281/zenodo\\.", config$concept_doi))

  # Check file configurations
  expect_true("ami_2022" %in% names(config$files))
  expect_true("fpl_2022" %in% names(config$files))
  expect_true("ami_2018" %in% names(config$files))
  expect_true("fpl_2018" %in% names(config$files))

  # Check file URLs are set
  expect_type(config$files$ami_2022$url, "character")
  expect_true(grepl("^https://zenodo\\.org/(api/)?records/", config$files$ami_2022$url))
})

test_that("Zenodo URLs are accessible", {
  skip_on_cran()
  skip_if_offline()

  config <- get_zenodo_config()

  # Test one file URL is reachable
  url <- config$files$ami_2022$url

  # Just check HTTP status (don't download full file)
  response <- httr::HEAD(url)
  expect_equal(httr::status_code(response), 200)
})

test_that("Can download from Zenodo (test environment)", {
  skip_on_cran()
  skip_if_offline()
  skip("Manual test only - requires clean test environment")

  # This test should be run manually to verify Zenodo downloads
  # It uses a SEPARATE test database, never touching production

  # Clear test environment (safe - only touches test DB)
  withr::defer(clear_test_environment())

  # TODO: Implement test-specific data loading that uses test DB
  # data <- load_cohort_data_test('ami', 'NC', '2022')
  # expect_gt(nrow(data), 0)
})

test_that("Database protection prevents accidental deletion", {
  # Trying to delete production DB without confirmation should fail
  expect_error(
    delete_db(test = FALSE, confirm = FALSE),
    "Cannot delete production database"
  )

  # Test DB can be deleted safely
  test_db <- get_db_path(test = TRUE)
  if (file.exists(test_db)) {
    expect_true(delete_db(test = TRUE))
  }
})

test_that("Test and production databases are separate", {
  test_path <- get_db_path(test = TRUE)
  prod_path <- get_db_path(test = FALSE)

  # Paths must be different
  expect_false(test_path == prod_path)

  # Test DB should have 'test' in name
  expect_true(grepl("test", basename(test_path)))

  # Production DB should NOT have 'test' in name
  expect_false(grepl("test", basename(prod_path)))
})

# Global variable bindings to satisfy R CMD check
utils::globalVariables(c(
  "total_electricity_spend", "total_gas_spend", "total_other_spend",
  "total_income", "total_spend", "geoid", "state_abbr", "nh",
  "vintage", "neb", ".data", "change_pp", "change_pct", "households"
))

#' Compare Energy Burden Between Years
#'
#' Compare household energy burden metrics across different data vintages,
#' using proper Net Energy Return (Nh) aggregation methodology.
#'
#' @param dataset Character, either "ami" or "fpl" for cohort data type
#' @param states Character vector of state abbreviations to filter by (optional)
#' @param group_by Character or character vector. Use keywords "income_bracket" (default),
#'   "state", or "none" for standard groupings. Or provide custom column name(s)
#'   for dynamic grouping (e.g., "geoid" for tract-level, c("state_abbr", "income_bracket")
#'   for multi-level grouping). Custom columns must exist in the loaded data.
#' @param counties Character vector of county names or FIPS codes to filter by (optional).
#'   Requires `states` to be specified.
#' @param vintage_1 Character, first vintage year: "2018" or "2022" (default "2022")
#' @param vintage_2 Character, second vintage year: "2018" or "2022" (default "2018")
#' @param format Logical, if TRUE returns formatted percentages (default TRUE)
#' @param strict_matching Logical, if TRUE (default) only compares income brackets
#'   that exist in both vintages and warns about mismatched brackets. If FALSE,
#'   compares all brackets (may result in NA values for brackets unique to one vintage).
#'
#' @return A data.frame with energy burden comparison showing:
#'   - neb_YYYY: Net Energy Burden for each vintage (where YYYY is the year)
#'   - change_pp: Absolute change in percentage points
#'   - change_pct: Relative percent change
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Single state comparison (fast, good for learning)
#' nc_comparison <- compare_energy_burden("ami", "NC", "income_bracket")
#'
#' # Overall comparison (no grouping)
#' compare_energy_burden("ami", "NC", "none")
#' }
#'
#' \donttest{
#' if (interactive()) {
#'   # Multi-state regional comparison (requires census data download)
#'   southeast <- compare_energy_burden(
#'     dataset = "fpl",
#'     states = c("NC", "SC", "GA", "FL"),
#'     group_by = "state"
#'   )
#'
#'   # Nationwide comparison by income bracket (all 51 states)
#'   us_comparison <- compare_energy_burden(
#'     dataset = "ami",
#'     group_by = "income_bracket"
#'   )
#'
#'   # Compare specific counties within a state (requires census data)
#'   compare_energy_burden("fpl", "NC", counties = c("Orange", "Durham", "Wake"))
#'
#'   # Custom grouping by tract-level geoid (requires census data)
#'   compare_energy_burden("ami", "NC", group_by = "geoid")
#' }
#' }
compare_energy_burden <- function(dataset = c("ami", "fpl"),
                                  states = NULL,
                                  group_by = "income_bracket",
                                  counties = NULL,
                                  vintage_1 = "2022",
                                  vintage_2 = "2018",
                                  format = TRUE,
                                  strict_matching = TRUE) {

  # Validate inputs
  dataset <- match.arg(dataset)

  # Handle group_by - can be keyword ("income_bracket", "state", "none")
  # or custom column name(s)
  valid_keywords <- c("income_bracket", "state", "none")

  if (length(group_by) == 1 && group_by %in% valid_keywords) {
    # Using standard keyword
    grouping_method <- group_by
  } else {
    # Using custom column name(s) - validate later when we have data
    grouping_method <- "custom"
    custom_group_cols <- group_by
  }

  # Handle common mistake: passing group_by keywords as counties argument
  # (though with new parameter order this is less likely)
  if (!is.null(counties)) {
    if (any(tolower(counties) %in% valid_keywords)) {
      # User likely meant group_by parameter - just ignore counties
      counties <- NULL
    }
  }

  # Load both vintages
  message("Loading ", vintage_1, " data...")
  data_1 <- load_cohort_data(
    dataset = dataset,
    states = states,
    counties = counties,
    vintage = vintage_1,
    verbose = FALSE
  )

  message("Loading ", vintage_2, " data...")
  data_2 <- load_cohort_data(
    dataset = dataset,
    states = states,
    counties = counties,
    vintage = vintage_2,
    verbose = FALSE
  )

  # Apply bracket harmonization if requested and grouping includes income brackets
  if (strict_matching && grouping_method %in% c("income_bracket", "custom")) {
    # Check if income_bracket is in the grouping
    if (grouping_method == "income_bracket" ||
        (grouping_method == "custom" && "income_bracket" %in% custom_group_cols)) {

      # Harmonize both vintages
      harm_1 <- harmonize_income_brackets(
        data = data_1,
        dataset = dataset,
        vintage = as.integer(vintage_1),
        strict_matching = TRUE,
        comparison_vintages = c(as.integer(vintage_1), as.integer(vintage_2))
      )

      harm_2 <- harmonize_income_brackets(
        data = data_2,
        dataset = dataset,
        vintage = as.integer(vintage_2),
        strict_matching = TRUE,
        comparison_vintages = c(as.integer(vintage_1), as.integer(vintage_2))
      )

      # Update data with harmonized versions
      data_1 <- harm_1$data
      data_2 <- harm_2$data

      # Display warnings
      all_warnings <- unique(c(harm_1$warnings, harm_2$warnings))
      if (length(all_warnings) > 0) {
        for (warn_msg in all_warnings) {
          message(warn_msg)
        }
      }

      # Inform about dropped brackets
      all_dropped <- unique(c(harm_1$dropped_brackets, harm_2$dropped_brackets))
      if (length(all_dropped) > 0) {
        message(
          "Note: Dropped brackets not present in both vintages: ",
          paste(all_dropped, collapse = ", ")
        )
      }
    }
  }

  # Select only required columns before combining
  # This ensures both datasets have matching column sets regardless of vintage
  required_cols <- c(
    "geoid",
    "income_bracket",
    "households",
    "total_income",
    "total_electricity_spend",
    "total_gas_spend",
    "total_other_spend"
  )

  data_1 <- data_1 |>
    dplyr::select(dplyr::all_of(required_cols))

  data_2 <- data_2 |>
    dplyr::select(dplyr::all_of(required_cols))

  # Combine datasets
  combined <- rbind(
    data_1 |> dplyr::mutate(vintage = vintage_1),
    data_2 |> dplyr::mutate(vintage = vintage_2)
  )

  # Calculate total spend for each row
  combined <- combined |>
    dplyr::mutate(
      total_spend = total_electricity_spend +
        dplyr::coalesce(total_gas_spend, 0) +
        dplyr::coalesce(total_other_spend, 0)
    )

  # Determine grouping variables based on grouping method
  if (grouping_method == "income_bracket") {
    group_vars <- c("vintage", "income_bracket")
  } else if (grouping_method == "state") {
    # Need to join with census tract data to get state
    tracts <- load_census_tract_data(states = states, verbose = FALSE)
    combined <- combined |>
      dplyr::left_join(
        tracts |> dplyr::select(geoid, state_abbr),
        by = "geoid"
      )
    group_vars <- c("vintage", "state_abbr")
  } else if (grouping_method == "none") {
    group_vars <- "vintage"
  } else {
    # Custom column grouping
    group_vars <- c("vintage", custom_group_cols)

    # Validate that custom columns exist in the data
    missing_cols <- setdiff(custom_group_cols, names(combined))
    if (length(missing_cols) > 0) {
      stop(
        "Custom grouping column(s) not found in data: ",
        paste(missing_cols, collapse = ", "),
        "\nAvailable columns: ",
        paste(names(combined), collapse = ", ")
      )
    }
  }

  # Aggregate by grouping variables
  # For aggregated cohort data, sum totals directly rather than using Nh
  aggregated <- combined |>
    dplyr::group_by(dplyr::across(dplyr::all_of(group_vars))) |>
    dplyr::summarise(
      total_income_sum = sum(total_income, na.rm = TRUE),
      total_spend_sum = sum(total_spend, na.rm = TRUE),
      households = sum(households, na.rm = TRUE),
      .groups = "drop"
    ) |>
    dplyr::mutate(
      neb = total_spend_sum / total_income_sum
    ) |>
    dplyr::select(-total_income_sum, -total_spend_sum)

  # Pivot to wide format for comparison
  result <- aggregated |>
    tidyr::pivot_wider(
      names_from = vintage,
      values_from = c(neb, households),
      names_glue = "{.value}_{vintage}"
    )

  # Calculate changes
  # Always calculate change in chronological order (later - earlier)
  # regardless of which vintage was specified first
  v1_year <- as.integer(vintage_1)
  v2_year <- as.integer(vintage_2)

  if (v2_year > v1_year) {
    # v2 is later: calculate v2 - v1 (normal case)
    neb_later <- paste0("neb_", vintage_2)
    neb_earlier <- paste0("neb_", vintage_1)
  } else {
    # v1 is later: calculate v1 - v2
    neb_later <- paste0("neb_", vintage_1)
    neb_earlier <- paste0("neb_", vintage_2)
  }

  result <- result |>
    dplyr::mutate(
      change_pp = .data[[neb_later]] - .data[[neb_earlier]],
      change_pct = (change_pp / .data[[neb_earlier]]) * 100
    )

  # Format if requested
  if (format) {
    result <- result |>
      dplyr::mutate(
        dplyr::across(
          dplyr::starts_with("neb_"),
          ~ sprintf("%.2f%%", .x * 100)
        ),
        change_pp = sprintf("%+.2f pp", change_pp * 100),
        change_pct = sprintf("%+.1f%%", change_pct)
      )
  }

  # Rename state_abbr back to state if that was the grouping
  if (group_by == "state" && "state_abbr" %in% names(result)) {
    result <- result |>
      dplyr::rename(state = state_abbr)
  }

  # Add metadata as attributes
  attr(result, "dataset") <- dataset
  attr(result, "states") <- states
  attr(result, "group_by") <- group_by
  attr(result, "vintage_1") <- vintage_1
  attr(result, "vintage_2") <- vintage_2

  message("Comparison complete: ", vintage_1, " vs ", vintage_2)
  return(result)
}


#' Print Comparison Summary
#'
#' Pretty-print a comparison table from compare_energy_burden()
#'
#' @param x Comparison result from compare_energy_burden()
#' @param ... Additional arguments (not used)
#'
#' @return Returns \code{x} invisibly for use in pipe chains.
#'
#' @export
print.energy_burden_comparison <- function(x, ...) {
  dataset <- attr(x, "dataset")
  states <- attr(x, "states")
  group_by <- attr(x, "group_by")
  v1 <- attr(x, "vintage_1")
  v2 <- attr(x, "vintage_2")

  cat("\n")
  cat("Energy Burden Comparison (", v1, " vs ", v2, ")\n", sep = "")
  cat(strrep("=", 60), "\n")
  cat("Dataset:  ", toupper(dataset), "\n", sep = "")
  cat("States:   ", paste(states, collapse = ", "), "\n", sep = "")
  cat("Group by: ", group_by, "\n", sep = "")
  cat(strrep("=", 60), "\n\n")

  print(as.data.frame(x), row.names = FALSE)

  cat("\n")
  invisible(x)
}

# Import pipe operator from dplyr
#' @importFrom dplyr %>%
#' @export
dplyr::`%>%`

# Declare global variables to avoid R CMD check NOTEs
utils::globalVariables(c(
  ".",
  "households",
  "group_households",
  "group_household_weights",
  "household_count",
  "households_below_cutoff",
  "FPL15",
  "HINCP.UNITS",
  "ELEP.UNITS",
  "GASP.UNITS",
  "FULP.UNITS"
))

# Zenodo Data Repository Functions
#
# This module handles downloading pre-processed energy burden datasets from Zenodo,
# providing faster downloads and better reliability than OpenEI for large datasets.

#' Get Zenodo Record Information
#'
#' Returns the Zenodo DOI and file information for emburden datasets.
#'
#' @return List with Zenodo record information
#' @keywords internal
get_zenodo_config <- function() {
  # Zenodo record for emburden processed datasets
  # Published: 2025-11-19
  # Scope: US Nationwide (51 states + DC)
  # This record contains pre-processed, analysis-ready datasets
  #
  # AUTO-GENERATED by .dev/update-zenodo-config.R
  # DO NOT EDIT MANUALLY - regenerate from state-manifest.json instead
  list(
    # Main repository DOI (concept DOI - always points to latest version)
    concept_doi = "10.5281/zenodo.17656636",

    # Version-specific DOI (for reproducibility)
    version_doi = "10.5281/zenodo.17656637",

    # Direct download URLs for each dataset
    # Format: zenodo_baseurl/records/RECORD_ID/files/FILENAME
    files = list(
      # 2022 Cohort Data (US Nationwide)
      ami_2022 = list(
        filename = "lead_ami_cohorts_2022_us.csv.gz",
        url = "https://zenodo.org/records/17656637/files/lead_ami_cohorts_2022_us.csv.gz",
        size_mb = 23.02,
        md5 = "cc847d89119a374bede6ee7f41060506"
      ),

      fpl_2022 = list(
        filename = "lead_fpl_cohorts_2022_us.csv.gz",
        url = "https://zenodo.org/records/17656637/files/lead_fpl_cohorts_2022_us.csv.gz",
        size_mb = 20.00,
        md5 = "767f2ff27193116f61e893999eb8bcf1"
      ),

      ami_2018 = list(
        filename = "lead_ami_cohorts_2018_us.csv.gz",
        url = "https://zenodo.org/records/17656637/files/lead_ami_cohorts_2018_us.csv.gz",
        size_mb = 17.20,
        md5 = "4941e3566daec1badc53eb44f34d95a8"
      ),

      fpl_2018 = list(
        filename = "lead_fpl_cohorts_2018_us.csv.gz",
        url = "https://zenodo.org/records/17656637/files/lead_fpl_cohorts_2018_us.csv.gz",
        size_mb = 17.36,
        md5 = "85ef6b7b4de244e80ff700f3d5becf3a"
      ),


      # Census Tract Data (not yet uploaded)
      census_tracts = list(
        filename = "census_tract_data.csv.gz",
        url = NULL,
        size_mb = NULL,
        md5 = NULL
      )
    ),

    # Metadata
    description = "Pre-processed DOE LEAD Tool data for emburden R package (US Nationwide)",
    license = "CC-BY-4.0",
    source = "DOE Low-Income Energy Affordability Data (LEAD) Tool"
  )
}


#' Download Dataset from Zenodo
#'
#' Downloads a pre-processed dataset from the emburden Zenodo repository.
#' Includes progress bars, checksum verification, and automatic retry logic.
#'
#' @param dataset Character, either "ami" or "fpl"
#' @param vintage Character, data vintage: "2018" or "2022"
#' @param verbose Logical, print progress messages (default TRUE)
#'
#' @return Tibble with downloaded data, or NULL if download fails
#' @keywords internal
download_from_zenodo <- function(dataset, vintage, verbose = FALSE) {

  # Get Zenodo configuration
  config <- get_zenodo_config()

  # Construct dataset key
  dataset_key <- paste0(dataset, "_", vintage)

  if (!dataset_key %in% names(config$files)) {
    if (verbose) {
      message("  Dataset '", dataset_key, "' not available on Zenodo")
    }
    return(NULL)
  }

  file_info <- config$files[[dataset_key]]

  # Check if URL is configured
  if (is.null(file_info$url) || file_info$url == "") {
    if (verbose) {
      message("  Zenodo URL not configured for ", dataset_key)
      message("  Falling back to OpenEI...")
    }
    return(NULL)
  }

  if (verbose) {
    message("Downloading from Zenodo repository...")
    if (!is.null(file_info$size_mb)) {
      message("  File size: ", file_info$size_mb, " MB")
    }
    message("  URL: ", file_info$url)
  }

  # Setup cache directory
  cache_dir <- get_cache_dir()
  cache_file <- file.path(
    cache_dir,
    paste0("lead_", vintage, "_", dataset, "_cohorts.csv")
  )

  # If already cached, load from cache
  if (file.exists(cache_file)) {
    if (verbose) {
      message("  Found in cache, loading...")
    }
    return(readr::read_csv(cache_file, show_col_types = FALSE))
  }

  # Download to temporary file
  temp_gz <- tempfile(fileext = ".csv.gz")

  tryCatch({

    # Check for httr package
    if (!requireNamespace("httr", quietly = TRUE)) {
      stop("Package 'httr' is required for downloading. Install it with: install.packages('httr')")
    }

    # Download with progress bar
    if (verbose) {
      response <- httr::GET(
        file_info$url,
        httr::write_disk(temp_gz, overwrite = TRUE),
        httr::progress()
      )
    } else {
      response <- httr::GET(
        file_info$url,
        httr::write_disk(temp_gz, overwrite = TRUE)
      )
    }

    # Check for HTTP errors
    if (httr::http_error(response)) {
      status_code <- httr::status_code(response)
      if (verbose) {
        message("  Zenodo download failed (HTTP ", status_code, ")")
        message("  Falling back to OpenEI...")
      }
      return(NULL)
    }

    # Verify checksum if available
    if (!is.null(file_info$md5)) {
      if (verbose) {
        message("  Verifying checksum...")
      }

      actual_md5 <- tools::md5sum(temp_gz)
      if (actual_md5 != file_info$md5) {
        warning("MD5 checksum mismatch! File may be corrupted.")
        if (verbose) {
          message("  Expected: ", file_info$md5)
          message("  Actual:   ", actual_md5)
          message("  Falling back to OpenEI...")
        }
        return(NULL)
      }
    }

    # Decompress and read
    if (verbose) {
      message("  Decompressing and reading data...")
    }

    # Read gzipped CSV directly
    data <- readr::read_csv(temp_gz, show_col_types = FALSE)

    # Save uncompressed to cache for faster subsequent loads
    readr::write_csv(data, cache_file)

    # Clean up
    unlink(temp_gz)

    if (verbose) {
      message("  Successfully downloaded from Zenodo")
    }

    # Import to database for even faster future loads
    try_import_to_database(data, dataset, vintage, verbose = verbose)

    return(data)

  }, error = function(e) {
    if (verbose) {
      message("  Zenodo download error: ", e$message)
      message("  Falling back to OpenEI...")
    }

    # Clean up on error
    if (file.exists(temp_gz)) {
      unlink(temp_gz)
    }

    return(NULL)
  })
}


#' Download Census Tract Data from Zenodo
#'
#' Downloads pre-processed census tract data from Zenodo.
#'
#' @param verbose Logical, print progress messages (default TRUE)
#'
#' @return Tibble with census tract data, or NULL if download fails
#' @keywords internal
download_tracts_from_zenodo <- function(verbose = FALSE) {

  config <- get_zenodo_config()
  file_info <- config$files$census_tracts

  # Check if configured
  if (is.null(file_info$url) || file_info$url == "") {
    if (verbose) {
      message("  Zenodo URL not configured for census tracts")
    }
    return(NULL)
  }

  if (verbose) {
    message("Downloading census tract data from Zenodo...")
  }

  # Setup cache
  cache_dir <- get_cache_dir()
  cache_file <- file.path(cache_dir, "census_tract_data.csv")

  if (file.exists(cache_file)) {
    if (verbose) {
      message("  Found in cache, loading...")
    }
    return(readr::read_csv(cache_file, show_col_types = FALSE))
  }

  # Download
  temp_gz <- tempfile(fileext = ".csv.gz")

  tryCatch({

    if (!requireNamespace("httr", quietly = TRUE)) {
      stop("Package 'httr' is required for downloading")
    }

    response <- httr::GET(
      file_info$url,
      httr::write_disk(temp_gz, overwrite = TRUE),
      if (verbose) httr::progress() else NULL
    )

    if (httr::http_error(response)) {
      if (verbose) {
        message("  Zenodo download failed")
      }
      return(NULL)
    }

    # Read and cache
    data <- readr::read_csv(temp_gz, show_col_types = FALSE)
    readr::write_csv(data, cache_file)
    unlink(temp_gz)

    if (verbose) {
      message("  Successfully downloaded from Zenodo")
    }

    return(data)

  }, error = function(e) {
    if (verbose) {
      message("  Zenodo download error: ", e$message)
    }
    if (file.exists(temp_gz)) unlink(temp_gz)
    return(NULL)
  })
}

#' Orange County NC Energy Burden Sample Data
#'
#' A sample dataset containing energy burden data for Orange County, North Carolina
#' (FIPS code 37135). This dataset includes both Federal Poverty Line (FPL) and
#' Area Median Income (AMI) cohort data for 2018 and 2022 vintages.
#'
#' This sample data is provided for quick demos, testing, and vignettes without
#' requiring a large download. For full state or national analysis, use
#' \code{\link{load_cohort_data}()} to download complete datasets from OpenEI.
#'
#' @format A named list with 4 data frames:
#' \describe{
#'   \item{fpl_2018}{Federal Poverty Line cohort data for 2018 (135 rows)}
#'   \item{fpl_2022}{Federal Poverty Line cohort data for 2022 (206 rows)}
#'   \item{ami_2018}{Area Median Income cohort data for 2018 (259 rows)}
#'   \item{ami_2022}{Area Median Income cohort data for 2022 (149 rows)}
#' }
#'
#' Each data frame contains:
#' \describe{
#'   \item{geoid}{11-digit census tract identifier (character)}
#'   \item{income_bracket}{Income bracket category (character)}
#'   \item{households}{Number of households in this cohort (numeric)}
#'   \item{total_income}{Total household income in dollars (numeric)}
#'   \item{total_electricity_spend}{Total electricity spending in dollars (numeric)}
#'   \item{total_gas_spend}{Total gas spending in dollars (numeric)}
#'   \item{total_other_spend}{Total other fuel spending in dollars (numeric)}
#' }
#'
#' @details
#' **Orange County NC** (Chapel Hill, Carrboro, Hillsborough):
#' \itemize{
#'   \item 2018: 27 census tracts
#'   \item 2022: 42 census tracts (tract boundaries changed)
#' }
#'
#' **Income Brackets**:
#' \itemize{
#'   \item FPL: 0-100%, 100-150%, 150-200%, 200-400%, 400%+
#'   \item AMI: very_low, low_mod, mid_high (aggregated from 6 AMI categories)
#' }
#'
#' @source
#' U.S. Department of Energy Low-Income Energy Affordability Data (LEAD) Tool
#' \itemize{
#'   \item 2018 vintage: \url{https://data.openei.org/submissions/573}
#'   \item 2022 vintage: \url{https://data.openei.org/submissions/6219}
#' }
#'
#' @examples
#' # Load sample data
#' data(orange_county_sample)
#'
#' # View structure
#' names(orange_county_sample)
#'
#' # Quick analysis of 2022 FPL data
#' library(dplyr)
#' orange_county_sample$fpl_2022 %>%
#'   group_by(income_bracket) %>%
#'   summarise(
#'     households = sum(households),
#'     avg_energy_burden = sum(total_electricity_spend + total_gas_spend + total_other_spend) /
#'                         sum(total_income)
#'   )
#'
#' @seealso
#' \itemize{
#'   \item \code{\link{load_cohort_data}} - Load full datasets for any state
#'   \item \code{\link{compare_energy_burden}} - Compare energy burden across vintages
#'   \item \code{\link{calculate_weighted_metrics}} - Calculate weighted metrics with grouping
#' }
"orange_county_sample"

#' List Available Income Brackets
#'
#' Returns the income brackets available for a given dataset and vintage.
#'
#' @param dataset Character, either "ami" or "fpl"
#' @param vintage Character, "2018" or "2022"
#'
#' @return Character vector of income bracket labels
#' @export
#'
#' @examples
#' list_income_brackets("ami", "2022")
#' list_income_brackets("fpl", "2018")
list_income_brackets <- function(dataset = c("ami", "fpl"),
                                  vintage = "2022") {
  dataset <- match.arg(dataset)

  if (!vintage %in% c("2018", "2022")) {
    stop("vintage must be '2018' or '2022'")
  }

  # Income brackets by dataset and vintage
  brackets <- list(
    ami_2022 = c(
      "0-30% AMI",
      "30-50% AMI",
      "50-80% AMI",
      "80-100% AMI",
      "100-120% AMI",
      "120%+ AMI"
    ),
    ami_2018 = c(
      "very_low",
      "low_mod",
      "moderate",
      "above_mod"
    ),
    fpl_2022 = c(
      "0-100%",
      "100-150%",
      "150-200%",
      "200-400%",
      "400%+"
    ),
    fpl_2018 = c(
      "0-100%",
      "100-150%",
      "150-200%",
      "200%+"
    )
  )

  key <- paste0(dataset, "_", vintage)
  return(brackets[[key]])
}


#' List Available States
#'
#' Returns all state abbreviations available in the LEAD dataset.
#'
#' @return Character vector of 51 state abbreviations (50 states + DC)
#' @export
#'
#' @examples
#' list_states()
list_states <- function() {
  c(
    "AL", "AK", "AZ", "AR", "CA", "CO", "CT", "DE", "DC", "FL",
    "GA", "HI", "ID", "IL", "IN", "IA", "KS", "KY", "LA", "ME",
    "MD", "MA", "MI", "MN", "MS", "MO", "MT", "NE", "NV", "NH",
    "NJ", "NM", "NY", "NC", "ND", "OH", "OK", "OR", "PA", "RI",
    "SC", "SD", "TN", "TX", "UT", "VT", "VA", "WA", "WV", "WI", "WY"
  )
}


#' List Available Columns in Cohort Data
#'
#' Returns column names and descriptions for LEAD cohort datasets.
#'
#' @param dataset Character, either "ami" or "fpl" (optional, affects available columns)
#' @param vintage Character, "2018" or "2022" (optional, affects available columns)
#'
#' @return Data frame with columns: column_name, description, data_type
#' @export
#'
#' @examples
#' list_cohort_columns()
#' list_cohort_columns("ami", "2022")
list_cohort_columns <- function(dataset = NULL, vintage = NULL) {
  # Core columns present in all datasets
  core_cols <- data.frame(
    column_name = c(
      "geoid",
      "income_bracket",
      "households",
      "total_income",
      "total_electricity_spend",
      "total_gas_spend",
      "total_other_spend"
    ),
    description = c(
      "11-digit census tract identifier (FIPS code)",
      "Income bracket category",
      "Number of households in this cohort",
      "Total household income ($)",
      "Total electricity spending ($)",
      "Total natural gas spending ($)",
      "Total other fuel spending (oil, propane, etc.) ($)"
    ),
    data_type = c(
      "character",
      "character",
      "numeric",
      "numeric",
      "numeric",
      "numeric",
      "numeric"
    ),
    stringsAsFactors = FALSE
  )

  return(core_cols)
}


#' Get Dataset Information
#'
#' Returns metadata about available LEAD datasets.
#'
#' @return Data frame with dataset information
#' @export
#'
#' @examples
#' get_dataset_info()
get_dataset_info <- function() {
  data.frame(
    dataset = c("ami", "ami", "fpl", "fpl"),
    vintage = c("2018", "2022", "2018", "2022"),
    full_name = c(
      "Area Median Income 2018",
      "Area Median Income 2022",
      "Federal Poverty Line 2018",
      "Federal Poverty Line 2022"
    ),
    income_brackets = c(4, 6, 4, 5),
    states_available = rep(51, 4),
    census_tracts = c(
      "~72,000",
      "~73,000",
      "~72,000",
      "~73,000"
    ),
    source_url = c(
      "https://data.openei.org/submissions/573",
      "https://data.openei.org/submissions/6219",
      "https://data.openei.org/submissions/573",
      "https://data.openei.org/submissions/6219"
    ),
    stringsAsFactors = FALSE
  )
}

# Tests for Metadata Functions

test_that("list_states returns correct number of states", {
  states <- list_states()

  expect_type(states, "character")
  expect_length(states, 51)  # 50 states + DC
})

test_that("list_states returns expected state abbreviations", {
  states <- list_states()

  # Check key states are included
  expect_true("NC" %in% states)
  expect_true("CA" %in% states)
  expect_true("TX" %in% states)
  expect_true("DC" %in% states)

  # Check PR is NOT included (as per documentation)
  expect_false("PR" %in% states)

  # All should be 2-character uppercase
  expect_true(all(nchar(states) == 2))
  expect_true(all(states == toupper(states)))
})

test_that("list_states has no duplicates", {
  states <- list_states()
  expect_equal(length(states), length(unique(states)))
})


# list_income_brackets tests -----

test_that("list_income_brackets works for all dataset/vintage combinations", {
  # AMI 2022
  ami_2022 <- list_income_brackets("ami", "2022")
  expect_type(ami_2022, "character")
  expect_length(ami_2022, 6)
  expect_true("0-30% AMI" %in% ami_2022)
  expect_true("120%+ AMI" %in% ami_2022)

  # AMI 2018
  ami_2018 <- list_income_brackets("ami", "2018")
  expect_type(ami_2018, "character")
  expect_length(ami_2018, 4)
  expect_true("very_low" %in% ami_2018)
  expect_true("above_mod" %in% ami_2018)

  # FPL 2022
  fpl_2022 <- list_income_brackets("fpl", "2022")
  expect_type(fpl_2022, "character")
  expect_length(fpl_2022, 5)
  expect_true("0-100%" %in% fpl_2022)
  expect_true("400%+" %in% fpl_2022)

  # FPL 2018
  fpl_2018 <- list_income_brackets("fpl", "2018")
  expect_type(fpl_2018, "character")
  expect_length(fpl_2018, 4)
  expect_true("0-100%" %in% fpl_2018)
  expect_true("200%+" %in% fpl_2018)
})

test_that("list_income_brackets validates input", {
  expect_error(
    list_income_brackets("ami", "2020"),
    "vintage must be '2018' or '2022'"
  )

  expect_error(
    list_income_brackets("invalid", "2022"),
    "'arg' should be one of"
  )
})

test_that("list_income_brackets defaults to 2022", {
  result <- list_income_brackets("ami")
  expect_equal(result, list_income_brackets("ami", "2022"))
})


# list_cohort_columns tests -----

test_that("list_cohort_columns returns data frame with correct structure", {
  cols <- list_cohort_columns()

  expect_s3_class(cols, "data.frame")
  expect_named(cols, c("column_name", "description", "data_type"))
  expect_true(nrow(cols) >= 7)  # At least core columns
})

test_that("list_cohort_columns includes core columns", {
  cols <- list_cohort_columns()

  core_columns <- c(
    "geoid",
    "income_bracket",
    "households",
    "total_income",
    "total_electricity_spend",
    "total_gas_spend",
    "total_other_spend"
  )

  expect_true(all(core_columns %in% cols$column_name))
})

test_that("list_cohort_columns data types are valid", {
  cols <- list_cohort_columns()

  valid_types <- c("character", "numeric", "integer", "logical")
  expect_true(all(cols$data_type %in% valid_types))
})


# get_dataset_info tests -----

test_that("get_dataset_info returns correct structure", {
  info <- get_dataset_info()

  expect_s3_class(info, "data.frame")
  expect_named(info, c(
    "dataset", "vintage", "full_name",
    "income_brackets", "states_available",
    "census_tracts", "source_url"
  ))
  expect_equal(nrow(info), 4)  # 2 datasets x 2 vintages
})

test_that("get_dataset_info has correct dataset combinations", {
  info <- get_dataset_info()

  # Should have all 4 combinations
  combos <- paste(info$dataset, info$vintage)
  expect_true("ami 2018" %in% combos)
  expect_true("ami 2022" %in% combos)
  expect_true("fpl 2018" %in% combos)
  expect_true("fpl 2022" %in% combos)
})

test_that("get_dataset_info has valid URLs", {
  info <- get_dataset_info()

  expect_true(all(grepl("^https://", info$source_url)))
  expect_true(all(grepl("openei.org", info$source_url)))
})

test_that("get_dataset_info shows 51 states available", {
  info <- get_dataset_info()

  expect_true(all(info$states_available == 51))
})

test_that("get_dataset_info income brackets match list_income_brackets", {
  info <- get_dataset_info()

  # Check each row
  for (i in 1:nrow(info)) {
    dataset <- info$dataset[i]
    vintage <- info$vintage[i]
    expected_count <- length(list_income_brackets(dataset, vintage))

    expect_equal(
      info$income_brackets[i],
      expected_count,
      info = paste("Mismatch for", dataset, vintage)
    )
  }
})

#' Calculate Energy Burden
#'
#' Calculates the energy burden as the ratio of energy spending to gross income.
#' Energy burden is defined as E_b = S/G, where S is energy spending and G is
#' gross income.
#'
#' @param g Numeric vector of gross income values
#' @param s Numeric vector of energy spending values
#' @param se Optional numeric vector of effective energy spending (defaults to s)
#'
#' @returns Numeric vector of energy burden values (ratio of spending to income)
#'
#' @export
#'
#' @examples
#' # Calculate energy burden for households
#' gross_income <- c(50000, 75000, 100000)
#' energy_spending <- c(3000, 3500, 4000)
#' energy_burden_func(gross_income, energy_spending)
energy_burden_func <- function(g, s, se = NULL) {
  if (is.null(se)) {
    se <- s
  }
  s / g
}

#' Calculate Net Energy Burden (NEB)
#'
#' Calculates Net Energy Burden with proper aggregation methodology via the
#' Net Energy Return (Nh) framework. For individual households, NEB = EB = S/G.
#' When aggregating across households (with weights), automatically uses the
#' Nh method to avoid 1-5% aggregation errors.
#'
#' @param g Numeric vector of gross income values
#' @param s Numeric vector of energy spending values
#' @importFrom stats weighted.mean
#' @param se Optional numeric vector of effective energy spending (defaults to s)
#' @param weights Optional numeric vector of weights for aggregation (e.g., household counts).
#'   When provided, uses Nh method: `1 / (1 + weighted.mean(nh, weights))`
#' @param aggregate Logical, if TRUE forces aggregation even without weights (uses unweighted mean).
#'   Default FALSE for backwards compatibility.
#'
#' @returns
#' - If `weights = NULL` and `aggregate = FALSE`: Numeric vector of individual NEB values (S/G)
#' - If `weights` provided or `aggregate = TRUE`: Single aggregated NEB value via Nh method
#'
#' @details
#' **Individual Level:** NEB = EB = S/G (mathematically identical)
#'
#' **Aggregation Modes:**
#' 1. **No aggregation** (default): Returns vector of individual NEB values
#'    ```
#'    neb_func(income, spending)  # Returns vector
#'    ```
#'
#' 2. **Weighted aggregation**: Automatically uses Nh method when weights provided
#'    ```
#'    neb_func(income, spending, weights = households)  # Returns single value
#'    ```
#'
#' 3. **Unweighted aggregation**: Use `aggregate = TRUE` for simple mean
#'    ```
#'    neb_func(income, spending, aggregate = TRUE)  # Returns single value
#'    ```
#'
#' **Why Nh Method?** Avoids 1-5% error from naive averaging:
#' - **CORRECT**: `neb_func(g, s, weights = w)` → Uses Nh internally
#' - **WRONG**: `weighted.mean(s/g, w)` → Introduces bias
#'
#' The Nh method: `1 / (1 + weighted.mean(nh, weights))` where `nh = (g-s)/se`
#' uses arithmetic mean instead of harmonic mean, providing computational
#' simplicity and numerical stability.
#'
#' @seealso [ner_func()] for the Net Energy Return (Nh) calculation
#' @seealso [energy_burden_func()] for simple EB without aggregation support
#' @export
#'
#' @examples
#' # Individual household - returns vector
#' neb_func(50000, 3000)  # 0.06
#' neb_func(c(30000, 50000), c(3000, 3500))  # c(0.10, 0.07)
#'
#' # Aggregation with weights - returns single value (CORRECT method)
#' incomes <- c(30000, 50000, 75000)
#' spending <- c(3000, 3500, 4000)
#' households <- c(100, 150, 200)
#' neb_func(incomes, spending, weights = households)
#'
#' # Unweighted aggregation
#' neb_func(incomes, spending, aggregate = TRUE)
#'
#' # Comparison: naive mean (WRONG) vs Nh method (CORRECT)
#' neb_naive <- weighted.mean(spending/incomes, households)  # Biased
#' neb_correct <- neb_func(incomes, spending, weights = households)  # Correct
#' abs(neb_naive - neb_correct) / neb_correct  # ~1-5% error
neb_func <- function(g, s, se = NULL, weights = NULL, aggregate = FALSE) {
  if (is.null(se)) {
    se <- s
  }

  # Individual household calculation (no aggregation) - backwards compatible
  if (is.null(weights) && !aggregate) {
    return(s / g)
  }

  # Aggregation via Nh method (proper methodology)
  nh <- ner_func(g, s, se)

  if (is.null(weights)) {
    # Unweighted aggregation (aggregate = TRUE case)
    nh_mean <- mean(nh, na.rm = TRUE)
  } else {
    # Weighted aggregation (when weights provided)
    nh_mean <- weighted.mean(nh, weights, na.rm = TRUE)
  }

  # Convert aggregated Nh back to NEB
  neb_aggregated <- 1 / (1 + nh_mean)

  return(neb_aggregated)
}

#' Calculate Energy Return on Investment (EROI)
#'
#' Calculates the Energy Return on Investment as the ratio of gross income to
#' effective energy spending. EROI = G/Se.
#'
#' @param g Numeric vector of gross income values
#' @param s Numeric vector of energy spending values
#' @param se Optional numeric vector of effective energy spending (defaults to s)
#'
#' @returns Numeric vector of EROI values
#'
#' @export
#'
#' @examples
#' # Calculate EROI for households
#' eroi_func(50000, 3000)
eroi_func <- function(g, s, se = NULL) {
  if (is.null(se)) {
    se <- s
  }
  g / se
}

#' Calculate Net Energy Return (Nh)
#'
#' Calculates the Net Energy Return using the formula Nh = (G - S) / Se,
#' where G is gross income, S is energy spending, and Se is effective energy
#' spending. This metric is the preferred aggregation variable as it properly
#' accounts for harmonic mean behavior when aggregating across households.
#'
#' @param g Numeric vector of gross income values
#' @param s Numeric vector of energy spending values
#' @param se Optional numeric vector of effective energy spending (defaults to s)
#'
#' @returns Numeric vector of Net Energy Return (Nh) values
#'
#' @details
#' The Net Energy Return is mathematically related to energy burden by:
#' E_b = 1 / (Nh + 1), or equivalently: Nh = (1/E_b) - 1
#'
#' **Why use Nh for aggregation?**
#'
#' For individual household data, the Nh method enables simple arithmetic weighted
#' mean aggregation:
#' - **Via Nh**: `neb = 1 / (1 + weighted.mean(nh, weights))` (arithmetic mean)
#' - **Direct EB**: `neb = 1 / weighted.mean(1/eb, weights)` (harmonic mean)
#'
#' **Computational advantages of the arithmetic mean approach:**
#' 1. **Simpler to compute** - Uses standard `weighted.mean()` function
#' 2. **More numerically stable** - Avoids division by very small EB values (e.g., 0.01)
#' 3. **More interpretable** - "Average net return per dollar spent on energy"
#' 4. **Prevents errors** - Makes it obvious you can't use arithmetic mean on EB directly
#'
#' For cohort data (pre-aggregated totals), direct calculation `sum(S)/sum(G)`
#' is mathematically equivalent to the Nh method but simpler.
#'
#' The 6% energy burden poverty threshold corresponds to Nh \eqn{\ge} 15.67.
#'
#' @export
#'
#' @examples
#' # Calculate Net Energy Return
#' gross_income <- 50000
#' energy_spending <- 3000
#' nh <- ner_func(gross_income, energy_spending)
#'
#' # Convert back to energy burden
#' energy_burden <- 1 / (nh + 1)
ner_func <- function(g, s, se = NULL) {
  if (is.null(se)) {
    se <- s
  }
  (g - s) / se
}

#' Calculate Disposable Energy-Adjusted Resources (DEAR)
#'
#' Calculates DEAR as the ratio of net income after energy spending to
#' gross income. DEAR = (G - S) / G.
#'
#' @param g Numeric vector of gross income values
#' @param s Numeric vector of energy spending values
#' @param se Optional numeric vector of effective energy spending (defaults to s)
#'
#' @returns Numeric vector of DEAR values (ratio of disposable income to gross income)
#'
#' @export
#'
#' @examples
#' # Calculate DEAR
#' dear_func(50000, 3000)
dear_func <- function(g, s, se = NULL) {
  if (is.null(se)) {
    se <- s
  }
  (g - s) / g
}

# Test NEB/EB/Nh Mathematical Equivalences and Aggregation Methods
# =================================================================
#
# This test suite demonstrates:
# 1. NEB = EB mathematically (at household level)
# 2. Proper aggregation via Nh (arithmetic mean) vs direct EB (harmonic mean)
# 3. Why arithmetic mean of EB is WRONG
# 4. Zero-removal requirements

library(testthat)

# Test 1: NEB function is identical to Energy Burden
# ===================================================
test_that("neb_func() is mathematically identical to energy_burden_func()", {
  # Single household
  income <- 50000
  spending <- 3000

  eb <- energy_burden_func(income, spending)
  neb <- neb_func(income, spending)

  expect_equal(neb, eb)
  expect_equal(neb, 0.06)

  # Vectorized
  incomes <- c(30000, 50000, 75000, 100000)
  spendings <- c(3000, 3500, 4000, 3500)

  eb_vec <- energy_burden_func(incomes, spendings)
  neb_vec <- neb_func(incomes, spendings)

  expect_equal(neb_vec, eb_vec)
})

# Test 2: NEB via Nh transformation
# ==================================
test_that("NEB can be calculated via Nh transformation", {
  income <- 50000
  spending <- 3000

  # Direct calculation
  neb_direct <- neb_func(income, spending)

  # Via Nh
  nh <- ner_func(income, spending)
  neb_via_nh <- 1 / (1 + nh)

  expect_equal(neb_via_nh, neb_direct)
  expect_equal(neb_via_nh, 0.06)

  # Verify the inverse relationship
  # nh = (G - S) / S
  # neb = S / G
  # neb = 1 / (1 + nh)
  expected_nh <- (income - spending) / spending
  expect_equal(nh, expected_nh)
  expect_equal(1 / (1 + nh), spending / income)
})

# Test 3: Cohort Data - Direct NEB equals NEB via Nh
# ====================================================
test_that("For cohort data, direct NEB calculation equals NEB via Nh", {
  # Simulate cohort data (pre-aggregated totals)
  # These are totals for different income groups
  total_incomes <- c(3000000, 7500000, 15000000, 20000000)
  total_spendings <- c(300000, 525000, 750000, 600000)
  households <- c(100, 150, 200, 200)

  # Method 1: Direct calculation (sum totals)
  neb_direct <- sum(total_spendings) / sum(total_incomes)

  # Method 2: Via Nh
  # For cohort data with totals, calculate aggregate Nh using weighted totals
  # Nh = (sum(G*h) - sum(S*h)) / sum(S*h) where h is household count per cohort
  # This is equivalent to: (sum(total_income) - sum(total_spending)) / sum(total_spending)
  total_income_all <- sum(total_incomes)
  total_spending_all <- sum(total_spendings)
  nh_aggregate <- (total_income_all - total_spending_all) / total_spending_all
  neb_via_nh <- 1 / (1 + nh_aggregate)

  # Should be equal for cohort data
  expect_equal(neb_via_nh, neb_direct, tolerance = 1e-10)

  # Verify: This is mathematically identical to direct NEB
  # NEB = S/G
  # Nh = (G-S)/S
  # NEB = 1/(1+Nh) = 1/(1+(G-S)/S) = S/(S+G-S) = S/G ✓
})

# Test 4: Individual HH Data - Nh Method vs Harmonic Mean
# ========================================================
test_that("For individual HH data, Nh method equals harmonic mean of EB", {
  # Simulate individual household data
  incomes <- c(30000, 50000, 75000, 100000, 150000)
  spendings <- c(3600, 3500, 4500, 4000, 5000)
  hh_weights <- c(100, 150, 200, 175, 125)

  # Calculate EB for each household
  eb <- energy_burden_func(incomes, spendings)

  # Method 1: Via Nh (arithmetic mean)
  nh <- ner_func(incomes, spendings)
  nh_mean <- weighted.mean(nh, hh_weights)
  neb_via_nh <- 1 / (1 + nh_mean)

  # Method 2: Harmonic mean of EB
  # Harmonic mean formula: 1 / weighted.mean(1/x, w)
  neb_harmonic <- 1 / weighted.mean(1 / eb, hh_weights)

  # These should be equal
  expect_equal(neb_via_nh, neb_harmonic, tolerance = 1e-10)
})

# Test 5: Arithmetic Mean of EB is WRONG
# =======================================
test_that("Arithmetic mean of EB introduces error (DO NOT USE)", {
  # Simulate data with varying energy burdens
  incomes <- c(25000, 50000, 75000, 100000, 150000)
  spendings <- c(3750, 3500, 4500, 4000, 4500)  # Different burden levels
  hh_weights <- c(150, 200, 180, 150, 100)

  # Calculate EB
  eb <- energy_burden_func(incomes, spendings)

  # CORRECT: Via Nh
  nh <- ner_func(incomes, spendings)
  nh_mean <- weighted.mean(nh, hh_weights)
  neb_correct <- 1 / (1 + nh_mean)

  # WRONG: Arithmetic mean of EB
  neb_wrong <- weighted.mean(eb, hh_weights)

  # Calculate error
  error <- abs(neb_wrong - neb_correct)
  error_pct <- (error / neb_correct) * 100

  # Error should be non-zero and typically 1-5%
  expect_gt(error, 0)
  expect_gt(error_pct, 0.1)  # At least 0.1% error

  # Document the error
  message(sprintf(
    "Arithmetic mean error: %.4f (%.2f%% relative error)",
    error, error_pct
  ))

  # The error increases with heterogeneity in energy burdens
  # This is why arithmetic mean is WRONG for ratios
})

# Test 6: Arithmetic Mean Error Increases with Heterogeneity
# ===========================================================
test_that("Arithmetic mean error increases with burden heterogeneity", {
  # Case 1: Homogeneous burdens (similar incomes/spending)
  incomes_homo <- c(50000, 52000, 48000, 51000, 49000)
  spendings_homo <- c(3000, 3120, 2880, 3060, 2940)
  weights <- c(100, 100, 100, 100, 100)

  eb_homo <- energy_burden_func(incomes_homo, spendings_homo)
  nh_homo <- ner_func(incomes_homo, spendings_homo)

  neb_correct_homo <- 1 / (1 + weighted.mean(nh_homo, weights))
  neb_wrong_homo <- weighted.mean(eb_homo, weights)
  error_homo <- abs(neb_wrong_homo - neb_correct_homo) / neb_correct_homo * 100

  # Case 2: Heterogeneous burdens (high variance)
  incomes_hetero <- c(25000, 50000, 100000, 150000, 200000)
  spendings_hetero <- c(5000, 4000, 5000, 4500, 4000)

  eb_hetero <- energy_burden_func(incomes_hetero, spendings_hetero)
  nh_hetero <- ner_func(incomes_hetero, spendings_hetero)

  neb_correct_hetero <- 1 / (1 + weighted.mean(nh_hetero, weights))
  neb_wrong_hetero <- weighted.mean(eb_hetero, weights)
  error_hetero <- abs(neb_wrong_hetero - neb_correct_hetero) / neb_correct_hetero * 100

  # Heterogeneous case should have larger error
  expect_gt(error_hetero, error_homo)

  message(sprintf(
    "Homogeneous error: %.2f%% | Heterogeneous error: %.2f%%",
    error_homo, error_hetero
  ))
})

# Test 7: Zero-Removal Requirement for Nh
# ========================================
test_that("Nh requires S > 0 (zero-removal is essential)", {
  # Test data with zero spending
  incomes <- c(50000, 60000, 70000)
  spendings <- c(3000, 0, 4000)  # One zero

  # EB technically works with S=0 (gives EB=0)
  eb <- energy_burden_func(incomes, spendings)
  expect_equal(eb[2], 0)  # Zero spending gives zero burden

  # But Nh has division by S in denominator
  # Nh = (G - S) / S  --> division by zero!
  nh_with_zeros <- ner_func(incomes, spendings)

  # Check for Inf at the zero-spending position
  expect_true(is.infinite(nh_with_zeros[2]))
  expect_true(is.finite(nh_with_zeros[1]))
  expect_true(is.finite(nh_with_zeros[3]))

  # This demonstrates why zero-removal is essential for Nh
  # Infinite values break aggregation calculations

  # SOLUTION: Remove zero-spending records before analysis
  valid_idx <- spendings > 0
  incomes_clean <- incomes[valid_idx]
  spendings_clean <- spendings[valid_idx]

  nh_clean <- ner_func(incomes_clean, spendings_clean)
  expect_true(all(is.finite(nh_clean)))
  expect_length(nh_clean, 2)  # Only 2 valid records

  # This is why the LEAD processing code filters out zero-energy records:
  # data <- data[data$energy_cost != 0, ]
  #
  # From R/lead_processing.R:208:
  # "Filter out zero-energy records (required for analysis)"
})

# Test 8: 6% Energy Burden Threshold Correspondence
# ==================================================
test_that("6% EB threshold corresponds to Nh = 15.67", {
  # The standard 6% energy burden poverty threshold
  eb_threshold <- 0.06

  # Calculate corresponding Nh
  # EB = 1/(1+Nh)
  # 0.06 = 1/(1+Nh)
  # 1+Nh = 1/0.06
  # Nh = (1/0.06) - 1
  nh_threshold <- (1 / eb_threshold) - 1

  expect_equal(nh_threshold, 15.666667, tolerance = 1e-5)

  # Verify the inverse
  eb_from_nh <- 1 / (1 + nh_threshold)
  expect_equal(eb_from_nh, eb_threshold, tolerance = 1e-10)

  # Using the ner_func with hypothetical data
  income <- 100
  spending <- 6  # 6% burden

  nh_calculated <- ner_func(income, spending)
  expect_equal(nh_calculated, nh_threshold, tolerance = 1e-5)
})

# Test 9: Computational Advantages Documentation
# ===============================================
test_that("Nh method is computationally simpler than harmonic mean", {
  # Simulate realistic data
  set.seed(42)
  n <- 1000
  incomes <- rlnorm(n, meanlog = 10.8, sdlog = 0.6) * 1000
  spendings <- incomes * runif(n, 0.02, 0.15)
  weights <- rpois(n, lambda = 50) + 1

  # Method 1: Nh approach (arithmetic mean)
  nh <- ner_func(incomes, spendings)

  time_nh <- system.time({
    neb_nh <- 1 / (1 + weighted.mean(nh, weights))
  })

  # Method 2: Direct EB harmonic mean
  eb <- energy_burden_func(incomes, spendings)

  time_harmonic <- system.time({
    neb_harmonic <- 1 / weighted.mean(1 / eb, weights)
  })

  # Results should be identical
  expect_equal(neb_nh, neb_harmonic, tolerance = 1e-10)

  # Nh method advantages:
  # 1. Uses standard weighted.mean() - simpler
  # 2. Avoids division by small EB values - more stable
  # 3. More interpretable - "average net return per dollar"
  # 4. Makes it obvious arithmetic mean on EB is wrong

  message(sprintf(
    "Nh method: %.6f sec | Harmonic method: %.6f sec | NEB: %.4f",
    time_nh[3], time_harmonic[3], neb_nh
  ))
})

# Test 10: Numerical Stability with Small EB Values
# ==================================================
test_that("Nh method is more numerically stable with small EB values", {
  # Create data with some very small energy burdens
  incomes <- c(200000, 150000, 100000, 50000, 30000)
  spendings <- c(1000, 2000, 3000, 4000, 4500)  # 0.5% to 15%
  weights <- c(50, 75, 100, 120, 130)

  eb <- energy_burden_func(incomes, spendings)

  # Nh method
  nh <- ner_func(incomes, spendings)
  neb_nh <- 1 / (1 + weighted.mean(nh, weights))

  # Harmonic mean (involves 1/eb where eb is very small)
  neb_harmonic <- 1 / weighted.mean(1 / eb, weights)

  # Both should be equal
  expect_equal(neb_nh, neb_harmonic, tolerance = 1e-10)

  # But Nh method avoids potential numerical issues from 1/0.005 = 200
  # vs Nh = 199 (subtraction instead of division)

  # Check that Nh values are well-behaved
  expect_true(all(is.finite(nh)))
  expect_true(all(nh > 0))  # All should be positive for real households
})

# Test 11: Edge Cases
# ===================
test_that("Functions handle edge cases appropriately", {
  # Very high burden (S > G would give negative Nh)
  # This shouldn't happen in real data but test boundary
  income_low <- 1000
  spending_high <- 1500

  # EB can be > 1 (spending exceeds income)
  eb_high <- energy_burden_func(income_low, spending_high)
  expect_equal(eb_high, 1.5)

  # Nh would be negative
  nh_negative <- ner_func(income_low, spending_high)
  expect_lt(nh_negative, 0)

  # Relationship still holds
  expect_equal(1 / (1 + nh_negative), eb_high, tolerance = 1e-10)
})

# Test 12: Real-World Example
# ============================
test_that("Real-world scenario demonstrates proper aggregation", {
  # Simulate data similar to NC energy burden analysis
  # Low income: high burden
  # High income: low burden

  income_brackets <- c("0-30% AMI", "30-50% AMI", "50-80% AMI",
                       "80-100% AMI", "100%+ AMI")
  mean_incomes <- c(15000, 35000, 55000, 75000, 120000)
  mean_spendings <- c(1800, 2500, 3000, 3200, 3500)
  households <- c(150000, 120000, 180000, 100000, 300000)

  # Calculate metrics
  eb <- energy_burden_func(mean_incomes, mean_spendings)
  nh <- ner_func(mean_incomes, mean_spendings)

  # CORRECT aggregation via Nh
  nh_weighted <- weighted.mean(nh, households)
  neb_correct <- 1 / (1 + nh_weighted)

  # WRONG aggregation (arithmetic mean)
  eb_wrong <- weighted.mean(eb, households)

  # Calculate error
  error_pct <- abs(eb_wrong - neb_correct) / neb_correct * 100

  # Document results
  message(sprintf("\n=== Real-World Aggregation Example ==="))
  message(sprintf("Correct NEB (via Nh): %.4f (%.2f%%)",
                  neb_correct, neb_correct * 100))
  message(sprintf("Wrong EB (arithmetic): %.4f (%.2f%%)",
                  eb_wrong, eb_wrong * 100))
  message(sprintf("Relative error: %.2f%%", error_pct))
  message(sprintf("Households with EB>6%%: %.1f%%",
                  sum(households[eb > 0.06]) / sum(households) * 100))

  # Error should be substantial
  expect_gt(error_pct, 0.5)

  # Verify individual bracket calculations are correct
  for (i in seq_along(income_brackets)) {
    neb_i <- neb_func(mean_incomes[i], mean_spendings[i])
    nh_i <- ner_func(mean_incomes[i], mean_spendings[i])
    expect_equal(neb_i, 1 / (1 + nh_i), tolerance = 1e-10)
  }
})

# Test 13: Performance Benchmark - Computational Efficiency
# ==========================================================
test_that("Nh method (arithmetic mean) is faster than harmonic mean", {
  # Timing-sensitive benchmark; skip on CRAN where check-machine load
  # makes wall-clock ratios unreliable (failed on r-devel-fedora-clang,
  # 2026-05-18).
  skip_on_cran()

  # Create realistic large dataset
  set.seed(123)
  n <- 10000  # 10,000 households
  incomes <- rlnorm(n, meanlog = 10.8, sdlog = 0.6) * 1000
  spendings <- incomes * runif(n, 0.02, 0.15)
  weights <- rpois(n, lambda = 50) + 1

  # Calculate metrics once
  eb <- energy_burden_func(incomes, spendings)
  nh <- ner_func(incomes, spendings)

  # Number of iterations for timing
  n_iter <- 1000

  # Method 1: Nh approach (arithmetic mean) - RECOMMENDED
  time_nh <- system.time({
    for (i in 1:n_iter) {
      neb_nh <- 1 / (1 + weighted.mean(nh, weights))
    }
  })[3]

  # Method 2: Harmonic mean of EB - CORRECT but slower
  time_harmonic <- system.time({
    for (i in 1:n_iter) {
      neb_harmonic <- 1 / weighted.mean(1 / eb, weights)
    }
  })[3]

  # Method 3: Arithmetic mean of EB - FAST but WRONG
  time_wrong <- system.time({
    for (i in 1:n_iter) {
      neb_wrong <- weighted.mean(eb, weights)
    }
  })[3]

  # Calculate final values for verification
  neb_nh_final <- 1 / (1 + weighted.mean(nh, weights))
  neb_harmonic_final <- 1 / weighted.mean(1 / eb, weights)
  neb_wrong_final <- weighted.mean(eb, weights)

  # Verify correctness
  expect_equal(neb_nh_final, neb_harmonic_final, tolerance = 1e-10)
  expect_gt(abs(neb_wrong_final - neb_nh_final), 0)  # Wrong method differs

  # Performance assertions
  # Nh method should be at least as fast as harmonic mean (typically faster)
  speedup_vs_harmonic <- time_harmonic / time_nh

  # Calculate error from wrong method
  error_pct <- abs(neb_wrong_final - neb_nh_final) / neb_nh_final * 100

  # Document results
  message("\n=== Performance Benchmark (", n, " households, ", n_iter, " iterations) ===")
  message("NOTE: Speed advantage applies to AGGREGATION operation only!")
  message("      For single household calculations, both methods are equivalent.")
  message(sprintf("Method 1 (Nh arithmetic):  %.4f sec  [RECOMMENDED - Correct & Fast]",
                  time_nh))
  message(sprintf("Method 2 (EB harmonic):    %.4f sec  [Correct but %.1fx slower]",
                  time_harmonic, speedup_vs_harmonic))
  message(sprintf("Method 3 (EB arithmetic):  %.4f sec  [WRONG - %.2f%% error!]",
                  time_wrong, error_pct))
  message(sprintf("\nSpeedup: Nh method is %.2fx faster than harmonic mean (for aggregation)", speedup_vs_harmonic))
  message(sprintf("Accuracy: Both correct methods agree within 1e-10"))
  message(sprintf("Error: Arithmetic mean of EB introduces %.2f%% error", error_pct))

  # Test should pass if Nh method is correct
  # Skip performance test on Windows - timing is too variable on CI
  skip_on_os("windows")
  expect_true(speedup_vs_harmonic > 0.5)  # At least not much slower
  expect_true(error_pct > 0.1)  # Wrong method has measurable error
})

# Test 14: Scalability - Performance with Different Dataset Sizes
# ================================================================
test_that("Nh method scales well with dataset size", {
  # Timing-sensitive benchmark; skip on CRAN (see Test 13 note).
  skip_on_cran()

  set.seed(456)

  dataset_sizes <- c(100, 1000, 10000)
  results <- data.frame(
    n = integer(),
    time_nh = numeric(),
    time_harmonic = numeric(),
    speedup = numeric()
  )

  for (n in dataset_sizes) {
    # Generate data
    incomes <- rlnorm(n, meanlog = 10.8, sdlog = 0.6) * 1000
    spendings <- incomes * runif(n, 0.02, 0.15)
    weights <- rpois(n, lambda = 50) + 1

    # Calculate metrics
    eb <- energy_burden_func(incomes, spendings)
    nh <- ner_func(incomes, spendings)

    # Benchmark with fewer iterations for larger datasets
    n_iter <- max(100, 10000 / n)

    # Time Nh method
    time_nh <- system.time({
      for (i in 1:n_iter) {
        neb <- 1 / (1 + weighted.mean(nh, weights))
      }
    })[3]

    # Time harmonic method
    time_harmonic <- system.time({
      for (i in 1:n_iter) {
        neb <- 1 / weighted.mean(1 / eb, weights)
      }
    })[3]

    # Store results
    results <- rbind(results, data.frame(
      n = n,
      time_nh = time_nh,
      time_harmonic = time_harmonic,
      speedup = time_harmonic / time_nh
    ))
  }

  message("\n=== Scalability Benchmark ===")
  message("NOTE: Speed advantage applies to AGGREGATION operation only, not single household")
  message(sprintf("%-10s %-15s %-15s %-10s",
                  "Size", "Nh (sec)", "Harmonic (sec)", "Speedup"))
  for (i in 1:nrow(results)) {
    message(sprintf("%-10d %-15.4f %-15.4f %.2fx",
                    results$n[i], results$time_nh[i],
                    results$time_harmonic[i], results$speedup[i]))
  }

  # Verify both methods complete successfully (no performance assertion - too noisy)
  expect_true(all(is.finite(results$time_nh)))
  expect_true(all(is.finite(results$time_harmonic)))
  # Note: speedup can be non-finite when timing is very small, which is acceptable
  # We're testing correctness, not enforcing specific performance ratios
})

# Phase 1: Energy Metrics Calculation Tests
# Tests for core energy burden and NER calculation functions

test_that("ner_func calculates Net Energy Return correctly", {
  # Standard case
  expect_equal(ner_func(50000, 5000), 9)

  # At poverty threshold (10% burden = NER of 9)
  expect_equal(ner_func(100000, 10000), 9)

  # Higher burden (20% burden = NER of 4)
  expect_equal(ner_func(10000, 2000), 4)
})

test_that("energy_burden_func calculates energy burden correctly", {
  # Standard case: 10% burden
  expect_equal(energy_burden_func(50000, 5000), 0.1)

  # 5% burden
  expect_equal(energy_burden_func(100000, 5000), 0.05)

  # 20% burden
  expect_equal(energy_burden_func(10000, 2000), 0.2)
})

test_that("neb_func is an alias for energy_burden_func", {
  # neb_func is actually an alias for energy_burden_func in the package
  # So it returns S/G, not (G-S)/G
  expect_equal(neb_func(50000, 5000), 0.1)  # 10% spent on energy
  expect_equal(neb_func(100000, 10000), 0.1)
  expect_equal(neb_func(10000, 2000), 0.2)  # 20% spent on energy

  # Verify it's the same as energy_burden_func
  expect_equal(neb_func(50000, 5000), energy_burden_func(50000, 5000))
})

test_that("energy burden and NER are inverse functions", {
  income <- 50000
  cost <- 5000

  eb <- energy_burden_func(income, cost)
  ner <- ner_func(income, cost)

  # Test inverse relationship: eb = 1 / (ner + 1)
  expect_equal(eb, 1 / (ner + 1))

  # Test inverse: ner = (1 / eb) - 1
  expect_equal(ner, (1 / eb) - 1)
})

test_that("energy poverty threshold (NER = 9) matches 10% burden", {
  # NER of 9 should equal 10% energy burden
  income <- 100000
  cost <- 10000

  ner_value <- ner_func(income, cost)
  eb_value <- energy_burden_func(income, cost)

  expect_equal(ner_value, 9)
  expect_equal(eb_value, 0.1)
})

test_that("ner_func handles zero income", {
  # Zero income means all spending comes from elsewhere
  # NER = (0 - S) / S = -1
  expect_equal(ner_func(0, 1000), -1)
  expect_equal(ner_func(0, 5000), -1)
})

test_that("ner_func handles zero energy cost", {
  # Zero energy cost means infinite return
  expect_equal(ner_func(50000, 0), Inf)
  expect_equal(ner_func(100000, 0), Inf)
})

test_that("ner_func handles negative income", {
  # Negative income (debt/losses) still produces valid NER
  expect_equal(ner_func(-5000, 2000), -3.5)  # (-5000 - 2000) / 2000

  # Should be finite
  expect_true(is.finite(ner_func(-1000, 1000)))
})

test_that("energy_burden_func handles zero income", {
  # Zero income means infinite burden
  expect_equal(energy_burden_func(0, 1000), Inf)
})

test_that("energy_burden_func handles zero cost", {
  # Zero cost means zero burden
  expect_equal(energy_burden_func(50000, 0), 0)
})

test_that("eroi_func calculates Energy Return on Investment", {
  # EROI = G / S
  expect_equal(eroi_func(50000, 5000), 10)
  expect_equal(eroi_func(100000, 10000), 10)
  expect_equal(eroi_func(10000, 2000), 5)
})

test_that("eroi_func and ner_func relationship", {
  # EROI = NER + 1
  income <- 50000
  cost <- 5000

  eroi <- eroi_func(income, cost)
  ner <- ner_func(income, cost)

  expect_equal(eroi, ner + 1)
})

test_that("dear_func calculates Discretionary Energy Affordability Ratio", {
  # DEAR = (G - S) / G
  # This is the same as NEB (net energy burden)
  expect_equal(dear_func(50000, 5000), 0.9)
  expect_equal(dear_func(100000, 10000), 0.9)
})

test_that("all metrics handle same input consistently", {
  income <- 75000
  cost <- 6000

  # Calculate all metrics
  eb <- energy_burden_func(income, cost)
  ner <- ner_func(income, cost)
  eroi <- eroi_func(income, cost)
  neb <- neb_func(income, cost)
  dear <- dear_func(income, cost)

  # Verify relationships
  expect_equal(eb, cost / income)
  expect_equal(ner, (income - cost) / cost)
  expect_equal(eroi, income / cost)
  expect_equal(neb, cost / income)  # neb_func is alias for energy_burden_func
  expect_equal(dear, (income - cost) / income)  # DEAR is net energy burden

  # Verify mathematical relationships
  expect_equal(eroi, ner + 1)
  expect_equal(eb + dear, 1)  # Energy burden + DEAR = 1
  expect_equal(eb, 1 / eroi)
  expect_equal(neb, eb)  # neb_func is alias for energy_burden_func
})

test_that("metrics work with vectors", {
  incomes <- c(50000, 60000, 70000)
  costs <- c(5000, 6000, 7000)

  ner_values <- ner_func(incomes, costs)

  expect_equal(length(ner_values), 3)
  expect_equal(ner_values[1], 9)
  expect_equal(ner_values[2], 9)
  expect_equal(ner_values[3], 9)
})

test_that("vectorized calculations match element-wise", {
  incomes <- c(50000, 100000, 25000)
  costs <- c(5000, 10000, 5000)

  # Vectorized
  ner_vec <- ner_func(incomes, costs)

  # Element-wise
  ner_elem <- c(
    ner_func(incomes[1], costs[1]),
    ner_func(incomes[2], costs[2]),
    ner_func(incomes[3], costs[3])
  )

  expect_equal(ner_vec, ner_elem)
})

test_that("metrics preserve NA values appropriately", {
  incomes <- c(50000, NA, 70000)
  costs <- c(5000, 6000, NA)

  ner_values <- ner_func(incomes, costs)

  expect_true(is.na(ner_values[2]))  # NA income -> NA NER
  expect_true(is.na(ner_values[3]))  # NA cost -> NA NER
  expect_false(is.na(ner_values[1]))  # Valid inputs -> valid NER
})

test_that("extreme values don't cause overflow", {
  # Very large income
  large_income <- 1e12
  cost <- 1000

  ner <- ner_func(large_income, cost)
  expect_true(is.finite(ner))
  expect_gt(ner, 0)

  # Very small income
  small_income <- 0.01
  small_cost <- 0.001

  ner_small <- ner_func(small_income, small_cost)
  expect_true(is.finite(ner_small))
})

test_that("energy burden never exceeds 1 for positive inputs", {
  # Unless cost > income
  incomes <- c(50000, 100000, 25000)
  costs <- c(5000, 10000, 5000)

  eb_values <- energy_burden_func(incomes, costs)

  expect_true(all(eb_values <= 1))
  expect_true(all(eb_values >= 0))
})

test_that("energy burden can exceed 1 when cost > income", {
  # Cost exceeds income
  eb <- energy_burden_func(10000, 15000)
  expect_gt(eb, 1)
  expect_equal(eb, 1.5)
})

test_that("NER is negative when cost exceeds income", {
  # NER = (G - S) / S
  # When S > G, numerator is negative
  ner <- ner_func(10000, 15000)
  expect_lt(ner, 0)
  expect_equal(ner, -5000 / 15000)
})

test_that("metrics handle recycling correctly", {
  # Single income, multiple costs
  income <- 50000
  costs <- c(2000, 5000, 10000)

  ner_values <- ner_func(income, costs)

  expect_equal(length(ner_values), 3)
  expect_equal(ner_values[1], (50000 - 2000) / 2000)
  expect_equal(ner_values[2], (50000 - 5000) / 5000)
  expect_equal(ner_values[3], (50000 - 10000) / 10000)
})

test_that("calculate_weighted_metrics works with sample data", {
  data <- create_sample_lead_data(n = 100)

  # Test that we can calculate metrics without errors
  expect_silent({
    data$ner <- ner_func(data$income, data$energy_cost)
    data$energy_burden <- energy_burden_func(data$income, data$energy_cost)
  })

  # Check that all values are finite (for valid data)
  valid_rows <- data$income > 0 & data$energy_cost > 0
  expect_true(all(is.finite(data$ner[valid_rows])))
  expect_true(all(is.finite(data$energy_burden[valid_rows])))
})

test_that("poverty rate calculation based on NER threshold", {
  data <- create_sample_lead_data(n = 200)
  data$ner <- ner_func(data$income, data$energy_cost)

  # Count households below poverty threshold (NER < 9)
  poverty_count <- sum(data$ner < 9, na.rm = TRUE)
  total_count <- sum(!is.na(data$ner))

  poverty_rate <- poverty_count / total_count

  # Poverty rate should be between 0 and 1
  expect_gte(poverty_rate, 0)
  expect_lte(poverty_rate, 1)

  # With random data, should have some variation
  expect_gt(poverty_count, 0)  # Should have some in poverty
  expect_lt(poverty_count, total_count)  # Not all in poverty
})

test_that("energy_burden_func calculates correctly", {
  # Basic calculation
  expect_equal(energy_burden_func(50000, 3000), 0.06)
  expect_equal(energy_burden_func(100000, 4000), 0.04)

  # Vector inputs
  income <- c(50000, 75000, 100000)
  spending <- c(3000, 4500, 5000)
  result <- energy_burden_func(income, spending)
  expect_equal(result, c(0.06, 0.06, 0.05))

  # Zero income should give Inf
  expect_equal(energy_burden_func(0, 1000), Inf)

  # Zero spending should give 0
  expect_equal(energy_burden_func(50000, 0), 0)
})

test_that("ner_func calculates correctly", {
  # Basic calculation: (50000-3000)/3000 = 15.67
  expect_equal(ner_func(50000, 3000), (50000 - 3000) / 3000)

  # Verify relationship with energy burden
  g <- 50000
  s <- 3000
  nh <- ner_func(g, s)
  eb <- 1 / (nh + 1)
  expect_equal(eb, s / g)

  # Energy poverty line: 6% burden corresponds to Nh = 15.67
  nh_poverty <- ner_func(1, 0.06)
  expect_equal(round(nh_poverty, 2), 15.67)

  # Vector inputs
  income <- c(50000, 75000, 100000)
  spending <- c(3000, 4500, 5000)
  result <- ner_func(income, spending)
  expect_length(result, 3)
  expect_true(all(result > 0))
})

test_that("eroi_func calculates correctly", {
  # EROI = G / S
  expect_equal(eroi_func(50000, 3000), 50000 / 3000)
  expect_equal(eroi_func(100000, 4000), 25)

  # Relationship: EROI = Nh + 1
  g <- 50000
  s <- 3000
  nh <- ner_func(g, s)
  eroi <- eroi_func(g, s)
  expect_equal(eroi, nh + 1)
})

test_that("dear_func calculates correctly", {
  # DEAR = (G - S) / G
  expect_equal(dear_func(50000, 3000), (50000 - 3000) / 50000)
  expect_equal(dear_func(100000, 10000), 0.9)

  # Relationship with energy burden: DEAR = 1 - E_b
  g <- 50000
  s <- 3000
  eb <- energy_burden_func(g, s)
  dear <- dear_func(g, s)
  expect_equal(dear, 1 - eb)
})

test_that("energy metrics handle effective spending parameter", {
  g <- 50000
  s <- 3000
  se <- 2500 # Different effective spending

  # With se parameter
  eb_with_se <- energy_burden_func(g, s, se)
  nh_with_se <- ner_func(g, s, se)

  # Should use se in calculations
  expect_equal(eb_with_se, s / g) # Still uses s for numerator
  expect_equal(nh_with_se, (g - s) / se) # Uses se for denominator
})

test_that("energy metrics handle edge cases", {
  # NA values
  expect_true(is.na(energy_burden_func(NA, 1000)))
  expect_true(is.na(ner_func(50000, NA)))

  # Negative values (unrealistic but should compute)
  expect_true(energy_burden_func(50000, -1000) < 0)
})

test_that("neb_func works without aggregation (backwards compatible)", {
  # Individual household - should return vector identical to EB
  expect_equal(neb_func(50000, 3000), 0.06)
  expect_equal(neb_func(50000, 3000), energy_burden_func(50000, 3000))

  # Multiple households without weights - should return vector
  incomes <- c(30000, 50000, 75000)
  spending <- c(3000, 3500, 4000)
  result <- neb_func(incomes, spending)

  expect_length(result, 3)
  expect_equal(result, spending / incomes)
  expect_equal(result, energy_burden_func(incomes, spending))
})

test_that("neb_func aggregates correctly with weights (Nh method)", {
  # Test data
  incomes <- c(30000, 50000, 75000)
  spending <- c(3000, 3500, 4000)
  households <- c(100, 150, 200)

  # Calculate via neb_func with weights
  neb_aggregated <- neb_func(incomes, spending, weights = households)

  # Verify it's a single value
  expect_length(neb_aggregated, 1)

  # Verify it uses Nh method internally
  nh <- ner_func(incomes, spending)
  nh_mean <- weighted.mean(nh, households)
  neb_expected <- 1 / (1 + nh_mean)
  expect_equal(neb_aggregated, neb_expected)

  # Verify it avoids naive averaging error
  neb_naive <- weighted.mean(spending / incomes, households)
  expect_false(isTRUE(all.equal(neb_aggregated, neb_naive)))
  # Error should be small but present (1-5%)
  error_pct <- abs(neb_naive - neb_aggregated) / neb_aggregated * 100
  expect_gt(error_pct, 0.1)  # At least 0.1% difference
})

test_that("neb_func aggregates without weights when aggregate=TRUE", {
  incomes <- c(30000, 50000, 75000)
  spending <- c(3000, 3500, 4000)

  # Aggregate without weights
  neb_agg <- neb_func(incomes, spending, aggregate = TRUE)

  # Should return single value
  expect_length(neb_agg, 1)

  # Should use unweighted mean of Nh
  nh <- ner_func(incomes, spending)
  nh_mean <- mean(nh)
  neb_expected <- 1 / (1 + nh_mean)
  expect_equal(neb_agg, neb_expected)
})

test_that("neb_func handles effective spending with aggregation", {
  incomes <- c(30000, 50000)
  spending <- c(3000, 3500)
  se <- c(2500, 3000)  # Effective spending
  weights <- c(100, 150)

  # With weights and effective spending
  neb_agg <- neb_func(incomes, spending, se = se, weights = weights)

  # Should use se in Nh calculation
  nh <- ner_func(incomes, spending, se)
  nh_mean <- weighted.mean(nh, weights)
  neb_expected <- 1 / (1 + nh_mean)
  expect_equal(neb_agg, neb_expected)
})

test_that("neb_func demonstrates Nh method superiority", {
  # Realistic income distribution
  incomes <- c(25000, 35000, 50000, 75000, 100000)
  spending <- c(3500, 3800, 4000, 4500, 5000)
  households <- c(200, 300, 250, 150, 100)

  # CORRECT: Via neb_func (uses Nh method)
  neb_correct <- neb_func(incomes, spending, weights = households)

  # WRONG: Naive weighted mean
  neb_naive <- weighted.mean(spending / incomes, households)

  # Should be different
  expect_false(isTRUE(all.equal(neb_correct, neb_naive, tolerance = 0.001)))

  # Verify the correct method gives reasonable value
  expect_gt(neb_correct, 0)
  expect_lt(neb_correct, 1)

  # Verify relationship: correct = 1/(1 + weighted.mean(Nh))
  nh <- ner_func(incomes, spending)
  nh_mean <- weighted.mean(nh, households)
  expect_equal(neb_correct, 1 / (1 + nh_mean))
})

test_that("neb_func handles NA values in aggregation", {
  incomes <- c(30000, NA, 75000)
  spending <- c(3000, 3500, 4000)
  weights <- c(100, 150, 200)

  # With weights - should handle NA gracefully
  result <- neb_func(incomes, spending, weights = weights)
  expect_false(is.na(result))  # Should compute with non-NA values
  expect_length(result, 1)
})

test_that("neb_func with weights matches manual Nh aggregation (documentation test)", {
  # This test verifies examples from documentation work correctly
  # Simulates cohort data (pre-aggregated means)
  mean_incomes <- c(30000, 50000, 75000)
  mean_spending <- c(3000, 3500, 4000)
  households <- c(100, 150, 200)

  # Manual Nh method (documented approach)
  nh <- ner_func(mean_incomes, mean_spending)
  nh_mean <- weighted.mean(nh, households)
  neb_manual <- 1 / (1 + nh_mean)

  # Using neb_func() with weights (new approach)
  neb_auto <- neb_func(mean_incomes, mean_spending, weights = households)

  # Should be identical
  expect_equal(neb_auto, neb_manual)

  # Verify both differ from naive averaging (which is wrong)
  neb_naive <- weighted.mean(mean_spending / mean_incomes, households)
  expect_false(isTRUE(all.equal(neb_auto, neb_naive, tolerance = 0.001)))
})

test_that("neb_func aggregation works in dplyr workflows", {
  # Simulate grouped aggregation as in vignettes
  library(dplyr)

  cohort_data <- data.frame(
    group = rep(c("A", "B"), each = 3),
    mean_income = c(30000, 50000, 75000, 40000, 60000, 90000),
    mean_spending = c(3000, 3500, 4000, 3200, 3800, 4500),
    households = c(100, 150, 200, 120, 180, 150)
  )

  # Method 1: Manual Nh aggregation
  result_manual <- cohort_data %>%
    group_by(group) %>%
    summarise(
      nh_mean = weighted.mean(ner_func(mean_income, mean_spending), households),
      neb = 1 / (1 + nh_mean),
      .groups = "drop"
    )

  # Method 2: Using neb_func() with weights
  result_auto <- cohort_data %>%
    group_by(group) %>%
    summarise(
      neb = neb_func(mean_income, mean_spending, weights = households),
      .groups = "drop"
    )

  # Should give identical results
  expect_equal(result_auto$neb, result_manual$neb)
})

---
title: "emburden: Temporal Analysis of Household Energy Burden Using Net Energy Return Metrics"
author: "Eric Scheier"
date: "`r Sys.Date()`"
output:
  html_document:
    toc: true
    toc_depth: 3
bibliography: references.bib
vignette: >
  %\VignetteIndexEntry{emburden: Temporal Energy Burden Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteBuilder{knitr}
---

# Abstract

Energy burden---the proportion of household income spent on energy---is a critical metric for understanding energy poverty and inequity. However, traditional energy burden ratios present analytical challenges including difficulties with aggregation and visualization of extreme values. The emburden package for R implements Net Energy Return (Nh) methodology to address these limitations while enabling temporal analysis of household energy characteristics. This paper introduces the package's design and demonstrates its application to comparing Low-Income Energy Affordability Data (LEAD) Tool vintages from 2018 and 2022 across geographic and demographic dimensions. The package provides functions for downloading, processing, and analyzing census tract-level energy burden data for all U.S. states, with particular attention to proper weighted aggregation and schema normalization across data vintages. We demonstrate the package's capabilities through examples ranging from state-level summaries to fine-grained census tract comparisons, illustrating how policy-relevant insights can be extracted at multiple scales.

# Introduction

Household energy affordability is a persistent challenge affecting millions of households in the United States. Low-income households face disproportionate energy burdens, often spending more than 6% of their income on energy costs compared to 2-3% for higher-income households [@ross2018high; @drehobl2016lifting]. Understanding these disparities and tracking changes over time is essential for designing effective energy assistance programs and policies.

The traditional energy burden metric---the ratio of energy expenditures ($S$) to gross income ($G$)---has several analytical limitations. As a ratio with income in the denominator, energy burden ($E_b = S/G$) approaches infinity for households with very low incomes, creating challenges for aggregation and visualization. Additionally, the metric requires harmonic mean aggregation rather than arithmetic means, which is not widely understood or consistently applied [@scheier2022measurement].

## Mathematical foundations

The `emburden` package for R addresses these challenges by implementing Net Energy Return (NER) methodology, adapted from macro-energy systems analysis [@hall2011eroi; @brandtcalculating2013; @carbajalesdale2014better]. Net energy analysis estimates the net energy return of a process as a relationship between gross resources extracted and embodied energy directed toward extraction:

$$G = Gross\ Resource\ Extracted$$

$$S = Spending\ on\ Extraction\ Process$$

$$Net\ Energy\ Return\ (NER) = \frac{G - S}{S}$$

For households extracting income from the economy, these ratios become:

$$G_{income} = Gross\ Income$$

$$S_{energy} = Spending\ on\ Energy$$

$$NER_{household} = \frac{G_{income} - S_{energy}}{S_{energy}}$$

This metric represents the net earnings a household receives for every dollar of expenditure on secondary energy. For notational simplicity, we use $N_h$ to denote household Net Energy Return throughout this paper, where $N_h = NER_{household}$.

### Comparison with energy burden

Energy burden, the traditional metric in energy poverty analysis, is defined as:

$$Energy\ Burden = E_b = \frac{S_{energy}}{G_{income}}$$

While energy burden is intuitive as a percentage, it has several mathematical limitations. The Net Energy Return transformation addresses these by preventing double-counting of energy expenditures (income in the numerator already includes the portion spent on energy) and enabling proper weighted mean aggregation:

$$\overline{N_h} = \frac{\sum (N_h \times households)}{\sum households}$$

In contrast, energy burden requires harmonic mean aggregation:

$$\overline{E_b} = \frac{1}{\overline{1/E_b}}$$

The two metrics are mathematically related through the transformation $E_b = 1/(N_h + 1)$, allowing seamless conversion between representations.

### Energy poverty threshold

Energy poverty is commonly defined as spending greater than 10% of household income on energy [@bednarrecognition2020]:

$$E_b^{*} = \frac{S_{energy}}{G_{income}} > 10\%$$

Translated to Net Energy Return, the energy poverty threshold becomes:

$$N_h^{*} < 9: Household\ at\ Energy\ Poverty\ Line$$

This means a household earning less than \$9 of income for every dollar spent on secondary energy is considered to be in energy poverty by the traditional energy burden accounting method. A Net Energy Return of 9 or lower is equivalent to an energy burden of 10% or higher. While this threshold is somewhat arbitrary and may not be suitable in all situations, it provides a useful benchmark for comparing results to the energy poverty literature.

## The LEAD Tool and temporal analysis

The U.S. Department of Energy's Low-Income Energy Affordability Data (LEAD) Tool [@ma2019lowincome] provides census tract-level estimates of household energy characteristics based on American Community Survey microdata. The tool uses iterative proportional fitting to allocate households to census tracts while calibrating to utility-reported sales and revenues.

Multiple vintages of LEAD Tool data have been released:

- **2018 Update**: Based on 2018 5-year ACS data, released July 2020
- **2022 Update**: Based on 2022 5-year ACS data, released August 2024

These vintages enable temporal analysis of energy burden trends, but require careful handling of schema differences and income bracket definitions.

## Package design philosophy

The `emburden` package is designed around several key principles:

1. **Proper aggregation**: Implements weighted mean aggregation using Net Energy Return, with household counts as weights
2. **Temporal consistency**: Normalizes schema differences between LEAD Tool vintages to enable valid comparisons
3. **Flexible workflows**: Supports both database and CSV-based data access with automatic fallback
4. **Geographic flexibility**: Enables analysis from national level down to individual census tracts

# Methodology

## Data sources

The `emburden` package provides access to three primary datasets for household energy burden analysis:

### LEAD Tool

The Low-Income Energy Affordability Data (LEAD) Tool [@ma2019lowincome] portrays average income, electricity expenditures, gas expenditures, and other fuel expenditures for cohorts of households segmented by location (census tract, county, state) and household characteristics (ownership status, building age, number of units, attachment status, primary heating fuel).

The dataset is assembled using iterative proportional fitting (IPF), a widely used spatial microsimulation method to allocate households to census tracts while calibrating characteristics to known quantities. The IPF algorithm processes cross-tabulations of household responses from the American Community Survey (ACS) Public Use Microdata Samples, scaling them to match aggregate annual values from utility sales and revenues reported in Energy Information Administration forms 861 (electricity) and 176 (natural gas).

Multiple vintages are available:

- **2018 Update**: Based on 2016 5-year ACS data (2012-2016), released July 2020
- **2022 Update**: Based on 2018 5-year ACS data (2014-2018), released August 2024

### REPLICA dataset

The Renewable Energy Potential of Low-Income Communities in America (REPLICA) dataset [@sigrinRooftopSolarTechnical2018] adds technical rooftop solar potential and additional techno-economic variables including demographics and electricity rates. The package can merge REPLICA data with LEAD data to enrich analyses with utility type, locale classification, and solar generation potential.

### Schema normalization across vintages

A critical challenge in temporal analysis is handling schema differences between LEAD Tool vintages. The package implements automatic normalization through the following transformations:

**Income bracket aggregation**: The LEAD Tool provides income as a fraction of Area Median Income (AMI) or Federal Poverty Level (FPL). For AMI data, the package can aggregate detailed brackets into simplified categories matching the REPLICA schema:

- 0-30% AMI: Very Low Income
- 30-80% AMI: Low-to-Moderate Income
- 80%+ AMI: Middle-to-High Income

For FPL data, the aggregation follows poverty line definitions:

- 0-100% FPL: In Poverty
- 100%+ FPL: Not In Poverty

**Building type simplification**: Housing units are classified as:

- 1 Unit: Single-Family
- >1 Unit: Multi-Family
- Other Unit: Excluded from analysis

These normalizations enable valid temporal comparisons despite underlying schema evolution between vintages.

## Data processing

The package processes raw LEAD Tool data through several stages:

### Energy burden indicator calculation

For each household cohort, the package calculates:

$$s = electricity + natural\ gas + other\ fuels$$

$$g = annual\ household\ income$$

From these base metrics, all energy burden indicators are derived using the formulas presented in Section 1.1.

### Weighted aggregation

The package implements proper weighted aggregation using household counts as weights. For Net Energy Return:

```{r, eval=FALSE}
calculate_weighted_metrics(
  data,
  group_columns = c("state", "income_bracket"),
  metric_name = "ner"
)
```

This function:

1. Filters data to specified groups
2. Calculates weighted means using household counts
3. Computes poverty rates below specified thresholds
4. Returns summary statistics including quantiles and standard deviations

The key insight is that Net Energy Return allows arithmetic weighted means, while energy burden would require harmonic mean aggregation---a distinction that significantly impacts the validity and interpretability of aggregate statistics.

### Data quality considerations

Iterative proportional fitting has limitations as an estimation procedure. The relationship between constraint variables tends toward the average of the initializing dataset, potentially depressing variations among otherwise similar regions. This may explain the large quantities of households estimated to have very low incomes. Validating these estimated data would require randomized surveys along the dimensions of interest.

Additionally, the "primary heating fuel" category derives from the ACS question "Which fuel is used most for heating this house, apartment, or mobile home?" The predictive power of this question for energy expenditures is not fully understood and warrants caution in interpretation.

Though REPLICA relies on a different LEAD vintage (2017) than recent analyses (2019, 2022), the package still enables useful cross-dataset analysis. However, inferring differences among annual estimates should account for the standard error of the data [@ma2019lowincome]. Rigorous temporal analysis benefits from comparing identically-processed vintages.

# Package architecture

The `emburden` package is organized into several functional modules:

## Core functions

```{r, eval=FALSE}
library(emburden)

# Energy metric calculations
energy_burden_func(gross_income, energy_spending)
ner_func(gross_income, energy_spending)  # Net Energy Return
eroi_func(gross_income, energy_spending)  # EROI
dear_func(gross_income, energy_spending)  # DEAR

# Statistical aggregation
calculate_weighted_metrics(
  graph_data,
  group_columns = "state",
  metric_name = "ner"
)
```

## Data loading functions

The package provides automatic data downloading and caching:

```{r, eval=FALSE}
# Load census tract data (auto-downloads if not available)
nc_tracts <- load_census_tract_data(states = "NC")

# Load cohort data by income bracket
nc_ami <- load_cohort_data(
  dataset = "ami",
  states = "NC",
  vintage = "2022"
)

# Compare vintages
comparison <- compare_energy_burden(
  dataset = "ami",
  states = "NC",
  group_by = "state"
)
```

# Analysis examples

The `emburden` package's primary contribution is enabling temporal analysis of energy burden through proper schema normalization and aggregation. This section demonstrates the package's capabilities through progressively detailed examples.

## Temporal comparison workflow

The `compare_energy_burden()` function provides the core temporal analysis functionality:

```{r basic-comparison, eval=FALSE}
library(emburden)

# Compare North Carolina energy burden: 2018 vs 2022
nc_comparison <- compare_energy_burden(
  dataset = "ami",
  states = "NC",
  group_by = "income_bracket"
)

# View formatted comparison table
print(nc_comparison)
```

The function automatically:

1. Downloads both vintages if not cached locally
2. Normalizes schema differences between vintages
3. Performs proper $N_h$-based weighted aggregation
4. Calculates energy burden for both periods
5. Computes changes in percentage points

### Understanding the output

The comparison object contains multiple metrics:

```{r comparison-metrics, eval=FALSE}
# Energy burden in 2018 and 2022
nc_comparison$neb_2018
nc_comparison$neb_2022

# Change in energy burden (percentage points)
nc_comparison$neb_change_pp

# Net Energy Return values
nc_comparison$ner_2018
nc_comparison$ner_2022

# Household counts
nc_comparison$households_2018
nc_comparison$households_2022
```

## Example 1: State-level temporal analysis

To examine overall state changes without grouping by demographic characteristics:

```{r state-level, eval=FALSE}
# Overall state comparison
nc_state <- compare_energy_burden(
  dataset = "ami",
  states = "NC",
  group_by = "none"
)

# Extract key findings
cat(sprintf(
  "North Carolina energy burden changed from %.1f%% (2018) to %.1f%% (2022)\n",
  nc_state$neb_2018 * 100,
  nc_state$neb_2022 * 100
))

cat(sprintf(
  "Change: %+.2f percentage points\n",
  nc_state$neb_change_pp * 100
))
```

## Example 2: Income bracket analysis

Disaggregating by income bracket reveals which populations experienced the largest changes:

```{r income-bracket, eval=FALSE}
# Compare by income bracket
nc_income <- compare_energy_burden(
  dataset = "ami",
  states = "NC",
  group_by = "income_bracket"
)

# Visualize changes
library(ggplot2)

ggplot(nc_income, aes(x = income_bracket, y = neb_change_pp * 100)) +
  geom_col(fill = "steelblue") +
  geom_hline(yintercept = 0, linetype = "dashed") +
  labs(
    title = "Change in Energy Burden by Income Bracket",
    subtitle = "North Carolina, 2018 to 2022",
    x = "Income Bracket (% of Area Median Income)",
    y = "Change in Energy Burden (percentage points)"
  ) +
  theme_minimal()
```

Typical findings show that very low-income households (0-30% AMI) experience the highest energy burdens and are most vulnerable to changes in energy costs or income levels.

## Example 3: Multi-state comparison

Comparing multiple states reveals regional patterns and policy impacts:

```{r multi-state, eval=FALSE}
# Compare Southern states
southern_states <- compare_energy_burden(
  dataset = "ami",
  states = c("NC", "SC", "GA", "FL"),
  group_by = "state"
)

# Which states improved most?
southern_states %>%
  arrange(neb_change_pp) %>%
  select(state_abbr, neb_2018, neb_2022, neb_change_pp)

# Visualize state comparison
ggplot(southern_states, aes(x = reorder(state_abbr, neb_2022),
                             y = neb_2022 * 100)) +
  geom_col(fill = "darkgreen") +
  geom_point(aes(y = neb_2018 * 100), color = "red", size = 3) +
  labs(
    title = "Energy Burden by State: 2022 (bars) vs 2018 (points)",
    x = "State",
    y = "Energy Burden (%)"
  ) +
  theme_minimal()
```

## Example 4: Housing tenure analysis

Energy burden often varies significantly between renters and homeowners:

```{r housing-tenure, eval=FALSE}
# Compare by housing tenure
nc_tenure <- compare_energy_burden(
  dataset = "ami",
  states = "NC",
  group_by = "housing_tenure"
)

# Calculate the renter-owner gap
gap_2018 <- nc_tenure$neb_2018[nc_tenure$housing_tenure == "RENTER"] -
            nc_tenure$neb_2018[nc_tenure$housing_tenure == "OWNER"]

gap_2022 <- nc_tenure$neb_2022[nc_tenure$housing_tenure == "RENTER"] -
            nc_tenure$neb_2022[nc_tenure$housing_tenure == "OWNER"]

cat(sprintf(
  "Renter-Owner energy burden gap: %.2f pp (2018) → %.2f pp (2022)\n",
  gap_2018 * 100,
  gap_2022 * 100
))
```

Renters typically face higher energy burdens due to split-incentive problems where landlords make efficiency investment decisions but tenants pay energy bills.

## Example 5: Federal Poverty Line analysis

For policy applications targeting households below the federal poverty line:

```{r fpl-analysis, eval=FALSE}
# Use FPL dataset instead of AMI
nc_fpl <- compare_energy_burden(
  dataset = "fpl",
  states = "NC",
  group_by = "income_bracket"
)

# Compare poverty vs non-poverty households
nc_fpl %>%
  filter(income_bracket %in% c("Below Federal Poverty Line",
                                "Above Federal Poverty Line")) %>%
  select(income_bracket, neb_2018, neb_2022, neb_change_pp)
```

This analysis is particularly relevant for programs like the Low-Income Home Energy Assistance Program (LIHEAP) which target households below specific poverty thresholds.

## Example 6: Census tract-level analysis

For fine-grained spatial analysis, load tract-level data directly:

```{r tract-level, eval=FALSE}
# Load 2022 census tract data
nc_tracts_2022 <- load_census_tract_data(
  states = "NC",
  vintage = "2022"
)

# Calculate county-level statistics
nc_counties <- calculate_weighted_metrics(
  nc_tracts_2022,
  group_columns = "county_name",
  metric_name = "ner"
)

# Identify counties with highest energy burden
nc_counties %>%
  mutate(energy_burden = 1 / (ner + 1)) %>%
  arrange(desc(energy_burden)) %>%
  head(10) %>%
  select(county_name, energy_burden, household_count)
```

Census tract data enables spatial analysis and mapping applications, revealing urban-rural disparities and identifying communities in need of targeted assistance.

# Discussion

## Policy implications

The ability to track energy burden changes over time has important policy implications. Programs like LIHEAP (Low-Income Home Energy Assistance Program) and WAP (Weatherization Assistance Program) target households experiencing energy insecurity, but evaluating their effectiveness requires robust temporal analysis.

The `emburden` package enables researchers and policymakers to:

1. **Track program impacts**: Compare energy burden before and after policy interventions
2. **Identify vulnerable populations**: Disaggregate trends by income, tenure, and geography
3. **Allocate resources effectively**: Target communities with worsening energy affordability
4. **Benchmark across jurisdictions**: Compare state and local policy outcomes

### Split-incentive and principal-agent problems

A persistent challenge in energy equity is the split-incentive problem: landlords make energy efficiency investment decisions, but tenants pay the energy bills. This misalignment of incentives leads to underinvestment in efficiency improvements for rental properties.

The package's ability to analyze energy burden by housing tenure reveals the magnitude of this problem:

```{r split-incentive-example, eval=FALSE}
# Quantify the renter-owner gap
tenure_comparison <- compare_energy_burden(
  dataset = "ami",
  states = "all",  # National analysis
  group_by = "housing_tenure"
)

# Calculate disparity
renter_burden <- tenure_comparison$neb_2022[
  tenure_comparison$housing_tenure == "RENTER"
]
owner_burden <- tenure_comparison$neb_2022[
  tenure_comparison$housing_tenure == "OWNER"
]

disparity_ratio <- renter_burden / owner_burden
```

Addressing this gap requires policy interventions such as:

- On-bill financing programs
- Landlord incentive programs
- Energy efficiency standards for rental properties
- Community-scale renewable energy projects

## Data limitations and considerations

Users should be aware of several data limitations:

### Iterative proportional fitting constraints

The LEAD Tool uses IPF to allocate households to census tracts, which has important implications:

1. **Regression toward the mean**: IPF tends to depress variations among similar regions
2. **Estimation uncertainty**: Standard errors are substantial, especially for small cohorts
3. **Temporal comparability**: Different ACS vintages may have methodological differences

### Income measurement challenges

Household income as reported in the ACS has known limitations:

- **Underreporting**: Particularly for benefits and informal income
- **Timing**: Income is annual but energy costs vary seasonally
- **Household composition**: Per-capita income may be more relevant for some analyses

### Energy expenditure estimation

The "primary heating fuel" categorization derives from a single ACS question and may not fully capture:

- Mixed-fuel households
- Behavioral patterns
- Appliance efficiency variations
- Climate variations within states

Despite these limitations, the LEAD Tool represents the most comprehensive spatial dataset available for energy burden analysis in the United States.

## Future research directions

Several extensions would enhance the package's capabilities:

### Additional vintages

As DOE releases new LEAD Tool vintages (potentially 2024, 2026, etc.), the package can incorporate them to enable longer-term trend analysis. This would support:

- Multi-year trend identification
- Correlation with economic cycles
- Climate change impact assessment

### Additional metrics

The package currently implements Net Energy Return, EROI, and DEAR. Future versions could add:

- **Disposable income ratios**: Accounting for essential expenses beyond energy
- **Energy poverty depth**: How far below thresholds households fall
- **Vulnerability indices**: Combining burden with demographic risk factors

### Spatial analysis enhancements

Geographic extensions could include:

- Integration with climate zone data
- Utility service territory analysis
- Transportation energy burden incorporation
- Built environment characteristics

### Causal analysis tools

Methodological extensions for policy evaluation:

- Difference-in-differences estimation
- Synthetic control methods
- Regression discontinuity designs
- Propensity score matching

## Comparison with existing tools

Several tools exist for energy burden analysis, each with different strengths:

- **LEAD Tool web interface**: Interactive but limited temporal comparison
- **State energy office tools**: Customized but not standardized across states
- **Academic datasets**: Rich but often one-time snapshots
- **`emburden`**: Focused on temporal analysis with proper aggregation methodology

The `emburden` package fills a gap by providing programmatic access to multiple vintages with automated schema normalization, enabling reproducible temporal analyses at scale.

# Conclusion

The `emburden` package provides a robust framework for temporal analysis of household energy burden using proper Net Energy Return methodology. By automating data access, normalizing schema differences, and implementing correct aggregation methods, the package enables researchers and policymakers to track energy affordability trends across multiple scales.

Key contributions include:

1. **Mathematical foundations**: Proper Net Energy Return aggregation avoiding double-counting
2. **Temporal consistency**: Automated schema normalization across LEAD Tool vintages
3. **Flexible analysis**: Functions supporting national, state, county, and tract-level analysis
4. **Policy relevance**: Direct support for energy assistance program evaluation

The package is available from GitHub at \url{https://github.com/ericscheier/emburden} and is licensed under AGPL-3+. Documentation, vignettes, and issue tracking are available through the package website.

# References

---
title: "Net Energy Return Methodology"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Net Energy Return Methodology}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(emburden)
library(dplyr)
```

## Introduction

This vignette explains the mathematical foundations of the Net Energy Return (Nh) methodology for analyzing household energy burden. We'll show why this approach is both theoretically sound and computationally advantageous for aggregation across households.

## The Problem: Aggregating Ratios

Energy burden is defined as the ratio of energy spending to gross income:

**Energy Burden (EB) = S / G**

Where:
- **S** = Total energy spending (electricity + gas + other fuels)
- **G** = Gross household income

While this definition is straightforward for a single household, **aggregating energy burden across multiple households is not trivial**.

### Why Arithmetic Mean Fails

Consider three households:

```{r simple-example}
# Three households with different income/spending patterns
households <- data.frame(
  id = 1:3,
  income = c(30000, 50000, 100000),
  spending = c(3000, 3500, 4000)
)

households$eb <- energy_burden_func(households$income, households$spending)
print(households)
```

What is the "average" energy burden across these three households?

```{r wrong-aggregation}
# Attempt 1: Simple arithmetic mean (WRONG!)
mean_eb_wrong <- mean(households$eb)
print(paste("Simple mean EB:", scales::percent(mean_eb_wrong)))
```

This is incorrect because it treats all households equally, ignoring that they represent different amounts of total income and spending.

```{r weighted-wrong}
# Attempt 2: Weighted arithmetic mean (STILL WRONG!)
# Let's weight by number of households (all equal here, but principle matters)
weights <- c(100, 150, 200)  # Different household counts

eb_arithmetic_mean <- weighted.mean(households$eb, weights)
print(paste("Weighted arithmetic mean EB:", scales::percent(eb_arithmetic_mean)))
```

**Why is this still wrong?** Energy burden is a ratio, and ratios cannot be aggregated using arithmetic means. The correct approach requires the **harmonic mean**.

### The Correct Approach: Harmonic Mean

For ratios like energy burden, the mathematically correct aggregation uses the harmonic mean:

```{r harmonic-mean}
# Correct aggregation: Weighted harmonic mean
eb_harmonic <- 1 / weighted.mean(1 / households$eb, weights)
print(paste("Weighted harmonic mean EB:", scales::percent(eb_harmonic)))

# Verify by calculating from totals
total_spending <- sum(households$spending * weights)
total_income <- sum(households$income * weights)
eb_from_totals <- total_spending / total_income
print(paste("EB from totals:", scales::percent(eb_from_totals)))

# These should be identical
print(paste("Difference:", abs(eb_harmonic - eb_from_totals)))
```

The harmonic mean gives the same result as calculating energy burden from the aggregated totals, which is what we want!

## The Solution: Net Energy Return (Nh)

While the harmonic mean is correct, it has some practical drawbacks:
1. **Computational complexity**: Requires division by each individual EB value
2. **Numerical instability**: Very small EB values (e.g., 0.01) become very large (100) after inversion
3. **Less intuitive**: Harmonic means are less familiar to most analysts
4. **Error-prone**: Easy to accidentally use arithmetic mean instead

The Net Energy Return (Nh) transformation solves these issues.

### Mathematical Relationship

Net Energy Return is defined as:

**Nh = (G - S) / S**

This represents "net energy return per unit of energy spending" - similar to concepts in biophysical economics.

**Key insight**: Nh can be aggregated using simple arithmetic mean, then converted back to energy burden!

Let's verify the mathematical relationship:

```{r nh-identity}
# Starting from Nh = (G - S) / S
# Let's solve for EB = S / G

# Nh = (G - S) / S
# Nh = G/S - S/S
# Nh = G/S - 1
# Nh + 1 = G/S
# 1 / (Nh + 1) = S/G = EB

# Therefore: EB = 1 / (Nh + 1)

# Verify with our example
households$nh <- ner_func(households$income, households$spending)
households$eb_from_nh <- 1 / (households$nh + 1)

# Compare to original EB
households$identical <- all.equal(households$eb, households$eb_from_nh)
print(households[, c("id", "eb", "eb_from_nh", "nh")])
```

### Aggregation via Arithmetic Mean

Now we can aggregate using simple arithmetic mean:

```{r nh-aggregation}
# Step 1: Calculate Nh for each household
nh_values <- ner_func(households$income, households$spending)

# Step 2: Arithmetic weighted mean (simple!)
nh_mean <- weighted.mean(nh_values, weights)
print(paste("Weighted mean Nh:", round(nh_mean, 2)))

# Step 3: Convert back to EB
eb_from_nh <- 1 / (nh_mean + 1)
print(paste("EB from Nh method:", scales::percent(eb_from_nh)))

# Compare to harmonic mean result
print(paste("EB from harmonic mean:", scales::percent(eb_harmonic)))
print(paste("Difference:", abs(eb_from_nh - eb_harmonic)))
```

**Result**: Both methods give identical results, but the Nh method uses simple arithmetic mean instead of harmonic mean!

## Why This Works: The Mathematics

The key mathematical insight is that the Nh transformation linearizes the aggregation problem.

For a group of households with weights $w_i$, incomes $G_i$, and spending $S_i$:

**Harmonic mean approach**:
$$\text{EB}_{\text{agg}} = \frac{1}{\sum_i w_i \cdot \frac{1}{\text{EB}_i}} = \frac{1}{\sum_i w_i \cdot \frac{G_i}{S_i}}$$

**Nh approach**:
$$\text{Nh}_{\text{mean}} = \sum_i w_i \cdot \text{Nh}_i = \sum_i w_i \cdot \frac{G_i - S_i}{S_i}$$

$$\text{EB}_{\text{agg}} = \frac{1}{1 + \text{Nh}_{\text{mean}}}$$

Both formulations are mathematically equivalent when computing from the same underlying data.

## Computational Advantages (For Aggregation)

The Nh method provides several advantages **when aggregating across multiple households**:

```{r computational-comparison}
# Simulate larger dataset
set.seed(42)
n <- 10000
large_data <- data.frame(
  income = rlnorm(n, meanlog = 10.8, sdlog = 0.8),  # Log-normal income distribution
  spending = NA
)
large_data$spending <- pmin(
  rlnorm(n, meanlog = 8.2, sdlog = 0.5),  # Log-normal spending
  large_data$income * 0.5  # Cap at 50% of income
)
weights <- sample(50:500, n, replace = TRUE)

# Method 1: Nh with arithmetic mean
system.time({
  nh <- ner_func(large_data$income, large_data$spending)
  nh_mean <- weighted.mean(nh, weights)
  eb_nh <- 1 / (1 + nh_mean)
})

# Method 2: Harmonic mean
system.time({
  eb_direct <- energy_burden_func(large_data$income, large_data$spending)
  eb_harmonic <- 1 / weighted.mean(1 / eb_direct, weights)
})

# Verify results are identical
print(paste("EB via Nh:", scales::percent(eb_nh)))
print(paste("EB via harmonic mean:", scales::percent(eb_harmonic)))
print(paste("Difference:", abs(eb_nh - eb_harmonic)))
```

**Note**: The computational advantage applies specifically to **aggregation across households**. For single household calculations, both methods are equivalent (EB = S/G = 1/(1+Nh)) and require the same basic operations.

### Numerical Stability

The Nh method is also more numerically stable:

```{r numerical-stability}
# Households with very low energy burden
low_burden <- data.frame(
  income = c(200000, 500000, 1000000),
  spending = c(2000, 3000, 5000)  # Very low spending relative to income
)

low_burden$eb <- energy_burden_func(low_burden$income, low_burden$spending)
low_burden$nh <- ner_func(low_burden$income, low_burden$spending)

print("Energy Burden (direct):")
print(low_burden$eb)

print("\nReciprocal of EB (used in harmonic mean):")
print(1 / low_burden$eb)  # Very large numbers!

print("\nNet Energy Return (Nh):")
print(low_burden$nh)  # More reasonable range
```

Very low energy burdens (e.g., 0.01 = 1%) become very large values (100) when inverted for harmonic mean calculation. The Nh values remain in a more reasonable range, reducing numerical precision issues.

## Error from Incorrect Aggregation

Let's quantify the error introduced by incorrectly using arithmetic mean on EB values:

```{r error-demonstration}
# Use realistic North Carolina-like data
set.seed(123)
n_households <- 5000

realistic_data <- data.frame(
  income_bracket = sample(
    c("0-30% AMI", "30-50% AMI", "50-80% AMI", "80-100% AMI", "100%+ AMI"),
    n_households,
    replace = TRUE,
    prob = c(0.15, 0.12, 0.20, 0.10, 0.43)
  ),
  income = rlnorm(n_households, meanlog = 10.8, sdlog = 0.8),
  households = sample(10:100, n_households, replace = TRUE)
)

realistic_data$spending <- realistic_data$income * rlnorm(
  n_households,
  meanlog = log(0.05),
  sdlog = 0.6
)

# Calculate metrics
realistic_data <- realistic_data %>%
  mutate(
    eb = energy_burden_func(income, spending),
    nh = ner_func(income, spending),
    neb = neb_func(income, spending)
  )

# WRONG: Arithmetic mean of EB
eb_wrong <- weighted.mean(realistic_data$eb, realistic_data$households)

# CORRECT: Via Nh
nh_mean <- weighted.mean(realistic_data$nh, realistic_data$households)
eb_correct <- 1 / (1 + nh_mean)

# Calculate error
absolute_error <- eb_wrong - eb_correct
relative_error_pct <- (absolute_error / eb_correct) * 100

cat(sprintf("WRONG (arithmetic mean EB): %.2f%%\n", eb_wrong * 100))
cat(sprintf("CORRECT (via Nh method):   %.2f%%\n", eb_correct * 100))
cat(sprintf("Absolute error:             %.4f\n", absolute_error))
cat(sprintf("Relative error:             %.2f%%\n", relative_error_pct))
```

The error from using arithmetic mean instead of proper aggregation is typically 1-5% in relative terms, which can be significant for policy analysis and equity assessments.

## Practical Workflow

Here's the recommended workflow for energy burden analysis:

```{r practical-workflow}
# Step 1: Calculate Nh for all observations
data_with_metrics <- realistic_data %>%
  mutate(
    nh = ner_func(income, spending),
    neb = neb_func(income, spending)  # Same as eb, but emphasizes proper aggregation
  )

# Step 2: Aggregate by groups using arithmetic weighted mean
by_bracket <- data_with_metrics %>%
  group_by(income_bracket) %>%
  summarise(
    total_households = sum(households),
    nh_mean = weighted.mean(nh, households),
    neb = 1 / (1 + nh_mean),  # Correct aggregation
    .groups = "drop"
  ) %>%
  arrange(desc(neb))

print(by_bracket)

# Step 3: Identify high-burden households
high_burden_threshold <- 0.06  # 6% energy burden threshold
nh_threshold <- (1 / high_burden_threshold) - 1  # = 15.67

high_burden_count <- sum(
  data_with_metrics$households[data_with_metrics$nh < nh_threshold]
)
total_households <- sum(data_with_metrics$households)

cat(sprintf("\nHouseholds with >6%% energy burden: %.1f%%\n",
            (high_burden_count / total_households) * 100))
```

## Summary

### Key Principles

1. **Energy burden is a ratio** and requires harmonic mean for proper aggregation
2. **Net Energy Return (Nh)** transformation enables arithmetic mean aggregation
3. **Both methods are mathematically equivalent** and give identical results
4. **Nh method advantages** (for aggregation across households):
   - Simpler computation (arithmetic vs harmonic mean)
   - Better numerical stability
   - More interpretable (net return per dollar of energy spending)
   - Makes improper aggregation obviously wrong

### The Correct Formula

**For aggregation across households**:

```r
# Step 1: Calculate Nh for each household/cohort
data$nh <- ner_func(data$income, data$spending)

# Step 2: Weighted arithmetic mean
nh_mean <- weighted.mean(data$nh, data$weights)

# Step 3: Convert to energy burden
eb_aggregate <- 1 / (1 + nh_mean)
```

### Common Mistakes to Avoid

**❌ NEVER do this**:
```r
# WRONG: Arithmetic mean of energy burden
eb_wrong <- weighted.mean(energy_burden_func(income, spending), weights)
```

**✅ ALWAYS do this**:
```r
# CORRECT: Nh method with arithmetic mean, then convert
nh <- ner_func(income, spending)
nh_mean <- weighted.mean(nh, weights)
eb_correct <- 1 / (1 + nh_mean)
```

## References

For a quick reference guide, see `NEB_QUICKSTART.md` in the package repository.

For practical examples with real data, see `vignette("getting-started")`.

## Mathematical Appendix

### Identity Proofs

**Proof that NEB = EB at household level**:

$$\text{Nh} = \frac{G - S}{S}$$

$$\text{NEB} = \frac{1}{1 + \text{Nh}} = \frac{1}{1 + \frac{G-S}{S}} = \frac{1}{\frac{S + G - S}{S}} = \frac{1}{\frac{G}{S}} = \frac{S}{G} = \text{EB}$$

**Proof that harmonic mean of EB equals arithmetic mean via Nh**:

For weighted aggregation with weights $w_i$:

$$\text{EB}_{\text{harmonic}} = \frac{1}{\sum_i w_i \cdot \frac{1}{\text{EB}_i}}$$

Since $\text{EB}_i = S_i / G_i$:

$$\text{EB}_{\text{harmonic}} = \frac{1}{\sum_i w_i \cdot \frac{G_i}{S_i}}$$

Now via Nh method:

$$\text{Nh}_i = \frac{G_i - S_i}{S_i} = \frac{G_i}{S_i} - 1$$

$$\overline{\text{Nh}} = \sum_i w_i \cdot \text{Nh}_i$$

$$\text{EB}_{\text{via Nh}} = \frac{1}{1 + \overline{\text{Nh}}} = \frac{1}{1 + \sum_i w_i \cdot \left(\frac{G_i}{S_i} - 1\right)}$$

$$= \frac{1}{\sum_i w_i \cdot \frac{G_i}{S_i} - \sum_i w_i + 1}$$

When weights are normalized ($\sum_i w_i = 1$), this simplifies to:

$$= \frac{1}{\sum_i w_i \cdot \frac{G_i}{S_i}} = \text{EB}_{\text{harmonic}}$$

Therefore, both methods are mathematically equivalent. ∎

---
title: "Getting Started with emburden"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with emburden}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

The **emburden** package provides tools for analyzing household energy burden using the Net Energy Return (Nh) methodology. This vignette will walk you through the basic workflow for calculating and analyzing energy burden metrics.

## Installation

You can install emburden from GitHub:

```{r installation, eval=FALSE}
# install.packages("devtools")
devtools::install_github("ericscheier/emburden")
```

```{r setup}
library(emburden)
library(dplyr)
```

## What is Energy Burden?

Energy burden is the ratio of household energy spending to gross income:

**Energy Burden (EB) = S / G**

Where:
- **S** = Energy spending (electricity, gas, other fuels)
- **G** = Gross household income

A household spending $3,000 on energy with $50,000 income has a 6% energy burden.

## Quick Example: Single Household

```{r single-household}
# Calculate energy burden for a single household
gross_income <- 50000
energy_spending <- 3000

# Method 1: Direct energy burden
eb <- energy_burden_func(gross_income, energy_spending)
print(paste("Energy Burden:", scales::percent(eb)))

# Method 2: Via Net Energy Return (mathematically identical)
nh <- ner_func(gross_income, energy_spending)
neb <- 1 / (nh + 1)
print(paste("Net Energy Burden:", scales::percent(neb)))
print(paste("Net Energy Return:", round(nh, 2)))
```

For a single household, both methods give the same result: **6% energy burden**.

## Loading Data

The package automatically downloads data from OpenEI on first use:

```{r load-data, eval=FALSE}
# Load census tract data for North Carolina
nc_tracts <- load_census_tract_data(states = "NC")

# Load household cohort data by Area Median Income
nc_ami <- load_cohort_data(dataset = "ami", states = "NC")

# View structure
head(nc_ami)
```

## Calculating Metrics from Cohort Data

When working with pre-aggregated cohort data (total income and spending), calculate metrics from the totals:

```{r cohort-example, eval=FALSE}
# Calculate mean income and spending from totals
nc_data <- nc_ami %>%
  mutate(
    mean_income = total_income / households,
    mean_energy_spending = (total_electricity_spend +
                           coalesce(total_gas_spend, 0) +
                           coalesce(total_other_spend, 0)) / households
  ) %>%
  filter(!is.na(mean_income), !is.na(mean_energy_spending), households > 0) %>%
  mutate(
    eb = energy_burden_func(mean_income, mean_energy_spending),
    nh = ner_func(mean_income, mean_energy_spending),
    neb = neb_func(mean_income, mean_energy_spending)
  )
```

## Aggregating Energy Burden (Critical!)

**Important**: Energy burden is a ratio and **cannot be aggregated using arithmetic mean**!

### The WRONG Way

```{r wrong-aggregation, eval=FALSE}
# ❌ WRONG: Direct averaging of energy burden introduces ~1-5% error
eb_wrong <- weighted.mean(nc_data$eb, nc_data$households)
```

### The CORRECT Way: Via Net Energy Return

```{r correct-aggregation, eval=FALSE}
# ✅ CORRECT Method 1: Aggregate using Nh, then convert to NEB
nh_mean <- weighted.mean(nc_data$nh, nc_data$households)
neb_correct <- 1 / (1 + nh_mean)

# ✅ CORRECT Method 2: Use neb_func() with weights (simpler!)
# neb_func() automatically uses the Nh method internally
neb_correct2 <- neb_func(nc_data$mean_income,
                         nc_data$mean_energy_spending,
                         weights = nc_data$households)

print(paste("Correct NEB (manual Nh):", scales::percent(neb_correct)))
print(paste("Correct NEB (neb_func): ", scales::percent(neb_correct2)))
# Both give identical results!
```

**Why does this work?** The Nh transformation allows us to use simple arithmetic weighted mean instead of harmonic mean, making aggregation both simpler and more intuitive. The `neb_func()` with weights does this automatically.

## Analysis by Income Bracket

```{r by-income, eval=FALSE}
# Method 1: Manual Nh aggregation
nc_by_income <- nc_data %>%
  group_by(income_bracket) %>%
  summarise(
    households = sum(households),
    nh_mean = weighted.mean(nh, households),
    neb = 1 / (1 + nh_mean),  # Correct aggregation
    .groups = "drop"
  )

# Method 2: Using neb_func() with weights (simpler!)
nc_by_income2 <- nc_data %>%
  group_by(income_bracket) %>%
  summarise(
    neb = neb_func(mean_income, mean_energy_spending, weights = households),
    households = sum(households),
    .groups = "drop"
  )

print(nc_by_income)
```

## Identifying High Energy Burden Households

The 6% energy burden threshold is commonly used to identify energy poverty:

```{r high-burden, eval=FALSE}
# 6% energy burden corresponds to Nh = 15.67
high_burden_threshold <- 15.67

high_burden_households <- sum(nc_data$households[nc_data$nh < high_burden_threshold])
total_households <- sum(nc_data$households)
high_burden_pct <- (high_burden_households / total_households) * 100

print(paste("Households with >6% energy burden:",
            scales::percent(high_burden_pct/100)))
```

## Using calculate_weighted_metrics()

For more complex grouped analysis, use the built-in function:

```{r weighted-metrics, eval=FALSE}
results <- calculate_weighted_metrics(
  graph_data = nc_ami,
  group_columns = "income_bracket",
  metric_name = "ner",
  metric_cutoff_level = 15.67,  # 6% burden threshold
  upper_quantile_view = 0.95,
  lower_quantile_view = 0.05
)

# Format for publication
results$formatted_median <- to_percent(results$metric_median)
print(results)
```

## Key Takeaways

1. **For single households**: Both EB and NEB give identical results
2. **For aggregation**: Always use the Nh method to avoid errors
3. **Never**: Directly average energy burden values
4. **Data loading**: Automatic from OpenEI (2018 and 2022 vintages available)
5. **Threshold**: 6% energy burden (Nh ≥ 15.67) identifies high burden households

## Temporal Comparison

The package provides a dedicated function for comparing energy burden across data vintages (2018 vs 2022):

```{r temporal-comparison, eval=FALSE}
# Compare by income bracket
comparison <- compare_energy_burden(
  dataset = "ami",
  states = "NC",
  group_by = "income_bracket"
)

# View results
print(comparison)

# The function automatically:
# - Loads both 2018 and 2022 data
# - Normalizes schema differences (4 vs 6 AMI brackets)
# - Performs proper Nh-based aggregation
# - Calculates changes in energy burden

# Grouping options:
# - "income_bracket": Compare by AMI/FPL brackets (default)
# - "state": Compare multiple states
# - "none": Overall state-level comparison

# Example: State-level comparison
state_comparison <- compare_energy_burden(
  dataset = "ami",
  states = "NC",
  group_by = "none"
)

# Access specific metrics
state_comparison$neb_2018         # 2018 energy burden
state_comparison$neb_2022         # 2022 energy burden
state_comparison$neb_change_pp    # Change in percentage points
state_comparison$neb_change_pct   # Relative change percentage
```

This is much simpler than manually loading and aggregating both vintages!

## Analyzing Energy Burden by Housing Characteristics

The LEAD Tool data includes detailed housing characteristics that enable analysis of how building attributes affect energy burden. Four key housing dimension columns are available:

- **TEN**: Housing tenure (1=Owned free/clear, 2=Owned with mortgage, 3=Rented, 4=Occupied without rent)
- **TEN-YBL6**: Tenure crossed with year structure built (6 categories)
- **TEN-BLD**: Tenure crossed with building type (single-family, multi-unit, etc.)
- **TEN-HFL**: Tenure crossed with primary heating fuel type (gas, electric, oil, etc.)

These columns preserve granular housing detail through the data aggregation process, allowing you to analyze energy burden patterns across different housing types.

### Example: Comparing Renters vs Owners by Heating Fuel

```{r housing-analysis, eval=FALSE}
# Load data with housing characteristics
nc_housing <- load_cohort_data(dataset = "ami", states = "NC")

# Analyze energy burden by tenure and heating fuel
housing_analysis <- nc_housing %>%
  filter(!is.na(TEN), !is.na(`TEN-HFL`)) %>%
  mutate(
    mean_income = total_income / households,
    mean_energy_spending = (total_electricity_spend +
                           coalesce(total_gas_spend, 0) +
                           coalesce(total_other_spend, 0)) / households,
    nh = ner_func(mean_income, mean_energy_spending)
  ) %>%
  group_by(TEN, `TEN-HFL`) %>%
  summarise(
    total_households = sum(households),
    nh_mean = weighted.mean(nh, households),
    neb = 1 / (1 + nh_mean),
    .groups = "drop"
  ) %>%
  arrange(desc(neb))

# View the top 10 tenure-heating fuel combinations with highest burden
head(housing_analysis, 10)
```

### Example: Energy Burden by Building Age and Type

```{r building-analysis, eval=FALSE}
# Analyze by building characteristics
building_analysis <- nc_housing %>%
  filter(!is.na(`TEN-YBL6`), !is.na(`TEN-BLD`)) %>%
  mutate(
    mean_income = total_income / households,
    mean_energy_spending = (total_electricity_spend +
                           coalesce(total_gas_spend, 0) +
                           coalesce(total_other_spend, 0)) / households,
    nh = ner_func(mean_income, mean_energy_spending)
  ) %>%
  group_by(`TEN-YBL6`, `TEN-BLD`) %>%
  summarise(
    total_households = sum(households),
    nh_mean = weighted.mean(nh, households),
    neb = 1 / (1 + nh_mean),
    .groups = "drop"
  )

# Identify building age/type combinations with highest burden
high_burden_buildings <- building_analysis %>%
  filter(neb > 0.06) %>%  # Above 6% burden threshold
  arrange(desc(neb))

print(high_burden_buildings)
```

### Key Insights from Housing Analysis

Housing characteristic analysis can reveal:

1. **Tenure effects**: How renters vs owners experience different energy burdens
2. **Heating fuel disparities**: Which fuel types create higher burden (often oil/propane)
3. **Building age impacts**: Older buildings typically have higher burden due to poor insulation
4. **Structure type patterns**: Multi-family vs single-family burden differences
5. **Vulnerable populations**: Combinations like "renter + old building + expensive fuel" often show extreme burden

This granular analysis helps target energy efficiency interventions to the housing types and populations that need them most.

## Next Steps

- See `vignette("methodology")` for mathematical details
- See `NEB_QUICKSTART.md` for quick reference
- Run example scripts in `analysis/scripts/` directory
- Read full documentation: `?energy_burden_func`, `?ner_func`

## References

- **Paper**: "Net energy metrics reveal striking disparities across United States household energy burdens"
- **LEAD Tool Data**: https://data.openei.org/
- **GitHub**: https://github.com/ericscheier/emburden

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/database-helpers.R
\name{db_exists}
\alias{db_exists}
\title{Check if Database Exists}
\usage{
db_exists(test = FALSE)
}
\arguments{
\item{test}{Logical, check test database instead of production}
}
\value{
Logical, TRUE if database exists
}
\description{
Check if Database Exists
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energy_ratios.R
\name{energy_burden_func}
\alias{energy_burden_func}
\title{Calculate Energy Burden}
\usage{
energy_burden_func(g, s, se = NULL)
}
\arguments{
\item{g}{Numeric vector of gross income values}

\item{s}{Numeric vector of energy spending values}

\item{se}{Optional numeric vector of effective energy spending (defaults to s)}
}
\value{
Numeric vector of energy burden values (ratio of spending to income)
}
\description{
Calculates the energy burden as the ratio of energy spending to gross income.
Energy burden is defined as E_b = S/G, where S is energy spending and G is
gross income.
}
\examples{
# Calculate energy burden for households
gross_income <- c(50000, 75000, 100000)
energy_spending <- c(3000, 3500, 4000)
energy_burden_func(gross_income, energy_spending)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energy_ratios.R
\name{neb_func}
\alias{neb_func}
\title{Calculate Net Energy Burden (NEB)}
\usage{
neb_func(g, s, se = NULL, weights = NULL, aggregate = FALSE)
}
\arguments{
\item{g}{Numeric vector of gross income values}

\item{s}{Numeric vector of energy spending values}

\item{se}{Optional numeric vector of effective energy spending (defaults to s)}

\item{weights}{Optional numeric vector of weights for aggregation (e.g., household counts).
When provided, uses Nh method: \code{1 / (1 + weighted.mean(nh, weights))}}

\item{aggregate}{Logical, if TRUE forces aggregation even without weights (uses unweighted mean).
Default FALSE for backwards compatibility.}
}
\value{
\itemize{
\item If \code{weights = NULL} and \code{aggregate = FALSE}: Numeric vector of individual NEB values (S/G)
\item If \code{weights} provided or \code{aggregate = TRUE}: Single aggregated NEB value via Nh method
}
}
\description{
Calculates Net Energy Burden with proper aggregation methodology via the
Net Energy Return (Nh) framework. For individual households, NEB = EB = S/G.
When aggregating across households (with weights), automatically uses the
Nh method to avoid 1-5\% aggregation errors.
}
\details{
\strong{Individual Level:} NEB = EB = S/G (mathematically identical)

\strong{Aggregation Modes:}
\enumerate{
\item \strong{No aggregation} (default): Returns vector of individual NEB values

\if{html}{\out{<div class="sourceCode">}}\preformatted{neb_func(income, spending)  # Returns vector
}\if{html}{\out{</div>}}
\item \strong{Weighted aggregation}: Automatically uses Nh method when weights provided

\if{html}{\out{<div class="sourceCode">}}\preformatted{neb_func(income, spending, weights = households)  # Returns single value
}\if{html}{\out{</div>}}
\item \strong{Unweighted aggregation}: Use \code{aggregate = TRUE} for simple mean

\if{html}{\out{<div class="sourceCode">}}\preformatted{neb_func(income, spending, aggregate = TRUE)  # Returns single value
}\if{html}{\out{</div>}}
}

\strong{Why Nh Method?} Avoids 1-5\% error from naive averaging:
\itemize{
\item \strong{CORRECT}: \code{neb_func(g, s, weights = w)} → Uses Nh internally
\item \strong{WRONG}: \code{weighted.mean(s/g, w)} → Introduces bias
}

The Nh method: \code{1 / (1 + weighted.mean(nh, weights))} where \code{nh = (g-s)/se}
uses arithmetic mean instead of harmonic mean, providing computational
simplicity and numerical stability.
}
\examples{
# Individual household - returns vector
neb_func(50000, 3000)  # 0.06
neb_func(c(30000, 50000), c(3000, 3500))  # c(0.10, 0.07)

# Aggregation with weights - returns single value (CORRECT method)
incomes <- c(30000, 50000, 75000)
spending <- c(3000, 3500, 4000)
households <- c(100, 150, 200)
neb_func(incomes, spending, weights = households)

# Unweighted aggregation
neb_func(incomes, spending, aggregate = TRUE)

# Comparison: naive mean (WRONG) vs Nh method (CORRECT)
neb_naive <- weighted.mean(spending/incomes, households)  # Biased
neb_correct <- neb_func(incomes, spending, weights = households)  # Correct
abs(neb_naive - neb_correct) / neb_correct  # ~1-5\% error
}
\seealso{
\code{\link[=ner_func]{ner_func()}} for the Net Energy Return (Nh) calculation

\code{\link[=energy_burden_func]{energy_burden_func()}} for simple EB without aggregation support
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/harmonize_brackets.R
\name{get_income_brackets}
\alias{get_income_brackets}
\title{Get Available Income Brackets for a Dataset and Vintage}
\usage{
get_income_brackets(dataset, vintage)
}
\arguments{
\item{dataset}{Character, either "ami" or "fpl"}

\item{vintage}{Integer, the year of the data vintage (e.g., 2018, 2022)}
}
\value{
Character vector of income bracket names
}
\description{
Returns the expected income brackets for a given dataset and vintage year.
Useful for understanding what brackets are available before running analyses.
}
\examples{
# Get AMI brackets for 2022
get_income_brackets("ami", 2022)

# Get FPL brackets for 2018
get_income_brackets("fpl", 2018)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metrics.R
\name{calculate_weighted_metrics}
\alias{calculate_weighted_metrics}
\title{Calculate Weighted Metrics for Energy Burden Analysis}
\usage{
calculate_weighted_metrics(
  graph_data,
  group_columns,
  metric_name,
  metric_cutoff_level,
  upper_quantile_view = 1,
  lower_quantile_view = 0
)
}
\arguments{
\item{graph_data}{A data frame containing household energy burden data with
columns for the metric of interest, household counts, and optional grouping
variables}

\item{group_columns}{Character vector of column names to group by, or NULL
for no grouping (calculates overall statistics)}

\item{metric_name}{Character string specifying the column name of the metric
to analyze (e.g., "ner" for Net Energy Return)}

\item{metric_cutoff_level}{Numeric value defining the poverty threshold for
the metric (e.g., 15.67 for Nh corresponding to 6\% energy burden)}

\item{upper_quantile_view}{Numeric between 0 and 1 specifying the upper
quantile to calculate (default: 1.0 for maximum)}

\item{lower_quantile_view}{Numeric between 0 and 1 specifying the lower
quantile to calculate (default: 0.0 for minimum)}
}
\value{
A data frame with one row per group (or one row if ungrouped)
containing:
\item{household_count}{Total number of households in the group}
\item{households_below_cutoff}{Number of households below poverty threshold}
\item{pct_in_group_below_cutoff}{Proportion of group below threshold}
\item{metric_mean}{Weighted mean of the metric}
\item{metric_median}{Weighted median of the metric}
\item{metric_upper}{Upper quantile value}
\item{metric_lower}{Lower quantile value}
\item{metric_max}{Maximum value in group}
\item{metric_min}{Minimum value in group}
}
\description{
Calculates weighted statistical metrics (mean, median, quantiles) for a
specified energy metric, with optional grouping by geographic or demographic
categories. This is the primary function for aggregating household-level
energy burden data using proper weighting by household counts.
}
\details{
This function requires the \code{spatstat} package for weighted quantile
calculations. It automatically handles missing values and ensures that
statistics are only calculated when sufficient data points exist (n >= 3).

The function adds an "All" category row that aggregates across all groups,
in addition to the individual group statistics.
}
\examples{
\donttest{
# Calculate metrics for NC cooperatives using Nh
library(dplyr)

# Sample data
data <- data.frame(
  cooperative = rep(c("Coop A", "Coop B"), each = 3),
  ner = c(20, 15, 25, 18, 22, 12),
  households = c(1000, 500, 750, 900, 600, 400)
)

# Calculate weighted metrics by cooperative
results <- calculate_weighted_metrics(
  graph_data = data,
  group_columns = "cooperative",
  metric_name = "ner",
  metric_cutoff_level = 15.67,
  upper_quantile_view = 0.95,
  lower_quantile_view = 0.05
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{try_load_tracts_from_database}
\alias{try_load_tracts_from_database}
\title{Try to load census tract data from database}
\usage{
try_load_tracts_from_database(verbose = FALSE)
}
\description{
Try to load census tract data from database
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache_utils.R
\name{get_database_path}
\alias{get_database_path}
\title{Get the full path to the emburden database file}
\usage{
get_database_path()
}
\description{
Get the full path to the emburden database file
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{download_lead_data}
\alias{download_lead_data}
\title{Download LEAD data from OpenEI}
\usage{
download_lead_data(dataset, vintage, states = NULL, verbose = FALSE)
}
\description{
Download LEAD data from OpenEI
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{try_import_to_database}
\alias{try_import_to_database}
\title{Try to import cohort data to database}
\usage{
try_import_to_database(data, dataset, vintage, verbose = FALSE)
}
\description{
Try to import cohort data to database
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache_utils.R
\name{cache_utils}
\alias{cache_utils}
\title{Cache and Database Management Utilities}
\description{
Cache and Database Management Utilities
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_processing.R
\name{raw_to_lead}
\alias{raw_to_lead}
\title{Process raw LEAD data into clean format}
\usage{
raw_to_lead(data, vintage)
}
\arguments{
\item{data}{A data frame of raw LEAD data from OpenEI}

\item{vintage}{Character string indicating the ACS vintage year ("2016", "2018", "2022", etc.)}
}
\value{
A data frame with standardized column names:
\item{geoid}{11-digit census tract GEOID as character}
\item{state_abbr}{2-letter state abbreviation (2018+ only)}
\item{housing_tenure}{Housing tenure category}
\item{year_constructed}{Year building was constructed category}
\item{building_type}{Building type category}
\item{min_units}{Minimum number of units in building}
\item{detached}{Whether building is detached (1/0)}
\item{primary_heating_fuel}{Primary heating fuel type}
\item{income_bracket}{Income bracket category (depends on dataset: AMI, FPL, etc.)}
\item{households}{Number of households}
\item{income}{Annual income}
\item{electricity_spend}{Annual electricity spending}
\item{gas_spend}{Annual gas spending}
\item{other_spend}{Annual other fuel spending}
}
\description{
Converts raw LEAD data downloaded from OpenEI into a standardized clean format
suitable for analysis. Handles both 2016 (SH) and 2018+ ACS vintages.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{get_all_states}
\alias{get_all_states}
\title{Get all state abbreviations}
\usage{
get_all_states()
}
\value{
Character vector of all 51 state abbreviations (50 states + DC)
}
\description{
Get all state abbreviations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_processing.R
\name{process_lead_cohort_data}
\alias{process_lead_cohort_data}
\title{Process raw LEAD data into analysis-ready format with energy metrics}
\usage{
process_lead_cohort_data(data, dataset, vintage, aggregate_poverty = FALSE)
}
\arguments{
\item{data}{A data frame of raw LEAD data from OpenEI}

\item{dataset}{Character string indicating dataset type ("ami" or "fpl")}

\item{vintage}{Character string indicating ACS vintage year}

\item{aggregate_poverty}{Logical; if TRUE, aggregate to poverty status level}
}
\value{
A data frame ready for analysis with all energy metrics
}
\description{
This is the main processing workflow that:
\enumerate{
\item Converts raw OpenEI data to clean format
\item Optionally aggregates by poverty status
\item Adds energy burden and related metrics
\item Filters out zero-energy records
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nc_sample-data.R
\docType{data}
\name{nc_sample}
\alias{nc_sample}
\title{North Carolina Complete Energy Burden Sample Data}
\format{
A named list with 4 data frames:
\describe{
\item{fpl_2018}{Federal Poverty Line cohort data for 2018 (~10,805 rows)}
\item{fpl_2022}{Federal Poverty Line cohort data for 2022 (~13,185 rows)}
\item{ami_2018}{Area Median Income cohort data for 2018 (~6,484 rows)}
\item{ami_2022}{Area Median Income cohort data for 2022 (~5,091 rows)}
}

Each data frame contains:
\describe{
\item{geoid}{11-digit census tract identifier (character)}
\item{income_bracket}{Income bracket category (character)}
\item{households}{Number of households in this cohort (numeric)}
\item{total_income}{Total household income in dollars (numeric)}
\item{total_electricity_spend}{Total electricity spending in dollars (numeric)}
\item{total_gas_spend}{Total gas spending in dollars (numeric)}
\item{total_other_spend}{Total other fuel spending in dollars (numeric)}
}
}
\source{
U.S. Department of Energy Low-Income Energy Affordability Data (LEAD) Tool
\itemize{
\item 2018 vintage: \url{https://data.openei.org/submissions/573}
\item 2022 vintage: \url{https://data.openei.org/submissions/6219}
}
}
\usage{
nc_sample
}
\description{
A comprehensive dataset containing energy burden data for all counties in North Carolina.
This dataset includes both Federal Poverty Line (FPL) and Area Median Income (AMI) cohort
data for 2018 and 2022 vintages, aggregated to the census tract × income bracket level.
}
\details{
This sample data provides full state coverage for more comprehensive analysis, testing,
and demonstrations. For lightweight quick demos, see \code{\link{orange_county_sample}}.

\strong{North Carolina} (all 100 counties):
\itemize{
\item 2018: 2,163 census tracts
\item 2022: 2,642 census tracts (tract boundaries changed)
}

\strong{Income Brackets}:
\itemize{
\item FPL: 0-100\%, 100-150\%, 150-200\%, 200-400\%, 400\%+
\item AMI: Varies by vintage (4-6 categories)
}

\strong{Size}: 1.3 MB compressed (.rda)
}
\examples{
# Load sample data
data(nc_sample)

# View structure
names(nc_sample)

# Analyze energy burden by county
library(dplyr)

# Extract county FIPS (first 5 digits of geoid)
nc_sample$fpl_2022 \%>\%
  mutate(county_fips = substr(geoid, 1, 5)) \%>\%
  group_by(county_fips, income_bracket) \%>\%
  summarise(
    households = sum(households),
    avg_energy_burden = sum(total_electricity_spend + total_gas_spend + total_other_spend) /
                        sum(total_income),
    .groups = "drop"
  ) \%>\%
  filter(county_fips == "37183")  # Wake County

# Compare urban vs rural counties
urban_counties <- c("37119", "37063", "37183")  # Mecklenburg, Durham, Wake
rural_counties <- c("37069", "37095", "37131")  # Franklin, Hyde, Northampton

nc_sample$fpl_2022 \%>\%
  mutate(
    county_fips = substr(geoid, 1, 5),
    region = case_when(
      county_fips \%in\% urban_counties ~ "Urban",
      county_fips \%in\% rural_counties ~ "Rural",
      TRUE ~ "Other"
    )
  ) \%>\%
  filter(region != "Other") \%>\%
  group_by(region, income_bracket) \%>\%
  summarise(
    households = sum(households),
    energy_burden = sum(total_electricity_spend + total_gas_spend + total_other_spend) /
                    sum(total_income),
    .groups = "drop"
  )

}
\seealso{
\itemize{
\item \code{\link{orange_county_sample}} - Lightweight sample (94 KB) for quick demos
\item \code{\link{load_cohort_data}} - Load data for any state with county filtering
\item \code{\link{compare_energy_burden}} - Compare energy burden across vintages
\item \code{\link{calculate_weighted_metrics}} - Calculate weighted metrics with grouping
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{check_data_sources}
\alias{check_data_sources}
\title{Check Available Data Sources}
\usage{
check_data_sources(verbose = TRUE)
}
\arguments{
\item{verbose}{Logical, print detailed status (default TRUE)}
}
\value{
A list with status of each data source
}
\description{
Check which data sources are available locally (database, CSV files, or
will require download from OpenEI).
}
\examples{
\donttest{
# Check what data is available
check_data_sources()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{list_states}
\alias{list_states}
\title{List Available States}
\usage{
list_states()
}
\value{
Character vector of 51 state abbreviations (50 states + DC)
}
\description{
Returns all state abbreviations available in the LEAD dataset.
}
\examples{
list_states()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{download_census_tract_data}
\alias{download_census_tract_data}
\title{Download census tract data from OpenEI}
\usage{
download_census_tract_data(verbose = FALSE)
}
\description{
Download census tract data from OpenEI
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatting.R
\name{to_big}
\alias{to_big}
\title{Format Large Numbers with Thousand Separators}
\usage{
to_big(x)
}
\arguments{
\item{x}{Numeric vector to format}
}
\value{
Character vector of formatted numbers
}
\description{
Converts numeric values to formatted strings with thousand separators (commas).
}
\examples{
# Format large numbers
to_big(c(1000, 25000, 1000000))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zenodo.R
\name{get_zenodo_config}
\alias{get_zenodo_config}
\title{Get Zenodo Record Information}
\usage{
get_zenodo_config()
}
\value{
List with Zenodo record information
}
\description{
Returns the Zenodo DOI and file information for emburden datasets.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatting.R
\name{to_dollar}
\alias{to_dollar}
\title{Format Number as Dollar Amount}
\usage{
to_dollar(x, latex = FALSE)
}
\arguments{
\item{x}{Numeric vector to format}

\item{latex}{Logical indicating whether to escape dollar sign for LaTeX
(default: FALSE)}
}
\value{
Character vector of formatted dollar amounts
}
\description{
Converts numeric values to formatted dollar strings with appropriate
decimal places and thousand separators.
}
\examples{
# Format dollar amounts
to_dollar(c(1000, 2500.50, 10000))

# LaTeX-escaped format
to_dollar(c(1000, 2500.50), latex = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emburden-package.R
\docType{package}
\name{emburden-package}
\alias{emburden}
\alias{emburden-package}
\title{emburden: Energy Burden Analysis Using Net Energy Return Methodology}
\description{
Calculate and analyze household energy burden using the Net Energy Return aggregation methodology. Functions support weighted statistical calculations across geographic and demographic cohorts, with utilities for formatting results into publication-ready tables. Methods are based on Scheier & Kittner (2022) \doi{10.1038/s41467-021-27673-y}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ericscheier/emburden}
  \item \url{https://ericscheier.info/emburden/}
  \item Report bugs at \url{https://github.com/ericscheier/emburden/issues}
}

}
\author{
\strong{Maintainer}: Eric Scheier \email{eric@scheier.org} [copyright holder]

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zenodo.R
\name{download_from_zenodo}
\alias{download_from_zenodo}
\title{Download Dataset from Zenodo}
\usage{
download_from_zenodo(dataset, vintage, verbose = FALSE)
}
\arguments{
\item{dataset}{Character, either "ami" or "fpl"}

\item{vintage}{Character, data vintage: "2018" or "2022"}

\item{verbose}{Logical, print progress messages (default TRUE)}
}
\value{
Tibble with downloaded data, or NULL if download fails
}
\description{
Downloads a pre-processed dataset from the emburden Zenodo repository.
Includes progress bars, checksum verification, and automatic retry logic.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatting.R
\name{colorize}
\alias{colorize}
\title{Colorize Text for Knitted Documents}
\usage{
colorize(x, color)
}
\arguments{
\item{x}{Character string to colorize}

\item{color}{Character string specifying the color name (e.g., "red", "blue")}
}
\value{
Character string wrapped in LaTeX or HTML color commands, or
unchanged if output format is neither
}
\description{
Wraps text in color formatting appropriate for the output format (LaTeX or HTML).
This function is intended for use within R Markdown/knitr documents.
}
\details{
This function detects the knitr output format and applies appropriate color
formatting. For LaTeX output, it uses \verb{\\\\textcolor\{\}}. For HTML output, it
uses \verb{<span style='color: ...'>}.
}
\examples{
\donttest{
# In an R Markdown document:
colorize("Important text", "red")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache_utils.R, R/lead_data_loaders.R
\name{get_cache_dir}
\alias{get_cache_dir}
\title{Get the emburden cache directory}
\usage{
get_cache_dir()

get_cache_dir()
}
\description{
Get the emburden cache directory

Get cache directory for downloaded files
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{get_county_fips}
\alias{get_county_fips}
\title{Convert county identifiers to FIPS codes}
\usage{
get_county_fips(counties, states)
}
\arguments{
\item{counties}{Character vector of county identifiers (FIPS codes or names)}

\item{states}{Character vector of state abbreviations for context}
}
\value{
Character vector of 3-digit county FIPS codes
}
\description{
Supports both 3-digit county FIPS codes and 5-digit state+county FIPS codes.
County names can be matched from the orange_county_sample or nc_sample datasets.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energy_ratios.R
\name{ner_func}
\alias{ner_func}
\title{Calculate Net Energy Return (Nh)}
\usage{
ner_func(g, s, se = NULL)
}
\arguments{
\item{g}{Numeric vector of gross income values}

\item{s}{Numeric vector of energy spending values}

\item{se}{Optional numeric vector of effective energy spending (defaults to s)}
}
\value{
Numeric vector of Net Energy Return (Nh) values
}
\description{
Calculates the Net Energy Return using the formula Nh = (G - S) / Se,
where G is gross income, S is energy spending, and Se is effective energy
spending. This metric is the preferred aggregation variable as it properly
accounts for harmonic mean behavior when aggregating across households.
}
\details{
The Net Energy Return is mathematically related to energy burden by:
E_b = 1 / (Nh + 1), or equivalently: Nh = (1/E_b) - 1

\strong{Why use Nh for aggregation?}

For individual household data, the Nh method enables simple arithmetic weighted
mean aggregation:
\itemize{
\item \strong{Via Nh}: \code{neb = 1 / (1 + weighted.mean(nh, weights))} (arithmetic mean)
\item \strong{Direct EB}: \code{neb = 1 / weighted.mean(1/eb, weights)} (harmonic mean)
}

\strong{Computational advantages of the arithmetic mean approach:}
\enumerate{
\item \strong{Simpler to compute} - Uses standard \code{weighted.mean()} function
\item \strong{More numerically stable} - Avoids division by very small EB values (e.g., 0.01)
\item \strong{More interpretable} - "Average net return per dollar spent on energy"
\item \strong{Prevents errors} - Makes it obvious you can't use arithmetic mean on EB directly
}

For cohort data (pre-aggregated totals), direct calculation \code{sum(S)/sum(G)}
is mathematically equivalent to the Nh method but simpler.

The 6\% energy burden poverty threshold corresponds to Nh \eqn{\ge} 15.67.
}
\examples{
# Calculate Net Energy Return
gross_income <- 50000
energy_spending <- 3000
nh <- ner_func(gross_income, energy_spending)

# Convert back to energy burden
energy_burden <- 1 / (nh + 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energy_ratios.R
\name{eroi_func}
\alias{eroi_func}
\title{Calculate Energy Return on Investment (EROI)}
\usage{
eroi_func(g, s, se = NULL)
}
\arguments{
\item{g}{Numeric vector of gross income values}

\item{s}{Numeric vector of energy spending values}

\item{se}{Optional numeric vector of effective energy spending (defaults to s)}
}
\value{
Numeric vector of EROI values
}
\description{
Calculates the Energy Return on Investment as the ratio of gross income to
effective energy spending. EROI = G/Se.
}
\examples{
# Calculate EROI for households
eroi_func(50000, 3000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{orange_county_sample}
\alias{orange_county_sample}
\title{Orange County NC Energy Burden Sample Data}
\format{
A named list with 4 data frames:
\describe{
\item{fpl_2018}{Federal Poverty Line cohort data for 2018 (135 rows)}
\item{fpl_2022}{Federal Poverty Line cohort data for 2022 (206 rows)}
\item{ami_2018}{Area Median Income cohort data for 2018 (259 rows)}
\item{ami_2022}{Area Median Income cohort data for 2022 (149 rows)}
}

Each data frame contains:
\describe{
\item{geoid}{11-digit census tract identifier (character)}
\item{income_bracket}{Income bracket category (character)}
\item{households}{Number of households in this cohort (numeric)}
\item{total_income}{Total household income in dollars (numeric)}
\item{total_electricity_spend}{Total electricity spending in dollars (numeric)}
\item{total_gas_spend}{Total gas spending in dollars (numeric)}
\item{total_other_spend}{Total other fuel spending in dollars (numeric)}
}
}
\source{
U.S. Department of Energy Low-Income Energy Affordability Data (LEAD) Tool
\itemize{
\item 2018 vintage: \url{https://data.openei.org/submissions/573}
\item 2022 vintage: \url{https://data.openei.org/submissions/6219}
}
}
\usage{
orange_county_sample
}
\description{
A sample dataset containing energy burden data for Orange County, North Carolina
(FIPS code 37135). This dataset includes both Federal Poverty Line (FPL) and
Area Median Income (AMI) cohort data for 2018 and 2022 vintages.
}
\details{
This sample data is provided for quick demos, testing, and vignettes without
requiring a large download. For full state or national analysis, use
\code{\link{load_cohort_data}()} to download complete datasets from OpenEI.

\strong{Orange County NC} (Chapel Hill, Carrboro, Hillsborough):
\itemize{
\item 2018: 27 census tracts
\item 2022: 42 census tracts (tract boundaries changed)
}

\strong{Income Brackets}:
\itemize{
\item FPL: 0-100\%, 100-150\%, 150-200\%, 200-400\%, 400\%+
\item AMI: very_low, low_mod, mid_high (aggregated from 6 AMI categories)
}
}
\examples{
# Load sample data
data(orange_county_sample)

# View structure
names(orange_county_sample)

# Quick analysis of 2022 FPL data
library(dplyr)
orange_county_sample$fpl_2022 \%>\%
  group_by(income_bracket) \%>\%
  summarise(
    households = sum(households),
    avg_energy_burden = sum(total_electricity_spend + total_gas_spend + total_other_spend) /
                        sum(total_income)
  )

}
\seealso{
\itemize{
\item \code{\link{load_cohort_data}} - Load full datasets for any state
\item \code{\link{compare_energy_burden}} - Compare energy burden across vintages
\item \code{\link{calculate_weighted_metrics}} - Calculate weighted metrics with grouping
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zenodo.R
\name{download_tracts_from_zenodo}
\alias{download_tracts_from_zenodo}
\title{Download Census Tract Data from Zenodo}
\usage{
download_tracts_from_zenodo(verbose = FALSE)
}
\arguments{
\item{verbose}{Logical, print progress messages (default TRUE)}
}
\value{
Tibble with census tract data, or NULL if download fails
}
\description{
Downloads pre-processed census tract data from Zenodo.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache_utils.R
\name{validate_before_caching}
\alias{validate_before_caching}
\title{Validate data before caching to database}
\usage{
validate_before_caching(
  data,
  dataset,
  vintage,
  expected_states = 51,
  strict = TRUE
)
}
\arguments{
\item{data}{Data frame to validate}

\item{dataset}{Character, "ami" or "fpl"}

\item{vintage}{Character, "2018" or "2022"}

\item{expected_states}{Integer, expected number of states (51 for nationwide)}

\item{strict}{Logical, if TRUE throws errors; if FALSE returns list with validation results}
}
\value{
If strict=FALSE, returns list with: valid (logical), issues (character vector)
If strict=TRUE, throws error on validation failure
}
\description{
Performs comprehensive validation BEFORE data is saved to database or cache.
Prevents corrupted data from being cached in the first place.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{list_income_brackets}
\alias{list_income_brackets}
\title{List Available Income Brackets}
\usage{
list_income_brackets(dataset = c("ami", "fpl"), vintage = "2022")
}
\arguments{
\item{dataset}{Character, either "ami" or "fpl"}

\item{vintage}{Character, "2018" or "2022"}
}
\value{
Character vector of income bracket labels
}
\description{
Returns the income brackets available for a given dataset and vintage.
}
\examples{
list_income_brackets("ami", "2022")
list_income_brackets("fpl", "2018")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{get_dataset_info}
\alias{get_dataset_info}
\title{Get Dataset Information}
\usage{
get_dataset_info()
}
\value{
Data frame with dataset information
}
\description{
Returns metadata about available LEAD datasets.
}
\examples{
get_dataset_info()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{get_state_fips}
\alias{get_state_fips}
\title{Get state FIPS codes from abbreviations}
\usage{
get_state_fips(state_abbrs)
}
\description{
Get state FIPS codes from abbreviations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatting.R
\name{to_billion_dollar}
\alias{to_billion_dollar}
\title{Format Dollar Amounts in Billions}
\usage{
to_billion_dollar(x, suffix = " billion", override_to_k = TRUE)
}
\arguments{
\item{x}{Numeric vector to format}

\item{suffix}{Character string to append after "billion" (default: " billion")}

\item{override_to_k}{Logical (currently unused, kept for compatibility)}
}
\value{
Character vector of formatted dollar amounts with "billion" or "m" suffix
}
\description{
Converts large dollar values to billions format with dollar sign prefix.
Values less than 1 billion are shown in millions.
}
\examples{
# Format in billions
to_billion_dollar(c(5000000, 1000000000, 2500000000))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/harmonize_brackets.R
\name{harmonize_income_brackets}
\alias{harmonize_income_brackets}
\title{Harmonize Income Brackets Across Vintages}
\usage{
harmonize_income_brackets(
  data,
  dataset,
  vintage,
  strict_matching = TRUE,
  comparison_vintages = NULL
)
}
\arguments{
\item{data}{A data frame containing income bracket data}

\item{dataset}{Character, either "ami" or "fpl"}

\item{vintage}{Integer, the year of the data vintage}

\item{strict_matching}{Logical, if TRUE (default) only keeps brackets that
exist in both vintages being compared. If FALSE, keeps all brackets.}

\item{comparison_vintages}{Integer vector of length 2, the vintages being
compared (e.g., c(2018, 2022)). Required when strict_matching = TRUE.}
}
\value{
A list with components:
\itemize{
\item \code{data}: The harmonized data frame
\item \code{warnings}: Character vector of any warnings about bracket mismatches
\item \code{dropped_brackets}: Character vector of brackets that were dropped
}
}
\description{
Harmonizes income bracket categories when comparing data across different
vintage years. This is necessary because some datasets (particularly AMI)
have different bracket definitions across years.
}
\details{
\subsection{Dataset-Specific Bracket Definitions}{
\subsection{AMI (Area Median Income)}{
\itemize{
\item \strong{2018}: 3 brackets
\itemize{
\item \code{very_low}: Very low income (typically <50\% AMI)
\item \code{low_mod}: Low to moderate income (typically 50-80\% AMI)
\item \code{mid_high}: Middle to high income (typically >80\% AMI)
}
\item \strong{2022}: 5 brackets
\itemize{
\item \code{very_low}: Very low income (same as 2018)
\item \code{low_mod}: Low to moderate income (same as 2018)
\item \code{mid_high}: Middle to high income (narrower than 2018)
\item \verb{100-150\%}: 100-150\% of AMI (new in 2022)
\item \verb{150\%+}: Above 150\% of AMI (new in 2022)
}
}
}

\subsection{FPL (Federal Poverty Level)}{
\itemize{
\item \strong{Both 2018 and 2022}: 5 brackets
\itemize{
\item \verb{0-100\%}: Below poverty line
\item \verb{100-150\%}: 100-150\% of FPL
\item \verb{150-200\%}: 150-200\% of FPL
\item \verb{200-400\%}: 200-400\% of FPL
\item \verb{400\%+}: Above 400\% of FPL
}
}
}

}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_burden.R
\name{print.energy_burden_comparison}
\alias{print.energy_burden_comparison}
\title{Print Comparison Summary}
\usage{
\method{print}{energy_burden_comparison}(x, ...)
}
\arguments{
\item{x}{Comparison result from compare_energy_burden()}

\item{...}{Additional arguments (not used)}
}
\value{
Returns \code{x} invisibly for use in pipe chains.
}
\description{
Pretty-print a comparison table from compare_energy_burden()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache_utils.R
\name{clear_all_cache}
\alias{clear_all_cache}
\title{Clear all emburden cache and database}
\usage{
clear_all_cache(confirm = FALSE, verbose = TRUE)
}
\arguments{
\item{confirm}{Logical, must be TRUE to proceed (safety check)}

\item{verbose}{Logical, print progress messages}
}
\value{
Invisibly returns list with: cache_cleared (logical), db_cleared (logical)
}
\description{
Nuclear option: clears ALL cached data and database.
Use with caution - will require re-downloading all data.
}
\examples{
\donttest{
# Clear everything (requires confirm = TRUE)
clear_all_cache(confirm = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{load_census_tract_data}
\alias{load_census_tract_data}
\title{Load Census Tract Data}
\usage{
load_census_tract_data(states = NULL, verbose = TRUE)
}
\arguments{
\item{states}{Character vector of state abbreviations to filter by (optional)}

\item{verbose}{Logical, print status messages (default TRUE)}
}
\value{
A tibble with columns:
\itemize{
\item geoid: Census tract identifier
\item state_abbr: State abbreviation
\item county_name: County name
\item tract_name: Tract name
\item utility_name: Electric utility serving this tract
\item Additional demographic columns
}
}
\description{
Load census tract demographics and utility service territory information
with automatic fallback to CSV or OpenEI download.
}
\examples{
\donttest{
if (interactive()) {
  # Single state (requires census data download)
  nc_tracts <- load_census_tract_data(states = "NC")

  # Multiple states (regional)
  southeast <- load_census_tract_data(states = c("NC", "SC", "GA", "FL"))

  # Nationwide (all ~73,000 census tracts)
  us_tracts <- load_census_tract_data()  # No filter = all states
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/database-helpers.R
\name{backup_db}
\alias{backup_db}
\title{Backup Production Database}
\usage{
backup_db()
}
\value{
Path to backup file, or NULL if no database exists
}
\description{
Creates a timestamped backup of the production database
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{aggregate_cohort_data}
\alias{aggregate_cohort_data}
\title{Aggregate cohort data by census tract and income bracket}
\usage{
aggregate_cohort_data(data, dataset, vintage, verbose = FALSE)
}
\description{
Aggregate cohort data by census tract and income bracket
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{standardize_cohort_columns}
\alias{standardize_cohort_columns}
\title{Standardize cohort column names across vintages}
\usage{
standardize_cohort_columns(data, dataset, vintage)
}
\description{
Standardize cohort column names across vintages
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/database-helpers.R
\name{get_db_path}
\alias{get_db_path}
\title{Get Database Path}
\usage{
get_db_path(test = FALSE)
}
\arguments{
\item{test}{Logical, whether to use test database (default FALSE)}
}
\value{
Path to database file
}
\description{
Returns the path to the database, with protection against accidental deletion.
For tests, use a separate test database.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/database-helpers.R
\name{delete_db}
\alias{delete_db}
\title{Delete Database (PROTECTED)}
\usage{
delete_db(test = TRUE, confirm = FALSE)
}
\arguments{
\item{test}{Logical, delete test database (default TRUE)}

\item{confirm}{Logical, must be TRUE to delete production database}
}
\value{
Logical, TRUE if deleted successfully
}
\description{
Deletes a database with safety checks. Production database requires
explicit confirmation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache_utils.R
\name{clear_dataset_cache}
\alias{clear_dataset_cache}
\title{Clear cache for a specific dataset}
\usage{
clear_dataset_cache(
  dataset = c("ami", "fpl"),
  vintage = c("2018", "2022"),
  verbose = TRUE
)
}
\arguments{
\item{dataset}{Character, "ami" or "fpl"}

\item{vintage}{Character, "2018" or "2022"}

\item{verbose}{Logical, print progress messages}
}
\value{
Invisibly returns number of items cleared
}
\description{
Removes cached CSV files and database entries for a specific dataset/vintage.
Useful when you know a specific dataset is corrupted.
}
\examples{
\donttest{
# Clear corrupted AMI 2018 cache
clear_dataset_cache("ami", "2018")

# Clear FPL 2022 cache
clear_dataset_cache("fpl", "2022", verbose = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energy_ratios.R
\name{dear_func}
\alias{dear_func}
\title{Calculate Disposable Energy-Adjusted Resources (DEAR)}
\usage{
dear_func(g, s, se = NULL)
}
\arguments{
\item{g}{Numeric vector of gross income values}

\item{s}{Numeric vector of energy spending values}

\item{se}{Optional numeric vector of effective energy spending (defaults to s)}
}
\value{
Numeric vector of DEAR values (ratio of disposable income to gross income)
}
\description{
Calculates DEAR as the ratio of net income after energy spending to
gross income. DEAR = (G - S) / G.
}
\examples{
# Calculate DEAR
dear_func(50000, 3000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{load_cohort_data}
\alias{load_cohort_data}
\title{Load DOE LEAD Tool Cohort Data}
\usage{
load_cohort_data(
  dataset = c("ami", "fpl"),
  states = NULL,
  counties = NULL,
  vintage = "2022",
  income_brackets = NULL,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{dataset}{Character, either "ami" (Area Median Income) or "fpl"
(Federal Poverty Line)}

\item{states}{Character vector of state abbreviations to filter by (optional)}

\item{counties}{Character vector of county names or FIPS codes to filter by (optional).
County names are matched case-insensitively. Requires \code{states} to be specified.}

\item{vintage}{Character, data vintage: "2018" or "2022" (default "2022")}

\item{income_brackets}{Character vector of income brackets to filter by (optional)}

\item{verbose}{Logical, print status messages (default TRUE)}

\item{...}{Additional filter expressions passed to dplyr::filter() for dynamic filtering.
Allows filtering by any column in the dataset using tidyverse syntax.
Example: \verb{households > 100, total_income > 50000}}
}
\value{
A tibble with columns:
\itemize{
\item geoid: Census tract identifier
\item income_bracket: Income bracket label
\item households: Number of households
\item total_income: Total household income ($)
\item total_electricity_spend: Total electricity spending ($)
\item total_gas_spend: Total gas spending ($)
\item total_other_spend: Total other fuel spending ($)
\item TEN: Housing tenure category (1=Owned free/clear, 2=Owned with mortgage,
3=Rented, 4=Occupied without rent). Enables analysis of energy burden
differences between renters and owners.
\item TEN-YBL6: Housing tenure crossed with year structure built (6 categories).
Allows analysis of how building age and ownership status interact to affect
energy burden (e.g., older rental units vs newer owner-occupied homes).
\item TEN-BLD: Housing tenure crossed with building type (e.g., single-family,
multi-unit). Enables analysis of energy burden across different housing
structures and ownership patterns.
\item TEN-HFL: Housing tenure crossed with primary heating fuel type (e.g., gas,
electric, oil). Critical for analyzing how heating fuel choice and tenure
status jointly influence energy costs and burden.
}
}
\description{
Load household energy burden cohort data with automatic fallback:
\enumerate{
\item Try local database
\item Fall back to local CSV files
\item Auto-download from OpenEI if neither exists
\item Auto-import downloaded data to database for future use
}
}
\examples{
\donttest{
# Single state (fast, good for learning)
nc_ami <- load_cohort_data(dataset = "ami", states = "NC")

# Load specific vintage
nc_2018 <- load_cohort_data(dataset = "ami", states = "NC", vintage = "2018")
}

\donttest{
if (interactive()) {
  # Multiple states (regional analysis - requires data download)
  southeast <- load_cohort_data(dataset = "fpl", states = c("NC", "SC", "GA", "FL"))

  # Nationwide (all 51 states - no filter)
  us_data <- load_cohort_data(dataset = "ami", vintage = "2022")

# Filter to specific income brackets
low_income <- load_cohort_data(
  dataset = "ami",
  states = "NC",
  income_brackets = c("0-30\% AMI", "30-50\% AMI")
)

# Filter to specific counties within a state
triangle <- load_cohort_data(
  dataset = "fpl",
  states = "NC",
  counties = c("Orange", "Durham", "Wake")
)

# Or use county FIPS codes
orange <- load_cohort_data(
  dataset = "fpl",
  states = "NC",
  counties = "37135"
)

# Use dynamic filtering for custom criteria
high_burden <- load_cohort_data(
  dataset = "ami",
  states = "NC",
  households > 100,
  total_electricity_spend / total_income > 0.06
)

# Analyze energy burden by housing characteristics
# Compare renters vs owners by heating fuel type
nc_housing <- load_cohort_data(dataset = "ami", states = "NC")
library(dplyr)

# Group by tenure and heating fuel to analyze energy burden patterns
housing_analysis <- nc_housing \%>\%
  filter(!is.na(TEN), !is.na(`TEN-HFL`)) \%>\%
  group_by(TEN, `TEN-HFL`) \%>\%
  summarise(
    total_households = sum(households),
    avg_energy_burden = weighted.mean(
      (total_electricity_spend + total_gas_spend + total_other_spend) / total_income,
      w = households,
      na.rm = TRUE
    ),
    .groups = "drop"
  )
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatting.R
\name{to_million}
\alias{to_million}
\title{Format Numbers in Millions}
\usage{
to_million(x, suffix = " million", override_to_k = TRUE)
}
\arguments{
\item{x}{Numeric vector to format}

\item{suffix}{Character string to append after "million" (default: " million")}

\item{override_to_k}{Logical indicating whether to show values < 1M as
thousands (default: TRUE)}
}
\value{
Character vector of formatted numbers with "million" or "k" suffix
}
\description{
Converts large numeric values to millions format with appropriate suffix.
Values less than 1 million are shown in thousands.
}
\examples{
# Format in millions
to_million(c(5000, 1000000, 2500000))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_burden.R
\name{compare_energy_burden}
\alias{compare_energy_burden}
\title{Compare Energy Burden Between Years}
\usage{
compare_energy_burden(
  dataset = c("ami", "fpl"),
  states = NULL,
  group_by = "income_bracket",
  counties = NULL,
  vintage_1 = "2022",
  vintage_2 = "2018",
  format = TRUE,
  strict_matching = TRUE
)
}
\arguments{
\item{dataset}{Character, either "ami" or "fpl" for cohort data type}

\item{states}{Character vector of state abbreviations to filter by (optional)}

\item{group_by}{Character or character vector. Use keywords "income_bracket" (default),
"state", or "none" for standard groupings. Or provide custom column name(s)
for dynamic grouping (e.g., "geoid" for tract-level, c("state_abbr", "income_bracket")
for multi-level grouping). Custom columns must exist in the loaded data.}

\item{counties}{Character vector of county names or FIPS codes to filter by (optional).
Requires \code{states} to be specified.}

\item{vintage_1}{Character, first vintage year: "2018" or "2022" (default "2022")}

\item{vintage_2}{Character, second vintage year: "2018" or "2022" (default "2018")}

\item{format}{Logical, if TRUE returns formatted percentages (default TRUE)}

\item{strict_matching}{Logical, if TRUE (default) only compares income brackets
that exist in both vintages and warns about mismatched brackets. If FALSE,
compares all brackets (may result in NA values for brackets unique to one vintage).}
}
\value{
A data.frame with energy burden comparison showing:
\itemize{
\item neb_YYYY: Net Energy Burden for each vintage (where YYYY is the year)
\item change_pp: Absolute change in percentage points
\item change_pct: Relative percent change
}
}
\description{
Compare household energy burden metrics across different data vintages,
using proper Net Energy Return (Nh) aggregation methodology.
}
\examples{
\donttest{
# Single state comparison (fast, good for learning)
nc_comparison <- compare_energy_burden("ami", "NC", "income_bracket")

# Overall comparison (no grouping)
compare_energy_burden("ami", "NC", "none")
}

\donttest{
if (interactive()) {
  # Multi-state regional comparison (requires census data download)
  southeast <- compare_energy_burden(
    dataset = "fpl",
    states = c("NC", "SC", "GA", "FL"),
    group_by = "state"
  )

  # Nationwide comparison by income bracket (all 51 states)
  us_comparison <- compare_energy_burden(
    dataset = "ami",
    group_by = "income_bracket"
  )

  # Compare specific counties within a state (requires census data)
  compare_energy_burden("fpl", "NC", counties = c("Orange", "Durham", "Wake"))

  # Custom grouping by tract-level geoid (requires census data)
  compare_energy_burden("ami", "NC", group_by = "geoid")
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatting.R
\name{to_percent}
\alias{to_percent}
\title{Format Number as Percentage}
\usage{
to_percent(x, latex = FALSE)
}
\arguments{
\item{x}{Numeric vector to format (as proportions, not percentages)}

\item{latex}{Logical indicating whether to escape percent sign for LaTeX
(default: FALSE)}
}
\value{
Character vector of formatted percentages
}
\description{
Converts numeric values to formatted percentage strings with no decimal
places by default.
}
\examples{
# Format percentages
to_percent(c(0.25, 0.50, 0.123))

# LaTeX-escaped format
to_percent(c(0.25, 0.50), latex = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache_utils.R
\name{detect_database_corruption}
\alias{detect_database_corruption}
\title{Detect potentially corrupted database data}
\usage{
detect_database_corruption(
  data,
  dataset,
  vintage,
  states = NULL,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame to check}

\item{dataset}{Character, "ami" or "fpl"}

\item{vintage}{Character, "2018" or "2022"}

\item{states}{Character vector of expected states (NULL = all US states)}

\item{verbose}{Logical, print warnings}
}
\value{
List with: is_corrupted (logical), issues (character vector), recommendation (character)
}
\description{
Checks if loaded data appears corrupted (too small, missing states, missing columns).
\strong{Does NOT automatically delete} - only warns and provides recommendations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/database-helpers.R
\name{clear_test_environment}
\alias{clear_test_environment}
\title{Clear Test Database and Cache}
\usage{
clear_test_environment()
}
\description{
Safe function to clear test database and cache for testing.
NEVER touches production database.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/database-helpers.R
\name{get_db_connection}
\alias{get_db_connection}
\title{Get Database Connection}
\usage{
get_db_connection(test = FALSE)
}
\arguments{
\item{test}{Logical, connect to test database instead of production}
}
\value{
DBI connection object
}
\description{
Get Database Connection
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{list_cohort_columns}
\alias{list_cohort_columns}
\title{List Available Columns in Cohort Data}
\usage{
list_cohort_columns(dataset = NULL, vintage = NULL)
}
\arguments{
\item{dataset}{Character, either "ami" or "fpl" (optional, affects available columns)}

\item{vintage}{Character, "2018" or "2022" (optional, affects available columns)}
}
\value{
Data frame with columns: column_name, description, data_type
}
\description{
Returns column names and descriptions for LEAD cohort datasets.
}
\examples{
list_cohort_columns()
list_cohort_columns("ami", "2022")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{download_and_merge_states}
\alias{download_and_merge_states}
\title{Download and merge data from multiple states}
\usage{
download_and_merge_states(dataset, vintage, states, verbose = TRUE)
}
\arguments{
\item{dataset}{Character, "ami" or "fpl"}

\item{vintage}{Character, "2018" or "2022"}

\item{states}{Character vector of state abbreviations}

\item{verbose}{Logical, print progress messages}
}
\value{
Combined tibble with data from all states
}
\description{
Download and merge data from multiple states
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_processing.R
\name{lead_to_poverty}
\alias{lead_to_poverty}
\title{Aggregate LEAD data by poverty status}
\usage{
lead_to_poverty(data, dataset)
}
\arguments{
\item{data}{A data frame of processed LEAD data (output from raw_to_lead)}

\item{dataset}{Character string indicating income metric: "ami", "fpl", or "smi"}
}
\value{
A data frame aggregated by geoid, poverty status, housing attributes
}
\description{
Consolidates LEAD cohort data by poverty status, aggregating households
and computing weighted averages for income and spending.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lead_data_loaders.R
\name{find_emburden_db}
\alias{find_emburden_db}
\title{Find emburden_db.sqlite database}
\usage{
find_emburden_db()
}
\description{
Find emburden_db.sqlite database
}
\keyword{internal}
